{********************************************************************}
{                                                                    }
{ written by TMS Software                                            }
{            Copyright (c) 2018 - 2021                               }
{            Email : info@tmssoftware.com                            }
{            Web : http://www.tmssoftware.com                        }
{                                                                    }
{ The source code is given as is. The author is not responsible      }
{ for any possible damage done due to the use of this code.          }
{ The complete source code remains property of the author and may    }
{ not be distributed, published, given or sold in any form as such.  }
{ No parts of the source code can be included in any other component }
{ or application without written authorization of the author.        }
{********************************************************************}

unit WEBLib.Buttons;

interface

uses
  {$IFNDEF LCLTMSWEB}
  System.Classes, Winapi.Windows, Winapi.Messages, Vcl.Controls, WEBLib.Controls,
  SysUtils, Vcl.Graphics, Types, WEBLib.ComCtrls, Vcl.Forms, Vcl.Buttons, Vcl.StdCtrls;
  {$ENDIF}

  {$IFDEF LCLTMSWEB}
  Classes, SysUtils, Types, LCLType, LCLIntf, Messages, Controls, StdCtrls,
  Graphics, Forms, Buttons, WEBLib.Controls, WEBLib.ComCtrls;
  {$ENDIF}

type
  TCustomWebSpeedButton = class(TCustomControl)
  private
  public
    constructor Create(ID: string); reintroduce; overload; virtual;
    constructor Create(AOwner: TComponent); overload; override;
  published
  end;

  {$IFNDEF LCLTMSWEB}
  [ComponentPlatforms(TMSWebPlatform)]
  {$ENDIF}
  TWebSpeedButton = class(TCustomWebSpeedButton)
  private
    FMaterialGlyph: TMaterialGlyph;
    FAllowUp: boolean;
    FGroupIndex: integer;
    FFlat: boolean;
    FGlyph: TPicture;
    FElementID: TElementID;
    FElementClassName: TElementClassName;
    FElementFont: TElementFont;
    FElementPosition: TElementPosition;
    FLayout: TButtonLayout;
    FChildOrder: integer;
    FGlyphURL: String;
    FDown: boolean;
    FMaterialGlyphColor: TColor;
    FMaterialGlyphSize: integer;
    procedure CMTextChanged(var Message: TMessage); message CM_TEXTCHANGED;
    procedure SetMaterialGlyph(const Value: TMaterialGlyph);
    procedure SetGlyph(const Value: TPicture);
    procedure SetLayout(const Value: TButtonLayout);
    procedure SetMaterialGlyphColor(const Value: TColor);
    procedure SetMaterialGlyphSize(const Value: integer);
  protected
    procedure Paint; override;
    procedure GlyphChanged(Sender: TObject);
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  published
    property Align;
    {$IFNDEF LCLTMSWEB}
    property AlignWithMargins;
    {$ENDIF}
    property Anchors;
    property AllowAllUp: boolean read FAllowUp write FAllowUp default false;
    property Caption;
    property ChildOrder: integer read FChildOrder write FChildOrder default 0;
    property Color;
    property Down: boolean read FDown write FDown default false;
    property Enabled;
    property ElementClassName: TElementClassName read FElementClassName write FElementClassName;
    property ElementID: TElementID read FElementID write FElementID;
    property ElementFont: TElementFont read FElementFont write FElementFont default efProperty;
    property ElementPosition: TElementPosition read FElementPosition write FElementPosition default epAbsolute;
    property Flat: boolean read FFlat write FFlat default false;
    property Font;
    property Glyph: TPicture read FGlyph write SetGlyph;
    property GlyphURL: String read FGlyphURL write FGlyphURL;
    property GroupIndex: integer read FGroupIndex write FGroupIndex default 0;
    property Height;
    property Hint;
    property Layout: TButtonLayout read FLayout write SetLayout default blGlyphLeft;
    {$IFNDEF LCLTMSWEB}
    property Margins;
    {$ENDIF}
    property MaterialGlyph: TMaterialGlyph read FMaterialGlyph write SetMaterialGlyph;
    property MaterialGlyphColor: TColor read FMaterialGlyphColor write SetMaterialGlyphColor default clNone;
    property MaterialGlyphSize: integer read FMaterialGlyphSize write SetMaterialGlyphSize default 18;
    property ParentFont;
    property ShowHint;
    property Visible;
    property Width;
    property OnClick;
    property OnDblClick;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;
    property OnMouseDown;
    property OnMouseUp;
    property OnMouseMove;
    property OnMouseLeave;
    property OnMouseEnter;
    property OnEnter;
    property OnExit;
  end;

  TWebCustomBitBtn = class(TCustomControl)
  public
    constructor Create(ID: string); reintroduce; overload; virtual;
    constructor Create(AOwner: TComponent); overload; override;
  end;

  {$IFNDEF LCLTMSWEB}
  [ComponentPlatforms(TMSWebPlatform)]
  {$ENDIF}
  TWebBitBtn = class(TWebCustomBitBtn)
  private
    FMaterialGlyph: TMaterialGlyph;
    FFlat: boolean;
    FGlyph: TPicture;
    FLayout: TButtonLayout;
    FAlignment: TAlignment;
    FElementID: TElementID;
    FElementClassName: TElementClassName;
    FElementFont: TElementFont;
    FElementPosition: TElementPosition;
    FWidthStyle: TSizeStyle;
    FHeightPercent: TPercentSize;
    FHeightStyle: TSizeStyle;
    FWidthPercent: TPercentSize;
    FChildOrder: integer;
    FGlyphURL: String;
    FMaterialGlyphColor: TColor;
    FMaterialGlyphSize: integer;
    procedure CMTextChanged(var Message: TMessage); message CM_TEXTCHANGED;
    procedure SetGlyph(const Value: TPicture);
    procedure SetMaterialGlyph(const Value: TMaterialGlyph);
    procedure SetLayout(const Value: TButtonLayout);
    procedure SetAlignment(const Value: TAlignment);
    procedure SetHeightPercent(const Value: TPercentSize);
    procedure SetHeightStyle(const Value: TSizeStyle);
    procedure SetWidthPercent(const Value: TPercentSize);
    procedure SetWidthStyle(const Value: TSizeStyle);
    procedure GlyphChanged(Sender: TObject);
    procedure SetMaterialGlyphColor(const Value: TColor);
    procedure SetMaterialGlyphSize(const Value: integer);
  protected
    procedure Paint; override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  published
    property Align;
    {$IFNDEF LCLTMSWEB}
    property AlignWithMargins;
    {$ENDIF}
    property Alignment: TAlignment read FAlignment write SetAlignment default taCenter;
    property Anchors;
    property Caption;
    property ChildOrder: integer read FChildOrder write FChildOrder default 0;
    property Color;
    property DragMode;
    property ElementClassName: TElementClassName read FElementClassName write FElementClassName;
    property ElementID: TElementID read FElementID write FElementID;
    property ElementFont: TElementFont read FElementFont write FElementFont default efProperty;
    property ElementPosition: TElementPosition read FElementPosition write FElementPosition default epAbsolute;
    property Enabled;
    property Flat: boolean read FFlat write FFlat default false;
    property Font;
    property Glyph: TPicture read FGlyph write SetGlyph;
    property GlyphURL: String read FGlyphURL write FGlyphURL;
    property HeightStyle: TSizeStyle read FHeightStyle write SetHeightStyle default ssAbsolute;
    property HeightPercent: TPercentSize read FHeightPercent write SetHeightPercent;
    property Layout: TButtonLayout read FLayout write SetLayout default blGlyphLeft;
    property MaterialGlyph: TMaterialGlyph read FMaterialGlyph write SetMaterialGlyph;
    property MaterialGlyphColor: TColor read FMaterialGlyphColor write SetMaterialGlyphColor default clNone;
    property MaterialGlyphSize: integer read FMaterialGlyphSize write SetMaterialGlyphSize default 18;
    property Height;
    property Hint;
    property ParentFont;
    property ShowHint;
    property TabOrder;
    property TabStop;
    property Visible;

    property WidthStyle: TSizeStyle read FWidthStyle write SetWidthStyle default ssAbsolute;
    property WidthPercent: TPercentSize read FWidthPercent write SetWidthPercent;

    property Width;
    property OnClick;
    property OnDblClick;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;
    property OnMouseDown;
    property OnMouseUp;
    property OnMouseMove;
    property OnMouseLeave;
    property OnMouseEnter;
    property OnEnter;
    property OnExit;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnStartDrag;
  end;

  TCustomToolBar = class(TWebCustomControl)
  private
  public
    constructor Create(AOwner: TComponent); override;
  published
  end;

  {$IFNDEF LCLTMSWEB}
  [ComponentPlatforms(TMSWebPlatform)]
  {$ENDIF}
  TWebToolBar = class(TCustomToolBar)
  private
    FElementClassName: TElementClassName;
    FElementID: TElementID;
    FElementFont: TElementFont;
    FElementPosition: TElementPosition;
    FChildOrder: integer;
  public
    constructor Create(AOwner: TComponent); override;
  published
    property Align;
    {$IFNDEF LCLTMSWEB}
    property AlignWithMargins;
    {$ENDIF}
    property Color default $CFCFCF;
    property DragMode;
    property ChildOrder: integer read FChildOrder write FChildOrder default 0;
    property Enabled;
    property ElementClassName: TElementClassName read FElementClassName write FElementClassName;
    property ElementID: TElementID read FElementID write FElementID;
    property ElementFont: TElementFont read FElementFont write FElementFont default efProperty;
    property ElementPosition: TElementPosition read FElementPosition write FElementPosition default epAbsolute;
    property Height;
    property Hint;
    property Left;
    {$IFNDEF LCLTMSWEB}
    property Margins;
    {$ENDIF}
    property ShowHint;
    property Top;
    property Visible;
    property Width;
    property OnClick;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnStartDrag;
  end;


  TRichEditBtn = (reFont, reFontSize, reBold, reItalic, reUnderline, reStrikeThrough,
    reAlignLeft, reAlignCenter, reAlignRight, reUnorderedList, reOrderedList, reForegroundColor, reBackgroundColor);

  TRichEditButtonSet = set of TRichEditBtn;

  {$IFNDEF LCLTMSWEB}
  [ComponentPlatforms(TMSWebPlatform)]
  {$ENDIF}
  TWebRichEditToolBar = class(TWebToolBar)
  private
    FTextColor: TColor;
    FRichEdit: TWebRichEdit;
    FOnSpeedButtonClick: TNotifyEvent;
    FBackgroundColor: TColor;
    FVisibleButtons: TRichEditButtonSet;
    FHints: TStrings;
    FFontSizePicker: TComboBox;
    FFontPicker: TComboBox;
    procedure SetHints(const Value: TStrings);
  protected
    procedure Paint; override;
  public
    constructor Create(ID: string); reintroduce; overload; virtual;
    constructor Create(AOwner: TComponent); overload; override;
    destructor Destroy; override;
    property OnSpeedButtonClick: TNotifyEvent read FOnSpeedButtonClick write FOnSpeedButtonClick;
    property FontSizePicker: TComboBox read FFontSizePicker;
    property FontPicker: TComboBox read FFontPicker;
  published
    property TextColor: TColor read FTextColor write FTextColor default clBlack;
    property BackgroundColor: TColor read FBackgroundColor write FBackgroundColor default clWhite;
    property Hints: TStrings read FHints write SetHints;
    property RichEdit: TWebRichEdit read FRichEdit write FRichEdit;
    property VisibleButtons: TRichEditButtonSet read FVisibleButtons write FVisibleButtons default [reFont, reFontSize, reBold, reItalic, reUnderline, reStrikeThrough,
      reAlignLeft, reAlignCenter, reAlignRight, reUnorderedList, reOrderedList, reForegroundColor, reBackgroundColor];
  end;

  TToggleButtonStyle = (tsRectangular, tsRounded);

  {$IFNDEF LCLTMSWEB}
  [ComponentPlatforms(TMSWebPlatform)]
  {$ENDIF}
  TWebToggleButton = class(TWebCSSControl)
  private
    FChecked: boolean;
    FStyle: TToggleButtonStyle;
    FElementFont: TElementFont;
    FElementID: TElementID;
    FElementPosition: TElementPosition;
    FElementClassName: TElementClassName;
    FChildOrder: integer;
    procedure SetChecked(const Value: boolean);
  protected
    procedure Paint; override;
  public
    constructor Create(AOwner: TComponent); override;
    procedure SetBounds(ALeft, ATop, AWidth, AHeight: integer); override;
    destructor Destroy; override;
  published
    property Align;
    {$IFNDEF LCLTMSWEB}
    property AlignWithMargins;
    {$ENDIF}
    property Anchors;
    property Checked: boolean read FChecked write SetChecked default false;
    property ChildOrder: integer read FChildOrder write FChildOrder default 0;
    property DragMode;
    property ElementClassName: TElementClassName read FElementClassName write FElementClassName;
    property ElementID: TElementID read FElementID write FElementID;
    property ElementFont: TElementFont read FElementFont write FElementFont default efProperty;
    property ElementPosition: TElementPosition read FElementPosition write FElementPosition default epAbsolute;
    property Style: TToggleButtonStyle read FStyle write FStyle default tsRectangular;
    property Visible;
    property OnClick;
    property OnMouseDown;
    property OnMouseUp;
    property OnMouseMove;
    property OnMouseLeave;
    property OnMouseEnter;
    property OnEnter;
    property OnExit;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnStartDrag;
  end;

implementation

uses
  {$IFNDEF LCLTMSWEB}
  System.UITypes,
  {$ENDIF}
  WEBLib.Graphics;

{ TWebRichEditToolBar }

constructor TWebRichEditToolBar.Create(ID: string);
begin
  Create(nil);
end;

{ TCustomWebSpeedButton }

constructor TCustomWebSpeedButton.Create(ID: string);
begin
  Create(nil);
end;

constructor TCustomWebSpeedButton.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
end;

constructor TWebRichEditToolBar.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FVisibleButtons := [reFont, reFontSize, reBold, reItalic, reUnderline, reStrikeThrough,
    reAlignLeft, reAlignCenter, reAlignRight, reUnorderedList, reOrderedList, reForegroundColor, reBackgroundColor];
  Width := 540;
  FHints := TStringList.Create;

  FHints.Add('Bold');
  FHints.Add('Italic');
  FHints.Add('Underline');
  FHints.Add('Strike Throught');

  FHints.Add('Text Color');
  FHints.Add('Background Color');
  FHints.Add('Align lef');
  FHints.Add('Align center');
  FHints.Add('Align right');
  FHints.Add('Numbered list');
  FHints.Add('List');

  FTextColor := clBlack;
  FBackgroundColor := clWhite;
end;

destructor TWebRichEditToolBar.Destroy;
begin
  FHints.Free;
  inherited;
end;

procedure TWebRichEditToolBar.Paint;
const
  btn: array[0..10] of string = ('B','I','U','S','FG','BK','L','C','R','UL','OL');
var
  i: integer;

  procedure DrawButton(R: TRect; s: string);
  begin
    Canvas.Brush.Color := clBtnFace;
    Canvas.Brush.Style := bsSolid;
    Canvas.Pen.Color := clBlack;
    Canvas.Pen.Width := 1;
    Canvas.Rectangle(R);
    Canvas.Pen.Color := clWhite;
    Canvas.MoveTo(R.Left + 1, R.Top + 1);
    Canvas.LineTo(R.Right - 1, R.Top + 1);
    Canvas.MoveTo(R.Left + 1, R.Top + 1);
    Canvas.LineTo(R.Left + 1, R.Bottom - 1);
    DrawText(Canvas.Handle, PChar(s), Length(s), R, DT_CENTER or DT_SINGLELINE or DT_VCENTER);
  end;


begin
  inherited;

  DrawButton(Rect(0,0,94,31), 'Fontname');
  DrawButton(Rect(96,0,96+40,31), 'Size');

  for i := 0 to 10 do
  begin
    DrawButton(Rect(132 + i *37,0,132 + 37 + i*37,31), btn[i]);
  end;
end;

procedure TWebRichEditToolBar.SetHints(const Value: TStrings);
begin
  FHints.Assign(Value);
end;

{ TCustomToolBar }

constructor TCustomToolBar.Create(AOwner: TComponent);
begin
  inherited;
  Height := 32;
  Width := 300;
  ControlStyle := ControlStyle + [csAcceptsControls];
end;

{ TWebSpeedButton }

procedure TWebSpeedButton.CMTextChanged(var Message: TMessage);
begin
  Invalidate;
end;

constructor TWebSpeedButton.Create(AOwner: TComponent);
begin
  inherited;
  Flat := false;
  Width := 24;
  Height := 24;
  Caption := Name;
  ControlStyle := ControlStyle + [csSetCaption];
  FGlyph := TPicture.Create;
  FGlyph.OnChange := GlyphChanged;
  FElementFont := efProperty;
  FElementPosition := epAbsolute;
  FMaterialGlyphSize := 18;
  FMaterialGlyphColor := clNone;
end;

destructor TWebSpeedButton.Destroy;
begin
  FGlyph.Free;
  inherited;
end;

procedure TWebSpeedButton.GlyphChanged(Sender: TObject);
begin
  Invalidate;
end;

procedure TWebSpeedButton.Paint;
var
  r: TRect;
  ch: char;
  s: string;
  d: integer;
begin
  inherited;
  r := ClientRect;

  Canvas.Brush.Color := Color;
  Canvas.Brush.Style := bsSolid;
  Canvas.Pen.Style := psSolid;
  Canvas.Pen.Width := 1;
  Canvas.Rectangle(r);
  Canvas.Font.Assign(Font);


  InflateRect(r,-1,-1);

  if Assigned(Glyph) and Assigned(Glyph.Graphic) and not Glyph.Graphic.Empty then
  begin
    case Layout of
    blGlyphLeft:
      begin
        d := (r.Bottom - r.Top - Glyph.Height) div 2;
        Canvas.Draw(r.Left + 2, r.Top + d, Glyph.Graphic);
        r.Left := r.Left + Glyph.Width + 4;
      end;
    blGlyphRight:
      begin
        d := (r.Bottom - r.Top - Glyph.Height) div 2;
        Canvas.Draw(r.Right - 2 - Glyph.Width, r.Top + d, Glyph.Graphic);
        r.Right := r.Right - Glyph.Width - 4;
      end;
    blGlyphTop:
      begin
        d := (r.Right - r.Left - Glyph.Width) div 2;
        Canvas.Draw(r.Left + d, r.Top + 2, Glyph.Graphic);
        r.Top := r.Top + Glyph.Height + 4;
      end;
    blGlyphBottom:
      begin
        d := (r.Right - r.Top - Glyph.Width) div 2;
        Canvas.Draw(r.Left + d, r.Bottom - 2 - Glyph.Height, Glyph.Graphic);
        r.Bottom := r.Bottom - 4 - Glyph.Height;
      end;
    end;

  end
  else
  begin
    if (MaterialGlyph <> '') then
    begin
      Canvas.Font.Name := 'Material Icons';
      Canvas.Font.Size := Round(FMaterialGlyphSize * 72/96);

      if FMaterialGlyphColor = clNone then
        Canvas.Font.Color := clBlack
      else
        Canvas.Font.Color := FMaterialGlyphColor;

      //&#xE5DC;
      if (pos('&#', MaterialGlyph) = 1) then
      begin
        ch := chr(HTMLChar(MaterialGlyph));
        s := ch;
      end
      else
        s := MaterialGlyph;

      if (Caption = '') then
        DrawText(Canvas.Handle, PChar(s), Length(s), r, DT_SINGLELINE or DT_CENTER or DT_VCENTER)
      else
      begin

        case Layout of
        blGlyphLeft:
          begin
            DrawText(Canvas.Handle, PChar(s), Length(s), r, DT_SINGLELINE or DT_LEFT or DT_VCENTER);
            r.Left := r.Left + 20;
          end;
        blGlyphRight:
          begin
            DrawText(Canvas.Handle, PChar(s), Length(s), r, DT_SINGLELINE or DT_RIGHT or DT_VCENTER);
            r.Right := r.Right - 20;
          end;
        blGlyphTop:
          begin
            DrawText(Canvas.Handle, PChar(s), Length(s), r, DT_SINGLELINE or DT_CENTER or DT_TOP);
            r.Top := r.Top + 20;
          end;
        blGlyphBottom:
          begin
            DrawText(Canvas.Handle, PChar(s), Length(s), r, DT_SINGLELINE or DT_CENTER or DT_BOTTOM);
            r.Bottom := r.Bottom - 20;
          end;
        end;
      end;

      Canvas.Font.Assign(Font);
    end;
  end;


  DrawText(Canvas.Handle, PChar(Caption), Length(Caption),r, DT_SINGLELINE or DT_CENTER or DT_VCENTER);
end;

procedure TWebSpeedButton.SetGlyph(const Value: TPicture);
begin
  FGlyph.Assign(Value);
end;

procedure TWebSpeedButton.SetLayout(const Value: TButtonLayout);
begin
  if (FLayout <> Value) then
  begin
    FLayout := Value;
    Invalidate;
  end;
end;

procedure TWebSpeedButton.SetMaterialGlyph(const Value: TMaterialGlyph);
begin
  if (FMaterialGlyph <> Value) then
  begin
    FMaterialGlyph := Value;
    Invalidate;
  end;
end;

procedure TWebSpeedButton.SetMaterialGlyphColor(const Value: TColor);
begin
  if (FMaterialGlyphColor <> Value) then
  begin
    FMaterialGlyphColor := Value;
    Invalidate;
  end;
end;

procedure TWebSpeedButton.SetMaterialGlyphSize(const Value: integer);
begin
  if (FMaterialGlyphSize <> Value) and (Value > 0) and (Value <100) then
  begin
    FMaterialGlyphSize := Value;
    Invalidate;
  end;
end;

{ TWebToggleButton }

constructor TWebToggleButton.Create(AOwner: TComponent);
begin
  inherited;

  FChecked := false;
  FElementPosition := epAbsolute;
  FElementFont := efProperty;
end;

destructor TWebToggleButton.Destroy;
begin
  inherited;
end;

procedure TWebToggleButton.Paint;
var
  r: TRect;
begin
  inherited;

  R := Rect(0,0,44,22);

  Canvas.Brush.Color := clHighlight;
  Canvas.Brush.Style := bsSolid;
  Canvas.Pen.Color := Canvas.Brush.Color;
  Canvas.Rectangle(R);

  if not FChecked then
    R := Rect(3,3, 19, 19)
  else
    R := Rect(22, 3, 22 + 19, 19);

  Canvas.Brush.Color := clWhite;
  Canvas.Pen.Color := Canvas.Brush.Color;
  Canvas.Rectangle(R);
end;

procedure TWebToggleButton.SetBounds(ALeft, ATop, AWidth, AHeight: integer);
begin
  AWidth := 44;
  AHeight := 22;
  inherited SetBounds(ALeft, ATop, AWidth, AHeight);
end;

procedure TWebToggleButton.SetChecked(const Value: boolean);
begin
  FChecked := Value;
  Repaint;
end;

{ TWebBitBtn }

procedure TWebBitBtn.CMTextChanged(var Message: TMessage);
begin
  Invalidate;
end;

constructor TWebBitBtn.Create(AOwner: TComponent);
begin
  inherited;
  Flat := false;
  Width := 80;
  Height := 25;
  Caption := Name;
  Color := clNone;
  ControlStyle := ControlStyle + [csSetCaption];
  FGlyph := TPicture.Create;
  FGlyph.OnChange := GlyphChanged;
  FAlignment := taCenter;
  FElementPosition := epAbsolute;
  FElementFont := efProperty;

  FWidthStyle := ssAbsolute;
  FWidthPercent := 100;
  FHeightStyle := ssAbsolute;
  FHeightPercent := 100;
  FMaterialGlyphSize := 18;
  FMaterialGlyphColor := clNone;
end;

destructor TWebBitBtn.Destroy;
begin
  FGlyph.Free;
  inherited;
end;

procedure TWebBitBtn.GlyphChanged(Sender: TObject);
begin
  Invalidate;
end;

procedure TWebBitBtn.Paint;
var
  r: TRect;
  ch: char;
  s: string;
  d: integer;
  flg: DWORD;
begin
  inherited;
  r := ClientRect;

  Canvas.Brush.Color := Color;

  if Color = clNone then
    Canvas.Brush.Color := clBtnFace;

  Canvas.Brush.Style := bsSolid;
  Canvas.Pen.Style := psSolid;
  Canvas.Pen.Width := 1;
  Canvas.Rectangle(r);
  Canvas.Font.Assign(Font);

  InflateRect(r,-1,-1);

  flg := DT_CENTER;

  case Alignment of
  taLeftJustify: flg := DT_LEFT;
  taRightJustify: flg := DT_RIGHT;
  end;

  if (Layout in [blGlyphLeft, blGlyphRight]) then
  begin
    if (MaterialGlyph <> '') then
    begin
      Canvas.Font.Name := 'Material Icons';
      Canvas.Font.Size := Round(FMaterialGlyphSize * 72/96);

      if FMaterialGlyphColor = clNone then
        Canvas.Font.Color := clBlack
      else
        Canvas.Font.Color := FMaterialGlyphColor;


      //&#xE5DC;
      if (pos('&#', MaterialGlyph) = 1) then
      begin
        ch := chr(HTMLChar(MaterialGlyph));
        s := ch;
      end
      else
        s := MaterialGlyph;

      if Layout = blGlyphLeft then
      begin
        DrawText(Canvas.Handle, PChar(s), Length(s), r, DT_SINGLELINE or DT_LEFT or DT_VCENTER);
        r.Left := r.Left + 20;
      end
      else
      begin
        DrawText(Canvas.Handle, PChar(s), Length(s), r, DT_SINGLELINE or DT_RIGHT or DT_VCENTER);
        r.Right := r.Right - 20;
      end;

      Canvas.Font.Assign(Font);
    end;

    if Assigned(Glyph) and Assigned(Glyph.Graphic) and not Glyph.Graphic.Empty then
    begin
      d := (r.Bottom - r.Top - Glyph.Height) div 2;

      if Layout = blGlyphLeft then
      begin
        Canvas.Draw(r.Left + 2, r.Top + d, Glyph.Graphic);
        r.Left := r.Left + Glyph.Width + 4;
      end
      else
      begin
        Canvas.Draw(r.Right - Glyph.Width - 2, r.Top + d, Glyph.Graphic);
        r.Right := r.Right - Glyph.Width - 4;
      end;
    end;

    DrawText(Canvas.Handle, PChar(Caption), Length(Caption),r, DT_SINGLELINE or flg or DT_VCENTER);
  end
  else
  begin
    if (MaterialGlyph <> '') then
    begin
      Canvas.Font.Name := 'Material Icons';
      Canvas.Font.Size := 14;

      //&#xE5DC;
      if (pos('&#', MaterialGlyph) = 1) then
      begin
        ch := chr(HTMLChar(MaterialGlyph));
        s := ch;
      end
      else
        s := MaterialGlyph;

      if Layout = blGlyphTop then
      begin
        r.Top := r.Top + 2;
        DrawText(Canvas.Handle, PChar(s), Length(s), r, DT_SINGLELINE or DT_LEFT or DT_TOP);
        r.Top := r.Top + 20;
      end
      else
      begin
        r.Bottom := r.Bottom - 2;
        DrawText(Canvas.Handle, PChar(s), Length(s), r, DT_SINGLELINE or DT_RIGHT or DT_BOTTOM);
        r.Bottom := r.Bottom - 20;
      end;

      Canvas.Font.Assign(Font);
    end;

    if Assigned(Glyph) and Assigned(Glyph.Graphic) and not Glyph.Graphic.Empty then
    begin
      d := (r.Right - r.Left - Glyph.Width) div 2;

      if Layout = blGlyphTop then
      begin
        Canvas.Draw(r.Left + d, r.Top + 2, Glyph.Graphic);
        r.Top := r.Top + Glyph.Height + 4;
      end
      else
      begin
        Canvas.Draw(r.Left + d, r.Bottom - 2 - Glyph.Height, Glyph.Graphic);
        r.Bottom := r.Bottom - Glyph.Height - 4;
      end;
    end;

    DrawText(Canvas.Handle, PChar(Caption), Length(Caption),r, DT_SINGLELINE or flg or DT_VCENTER);
  end;
end;

procedure TWebBitBtn.SetAlignment(const Value: TAlignment);
begin
  FAlignment := Value;
  Invalidate;
end;

procedure TWebBitBtn.SetGlyph(const Value: TPicture);
begin
  FGlyph.Assign(Value);
  Invalidate;
end;

procedure TWebBitBtn.SetHeightPercent(const Value: TPercentSize);
begin
  FHeightPercent := Value;
end;

procedure TWebBitBtn.SetHeightStyle(const Value: TSizeStyle);
begin
  FHeightStyle := Value;
end;

procedure TWebBitBtn.SetLayout(const Value: TButtonLayout);
begin
  if (FLayout <> Value) then
  begin
    FLayout := Value;
    Invalidate;
  end;
end;

procedure TWebBitBtn.SetMaterialGlyph(const Value: TMaterialGlyph);
begin
  if (FMaterialGlyph <> Value) then
  begin
    FMaterialGlyph := Value;
    Invalidate;
  end;
end;

procedure TWebBitBtn.SetMaterialGlyphColor(const Value: TColor);
begin
  if (FMaterialGlyphColor <> Value) then
  begin
    FMaterialGlyphColor := Value;
    Invalidate;
  end;
end;

procedure TWebBitBtn.SetMaterialGlyphSize(const Value: integer);
begin
  if (FMaterialGlyphSize <> Value) and (Value >0) and (Value <100) then
  begin
    FMaterialGlyphSize := Value;
    Invalidate;
  end;
end;

procedure TWebBitBtn.SetWidthPercent(const Value: TPercentSize);
begin
  FWidthPercent := Value;
end;

procedure TWebBitBtn.SetWidthStyle(const Value: TSizeStyle);
begin
  FWidthStyle := Value;
end;

{ TWebCustomBitBtn }

constructor TWebCustomBitBtn.Create(ID: string);
begin
  Create(nil);
end;

constructor TWebCustomBitBtn.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
end;

{ TWebToolBar }

constructor TWebToolBar.Create(AOwner: TComponent);
begin
  inherited;
  FElementFont := efProperty;
  FElementPosition := epAbsolute;
  Color := $CFCFCF;
end;

end.
