{********************************************************************}
{                                                                    }
{ written by TMS Software                                            }
{            Copyright (c) 2018 - 2019                               }
{            Email : info@tmssoftware.com                            }
{            Web : http://www.tmssoftware.com                        }
{                                                                    }
{ The source code is given as is. The author is not responsible      }
{ for any possible damage done due to the use of this code.          }
{ The complete source code remains property of the author and may    }
{ not be distributed, published, given or sold in any form as such.  }
{ No parts of the source code can be included in any other component }
{ or application without written authorization of the author.        }
{********************************************************************}

unit WEBLib.EditAutocomplete;

interface

uses
  {$IFNDEF LCLTMSWEB}
  System.Classes, Winapi.Windows, SysUtils, Types, VCL.Controls, VCL.Graphics,
  WEBLib.Controls, WEBLib.StdCtrls, WEBLib.Http, System.UITypes, VCL.StdCtrls,
  VCL.Imaging.pngimage, VCL.Imaging.JPEG, VCL.Imaging.GIFImg, VCL.Dialogs, Web,
  VCL.Forms;
  {$ENDIF}
  {$IFDEF LCLTMSWEB}
  Classes, SysUtils, Types, Controls, Graphics,
  WEBLib.Controls, WEBLib.StdCtrls, WEBLib.Http, StdCtrls,
  Dialogs, Web;
  {$ENDIF}

type
  TEditAutoCompleteLookupType = (ltFirstCharacter, ltAnywhere);

  TEditAutoCompleteRenderItemEventArgs = class(TPersistent)
  private
    FItemElement: TJSHTMLElement;
  public
    property ItemElement: TJSHTMLElement read FItemElement write FItemElement;
  end;

  TEditAutoCompleteRenderItemEvent = procedure(Sender: TObject; Args: TEditAutoCompleteRenderItemEventArgs) of object;

  TEditAutoCompleteSelectEventArgs = class(TPersistent)
  private
    FItemIndex: Integer;
  public
    property ItemIndex: Integer read FItemIndex write FItemIndex;
  end;

  TEditAutoCompleteSelectEvent = procedure(Sender: TObject; Args: TEditAutoCompleteSelectEventArgs) of object;

  {$IFNDEF LCLTMSWEB}
  [ComponentPlatforms(TMSWebPlatform)]
  {$ENDIF}
  TWebEditAutoComplete = class(TWebCustomInput)
  strict private
    FOnChange: TNotifyEvent;
  private
    FOnRenderItem: TEditAutoCompleteRenderItemEvent;
    FOnSelect: TEditAutoCompleteSelectEvent;
    FReadOnly: boolean;
    FTextHint: string;
    FItemIndex: Integer;
    FPopupHeight: Integer;
    FItems: TStrings;
    FActiveItemClassName: string;
    FPopupClassName: string;
    FText: string;
    FItemClassName: string;
    FLookupCaseSensitive: Boolean;
    FlookupMinLength: Integer;
    FLookupType: TEditAutoCompleteLookupType;
    FShowFocus: boolean;
    procedure SetItems(const Value: TStrings);
    procedure SetItemIndex(const Value: Integer);
    procedure SetText(const Value: string);
  protected
    procedure Paint; override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  published
    property Align;
    {$IFNDEF LCLTMSWEB}
    property AlignWithMargins;
    {$ENDIF}
    property Anchors;
    property BorderStyle;
    property Color;
    property ElementClassName;
    property ElementID;
    property ElementFont;
    property ElementPosition;
    property Enabled;
    property Font;
    property Height;
    property HeightPercent;
    property Hint;
    property Left;
    {$IFNDEF LCLTMSWEB}
    property Margins;
    {$ENDIF}
    property ReadOnly: boolean read FReadOnly write FReadOnly default False;
    property ShowHint;
    property TextHint: string read FTextHint write FTextHint;
    property TabStop;
    property TabOrder;
    property Visible;
    property Width;
    property WidthPercent;
    property OnChange: TNotifyEvent read FOnChange write FOnChange;
    property OnClick;
    property OnDblClick;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;
    property OnMouseDown;
    property OnMouseUp;
    property OnMouseMove;
    property OnMouseLeave;
    property OnMouseEnter;
    property OnEnter;
    property OnExit;

    property Items: TStrings read FItems write SetItems;
    property ItemIndex: Integer read FItemIndex write SetItemIndex default -1;
    property ShowFocus: boolean read FShowFocus write FShowFocus default true;
    property Text: string read FText write SetText;
    property PopupHeight: Integer read FPopupHeight write FPopupHeight default 0;
    property PopupClassName: string read FPopupClassName write FPopupClassName;
    property ItemClassName: string read FItemClassName write FItemClassName;
    property ActiveItemClassName: string read FActiveItemClassName write FActiveItemClassName;
    property LookupType: TEditAutoCompleteLookupType read FLookupType write FLookupType default ltFirstCharacter;
    property LookupCaseSensitive: Boolean read FLookupCaseSensitive write FLookupCaseSensitive default False;
    property LookupMinLength: Integer read FlookupMinLength write FLookupMinLength default 1;

    property OnRenderItem: TEditAutoCompleteRenderItemEvent read FOnRenderItem write FOnRenderItem;
    property OnSelect: TEditAutoCompleteSelectEvent read FOnSelect write FOnSelect;
  end;


implementation

uses
  {$IFNDEF LCLTMSWEB}
  RTTI, TypInfo;
  {$ENDIF}

  {$IFDEF LCLTMSWEB}
  TypInfo;
  {$ENDIF}

{ TWebEditAutoComplete }

constructor TWebEditAutoComplete.Create(AOwner: TComponent);
begin
  inherited;
  ReadOnly := False;
  TabStop := True;
  Width := 121;
  Height := 21;
  FItemIndex := -1;
  FItems := TStringList.Create;
  FLookupType := ltFirstCharacter;
  FLookupMinLength := 1;
  FLookupCaseSensitive := False;
  FShowFocus := true;
end;

destructor TWebEditAutoComplete.Destroy;
begin
  FItems.Free;
  inherited;
end;

procedure TWebEditAutoComplete.Paint;
var
  lRect: TRect;
begin
  if Color = clNone then
    Canvas.Brush.Color := clWindow
  else
    Canvas.Brush.Color := Color;

  Canvas.Brush.Style := bsSolid;
  Canvas.Pen.Style := psSolid;

  if BorderStyle = bsSingle then
    Canvas.Pen.Color := BorderColor
  else
    Canvas.Pen.Color := Canvas.Brush.Color;

  if ElementFont = efProperty then
    Canvas.Font.Assign(Font);

  lRect := ClientRect;
  Canvas.Rectangle(lRect);

  if Text <> '' then
    Canvas.TextOut(4,4, Text)
  else
    Canvas.TextOut(4,4, Name);
end;

procedure TWebEditAutoComplete.SetItemIndex(const Value: Integer);
begin
  if Value < Items.Count then
  begin
    FItemIndex := Value;
    Text := Items[Value];
  end;
end;

procedure TWebEditAutoComplete.SetItems(const Value: TStrings);
begin
  FItems.Assign(Value);
end;

procedure TWebEditAutoComplete.SetText(const Value: string);
begin
  FText := Value;
  Invalidate;
end;

end.
