{********************************************************************}
{                                                                    }
{ written by TMS Software                                            }
{            Copyright (c) 2018 - 2020                               }
{            Email : info@tmssoftware.com                            }
{            Web : http://www.tmssoftware.com                        }
{                                                                    }
{ The source code is given as is. The author is not responsible      }
{ for any possible damage done due to the use of this code.          }
{ The complete source code remains property of the author and may    }
{ not be distributed, published, given or sold in any form as such.  }
{ No parts of the source code can be included in any other component }
{ or application without written authorization of the author.        }
{********************************************************************}

unit WEBLib.ExtCtrls;

interface

uses
  {$IFNDEF LCLTMSWEB}
  System.Classes, Winapi.Windows, SysUtils, Types, VCL.Controls, VCL.Graphics,
  WEBLib.Controls, WEBLib.StdCtrls, WEBLib.Http, System.UITypes, VCL.StdCtrls,
  VCL.Imaging.pngimage, VCL.Imaging.JPEG, VCL.Imaging.GIFImg, VCL.Dialogs, Web;
  {$ENDIF}
  {$IFDEF LCLTMSWEB}
  Classes, SysUtils, Types, LCLType, LCLIntf, Controls, Graphics,
  WEBLib.Controls, WEBLib.StdCtrls, WEBLib.Http, StdCtrls,
  Dialogs, Web;
  {$ENDIF}

type
  TCustomWebLinkLabel = class(TWebCustomLabel)
  strict private
    FOnLinkClick: TLinkClickEvent;
  private
    FWidthStyle: TSizeStyle;
    FHeightPercent: TPercentSize;
    FHeightStyle: TSizeStyle;
    FWidthPercent: TPercentSize;
    FElementHandle: TJSHTMLElement;
    FParentElementID: TJSElement;
    FParentElement: TJSElement;
    procedure SetHeightPercent(const Value: TPercentSize);
    procedure SetHeightStyle(const Value: TSizeStyle);
    procedure SetWidthPercent(const Value: TPercentSize);
    procedure SetWidthStyle(const Value: TSizeStyle);
  protected
    function GetLabelText: string; override;
    property Caption;
  public
    constructor Create(AOwner: TComponent); override;
    property ElementHandle: TJSHTMLElement read FElementHandle;
    property ParentElement: TJSElement read FParentElement write FParentElement;
    property ParentElementID: TJSElement read FParentElementID write FParentElementID;
  published
    property Align;
    property Alignment;
    {$IFNDEF LCLTMSWEB}
    property AlignWithMargins;
    {$ENDIF}
    property Anchors;
    property AutoSize;
    property ChildOrder;
    property Color;
    property DragMode;
    property ElementClassName;
    property ElementID;
    property ElementFont;
    {$IFNDEF LCLTMSWEB}
    property EllipsisPosition;
    {$ENDIF}
    property Enabled;
    property Font;
    property Height;
    property Hint;
    property Layout;
    property Left;
    {$IFNDEF LCLTMSWEB}
    property Margins;
    {$ENDIF}
    property ParentFont;
    property Role;
    property ShowHint;
    property Top;
    property Visible;
    property Width;
    property WordWrap;

    property HeightStyle: TSizeStyle read FHeightStyle write SetHeightStyle default ssAbsolute;
    property WidthStyle: TSizeStyle read FWidthStyle write SetWidthStyle default ssAbsolute;
    property HeightPercent: TPercentSize read FHeightPercent write SetHeightPercent;
    property WidthPercent: TPercentSize read FWidthPercent write SetWidthPercent;

    property OnClick;
    property OnDblClick;
    property OnLinkClick: TLinkClickEvent read FOnLinkClick write FOnLinkClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnStartDrag;
  end;

  {$IFNDEF LCLTMSWEB}
  [ComponentPlatforms(TMSWebPlatform)]
  {$ENDIF}
  TWebLinkLabel = class(TCustomWebLinkLabel)
  published
    property Caption;
  end;

  TTrackBarOrientation = (trHorizontal, trVertical);

  {$IFNDEF LCLTMSWEB}
  [ComponentPlatforms(TMSWebPlatform)]
  {$ENDIF}
  TWebTrackBar = class(TWebCustomInput)
  strict private
    FMax: Integer;
    FMin: Integer;
    FRole: string;
    FPosition: Integer;
    FOnChange: TNotifyEvent;
  private
    FTabOrder: integer;
    FOrientation: TTrackbarOrientation;
    procedure SetOrientation(const Value: TTrackbarOrientation);
  protected
    procedure Paint; override;
    procedure InitBootstrapStyle; override;
  public
    constructor Create(AOwner: TComponent); override;
  published
    property Align;
    {$IFNDEF LCLTMSWEB}
    property AlignWithMargins;
    {$ENDIF}
    property Anchors;
    property ChildOrder;
    property DragMode;
    property ElementClassName;
    property ElementID;
    property ElementFont;
    property ElementPosition;
    property Enabled;
    property Height;
    property Left;
    {$IFNDEF LCLTMSWEB}
    property Margins;
    {$ENDIF}
    property Max: Integer read FMax write FMax;
    property Min: Integer read FMin write FMin;
    property Orientation: TTrackbarOrientation read FOrientation write SetOrientation default trHorizontal;
    property Position: Integer read FPosition write FPosition;
    property Role: string read FRole write FRole;
    property ShowHint;
    property TabOrder: integer read FTabOrder write FTabOrder default 0;
    property TabStop;
    property Top;
    property Visible;
    property Width;
    property OnClick;
    property OnChange: TNotifyEvent read FOnChange write FOnChange;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnStartDrag;
  end;

  {$IFNDEF LCLTMSWEB}
  [ComponentPlatforms(TMSWebPlatform)]
  {$ENDIF}
  TWebTimer = class(TComponent)
  strict private
    FInterval: Integer;
    FOnTimer: TNotifyEvent;
    FEnabled: boolean;
  public
    constructor Create(AOwner: TComponent); override;
  published
    property Enabled: boolean read FEnabled write FEnabled;
    property Interval: Integer read FInterval write FInterval default 1000;
    property OnTimer: TNotifyEvent read FOnTimer write FOnTimer;
  end;

  {$IFNDEF LCLTMSWEB}
  [ComponentPlatforms(TMSWebPlatform)]
  {$ENDIF}
  TWebPaintBox = class(TWebCustomControl)
  strict private
    FOnPaint: TNotifyEvent;
    FCanvas: TCanvas;
  public
    property Canvas: TCanvas read FCanvas write FCanvas;
  published
    property Align;
    {$IFNDEF LCLTMSWEB}
    property AlignWithMargins;
    {$ENDIF}
    property Anchors;
    property ChildOrder;
    property DragMode;
    property ElementFont;
    property ElementPosition;
    property Enabled;
    property Height;
    property Left;
    {$IFNDEF LCLTMSWEB}
    property Margins;
    {$ENDIF}
    property Role;
    property Top;
    property Visible;
    property Width;
    property OnClick;
    property OnDblClick;
    property OnPaint: TNotifyEvent read FOnPaint write FOnPaint;
    property OnEnter;
    property OnExit;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;
    property OnMouseDown;
    property OnMouseEnter;
    property OnMouseLeave;
    property OnMouseMove;
    property OnMouseUp;
    property OnMouseWheel;
    property OnTouchStart;
    property OnTouchMove;
    property OnTouchEnd;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnStartDrag;
  end;

  TWebCustomImageControl = class(TWebGraphicControl)
  strict private
    FURL: string;
    FPicture: TPicture;
    FURLPicture: TPicture;
    FDesignTime: boolean;
  private
    FAutoSize: boolean;
    FOnLoaded: TNotifyEvent;
    FElementHandle: TJSHTMLElement;
    FOnError: THTTPErrorEvent;
    procedure SetAutoSize(const Value: boolean); reintroduce;
    procedure SetURL(const Value: string);
    function GetBase64Img: string;
  protected
    {$IFNDEF LCLTMSWEB}
    procedure SetParent(AParent: TWinControl); override;
    {$ENDIF}
    procedure PictureChanged(Sender: TObject);
    procedure SetPicture(const Value: TPicture);
    procedure Paint; override;
    procedure AutoSizeImage;
    procedure Download;
    property Picture: TPicture read FPicture write SetPicture;
    property URL: string read FURL write SetURL;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    function DataURL: string; overload;
    function DataURL(AWidth, AHeight: integer): string; overload;
    procedure ResizeImage; overload;
    procedure ResizeImage(AWidth, AHeight: integer); overload;
    procedure SetBounds(ALeft, ATop, AWidth, AHeight: integer); override;
    property Base64Image: string read GetBase64Img;
    property ElementHandle: TJSHTMLElement read FElementHandle;
    function ImageWidth: integer;
    function ImageHeight: integer;
  published
    property Align;
    {$IFNDEF LCLTMSWEB}
    property AlignWithMargins;
    {$ENDIF}
    property Anchors;
    property AutoSize: boolean read FAutoSize write SetAutoSize default false;
    property ChildOrder;
    property DragMode;
    property ElementClassName;
    property ElementID;
    property ElementFont;
    property ElementPosition;
    property Role;
    property ShowHint;
    property Visible;
    property OnClick;
    property OnDblClick;
    property OnMouseEnter;
    property OnMouseLeave;
    property OnMouseMove;
    property OnMouseDown;
    property OnMouseUp;
    property OnLoaded: TNotifyEvent read FOnLoaded write FOnLoaded;
    property OnError: THTTPErrorEvent read FOnError write FOnError;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnStartDrag;
  end;

  {$IFNDEF LCLTMSWEB}
  [ComponentPlatforms(TMSWebPlatform)]
  {$ENDIF}
  TWebImageControl = class(TWebCustomImageControl)
  published
    property ChildOrder;
    property Picture;
    property URL;
  end;

  TWebImageZoomControl = class;

  TImageZoomAppearance = class(TPersistent)
  private
    FOwner: TWebImageZoomControl;
    FHeightPercent: Integer;
    FWidthPercent: Integer;
    FResponsiveHeightPercent: Integer;
    FResponsiveWidthPercent: Integer;
    FResponsiveMaxWidth: Integer;
  public
    constructor Create(AOwner: TWebImageZoomControl); overload;
    destructor Destroy; override;
    procedure Assign(Source: TPersistent); override;
    function GetOwner: TPersistent; override;
  published
    property HeightPercent: Integer read FHeightPercent write FHeightPercent default 100;
    property WidthPercent: Integer read FWidthPercent write FWidthPercent default 100;
    property ResponsiveHeightPercent: Integer read FResponsiveHeightPercent write FResponsiveHeightPercent default 100;
    property ResponsiveWidthPercent: Integer read FResponsiveWidthPercent write FResponsiveWidthPercent default 100;
    property ResponsiveMaxWidth: Integer read FResponsiveMaxWidth write FResponsiveMaxWidth default 768;
  end;


  {$IFNDEF LCLTMSWEB}
  [ComponentPlatforms(TMSWebPlatform)]
  {$ENDIF}
  TWebImageZoomControl = class(TWebCustomImageControl)
  private
    FURLZoom: string;
    FAppearance: TImageZoomAppearance;
    FURLPictureZoom: TPicture;
    FPictureZoom: TPicture;
    procedure SetURLZoom(const Value: string);
    procedure SetAppearance(const Value: TImageZoomAppearance);
    procedure SetPictureZoom(const Value: TPicture);
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  published
    property Appearance: TImageZoomAppearance read FAppearance write SetAppearance;
    property ChildOrder;
    property Picture;
    property PictureZoom: TPicture read FPictureZoom write SetPictureZoom;
    property URL;
    property URLZoom: string read FURLZoom write SetURLZoom;
  end;

  TWebCustomPanel = class(TWebCustomControl)
  private
    FColor: TColor;
    FShowCaption: boolean;
    FElementHandle: TJSHTMLElement;
    FElementBodyClassName: TElementClassName;
    FAlignment: TAlignment;
    procedure SetColorEx(const Value: TColor);
    procedure SetShowCaption(const Value: boolean);
    procedure SetAlignment(const Value: TAlignment);
  protected
    procedure InitBootstrapStyle; override;
    procedure Paint; override;
  public
    constructor Create(AOwner: TComponent); override;
    property Alignment: TAlignment read FAlignment write SetAlignment default taCenter;
    property Color: TColor read FColor write SetColorEx default clBtnFace;
    property ElementHandle: TJSHTMLElement read FElementHandle;
    property ElementBodyClassName: TElementClassName read FElementBodyClassName write FElementBodyClassName;
    procedure UpdateAnchoring;
    property ShowCaption: boolean read FShowCaption write SetShowCaption default true;
  end;

  {$IFNDEF LCLTMSWEB}
  [ComponentPlatforms(TMSWebPlatform)]
  {$ENDIF}
  TWebPanel = class(TWebCustomPanel)
  published
    property Align;
    property Alignment;
    {$IFNDEF LCLTMSWEB}
    property AlignWithMargins;
    {$ENDIF}
    property Anchors;
    property AutoSize;
    property BorderColor;
    property BorderStyle;
    property Caption;
    property ChildOrder;
    property Color;
    property DragMode;
    property ElementBodyClassName;
    property ElementClassName;
    property ElementID;
    property ElementFont;
    property ElementPosition;
    property Enabled;
    property Font;
    property Height;
    property Hint;
    property Left;
    {$IFNDEF LCLTMSWEB}
    property Padding;
    {$ENDIF}
    property ParentFont;
    property Role;
    property ShowCaption;
    property ShowHint;
    property Top;
    property Visible;
    property Width;
    property OnClick;
    property OnDblClick;
    property OnEnter;
    property OnExit;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;
    property OnMouseDown;
    property OnMouseEnter;
    property OnMouseLeave;
    property OnMouseMove;
    property OnMouseUp;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnStartDrag;
  end;


  TWebCustomGroupBox = class(TWebCustomControl)
  private
    FColor: TColor;
    FElementHandle: TJSHTMLElement;
    procedure SetColorEx(const Value: TColor);
  protected
    procedure Paint; override;
  public
    constructor Create(AOwner: TComponent); override;
    property Color: TColor read FColor write SetColorEx default clBtnFace;
    property ElementHandle: TJSHTMLElement read FElementHandle;
  end;

  {$IFNDEF LCLTMSWEB}
  [ComponentPlatforms(TMSWebPlatform)]
  {$ENDIF}
  TWebGroupBox = class(TWebCustomGroupBox)
  published
    property Align;
    {$IFNDEF LCLTMSWEB}
    property AlignWithMargins;
    {$ENDIF}
    property Anchors;
    property BorderColor;
    property Caption;
    property ChildOrder;
    property Color;
    property DragMode;
    property ElementClassName;
    property ElementFont;
    property ElementID;
    property ElementPosition;
    property Enabled;
    property Font;
    property Height;
    property Hint;
    property Left;
    property ParentFont;
    property Role;
    property ShowHint;
    property Top;
    property Visible;
    property Width;
    property OnClick;
    property OnDblClick;
    property OnEnter;
    property OnExit;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;
    property OnMouseDown;
    property OnMouseEnter;
    property OnMouseLeave;
    property OnMouseMove;
    property OnMouseUp;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnStartDrag;
  end;

  {$IFNDEF LCLTMSWEB}
  [ComponentPlatforms(TMSWebPlatform)]
  {$ENDIF}
  TWebScrollBox = class(TWebCustomPanel)
  strict private
    FAutoScroll: Boolean;
  private
    FScrolLLeft: integer;
    FScrollTop: integer;
    FOnScroll: TNotifyEvent;
    {$IFNDEF LCLTMSWEB}
    FScrollBars: System.UITypes.TScrollStyle;
    {$ENDIF}
    {$IFDEF LCLTMSWEB}
    FScrollBars: TScrollStyle;
    {$ENDIF}
  public
    constructor Create(AOwner: TComponent); override;
    property ScrollLeft: integer read FScrolLLeft write FScrollLeft;
    property ScrollTop: integer read FScrollTop write FScrollTop;
  published
    property Align;
    {$IFNDEF LCLTMSWEB}
    property AlignWithMargins;
    {$ENDIF}
    property Anchors;
    property AutoScroll: Boolean read FAutoScroll write FAutoScroll default false;
    property BorderStyle;
    property ChildOrder;
    property Color;
    property DragMode;
    property ElementClassName;
    property ElementID;
    property Height;
    property Hint;
    property Left;
    property Role;
    {$IFNDEF LCLTMSWEB}
    property ScrollBars: System.UITypes.TScrollStyle read FScrollBars write FScrollBars default ssBoth;
    {$ENDIF}
    {$IFDEF LCLTMSWEB}
    property ScrollBars: TScrollStyle read FScrollBars write FScrollBars default ssBoth;
    {$ENDIF}
    property ShowHint;
    property Top;
    property Visible;
    property Width;
    property OnClick;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnScroll: TNotifyEvent read FOnScroll write FOnScroll;
    property OnStartDrag;
  end;

  TCustomDragControl = class(TWebCustomControl)
  strict private
    FText: string;
  published
    property Text: string read FText write FText;
  end;

  {$IFNDEF LCLTMSWEB}
  [ComponentPlatforms(TMSWebPlatform)]
  {$ENDIF}
  TWebSplitter = class(TWebCustomControl)
  private
    FGripColor: TColor;
    FOnMoved: TNotifyEvent;
    FOnMove: TNotifyEvent;
    function GetAlignEx: TAlign;
    procedure SetAlignEx(const Value: TAlign);
    procedure SetGripColor(const Value: TColor);
  protected
    procedure Paint; override;
  public
    constructor Create(AOwner: TComponent); override;
  published
    property Align: TAlign read GetAlignEx write SetAlignEx;
    property ChildOrder;
    property Color;
    property ElementClassName;
    property ElementID;
    property GripColor: TColor read FGripColor write SetGripColor default clWhite;
    property Height;
    property Hint;
    property Left;
    property Role;
    property ShowHint;
    property Top;
    property Visible;
    property Width;
    property OnClick;
    property OnMove: TNotifyEvent read FOnMove write FOnMove;
    property OnMoved: TNotifyEvent read FOnMoved write FOnMoved;
  end;

  {$IFNDEF LCLTMSWEB}
  TJSCoordinates = record
  private
    FLatitude: Double;
    FLongitude: Double;
    FAltitude: Double;
    FAccuracy: Double;
    FAltitudeAccuracy: Double;
    FHeading: Double;
    FSpeed: Double;
  public
    property Latitude: Double read FLatitude;
    property Longitude: Double read FLongitude;
    property Altitude: Double read FAltitude;
    property Accuracy: Double read FAccuracy;
    property AltitudeAccuracy: Double read FAltitudeAccuracy;
    property Heading: Double read FHeading;
    property Speed: Double read FSpeed;
  end;
  {$ENDIF}

  {$IFDEF LCLTMSWEB}
  TJSCoordinates = record
    Latitude: Double;
    Longitude: Double;
    Altitude: Double;
    Accuracy: Double;
    AltitudeAccuracy: Double;
    Heading: Double;
    Speed: Double;
  end;
  {$ENDIF}

  {$IFNDEF LCLTMSWEB}
  TJSPosition = record
  private
    FCoords: TJSCoordinates;
    FTimeStamp: integer;
  public
    property Coords: TJSCoordinates read FCoords;
    property TimeStamp: integer read FTimeStamp;
  end;
  {$ENDIF}

  {$IFDEF LCLTMSWEB}
  TJSPosition = record
    Coords: TJSCoordinates;
    TimeStamp: integer;
  end;
  {$ENDIF}


  TGeolocationEvent = procedure(Sender: TObject; Lat, Lon, Alt:double) of object;

  {$IFNDEF LCLTMSWEB}
  [ComponentPlatforms(TMSWebPlatform)]
  {$ENDIF}
  TWebGeoLocation = class(TComponent)
  private
    FOnGeolocation: TGeolocationEvent;
  public
    function HasGeolocation: boolean;
    procedure GetGeolocation;
  published
    property OnGeolocation: TGeolocationEvent read FOnGeolocation write FOnGeolocation;
  end;

  TVerticalAlignment = (vaTop, vaCenter, vaBottom);
  TGridPanelExpandStyle = (esAddRows, esAddColumns);

  TGridPanelRow = class(TCollectionItem)
  private
    FSizeStyle: TSizeStyle;
    FValue: integer;
    FMarginBottom: integer;
    FMarginTop: integer;
    FAlignment: TVerticalAlignment;
    FElementClassName: TElementClassName;
    procedure SetSizeStyle(const Value: TSizeStyle);
    procedure SetValue(const Value: integer);
    procedure SetMarginBottom(const Value: integer);
    procedure SetMarginTop(const Value: integer);
    procedure SetAlignment(const Value: TVerticalAlignment);
  public
    constructor Create(ACollection: TCollection); override;
  published
    property Alignment: TVerticalAlignment read FAlignment write SetAlignment default vaTop;
    property ElementClassName: TElementClassName read FElementClassName write FElementClassName;
    property MarginTop: integer read FMarginTop write SetMarginTop default 0;
    property MarginBottom: integer read FMarginBottom write SetMarginBottom default 0;
    property SizeStyle: TSizeStyle read FSizeStyle write SetSizeStyle default ssPercent;
    property Value: integer read FValue write SetValue;
  end;

  TGridPanelRows = class(TOwnedCollection)
  private
    FOnChange: TNotifyEvent;
    function GetItem(Index: integer): TGridPanelRow;
    procedure SetItem(Index: integer; const Value: TGridPanelRow);
  protected
    procedure Update(Item: TCollectionItem); override;
  public
    constructor Create(AOwner: TComponent);
    property Items[Index: integer]: TGridPanelRow read GetItem write SetItem;
    function Add: TGridPanelRow;
    function Insert(Index: Integer): TGridPanelRow;
    function NumPercent: integer;
    property OnChange: TNotifyEvent read FOnChange write FOnChange;
  end;

  TGridPanelColumn = class(TCollectionItem)
  private
    FSizeStyle: TSizeStyle;
    FValue: integer;
    FMarginLeft: integer;
    FMarginRight: integer;
    FAlignment: TAlignment;
    FElementClassName: TElementClassName;
    procedure SetSizeStyle(const Value: TSizeStyle);
    procedure SetValue(const Value: integer);
    procedure SetMarginLeft(const Value: integer);
    procedure SetMarginRight(const Value: integer);
    procedure SetAlignment(const Value: TAlignment);
  public
    constructor Create(ACollection: TCollection); override;
  published
    property Alignment: TAlignment read FAlignment write SetAlignment default taLeftJustify;
    property ElementClassName: TElementClassName read FElementClassName write FElementClassName;
    property MarginLeft: integer read FMarginLeft write SetMarginLeft default 0;
    property MarginRight: integer read FMarginRight write SetMarginRight default 0;
    property SizeStyle: TSizeStyle read FSizeStyle write SetSizeStyle default ssPercent;
    property Value: integer read FValue write SetValue;
  end;

  TGridPanelColumns = class(TOwnedCollection)
  private
    FOnChange: TNotifyEvent;
    function GetItem(Index: integer): TGridPanelColumn;
    procedure SetItem(Index: integer; const Value: TGridPanelColumn);
  protected
    procedure Update(Item: TCollectionItem); override;
  public
    constructor Create(AOwner: TComponent);
    property Items[Index: integer]: TGridPanelColumn read GetItem write SetItem;
    function Add: TGridPanelColumn;
    function Insert(Index: Integer): TGridPanelColumn;
    function NumPercent: integer;
    property OnChange: TNotifyEvent read FOnChange write FOnChange;
  end;

  TControlCollectionItem = class(TCollectionItem)
  private
    FControl: TWinControl;
    FRow: integer;
    FColumn: integer;
  protected
    procedure AssignTo(ADest: TPersistent); override;
  published
    property Column: integer read FColumn write FColumn;
    property Row: integer read FRow write FRow;
    property Control: TWinControl read FControl write FControl;
  end;

  TControlCollection = class(TOwnedCollection)
  private
    function GetItem(Index: Integer): TControlCollectionItem;
    procedure SetItem(Index: Integer; const Value: TControlCollectionItem);

  public
    constructor Create(AOwner: TComponent);
    function Add: TControlCollectionItem;
    function Insert(Index: Integer): TControlCollectionItem;
    property Items[Index: Integer]: TControlCollectionItem read GetItem write SetItem;
  end;

  {$IFNDEF LCLTMSWEB}
  [ComponentPlatforms(TMSWebPlatform)]
  {$ENDIF}
  TWebGridPanel = class(TWebCustomControl)
  private
    FRowCollection: TGridPanelRows;
    FColumnCollection: TGridPanelColumns;
    FControlCollection: TControlCollection;
    FGridLineWidth: integer;
    FGridLineColor: TColor;
    FExpandStyle: TGridPanelExpandStyle;
    FElementHandle: TJSHTMLElement;
    FIsAligning: boolean;
    procedure SetColumnCollection(const Value: TGridPanelColumns);
    procedure SetRowCollection(const Value: TGridPanelRows);
    procedure CMControlChange(var Message: TCMControlChange); message CM_CONTROLCHANGE;
    procedure SetControlCollection(const Value: TControlCollection);
    function CalcSize(X,Y: Integer): TSize;
    function CalcRect(X,Y: Integer): TRect;
    procedure SetGridLineColor(const Value: TColor);
    procedure SetGridLineWidth(const Value: integer);
  protected
    procedure ColRowChanged(Sender: TObject); virtual;
    procedure AlignControls(AControl: TControl; var Rect: TRect); override;
    procedure Paint; override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure AddControl(AControl: TWinControl);
    procedure RemoveControl(AControl: TWinControl);
    property ElementHandle: TJSHTMLElement read FElementHandle;
  published
    property Align;
    {$IFNDEF LCLTMSWEB}
    property AlignWithMargins;
    {$ENDIF}
    property ChildOrder;
    property ColumnCollection: TGridPanelColumns read FColumnCollection write SetColumnCollection;
    property ControlCollection: TControlCollection read FControlCollection write SetControlCollection;
    property Color;
    property DragMode;
    property ElementClassName;
    property ElementID;
    property ElementPosition;
    property ExpandStyle: TGridPanelExpandStyle read FExpandStyle write FExpandStyle default esAddRows;
    property GridLineWidth: integer read FGridLineWidth write SetGridLineWidth default 0;
    property GridLineColor: TColor read FGridLineColor write SetGridLineColor default clNone;
    property Hint;
    property Role;
    property RowCollection: TGridPanelRows read FRowCollection write SetRowCollection;
    property ShowHint;
    property Visible;
    property OnClick;
    property OnDblClick;
    property OnMouseMove;
    property OnMouseDown;
    property OnMouseUp;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnStartDrag;
  end;

  TMultiMediaType = (mtVideo, mtAudio);

  TMultiMediaVolume = 0..100;

  {$IFNDEF LCLTMSWEB}
  [ComponentPlatforms(TMSWebPlatform)]
  {$ENDIF}
  TWebMultimediaPlayer = class(TWebCustomControl)
  private
    FURL: string;
    FMultimediaType: TMultimediaType;
    FAutoPlay: boolean;
    FControls: boolean;
    FLoop: boolean;
    FVolume: TMultiMediaVolume;
    FMuted: boolean;
    FCurrentTime: integer;
    FDuration: double;
    FPaused: boolean;
    FEnded: boolean;
    FPlaybackRate: double;
  public
    constructor Create(AOwner: TComponent); override;
    procedure Play;
    procedure Pause;
    procedure ReLoad;
    property CurrentTime: integer read FCurrentTime write FCurrentTime;
    property Duration: double read FDuration;
    property Ended: boolean read FEnded;
    property Paused: boolean read FPaused;
  published
    property Align;
    {$IFNDEF LCLTMSWEB}
    property AlignWithMargins;
    {$ENDIF}
    property Anchors;
    property AutoPlay: boolean read FAutoPlay write FAutoPlay default false;
    property Controls: boolean read FControls write FControls default false;
    property Loop: boolean read FLoop write FLoop default false;
    property MultimediaType: TMultimediaType read FMultiMediaType write FMultiMediaType default mtVideo;
    property Muted: boolean read FMuted write FMuted default false;
    property PlaybackRate: double read FPlaybackRate write FPlaybackRate;
    property Role;
    property URL: string read FURL write FURL;
    property Visible;
    property Volume: TMultiMediaVolume read FVolume write FVolume default 100;
  end;

  {$IFNDEF LCLTMSWEB}
  [ComponentPlatforms(TMSWebPlatform)]
  {$ENDIF}
  TWebHTMLContainer = class(TWebCustomControl)
  private
    FHTML: TStringList;
    {$IFNDEF LCLTMSWEB}
    FScrollStyle: System.UITypes.TScrollStyle;
    {$ENDIF}
    {$IFDEF LCLTMSWEB}
    FScrollStyle: TScrollStyle;
    {$ENDIF}
    FElementHandle: TJSHTMLElement;
    FLinkTouchEvents: boolean;
    procedure SetHTML(const Value: TStringList);
  protected
    procedure HTMLChanged(Sender: TObject);
    function GetDesignText: string; override;
    procedure Paint; override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    property ElementHandle: TJSHTMLElement read FElementHandle;
    property LinkTouchEvents: boolean read FLinkTouchEvents write FLinkTouchEvents;
  published
    property Align;
    {$IFNDEF LCLTMSWEB}
    property AlignWithMargins;
    {$ENDIF}
    property Anchors;
    property ChildOrder;
    property DragMode;
    property ElementClassName;
    property ElementID;
    property ElementFont;
    property ElementPosition;
    property Font;
    property HTML: TStringList read FHTML write SetHTML;
    {$IFNDEF LCLTMSWEB}
    property Margins;
    {$ENDIF}
    property Role;
    {$IFNDEF LCLTMSWEB}
    property ScrollStyle: System.UITypes.TScrollStyle read FScrollStyle write FScrollStyle default ssBoth;
    {$ENDIF}
    {$IFDEF LCLTMSWEB}
    property ScrollStyle: TScrollStyle read FScrollStyle write FScrollStyle default ssBoth;
    {$ENDIF}
    property Visible;
    property OnClick;
    property OnDblClick;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property OnTouchStart;
    property OnTouchEnd;
    property OnTouchMove;
    property OnTouchCancel;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnStartDrag;
  end;

  {$IFNDEF LCLTMSWEB}
  [ComponentPlatforms(TMSWebPlatform)]
  {$ENDIF}
  TWebHTMLForm = class(TWebCustomControl)
  private
    FOnSubmit: TNotifyEvent;
    FElementHandle: TJSHTMLElement;
    FAction: string;
  protected
    procedure Paint; override;
  public
    constructor Create(AOwner: TComponent); override;
    function CheckValidity: boolean;
    property ElementHandle: TJSHTMLElement read FElementHandle;
  published
    property Action: string read FAction write FAction;
    property Align;
    property Anchors;
    property ElementPosition;
    property ElementFont;
    property Font;
    property Role;
    property OnSubmit: TNotifyEvent read FOnSubmit write FOnSubmit;
  end;

  {$IFNDEF LCLTMSWEB}
  [ComponentPlatforms(TMSWebPlatform)]
  {$ENDIF}
  TWebBadge = class(TWebCustomControl)
  private
    FColor: TColor;
    FTextColor: TColor;
    FText: string;
    procedure SetColorEx(const Value: TColor);
    procedure SetText(const Value: string);
    procedure SetTextColor(const Value: TColor);
  protected
    procedure Paint; override;
    procedure InitBootstrapStyle; override;
  public
    constructor Create(AOwner: TComponent); override;
  published
    property Align;
    {$IFNDEF LCLTMSWEB}
    property AlignWithMargins;
    {$ENDIF}
    property Anchors;
    property ChildOrder;
    property Color: TColor read FColor write SetColorEx default clRed;
    property ElementClassName;
    property ElementFont;
    property ElementID;
    property ElementPosition;
    {$IFNDEF LCLTMSWEB}
    property Margins;
    {$ENDIF}
    property Role;
    property Text: string read FText write SetText;
    property TextColor: TColor read FTextColor write SetTextColor default clWhite;
    property Visible;
    property OnClick;
    property OnDblClick;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
  end;

  TAccordionSection = class(TCollectionItem)
  private
    FCaption: string;
    FContent: string;
    FTag: integer;
    FExpanded: boolean;
    procedure SetCaption(const Value: string);
    procedure SetContent(const Value: string);
  public
    procedure Assign(Source: TPersistent); override;
    property Expanded: boolean read FExpanded write FExpanded;
    function CaptionElement: TJSHTMLElement;
    function PanelElement: TJSHTMLElement;
  published
    property Caption: string read FCaption write SetCaption;
    property Content: string read FContent write SetContent;
    property Tag: integer read FTag write FTag;
  end;

  TAccordionSections = class(TOwnedCollection)
  private
    FOnChange: TNotifyEvent;
    function GetItem(Index: integer): TAccordionSection; reintroduce;
    procedure SetItem(Index: integer; const Value: TAccordionSection);
  protected
    procedure Update(Item: TCollectionItem); override;
  public
    constructor Create(AOwner: TComponent); overload;
    function Add: TAccordionSection; reintroduce;
    function Insert(Index: integer): TAccordionSection; reintroduce;
    property Items[Index: integer]: TAccordionSection read GetItem write SetItem; default;
    property OnChange: TNotifyEvent read FOnChange write FOnChange;
  end;

  TAccordionSectionEvent =  procedure(Sender: TObject; ASection: TAccordionSection) of object;
  TAccordionSectionAllowEvent =  procedure(Sender: TObject; ASection: TAccordionSection; var Allow: boolean) of object;
  TAccordionSectionRenderEvent =  procedure(Sender: TObject; ASection: TAccordionSection; ACaption, APanel: TJSHTMLElementRecord) of object;


  {$IFNDEF LCLTMSWEB}
  [ComponentPlatforms(TMSWebPlatform)]
  {$ENDIF}
  TWebAccordion = class(TWebCustomControl)
  private
    FSections: TAccordionSections;
    FOnCollapsing: TAccordionSectionAllowEvent;
    FOnExpanding: TAccordionSectionAllowEvent;
    FOnCollapsed: TAccordionSectionEvent;
    FOnExpanded: TAccordionSectionEvent;
    FOnRenderSection: TAccordionSectionRenderEvent;
    FElementSectionClassName: TElementClassName;
    FElementContentClassName: TElementClassName;
    procedure SetSections(const Value: TAccordionSections);
  protected
    procedure Paint; override;
    procedure SectionsChanged(Sender: TObject);
    function GetDesignText: string; override;
    procedure InitBootstrapStyle; override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  published
    property Align;
    {$IFNDEF LCLTMSWEB}
    property AlignWithMargins;
    {$ENDIF}
    property Anchors;
    property ChildOrder;

    property ElementFont;
    property ElementSectionClassName: TElementClassName read FElementSectionClassName write FElementSectionClassName;
    property ElementContentClassName: TElementClassName read FElementContentClassName write FElementContentClassName;
    property ElementPosition;

    property Role;
    property Sections: TAccordionSections read FSections write SetSections;
    property ShowHint;
    property Visible;

    property OnCollapsed: TAccordionSectionEvent read FOnCollapsed write FOnCollapsed;
    property OnCollapsing: TAccordionSectionAllowEvent read FOnCollapsing write FOnCollapsing;
    property OnExpanded: TAccordionSectionEvent read FOnExpanded write FOnExpanded;
    property OnExpanding: TAccordionSectionAllowEvent read FOnExpanding write FOnExpanding;

    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property OnMouseLeave;
    property OnMouseEnter;

    property OnRenderSection: TAccordionSectionRenderEvent read FOnRenderSection write FOnRenderSection;
  end;


  TGridStyle = (gTemplateColumns, gTemplateRows);

  TResponsiveStyleString = string;

  TLayoutMargins = class(TPersistent)
  private
    FRight: integer;
    FBottom: integer;
    FTop: integer;
    FLeft: integer;
  public
    procedure Assign(Source: TPersistent); override;
  published
    property Bottom: integer read FBottom write FBottom default 0;
    property Left: integer read FLeft write FLeft default 0;
    property Right: integer read FRight write FRight default 0;
    property Top: integer read FTop write FTop default 0;
  end;

  TResponsiveLayoutItem = class(TCollectionItem)
  private
    FWidth: integer;
    FStyleType: TGridStyle;
    FStyle: TResponsiveStyleString;
    FRowGap: string;
    FColumnGap: string;
    FTag: NativeInt;
    FDescription: string;
    FMargins: TLayoutMargins;
    procedure SetColumnGap(const Value: string);
    procedure SetRowGap(const Value: string);
    procedure SetStyle(const Value: TResponsiveStyleString);
    procedure SetStyleType(const Value: TGridStyle);
    procedure SetWidth(const Value: integer);
    procedure SetMargins(const Value: TLayoutMargins);
  protected
    procedure Changed; virtual;
  public
    constructor Create(AOwner: TCollection); override;
    destructor Destroy; override;
    procedure Assign(Source: TPersistent); override;
  published
    property ColumnGap: string read FColumnGap write SetColumnGap;
    property Description: string read FDescription write FDescription;
    property Margins: TLayoutMargins read FMargins write SetMargins;
    property RowGap: string read FRowGap write SetRowGap;
    property StyleType: TGridStyle read FStyleType write SetStyleType default gTemplateColumns;
    property Style: TResponsiveStyleString read FStyle write SetStyle;
    property Tag: NativeInt read FTag write FTag default 0;
    property Width: integer read FWidth write SetWidth;
  end;

  TResponsiveLayout = class(TCollection)
  private
    FOwner: TComponent;
    FActiveLayoutItem: TResponsiveLayoutItem;
    FOnChange: TNotifyEvent;
    function GetItem(Index: integer): TResponsiveLayoutItem;
    procedure SetItem(Index: integer; const Value: TResponsiveLayoutItem);
  protected
    procedure Update(Item: TCollectionItem); override;
  public
    function GetLayoutForWidth(w: integer): TResponsiveLayoutItem;
    constructor Create(AOwner: TComponent); reintroduce;
    function Add(AWidth: integer; AStyle: string): TResponsiveLayoutItem; overload;
    function Add: TResponsiveLayoutItem; overload;
    function Insert(Index: integer): TResponsiveLayoutItem;
    property Items[Index: integer]: TResponsiveLayoutItem read GetItem write SetItem;
    property ActiveLayoutItem: TResponsiveLayoutItem read FActiveLayoutItem;
    property OnChange: TNotifyEvent read FOnChange write FOnChange;
  end;

  TResponsiveLayoutChangeEvent = procedure(Sender: TObject; ALayout: TResponsiveLayoutItem) of object;


  {$IFNDEF LCLTMSWEB}
  [ComponentPlatforms(TMSWebPlatform)]
  {$ENDIF}
  TWebResponsiveGridPanel = class(TWebCustomControl)
  private
    FColWidths: TList;
    FColGap: integer;
    FRowGap: integer;
    FElementHandle: TJSHTMLElement;
    FControlCollection: TControlCollection;
    FLayout: TResponsiveLayout;
    FSelectedStyle: string;
    FOnLayoutChange: TResponsiveLayoutChangeEvent;
    procedure CMControlChange(var Message: TCMControlChange); message CM_CONTROLCHANGE;
    procedure SetControlCollection(const Value: TControlCollection);
    procedure SetLayout(const Value: TResponsiveLayout);
  protected
    procedure AlignControls(AControl: TControl; var Rect: TRect); override;
    procedure Resize; override;
    procedure Paint; override;
    procedure ResolveColWidths;
    procedure LayoutChanged(Sender: TObject);
    procedure Loaded; override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure AddControl(AControl: TWinControl);
    procedure RemoveControl(AControl: TWinControl);
    property ElementHandle: TJSHTMLElement read FElementHandle;
    procedure UpdateLayout;
  published
    property Align;
    {$IFNDEF LCLTMSWEB}
    property AlignWithMargins;
    {$ENDIF}
    property Anchors;
    property ChildOrder;
    property ControlCollection: TControlCollection read FControlCollection write SetControlCollection;
    property Color;
    property ElementClassName;
    property ElementFont;
    property ElementID;
    property ElementPosition;
    property Layout: TResponsiveLayout read FLayout write SetLayout;
    property Role;
    property ShowHint;
    property Visible;
    property OnLayoutChange: TResponsiveLayoutChangeEvent read FOnLayoutChange write FOnLayoutChange;
  end;

  TLookupValueItem = class(TCollectionItem)
  private
    FValue: string;
    FDisplayText: string;
  public
    procedure Assign(Source: TPersistent); override;
  published
    property Value: string read FValue write FValue;
    property DisplayText: string read FDisplayText write FDisplayText;
  end;

  TLookupValues = class(TOwnedCollection)
  public
    constructor Create(AOwner: TComponent);
    procedure AddPair(AValue, ADisplayText: string);
  end;

  TCustomWebLookupComboBox = class(TCustomComboBox)
  private
    FLookupValues: TLookupValues;
    FOnChange: TNotifyEvent;
    FWidthStyle: TSizeStyle;
    FElementFont: TElementFont;
    FElementID: TElementID;
    FShowFocus: boolean;
    FChildOrder: integer;
    FRole: string;
    FHeightPercent: TPercentSize;
    FElementPosition: TElementPosition;
    FHeightStyle: TSizeStyle;
    FTextDirection: TTextDirection;
    FTabOrder: integer;
    FWidthPercent: TPercentSize;
    FElementClassName: TElementClassName;
    procedure SetLookupValues(const Value: TLookupValues);
    function GetHeightEx: integer;
    procedure SetHeightEx(const Value: integer);
    procedure SetHeightPercent(const Value: TPercentSize);
    procedure SetHeightStyle(const Value: TSizeStyle);
    procedure SetWidthPercent(const Value: TPercentSize);
    procedure SetWidthStyle(const Value: TSizeStyle);
    function GetDisplayText: string;
    function GetValue: string;
    procedure SetDisplayText(const Value: string);
    procedure SetValue(const Value: string);
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    property Value: string read GetValue write SetValue;
    property DisplayText: string read GetDisplayText write SetDisplayText;
    property LookupValues: TLookupValues read FLookupValues write SetLookupValues;
    property ItemIndex;
  published
    property Align;
    {$IFNDEF LCLTMSWEB}
    property AlignWithMargins;
    {$ENDIF}
    property Anchors;
    property ChildOrder: integer read FChildOrder write FChildOrder default 0;
    property Color;
    property ElementClassName: TElementClassName read FElementClassName write FElementClassName;
    property ElementID: TElementID read FElementID write FElementID;
    property ElementFont: TElementFont read FElementFont write FElementFont default efProperty;
    property ElementPosition: TElementPosition read FElementPosition write FElementPosition default epAbsolute;
    property Enabled;
    property Font;
    property Height: integer read GetHeightEx write SetHeightEx;
    property HeightStyle: TSizeStyle read FHeightStyle write SetHeightStyle default ssAbsolute;
    property HeightPercent: TPercentSize read FHeightPercent write SetHeightPercent;
    property Hint;
    property Left;

    {$IFNDEF LCLTMSWEB}
    property Margins;
    {$ENDIF}
    property ParentFont;
    property Role: string read FRole write FRole;
    property ShowFocus: boolean read FShowFocus write FShowFocus default true;
    property ShowHint;
    property TabOrder: integer read FTabOrder write FTabOrder default 0;
    property TabStop;
    property TextDirection: TTextDirection read FTextDirection write FTextDirection default tdDefault;
    {$IFNDEF LCLTMSWEB}
    property TextHint;
    {$ENDIF}
    property Top;
    property Visible;
    property Width;
    property WidthStyle: TSizeStyle read FWidthStyle write SetWidthStyle default ssAbsolute;
    property WidthPercent: TPercentSize read FWidthPercent write SetWidthPercent;

    property OnChange: TNotifyEvent read FOnChange write FOnChange;
    property OnClick;
    property OnDblClick;
    property OnEnter;
    property OnExit;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;
    property OnMouseDown;
    property OnMouseEnter;
    property OnMouseLeave;
    property OnMouseMove;
    property OnMouseUp;
  end;

  {$IFNDEF LCLTMSWEB}
  [ComponentPlatforms(TMSWebPlatform)]
  {$ENDIF}
  TWebLookupComboBox = class(TCustomWebLookupComboBox)
  published
    property ItemIndex;
    property LookupValues;
  end;


implementation

uses
  Math, TypInfo, Variants, WEBLib.HTML, WEBLib.Forms
  {$IFNDEF LCLTMSWEB}
  , RTTI
  {$ENDIF}
  ;

{$IFDEF LCLTMSWEB}
type
  TWinControlEx = class(TWinControl);
{$ENDIF}

{ TWebCustomPanel }

constructor TWebCustomPanel.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  ControlStyle := ControlStyle + [csAcceptsControls];
  FColor := clBtnFace;
  FAlignment := taCenter;
  Width := 150;
  Height := 60;
  ShowCaption := true;
end;

procedure TWebCustomPanel.InitBootstrapStyle;
begin
  inherited;
  ElementFont := efCSS;
  ElementClassName := 'card';
  ElementBodyClassName := 'card-body';
end;

procedure TWebCustomPanel.Paint;
var
  r: TRect;
  LAlignFlag: DWORD;
begin
  inherited Paint;

  if (Color = clNone) then
    Color := clWindow;

  Canvas.Brush.Color := Color;
  Canvas.Brush.Style := bsSolid;
  Canvas.Pen.Color := Canvas.Brush.Color;
  Canvas.Pen.Style := psSolid;

  Canvas.Rectangle(ClientRect);

  Canvas.Pen.Style := psSolid;
  Canvas.Pen.Color := clGray;

  Canvas.MoveTo(Width - 1, 0);
  Canvas.LineTo(Width - 1, Height - 1);
  Canvas.LineTo(0, Height - 1);

  Canvas.Pen.Style := psSolid;
  Canvas.Pen.Color := clWhite;

  Canvas.MoveTo(0, Height - 1);
  Canvas.LineTo(0, 0);
  Canvas.LineTo(Width - 1, 0);

  r := ClientRect;

  if ShowCaption then
  begin
    Canvas.Font.Assign(Font);
    LAlignFlag := DT_CENTER;
    case Alignment of
    taLeftJustify: LAlignFlag := DT_LEFT;
    taRightJustify:  LAlignFlag := DT_RIGHT;
    end;
    DrawText(Canvas.Handle, PChar(Caption), Length(Caption), r, LAlignFlag or DT_SINGLELINE or DT_VCENTER);
  end;
end;

procedure TWebCustomPanel.SetAlignment(const Value: TAlignment);
begin
  if (FAlignment <> Value) then
  begin
    FAlignment := Value;
    Invalidate;
  end;
end;

procedure TWebCustomPanel.SetColorEx(const Value: TColor);
begin
  if (FColor <> Value) then
  begin
    FColor := Value;
    Invalidate;
  end;
end;

procedure TWebCustomPanel.SetShowCaption(const Value: boolean);
begin
  if (FShowCaption <> Value) then
  begin
    FShowCaption := Value;
    Invalidate;
  end;
end;

procedure TWebCustomPanel.UpdateAnchoring;
begin

end;

{ TWebTrackBar }

constructor TWebTrackBar.Create(AOwner: TComponent);
begin
  inherited;
  FMax := 10;
  Width := 150;
  Height := 20;
  Orientation := trHorizontal;
end;

procedure TWebTrackBar.InitBootstrapStyle;
begin
  inherited;
  ElementFont := efCSS;
  ElementClassName := 'custom-range';
end;

procedure TWebTrackBar.Paint;
var
  R: TRect;
begin
  Canvas.Brush.Color := clBtnFace;
  Canvas.Brush.Style := bsSolid;
  Canvas.Pen.Color := clBtnFace;
  Canvas.Pen.Style := psSolid;

  Canvas.Rectangle(ClientRect);

  Canvas.Brush.Color := clGray;
  Canvas.Brush.Style := bsSolid;

  R := ClientRect;

  if Orientation = trHorizontal then
  begin
    R.Top := ((R.Bottom - R.Top) div 2) - 4;
    R.Bottom := R.Top + 8;
    R.Left := 2;
    R.Right := Width - 2;

    Canvas.Rectangle(R);

    R.Top := R.Top - 8;
    R.Bottom := R.Bottom + 8;
    R.Left := 10;
    R.Right := 18;

    Canvas.Rectangle(R);
  end
  else
  begin
    R.Left := ((R.Right - R.Left) div 2) - 4;
    R.Right := R.Left + 8;
    R.Top := 2;
    R.Bottom := Height - 2;

    Canvas.Rectangle(R);

    R.Left := R.Left - 8;
    R.Right := R.Right + 8;
    R.Top := 10;
    R.Bottom := 18;

    Canvas.Rectangle(R);
  end;
end;

procedure TWebTrackBar.SetOrientation(const Value: TTrackbarOrientation);
var
  ow,oh: integer;
begin
  if (FOrientation <> Value) then
  begin
    ow := Width;
    oh := Height;
    FOrientation := Value;
    if not (csLoading in ComponentState) then
    begin
      Width := oh;
      Height := ow;
    end;
  end;
end;

{ TCustomWebLinkLabel }

constructor TCustomWebLinkLabel.Create(AOwner: TComponent);
begin
  inherited;
  FWidthStyle := ssAbsolute;
  FWidthPercent := 100;
  FHeightStyle := ssAbsolute;
  FHeightPercent := 100;
end;

function TCustomWebLinkLabel.GetLabelText: string;
begin
  Result := HTMLStrip(Caption);
end;


procedure TCustomWebLinkLabel.SetHeightPercent(const Value: TPercentSize);
begin
  FHeightPercent := Value;
end;

procedure TCustomWebLinkLabel.SetHeightStyle(const Value: TSizeStyle);
begin
  FHeightStyle := Value;
end;

procedure TCustomWebLinkLabel.SetWidthPercent(const Value: TPercentSize);
begin
  FWidthPercent := Value;
end;

procedure TCustomWebLinkLabel.SetWidthStyle(const Value: TSizeStyle);
begin
  FWidthStyle := Value;
end;

{ TWebCustomImageControl }

constructor TWebCustomImageControl.Create(AOwner: TComponent);
begin
  inherited;
  FAutoSize := false;
  FPicture := TPicture.Create;
  FURLPicture := TPicture.Create;
  FPicture.OnChange := PictureChanged;
  FURLPicture.OnChange := PictureChanged;

  FDesignTime := (csDesigning in ComponentState) and not
                 ((csReading in AOwner.ComponentState) or (csLoading in AOwner.ComponentState));
end;

function TWebCustomImageControl.DataURL: string;
begin
  Result := '';
end;

function TWebCustomImageControl.DataURL(AWidth, AHeight: integer): string; 
begin
  Result := IntToStr(AWidth)+':'+IntToStr(AHeight);
end;

procedure TWebCustomImageControl.ResizeImage;
begin
end;

procedure TWebCustomImageControl.ResizeImage(AWidth, AHeight: integer);
begin
  raise Exception.Create(IntToStr(AWidth)+':'+IntToStr(AHeight));
end;

destructor TWebCustomImageControl.Destroy;
begin
  FPicture.Free;
  FURLPicture.Free;
  inherited;
end;

procedure TWebCustomImageControl.Download;
begin

end;

function TWebCustomImageControl.GetBase64Img: string;
begin
  Result := '';
end;

function TWebCustomImageControl.ImageHeight: integer;
begin
  Result := 0;
end;

function TWebCustomImageControl.ImageWidth: integer;
begin
  Result := 0;
end;

procedure TWebCustomImageControl.AutoSizeImage;
var
  w,h: integer;
begin
  w := 0;
  h := 0;

  if FURL <> '' then
  begin
    if Assigned(FURLPicture) and Assigned(FURLPicture.Graphic) and not FURLPicture.Graphic.Empty then
    begin
      w := FURLPicture.Width;
      h := FURLPicture.Height;
    end;
  end
  else
  if Assigned(Picture) and Assigned(Picture.Graphic) and not Picture.Graphic.Empty then
  begin
    w := Picture.Width;
    h := Picture.Height;
  end;

  if (w <> 0) and (h <> 0) then
    SetBounds(Left, Top, w, h);
end;

procedure TWebCustomImageControl.Paint;
begin
  if FURL <> '' then
  begin
    if Assigned(FURLPicture) and Assigned(FURLPicture.Graphic) and not FURLPicture.Graphic.Empty then
      Canvas.StretchDraw(ClientRect,FURLPicture.Graphic);
  end
  else
  if Assigned(Picture) and Assigned(Picture.Graphic) and not Picture.Graphic.Empty then
    Canvas.StretchDraw(ClientRect,Picture.Graphic);
end;

procedure TWebCustomImageControl.PictureChanged(Sender: TObject);
begin
  Repaint;
end;

procedure TWebCustomImageControl.SetAutoSize(const Value: boolean);
begin
  if (FAutoSize <> Value) then
  begin
    FAutoSize := Value;
    if FAutoSize then
      AutoSizeImage;
  end;
end;

procedure TWebCustomImageControl.SetBounds(ALeft, ATop, AWidth, AHeight: integer);
begin
  if AutoSize and (Align = alNone) then
  begin
    if FURL <> '' then
    begin
      if Assigned(FURLPicture) and Assigned(FURLPicture.Graphic) and not FURLPicture.Graphic.Empty then
      begin
        AWidth := FURLPicture.Width;
        AHeight := FURLPicture.Height;
      end;
    end
    else
    if Assigned(FPicture) and Assigned(FPicture.Graphic) and not FPicture.Graphic.Empty then
    begin
      AWidth := FPicture.Width;
      AHeight := FPicture.Height;
    end;
  end;

  inherited;
end;

{$IFNDEF LCLTMSWEB}
procedure TWebCustomImageControl.SetParent(AParent: TWinControl);
begin
  inherited;

  if FDesignTime and Assigned(Parent) and not (csDestroying in ComponentState) then
  begin
    if IsRelativeParent(AParent) then
    begin
      ElementPosition := epIgnore;
      ElementFont := efCSS;
      WidthStyle := ssAuto;
      HeightStyle := ssAuto;
    end;

    ChildOrder := AParent.ControlCount - 1;
  end;
end;
{$ENDIF}

procedure TWebCustomImageControl.SetPicture(const Value: TPicture);
begin
  FPicture.Assign(Value);
  if FAutoSize then
    AutoSizeImage;
end;

procedure TWebCustomImageControl.SetURL(const Value: string);
begin
  if (FURL <> Value) then
  begin
    FURL := Value;

    if (csDesigning in ComponentState) then
    begin
      if FURL = '' then
      begin
        FURLPicture.Assign(nil);
        Invalidate;
      end
      else
        TDownLoadThread.Create(FURL, FURLPicture);
    end;
  end;
end;

{ TWebTimer }

constructor TWebTimer.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  Interval := 1000;
  Enabled := true;
end;

{ TGeoLocation }

procedure TWebGeoLocation.GetGeolocation;
begin
  //
end;

function TWebGeoLocation.HasGeolocation: boolean;
begin
  //
  Result := true;
end;

{ TWebSplitter }

constructor TWebSplitter.Create(AOwner: TComponent);
begin
  inherited;
  Color := clBtnFace;
  FGripColor := clWhite;
  Width := 6;
  Height := 100;
end;

function TWebSplitter.GetAlignEx: TAlign;
begin
  Result := inherited Align;
end;

procedure TWebSplitter.Paint;
 var
   xofs,dx: integer;
   yofs,dy: integer;
   i: integer;
begin
  Canvas.Brush.Color := Color;
  Canvas.Brush.Style := bsSolid;
  Canvas.Pen.Color := Canvas.Brush.Color;
  Canvas.Pen.Style := psSolid;
  Canvas.Rectangle(ClientRect);

  dx := 0;
  dy := 0;

  if (Align in [alTop, alBottom]) then
  begin
    dx := 6;
    xofs := (Width div 2) - 9;
    yofs := (Height div 2) - 1;
  end
  else
  begin
    dy := 6;
    yofs := (Height div 2) - 9;
    xofs := (Width div 2) - 1;
  end;

  Canvas.Brush.Color := FGripColor;
  Canvas.Pen.Color := FGripColor;

  for i := 0 to 2 do
  begin
    Canvas.Rectangle(xofs,yofs, xofs + 2, yofs + 2);
    xofs := xofs + dx;
    yofs := yofs + dy;
  end;
end;

procedure TWebSplitter.SetAlignEx(const Value: TAlign);
begin
  inherited Align := Value;

  if not (csLoading in ComponentState) then
  begin

    if Align in [alLeft, alRight] then
    begin
      Width := 5;
    end;

    if Align in [alTop, alBottom] then
    begin
      Height := 5;
    end;
  end;
end;

procedure TWebSplitter.SetGripColor(const Value: TColor);
begin
  if (FGripColor <> Value) then
  begin
    FGripColor := Value;
    Invalidate;
  end;
end;

{ TWebGridPanel }

constructor TWebGridPanel.Create(AOwner: TComponent);
var
  FDesignTime: boolean;
begin
  inherited;
  ControlStyle := ControlStyle + [csAcceptsControls];
  FRowCollection := TGridPanelRows.Create(Self);
  FColumnCollection := TGridPanelColumns.Create(Self);
  FControlCollection := TControlCollection.Create(Self);
  FColumnCollection.OnChange := ColRowChanged;
  FRowCollection.OnChange := ColRowChanged;

  FExpandStyle := esAddRows;
  FIsAligning := false;

  FDesignTime := (csDesigning in ComponentState) and not
                 ((csReading in Owner.ComponentState) or (csLoading in Owner.ComponentState));

  if FDesignTime then
  begin
    FRowCollection.Add;
    FRowCollection.Items[0].SizeStyle := ssPercent;
    FRowCollection.Items[0].Value := 100;
    FColumnCollection.Add;
    FColumnCollection.Add;
    FColumnCollection.Items[0].SizeStyle := ssPercent;
    FColumnCollection.Items[0].Value := 50;
    FColumnCollection.Items[1].SizeStyle := ssPercent;
    FColumnCollection.Items[1].Value := 50;
  end;
end;

destructor TWebGridPanel.Destroy;
begin
  FRowCollection.Free;
  FColumnCollection.Free;
  FControlCollection.Free;
  inherited;
end;

procedure TWebGridPanel.Paint;
var
  i,x,y: integer;
  sz: TSize;
begin
  inherited;

  x := 0;
  y := 0;

  if Color <> clNone then
  begin
    Canvas.Brush.Style := bsSolid;
    Canvas.Pen.Color := Color;
    Canvas.Brush.Color := Color;
    Canvas.Rectangle(ClientRect);
  end;

  if (GridLineWidth > 0) and (GridLineColor <> clNone) then
  begin
    Canvas.Pen.Style := psSolid;
    Canvas.Pen.Color := GridLineColor;
    Canvas.Pen.Width := GridLineWidth;
  end
  else
  begin
    Canvas.Pen.Style := psDot;
    Canvas.Pen.Color := clGray;
  end;

  for i := 0 to ColumnCollection.Count - 1 do
  begin
    sz := CalcSize(i,0);
    Canvas.MoveTo(x,0);
    Canvas.LineTo(x,Height);
    x := x + sz.cx;
  end;

  Canvas.MoveTo(Width - 1,0);
  Canvas.LineTo(Width - 1,Height);

  for i := 0 to RowCollection.Count - 1 do
  begin
    sz := CalcSize(0,i);
    Canvas.MoveTo(0,y);
    Canvas.LineTo(Width,y);
    y := y + sz.cy;
  end;

  Canvas.MoveTo(0,Height - 1);
  Canvas.LineTo(Width - 1,Height - 1);
end;

procedure TWebGridPanel.RemoveControl(AControl: TWinControl);
var
  i: integer;
begin
  for i := 0 to FControlCollection.Count - 1 do
  begin
    if FControlCollection.Items[i].Control =  AControl then
    begin
      FControlCollection.Delete(i);
      break;
    end;
  end;
end;

procedure TWebGridPanel.SetColumnCollection(const Value: TGridPanelColumns);
begin
  FColumnCollection.Assign(Value);
end;

procedure TWebGridPanel.SetControlCollection(const Value: TControlCollection);
begin
  FControlCollection.Assign(Value);
end;

procedure TWebGridPanel.SetGridLineColor(const Value: TColor);
begin
  if (FGridLineColor <> Value) then
  begin
    FGridLineColor := Value;
    Invalidate;
  end;
end;

procedure TWebGridPanel.SetGridLineWidth(const Value: integer);
begin
  if (FGridLineWidth <> Value) then
  begin
    FGridLineWidth := Value;
    Invalidate;
  end;
end;

procedure TWebGridPanel.SetRowCollection(const Value: TGridPanelRows);
begin
  FRowCollection.Assign(Value);
end;

procedure TWebGridPanel.AddControl(AControl: TWinControl);
begin
  FControlCollection.Add.Control := AControl;

  if FControlCollection.Count > FColumnCollection.Count * FRowCollection.Count then
  begin
    if ExpandStyle = esAddRows then
      FRowCollection.Add
    else
      FColumnCollection.Add;
  end;
end;

procedure TWebGridPanel.AlignControls(AControl: TControl; var Rect: TRect);
var
  i,j,k,mw,mh,dx,dy: integer;
  ctrl: TWinControl;
  r: TRect;
  {$IFNDEF LCLTMSWEB}
  ctx: TRttiContext;
  tt: TRttiType;
  p: TRttiProperty;
  v: TValue;
  {$ENDIF}
begin
  {$IFDEF LCLTMSWEB}
  if FIsAligning then
    Exit;
  {$ENDIF}

  {$IFNDEF LCLTMSWEB}
  inherited;
  ctx := TRttiContext.Create;
  {$ENDIF}

  FIsAligning := true;

  i := 0;
  j := 0;

  for k := 0 to FControlCollection.Count - 1 do
  begin
    ctrl := FControlCollection.Items[k].Control;

    r := CalcRect(i,j);

    if Assigned(ctrl) then
    begin
      InflateRect(r,-2,-2);

      mh := r.Bottom - r.Top;
      mw := r.Right - r.Left;

      {$IFNDEF LCLTMSWEB}
      tt := ctx.GetType(ctrl.ClassType);
      p := tt.GetProperty('WidthStyle');
      if Assigned(p) then
      begin
         v := p.GetValue(ctrl);

        if v.AsOrdinal = integer(ssPercent) then
        begin
           p := tt.GetProperty('WidthPercent');
           if Assigned(p) then
           begin
             v := p.GetValue(ctrl);
             mw := Round(mw/100 * v.AsExtended);
           end;
        end
        else
          mw := ctrl.Width;
      end;

      p := tt.GetProperty('HeightStyle');
      if Assigned(p) then
      begin
         v := p.GetValue(ctrl);
         if v.AsOrdinal = integer(ssPercent) then
         begin
           p := tt.GetProperty('HeightPercent');
           if Assigned(p) then
           begin
             v := p.GetValue(ctrl);
             mh := Round(mh/100 * v.AsExtended);
           end;
         end
         else
           mh := ctrl.Height;
      end;
      {$ENDIF}

      dx := 0;
      dy := 0;

      {$IFNDEF LCLTMSWEB}
      case ctrl.Align of
      alClient:
        ctrl.SetBounds(r.Left, r.Top, r.Right - r.Left ,r.Bottom - r.Top);
      alLeft:
        ctrl.SetBounds(r.Left, r.Top, ctrl.Width ,r.Bottom - r.Top);
      alRight:
        ctrl.SetBounds(r.Right - ctrl.Width, r.Top, ctrl.Width ,r.Bottom - r.Top);
      alTop:
        ctrl.SetBounds(r.Left, r.Top, r.Right - r.Left , ctrl.Height);
      alBottom:
        ctrl.SetBounds(r.Left, r.Bottom - ctrl.Height, r.Right - r.Left , ctrl.Height);
      alNone:
        begin
          if (i < ColumnCollection.Count) and (ColumnCollection.Items[i].Alignment = taCenter) then
            dx := (r.Right - r.Left - mw) div 2;

          if (i < ColumnCollection.Count) and (ColumnCollection.Items[i].Alignment = taRightJustify) then
            dx := (r.Right - r.Left - mw);

          if (j < RowCollection.Count) and (RowCollection.Items[j].Alignment = vaCenter) then
            dy := (r.Bottom - r.Top - mh) div 2;

          if (j < RowCollection.Count) and (RowCollection.Items[j].Alignment = vaBottom) then
            dy := (r.Bottom - r.Top - mh);

          ctrl.SetBounds(r.Left + dx,r.Top + dy, mw, mh);
        end;
      end;
      {$ENDIF}
      {$IFDEF LCLTMSWEB}

      mh := ctrl.Height;
      mw := ctrl.Width;

      case ctrl.Align of
      alClient:
        TWinControlEx(ctrl).DoSetBounds(r.Left, r.Top, r.Right - r.Left ,r.Bottom - r.Top);
      alLeft:
        TWinControlEx(ctrl).DoSetBounds(r.Left, r.Top, ctrl.Width ,r.Bottom - r.Top);
      alRight:
        TWinControlEx(ctrl).DoSetBounds(r.Right - ctrl.Width, r.Top, ctrl.Width ,r.Bottom - r.Top);
      alTop:
        TWinControlEx(ctrl).DoSetBounds(r.Left, r.Top, r.Right - r.Left , ctrl.Height);
      alBottom:
        TWinControlEx(ctrl).DoSetBounds(r.Left, r.Bottom - ctrl.Height, r.Right - r.Left , ctrl.Height);
      alNone:
        begin
          if (i < ColumnCollection.Count) and (ColumnCollection.Items[i].Alignment = taCenter) then
            dx := (r.Right - r.Left - mw) div 2;

          if (i < ColumnCollection.Count) and (ColumnCollection.Items[i].Alignment = taRightJustify) then
            dx := (r.Right - r.Left - mw);

          if (j < RowCollection.Count) and (RowCollection.Items[j].Alignment = vaCenter) then
            dy := (r.Bottom - r.Top - mh) div 2;

          if (j < RowCollection.Count) and (RowCollection.Items[j].Alignment = vaBottom) then
            dy := (r.Bottom - r.Top - mh);

          TWinControlEx(ctrl).DoSetBounds(r.Left + dx,r.Top + dy, mw, mh);
        end;
      end;
      {$ENDIF}
    end;

    inc(i);
    if i = ColumnCollection.Count then
    begin
      inc(j);
      i := 0;
    end;
  end;

  {$IFNDEF LCLTMSWEB}
  ctx.Free;
  {$ENDIF}

  FIsAligning := false;
end;

function TWebGridPanel.CalcRect(X, Y: Integer): TRect;
var
  i: integer;
  xo,yo: integer;
  sz: TSize;
begin
  Result := Rect(0,0,0,0);

  if X > ColumnCollection.Count - 1 then
    Exit;
  if Y > RowCollection.Count - 1 then
    Exit;

  xo := 0;
  yo := 0;
  for i := 0 to X - 1 do
  begin
    sz := CalcSize(i,0);
    xo := xo + sz.cx;
  end;

  for i := 0 to Y - 1 do
  begin
    sz := CalcSize(0,i);
    yo := yo + sz.cy;
  end;

  sz := CalcSize(X,Y);

  Result := Rect(xo + ColumnCollection.Items[X].MarginLeft,
                 yo + RowCollection.Items[Y].MarginTop,
                 xo + sz.cx - ColumnCollection.Items[X].MarginRight,
                 yo + sz.cy - RowCollection.Items[Y].MarginBottom);
end;

function TWebGridPanel.CalcSize(X, Y: Integer): TSize;
var
  cx,cy,i: integer;
  dx,dy: integer;
  rx,ry: integer;
begin
  Result.cx := 0;
  Result.cy := 0;

  if X > ColumnCollection.Count - 1 then
    Exit;
  if Y > RowCollection.Count - 1 then
    Exit;

  cx := 0;
  cy := 0;

  for I := 0 to ColumnCollection.Count -1 do
  begin
    if ColumnCollection.Items[i].SizeStyle = ssAbsolute then
      cx := cx + ColumnCollection.Items[i].Value;
  end;

  for I := 0 to RowCollection.Count -1 do
  begin
    if RowCollection.Items[i].SizeStyle = ssAbsolute then
      cy := cy + RowCollection.Items[i].Value;
  end;

  // remaining percentage width
  dx := Width - cx;
  dy := Height - cy;

  if ColumnCollection.Items[X].SizeStyle = ssAbsolute then
    rx := ColumnCollection.Items[X].Value
  else
    rx := Round((dx / 100) * ColumnCollection.Items[X].Value);

  if RowCollection.Items[Y].SizeStyle = ssAbsolute then
    ry := RowCollection.Items[Y].Value
  else
    ry := Round((dy / 100) * RowCollection.Items[Y].Value);

  Result.cx := rx;
  Result.cy := ry;
end;

procedure TWebGridPanel.CMControlChange(var Message: TCMControlChange);
var
  r: TRect;
begin
  inherited;
  if not (csLoading in ComponentState) then
    if Message.Inserting and (Message.Control.Parent = Self) then
    begin
      r := ClientRect;
      DisableAlign;
      try
        Message.Control.Anchors := [];
        AddControl(TCustomControl(Message.Control));
        AlignControls(Self, r);
      finally
        EnableAlign;
      end;
    end
    else
    begin
      RemoveControl(TCustomControl(Message.Control));
    end;
end;


procedure TWebGridPanel.ColRowChanged(Sender: TObject);
var
  r: TRect;
  i,l: integer;
  hasZero,hasDiff: boolean;
begin
  if not HandleAllocated then
    Exit;

  hasZero := false;
  hasDiff := false;

  l := -1;
  for i := 0 to ColumnCollection.Count - 1 do
  begin
    if l = -1 then
     l := ColumnCollection.Items[i].Value;


    if ColumnCollection.Items[i].Value = 0 then
     hasZero := true
    else
      if (l <> ColumnCollection.Items[i].Value) then
        hasDiff := true;
  end;

  if hasZero and not hasDiff then
  begin
    for i := 0 to ColumnCollection.Count - 1 do
    begin
      ColumnCollection.Items[i].Value := Round(100/ColumnCollection.Count);
    end;
  end;

  hasZero := false;
  hasDiff := false;

  l := -1;
  for i := 0 to RowCollection.Count - 1 do
  begin
    if l = -1 then
     l := RowCollection.Items[i].Value;


    if RowCollection.Items[i].Value = 0 then
     hasZero := true
    else
      if (l <> RowCollection.Items[i].Value) then
        hasDiff := true;
  end;

  if hasZero and not hasDiff then
  begin
    for i := 0 to RowCollection.Count - 1 do
    begin
      RowCollection.Items[i].Value := Round(100/RowCollection.Count);
    end;
  end;


  Resize;
  r := ClientRect;
  AlignControls(Self, r);
  Repaint;
end;

{ TGridPanelRows }

function TGridPanelRows.Add: TGridPanelRow;
begin
  Result := TGridPanelRow(inherited Add);
end;

constructor TGridPanelRows.Create(AOwner: TComponent);
begin
  inherited Create(AOwner, TGridPanelRow);
end;

function TGridPanelRows.GetItem(Index: integer): TGridPanelRow;
begin
  Result := TGridPanelRow(inherited Items[Index]);
end;

function TGridPanelRows.Insert(Index: Integer): TGridPanelRow;
begin
  Result := TGridPanelRow(inherited Insert(Index));
end;

function TGridPanelRows.NumPercent: integer;
var
  i: Integer;
begin
  Result := 0;

  for i := 0 to Count - 1 do
  begin
    if Items[i].SizeStyle = ssPercent then
      inc(Result);
  end;
end;

procedure TGridPanelRows.SetItem(Index: integer; const Value: TGridPanelRow);
begin
  inherited Items[Index] := Value;
end;

procedure TGridPanelRows.Update(Item: TCollectionItem);
begin
  inherited;
  if Assigned(OnChange) then
    OnChange(Self);
end;

{ TGridPanelColumns }

function TGridPanelColumns.Add: TGridPanelColumn;
begin
  Result := TGridPanelColumn(inherited Add);
end;

constructor TGridPanelColumns.Create(AOwner: TComponent);
begin
  inherited Create(AOwner, TGridPanelColumn);
end;

function TGridPanelColumns.GetItem(Index: integer): TGridPanelColumn;
begin
  Result := TGridPanelColumn(inherited Items[Index]);
end;

function TGridPanelColumns.Insert(Index: Integer): TGridPanelColumn;
begin
  Result := TGridPanelColumn(inherited Insert(Index));
end;

function TGridPanelColumns.NumPercent: integer;
var
  i: Integer;
begin
  Result := 0;

  for i := 0 to Count - 1 do
  begin
    if Items[i].SizeStyle = ssPercent then
      inc(Result);
  end;
end;

procedure TGridPanelColumns.SetItem(Index: integer;
  const Value: TGridPanelColumn);
begin
  inherited Items[Index] := Value;
end;

procedure TGridPanelColumns.Update(Item: TCollectionItem);
begin
  inherited;
  if Assigned(OnChange) then
    OnChange(Self);
end;

{ TControlCollection }

function TControlCollection.Add: TControlCollectionItem;
begin
  Result := TControlCollectionItem(inherited Add);
end;

constructor TControlCollection.Create(AOwner: TComponent);
begin
  inherited Create(AOwner, TControlCollectionItem);
end;

function TControlCollection.GetItem(Index: Integer): TControlCollectionItem;
begin
  Result := TControlCollectionItem(inherited Items[Index]);
end;

function TControlCollection.Insert(Index: Integer): TControlCollectionItem;
begin
  Result := TControlCollectionItem(inherited Insert(Index));
end;

procedure TControlCollection.SetItem(Index: Integer;
  const Value: TControlCollectionItem);
begin
  inherited Items[Index] := Value;
end;

{ TGridPanelRow }

constructor TGridPanelRow.Create(ACollection: TCollection);
begin
  inherited;
  FSizeStyle := ssPercent;
end;

procedure TGridPanelRow.SetAlignment(const Value: TVerticalAlignment);
begin
  if (FAlignment <> Value) then
  begin
    FAlignment := Value;
    Changed(false);
  end;
end;

procedure TGridPanelRow.SetMarginBottom(const Value: integer);
begin
  if (FMarginBottom <> Value) then
  begin
    FMarginBottom := Value;
    Changed(false);
  end;
end;

procedure TGridPanelRow.SetMarginTop(const Value: integer);
begin
  if (FMarginTop <> Value) then
  begin
    FMarginTop := Value;
    Changed(false);
  end;
end;

procedure TGridPanelRow.SetSizeStyle(const Value: TSizeStyle);
begin
  if (FSizeStyle <> Value) then
  begin
    FSizeStyle := Value;
    Changed(false);
  end;
end;

procedure TGridPanelRow.SetValue(const Value: integer);
begin
  if (FValue <> Value) then
  begin
    FValue := Value;
    Changed(false);
  end;
end;

{ TGridPanelColumn }

constructor TGridPanelColumn.Create(ACollection: TCollection);
begin
  inherited;
  FSizeStyle := ssPercent;
end;

procedure TGridPanelColumn.SetAlignment(const Value: TAlignment);
begin
  if (FAlignment <> Value) then
  begin
    FAlignment := Value;
    Changed(false);
  end;
end;

procedure TGridPanelColumn.SetMarginLeft(const Value: integer);
begin
  if (FMarginLeft <> Value) then
  begin
    FMarginLeft := Value;
    Changed(false);
  end;
end;

procedure TGridPanelColumn.SetMarginRight(const Value: integer);
begin
  if (FMarginRight <> Value) then
  begin
    FMarginRight := Value;
    Changed(false);
  end;
end;

procedure TGridPanelColumn.SetSizeStyle(const Value: TSizeStyle);
begin
  if (FSizeStyle <> Value) then
  begin
    FSizeStyle := Value;
    Changed(false);
  end;
end;

procedure TGridPanelColumn.SetValue(const Value: integer);
begin
  if (FValue <> Value) then
  begin
    FValue := Value;
    Changed(false);
  end;
end;

{ TWebMultimediaPlayer }

constructor TWebMultimediaPlayer.Create(AOwner: TComponent);
begin
  inherited;
  FVolume := 100;
  FPlaybackRate := 1.0;
  FMuted := false;
  FAutoPlay := false;
end;

procedure TWebMultimediaPlayer.Pause;
begin
  //
end;

procedure TWebMultimediaPlayer.Play;
begin
  //
end;

procedure TWebMultimediaPlayer.ReLoad;
begin
  //
end;


{ TWebCustomGroupBox }

constructor TWebCustomGroupBox.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  ControlStyle := ControlStyle + [csAcceptsControls];
  FColor := clBtnFace;
  Width := 270;
  Height := 180;
end;

procedure TWebCustomGroupBox.Paint;
var
  r: TRect;
  tw,th: integer;
begin
  inherited Paint;

  Canvas.Font.Assign(Font);
  tw := Canvas.TextWidth(Caption);
  th := Canvas.TextHeight(Caption) div 2;

  Canvas.Brush.Color := Color;
  Canvas.Brush.Style := bsSolid;
  Canvas.Pen.Color := Canvas.Brush.Color;
  Canvas.Pen.Style := psSolid;

  Canvas.Rectangle(ClientRect);

  Canvas.Pen.Style := psSolid;
  Canvas.Pen.Color := clGray;

  Canvas.MoveTo(Width - 1, th);
  Canvas.LineTo(Width - 1, Height - 1);
  Canvas.LineTo(0, Height - 1);
  Canvas.LineTo(0, th);
  Canvas.LineTo(8, th);

  Canvas.MoveTo(10 + tw + 2, th);
  Canvas.LineTo(Width - 1, th);

  r := Rect(10,0,Width,Height);

  DrawText(Canvas.Handle, PChar(Caption), Length(Caption), r, DT_LEFT or DT_SINGLELINE or DT_TOP);
end;

procedure TWebCustomGroupBox.SetColorEx(const Value: TColor);
begin
  if (FColor <> Value) then
  begin
    FColor := Value;
    Invalidate;
  end;
end;

{ TWebHTMLContainer }

constructor TWebHTMLContainer.Create(AOwner: TComponent);
begin
  inherited;
  FHTML := TStringList.Create;
  FHTML.OnChange := HTMLChanged;
  FScrollStyle := ssBoth;
end;

destructor TWebHTMLContainer.Destroy;
begin
  FHTML.Free;
  inherited;
end;

function TWebHTMLContainer.GetDesignText: string;
begin
  if HTML.Text = '' then
    Result := inherited GetDesignText
  else
    Result := '';
end;

procedure TWebHTMLContainer.HTMLChanged(Sender: TObject);
begin
  Invalidate;
end;

procedure TWebHTMLContainer.Paint;
var
  r: TRect;
begin
  inherited;

  r := ClientRect;

  Canvas.Pen.Color := clSilver;
  Canvas.Pen.Style := psDot;
  Canvas.Brush.Style := bsClear;
  Canvas.Rectangle(r);

  InflateRect(r, -2, -2);
  DrawText(Canvas.Handle, PChar(FHTML.Text), Length(FHTML.Text), r, DT_WORDBREAK);
end;

procedure TWebHTMLContainer.SetHTML(const Value: TStringList);
begin
  FHTML.Assign(Value);
end;

{ TWebHTMLForm }

function TWebHTMLForm.CheckValidity: boolean;
begin
  Result := true;
end;

constructor TWebHTMLForm.Create(AOwner: TComponent);
begin
  inherited;
  ControlStyle := ControlStyle + [csAcceptsControls];
  Width := 400;
  Height := 300;
end;

procedure TWebHTMLForm.Paint;
var
  r: TRect;
begin
  r := ClientRect;

  Canvas.Brush.Color := clWindow;
  Canvas.Pen.Color := $F0F0F0;
  Canvas.Brush.Style := bsSolid;

  Canvas.Brush.Color := Canvas.Pen.Color;
  Canvas.Brush.Style := bsFDiagonal;

  Canvas.Rectangle(R);
end;


{ TWebBadge }

constructor TWebBadge.Create(AOwner: TComponent);
var
  FDesignTime: boolean;
begin
  inherited;
  FColor := clRed;
  FTextColor := clWhite;

  FDesignTime := (csDesigning in ComponentState) and not
                 ((csReading in Owner.ComponentState) or (csLoading in Owner.ComponentState));

  if FDesignTime then
    FText := '1';

  Width := 20;
  Height := 20;
end;

procedure TWebBadge.InitBootstrapStyle;
begin
  inherited;
  ElementFont := efCSS;
  ElementClassName := 'badge badge-primary badge-pill';
end;

procedure TWebBadge.Paint;
var
  r: TRect;
begin
  Canvas.Brush.Color := FColor;
  Canvas.Brush.Style := bsSolid;
  Canvas.Pen.Color := FColor;
  r := Rect(2,2,18,18);
  Canvas.Ellipse(r);
  Canvas.Font.Color := FTextColor;
  DrawText(Canvas.Handle, PChar(FText), Length(FText), r, DT_SINGLELINE or DT_CENTER or DT_VCENTER);
end;

procedure TWebBadge.SetColorEx(const Value: TColor);
begin
  FColor := Value;
  Invalidate;
end;

procedure TWebBadge.SetText(const Value: string);
begin
  FText := Value;
  Invalidate;
end;

procedure TWebBadge.SetTextColor(const Value: TColor);
begin
  FTextColor := Value;
  Invalidate;
end;

{ TWebScrollBox }

constructor TWebScrollBox.Create(AOwner: TComponent);
begin
  inherited;
  FAutoScroll := false;
  FScrollBars := ssBoth;
end;

{ TWebResponsiveGridPanel }

procedure TWebResponsiveGridPanel.AddControl(AControl: TWinControl);
begin
  FControlCollection.Add.Control := AControl;
end;

procedure TWebResponsiveGridPanel.AlignControls(AControl: TControl;
  var Rect: TRect);
var
  k,j: integer;
  ctrl: TControl;
  x,y,h: integer;
  {$IFNDEF LCLTMSWEB}
  w: integer;
  PropInfo: PPropInfo;
  {$ENDIF}
begin
  if (csLoading in Owner.ComponentState) then
    Exit;

  x := 0;
  y := 0;
  h := 0;

  Layout.GetLayoutForWidth(Width);
  ResolveColWidths;

  if FColWidths.Count > 0 then
  begin
    j := 0;
    for k := 0 to FControlCollection.Count - 1 do
    begin
      ctrl := FControlCollection.Items[k].Control;

      if Assigned(ctrl) then
      begin
        ctrl.Left := x + FColGap;
        ctrl.Top := y + FRowGap;

        {$IFNDEF LCLTMSWEB}
        PropInfo := GetPropInfo(ctrl.ClassInfo, 'WidthStyle');
        if Assigned(PropInfo) then
        begin
          w := GetPropValue(ctrl, PropInfo, false);
          if (w = integer(ssPercent)) then
          begin
            PropInfo := GetPropInfo(ctrl.ClassInfo, 'WidthPercent');
            if Assigned(PropInfo) then
            begin
              w := GetPropValue(ctrl, PropInfo, false);
              ctrl.Width := Round(integer(FColWidths.Items[j]) * w/100);
            end;
          end;
        end;
        {$ENDIF}

        if ctrl.Height > h then
          h := ctrl.Height;
      end;

      x := x + integer(FColWidths.Items[j]) + FColGap;

      inc(j);

      if j >= FColWidths.Count then
      begin
        x := 0;
        j := 0;
        y := y + h + FRowGap;
        h := 0;
      end;
    end;
  end
  else
  begin
    for k := 0 to FControlCollection.Count - 1 do
    begin
      ctrl := FControlCollection.Items[k].Control;

      if Assigned(ctrl) then
      begin
        ctrl.Left := x;
        ctrl.Top := y;

        if ctrl.Height > h then
         h := ctrl.Height;

        x := x + ctrl.Width;
        if x > Width then
        begin
          x := 0;
          y := y + Max(20,h);
          h := 0;
        end;
      end;
    end;
  end;
end;

procedure TWebResponsiveGridPanel.CMControlChange(
  var Message: TCMControlChange);
var
  r: TRect;
begin
  inherited;

  if not (csLoading in ComponentState) then
  begin
    if Message.Inserting and (Message.Control.Parent = Self) then
    begin
      r := ClientRect;
      DisableAlign;
      try
        Message.Control.Anchors := [];
        AddControl(TCustomControl(Message.Control));
        AlignControls(Self, r);
      finally
        EnableAlign;
      end;
    end
    else
    begin
      RemoveControl(TCustomControl(Message.Control));
    end;
  end;
end;

constructor TWebResponsiveGridPanel.Create(AOwner: TComponent);
var
  FDesignTime: boolean;
  li: TResponsiveLayoutItem;
begin
  inherited;
  ControlStyle := ControlStyle + [csAcceptsControls];
  FColWidths := TList.Create;
  FControlCollection := TControlCollection.Create(Self);
  FLayout := TResponsiveLayout.Create(Self);
  FLayout.OnChange := LayoutChanged;

  FDesignTime := (csDesigning in ComponentState) and not
                 ((csReading in Owner.ComponentState) or (csLoading in Owner.ComponentState));

  Width := 600;
  Height := 400;

  // initialize some default media queries
  if FDesignTime then
  begin
    li := FLayout.Add;
    li.Description := 'Smartphone';
    li.Width := 575;
    li.Style := '1fr';

    li := FLayout.Add;
    li.Description := 'Tablet';
    li.Width := 768;
    li.Style := '1fr 1fr';

    li := FLayout.Add;
    li.Description := 'Desktop';
    li.Width := 991;
    li.Style := '1fr 1fr 1fr';

    li := FLayout.Add;
    li.Description := 'Large Desktop';
    li.Width := 1199;
    li.Style := '1fr 1fr 1fr 1fr';
  end;

end;

destructor TWebResponsiveGridPanel.Destroy;
begin
  FColWidths.Free;
  FControlCollection.Free;
  FLayout.Free;
  inherited;
end;

procedure TWebResponsiveGridPanel.LayoutChanged(Sender: TObject);
begin
  if not (csLoading in ComponentState) then
    UpdateLayout;
end;

procedure TWebResponsiveGridPanel.Loaded;
begin
  inherited;
  UpdateLayout;
end;

procedure TWebResponsiveGridPanel.Paint;
var
  x,i: integer;
  LFont: TLogFont;
  hOldFont,hNewFont: HFont;
  LPerCol: boolean;
begin
  inherited;

  Canvas.TextOut(4, Height - 4 - Canvas.TextHeight('gh'), 'Active style:'+FSelectedStyle);

  if FSelectedStyle <> '' then
  begin
    ResolveColWidths;

    Canvas.Pen.Style := psDot;
    Canvas.Pen.Color := clSilver;
    Canvas.Pen.Width := 1;

    LPerCol := true;
    if Assigned(Layout.ActiveLayoutItem) and (Layout.ActiveLayoutItem.StyleType = gTemplateRows) then
      LPerCol := false;

    if LPerCol then
    begin
      x := FColGap;
      for i := 0 to FColWidths.Count - 1 do
      begin
        Canvas.MoveTo(x + integer(FColWidths.Items[i]),0);
        Canvas.LineTo(x + integer(FColWidths.Items[i]),Height);
        x := x + integer(FColWidths.Items[i]) + FColGap;
      end;
    end
    else
    begin
      x := FRowGap;
      for i := 0 to FColWidths.Count - 1 do
      begin
        Canvas.MoveTo(0, x + integer(FColWidths.Items[i]));
        Canvas.LineTo(Width, x + integer(FColWidths.Items[i]));
        x := x + integer(FColWidths.Items[i]) + FRowGap;
      end;
    end;

    Canvas.Pen.Style := psSolid;
    Canvas.Pen.Color := clBlack;
  end;

  GetObject(Canvas.Font.Handle, SizeOf(LFont), Addr(LFont));
  LFont.lfEscapement := +900;
  LFont.lfOrientation := +900;
  hNewFont := CreateFontIndirect(LFont);
  hOldFont := SelectObject(Canvas.Handle,hNewFont);

  for i := 0 to Layout.Count - 1 do
  begin
    Canvas.Brush.Style := bsSolid;
    Canvas.Brush.Color := clRed;
    Canvas.Pen.Color := clRed;
    x := Layout.Items[i].Width;
    Canvas.Rectangle(x-2, Height - 8, x+2, Height);

    Canvas.Brush.Style := bsClear;
    Canvas.TextOut(x, Height - 10, IntToStr(x)+'px');
  end;

  hNewFont := SelectObject(Canvas.Handle,hOldFont);
  DeleteObject(hNewFont);
end;

procedure TWebResponsiveGridPanel.RemoveControl(AControl: TWinControl);
var
  i: integer;
begin
  for i := 0 to FControlCollection.Count - 1 do
  begin
    if FControlCollection.Items[i].Control =  AControl then
    begin
      FControlCollection.Delete(i);
      break;
    end;
  end;

end;

procedure TWebResponsiveGridPanel.Resize;
begin
  inherited;
  {$IFNDEF LCLTMSWEB}
  UpdateLayout;
  {$ENDIF}
end;

procedure TWebResponsiveGridPanel.ResolveColWidths;
var
  LStyles: TStringList;
  i: integer;
  LColWidth: integer;
  LRowHeight: integer;
  LAvailWidth: integer;
  LAvailHeight: integer;
  LFrac: integer;
  s: string;
  LNumRows: integer;
  li: TResponsiveLayoutItem;
begin
  li := Layout.GetLayoutForWidth(Width);
  if Assigned(li) then
    FSelectedStyle := li.Style
  else
    Exit;

  LStyles := TStringList.Create;
  LStyles.Delimiter := ' ';
  LStyles.StrictDelimiter := true;
  LStyles.DelimitedText := FSelectedStyle;

  FColGap := 0;
  FRowGap := 0;

  if Assigned(Layout.ActiveLayoutItem) then
  begin
    s := Layout.ActiveLayoutItem.ColumnGap;

    if pos('px',s) > 0 then
    begin
      s := Copy(s, 1, pos('px',s) - 1);
      TryStrToInt(s, FColGap);
    end
    else
    if pos('%',s) > 0 then
    begin
      s := Copy(s, 1, pos('%',s) - 1);
      TryStrToInt(s, FColGap);
      if LStyles.Count > 0 then
        FColGap := Round((Width * FColGap / 100) / LStyles.Count);
    end
    else
    begin
      if s <> '' then
        TryStrToInt(s, FColGap);
    end;

    s := Layout.ActiveLayoutItem.RowGap;

    if pos('px',s) > 0 then
    begin
      s := Copy(s, 1, pos('px',s) - 1);
      TryStrToInt(s, FRowGap);
    end
    else
    if pos('%',s) > 0 then
    begin
      s := Copy(s, 1, pos('%',s) - 1);
      TryStrToInt(s, FRowGap);

      if LStyles.Count > 0 then
      begin
        LNumRows := ControlCollection.Count div LStyles.Count;
        if LNumRows > 0 then
          FRowGap := Round((Height * FRowGap / 100) / LNumRows);
      end;
    end
    else
    begin
      if s <> '' then
        TryStrToInt(s, FRowGap);
    end;
  end;

  FColWidths.Clear;

  for i := 0 to LStyles.Count - 1 do
    FColWidths.Add(Pointer(0));

  LAvailWidth := Width - (LStyles.Count * FColGap);
  LAvailHeight := Height - (LStyles.Count * FRowGap);

  if Assigned(Layout.ActiveLayoutItem) and (Layout.ActiveLayoutItem.StyleType = gTemplateColumns) then
  begin

    for i := 0 to LStyles.Count - 1 do
    begin
      s := LStyles[i];
      if pos('px',s) > 0 then
      begin
        s := Copy(s, 1, pos('px',s) - 1);
        TryStrToInt(s, LColWidth);
        FColWidths.Items[i] := pointer(LColWidth);
        LAvailWidth := LAvailWidth - LColWidth;
      end;
    end;

    for i := 0 to LStyles.Count - 1 do
    begin
      s := LStyles[i];
      if pos('%',s) > 0 then
      begin
        s := Copy(s, 1, pos('%',s) - 1);
        TryStrToInt(s, LColWidth);
        FColWidths.Items[i] := pointer(Round(LColWidth * LAvailWidth /100));
      end;
    end;

    LFrac := 0;
    for i := 0 to LStyles.Count - 1 do
    begin
      s := LStyles[i];
      if pos('fr',s) > 0 then
      begin
        s := Copy(s, 1, pos('fr',s) - 1);
        TryStrToInt(s, LColWidth);
        LFrac := LFrac + LColWidth;
      end;
    end;

    if LFrac > 0 then
    begin
      for i := 0 to LStyles.Count - 1 do
      begin
        s := LStyles[i];
        if pos('fr',s) > 0 then
        begin
          s := Copy(s, 1, pos('fr',s) - 1);
          TryStrToInt(s, LColWidth);
          FColWidths.Items[i] := pointer(Round(LColWidth * LAvailWidth / LFrac));
        end;
      end;
    end;

  end;

  if Assigned(Layout.ActiveLayoutItem) and (Layout.ActiveLayoutItem.StyleType = gTemplateRows) then
  begin
    for i := 0 to LStyles.Count - 1 do
    begin
      s := LStyles[i];
      if pos('px',s) > 0 then
      begin
        s := Copy(s, 1, pos('px',s) - 1);
        TryStrToInt(s, LRowHeight);
        FColWidths.Items[i] := pointer(LRowHeight);
        LAvailHeight := LAvailHeight - LRowHeight;
      end;
    end;

    for i := 0 to LStyles.Count - 1 do
    begin
      s := LStyles[i];
      if pos('%',s) > 0 then
      begin
        s := Copy(s, 1, pos('%',s) - 1);
        TryStrToInt(s, LRowHeight);
        FColWidths.Items[i] := pointer(Round(LRowHeight * LAvailHeight /100));
      end;
    end;

    LFrac := 0;
    for i := 0 to LStyles.Count - 1 do
    begin
      s := LStyles[i];
      if pos('fr',s) > 0 then
      begin
        s := Copy(s, 1, pos('fr',s) - 1);
        TryStrToInt(s, LRowHeight);
        LFrac := LFrac + LRowHeight;
      end;
    end;

    if LFrac > 0 then
    begin
      for i := 0 to LStyles.Count - 1 do
      begin
        s := LStyles[i];
        if pos('fr',s) > 0 then
        begin
          s := Copy(s, 1, pos('fr',s) - 1);
          TryStrToInt(s, LRowHeight);
          FColWidths.Items[i] := pointer(Round(LRowHeight * LAvailHeight / LFrac));
        end;
      end;
    end;
  end;

  LStyles.Free;
end;

procedure TWebResponsiveGridPanel.SetControlCollection(
  const Value: TControlCollection);
begin
  FControlCollection := Value;
end;

procedure TWebResponsiveGridPanel.SetLayout(const Value: TResponsiveLayout);
begin
  FLayout.Assign(Value);
end;

procedure TWebResponsiveGridPanel.UpdateLayout;
begin
  ResolveColWidths;
  Realign;
  Invalidate;
end;

{ TResponsiveLayout }

function TResponsiveLayout.Add: TResponsiveLayoutItem;
begin
  Result := TResponsiveLayoutItem(inherited Add);
end;

function TResponsiveLayout.Add(AWidth: integer;
  AStyle: string): TResponsiveLayoutItem;
begin
  Result := Add;
  Result.Width := AWidth;
  Result.Style := AStyle;
end;

constructor TResponsiveLayout.Create(AOwner: TComponent);
begin
  inherited Create(TResponsiveLayoutItem);
  FOwner := AOwner;
end;

function TResponsiveLayout.GetItem(Index: integer): TResponsiveLayoutItem;
begin
  Result := TResponsiveLayoutItem(inherited Items[Index]);
end;

function TResponsiveLayout.GetLayoutForWidth(w: integer): TResponsiveLayoutItem;
var
  i,d,j,l,mx: integer;

begin
  Result := nil;
  FActiveLayoutItem := nil;
  if Count = 0 then
    Exit;

  d := $FFFF;
  j := -1;
  l := 0;
  mx := 0;

  for i := 0 to Count - 1 do
  begin
    if Items[i].Width > mx then
    begin
      mx := Items[i].Width;
      l := i;
    end;

    if w < Items[i].Width then
    begin
      if Items[i].Width - w < d then
      begin
        d := Items[i].Width - w;
        j := i;
      end;
    end;
  end;

  if j = -1 then
    j := l;

  Result := Items[j];
  FActiveLayoutItem := Result;
end;

function TResponsiveLayout.Insert(Index: integer): TResponsiveLayoutItem;
begin
  Result := TResponsiveLayoutItem(inherited Insert(Index));
end;

procedure TResponsiveLayout.SetItem(Index: integer; const Value: TResponsiveLayoutItem);
begin
  inherited Items[Index] := Value;
end;

procedure TResponsiveLayout.Update(Item: TCollectionItem);
begin
  inherited;
  if Assigned(OnChange) then
    OnChange(Self);
end;

{ TResponsiveLayoutItem }

procedure TResponsiveLayoutItem.Assign(Source: TPersistent);
begin
  if (Source is TResponsiveLayoutItem) then
  begin
    FColumnGap := (Source as TResponsiveLayoutItem).ColumnGap;
    FDescription := (Source as TResponsiveLayoutItem).Description;
    FRowGap := (Source as TResponsiveLayoutItem).RowGap;
    FStyle := (Source as TResponsiveLayoutItem).Style;
    FStyleType := (Source as TResponsiveLayoutItem).StyleType;
    FWidth := (Source as TResponsiveLayoutItem).Width;
    FTag := (Source as TResponsiveLayoutItem).Tag;
    FMargins.Assign((Source as TResponsiveLayoutItem).Margins);
  end;
end;


procedure TResponsiveLayoutItem.Changed;
begin
  (Collection as TResponsiveLayout).Update(Self);
end;

constructor TResponsiveLayoutItem.Create(AOwner: TCollection);
begin
  inherited;
  FMargins := TLayoutMargins.Create;
end;

destructor TResponsiveLayoutItem.Destroy;
begin
  FMargins.Free;
  inherited;
end;

procedure TResponsiveLayoutItem.SetColumnGap(const Value: string);
begin
  if (FColumnGap <> Value) then
  begin
    FColumnGap := Value;
    Changed;
  end;
end;

procedure TResponsiveLayoutItem.SetMargins(const Value: TLayoutMargins);
begin
  FMargins.Assign(Value);
end;

procedure TResponsiveLayoutItem.SetRowGap(const Value: string);
begin
  if (FRowGap <> Value) then
  begin
    FRowGap := Value;
    Changed;
  end;
end;

procedure TResponsiveLayoutItem.SetStyle(const Value: TResponsiveStyleString);
begin
  if (FStyle <> Value) then
  begin
    FStyle := Value;
    Changed;
  end;
end;

procedure TResponsiveLayoutItem.SetStyleType(const Value: TGridStyle);
begin
  if (FStyleType <> Value) then
  begin
    FStyleType := Value;
    Changed;
  end;
end;

procedure TResponsiveLayoutItem.SetWidth(const Value: integer);
begin
  if (FWidth <> Value) then
  begin
    FWidth := Value;
    Changed;
  end;
end;

{ TAccordionSections }

function TAccordionSections.Add: TAccordionSection;
begin
  Result := TAccordionSection(inherited Add);
end;

constructor TAccordionSections.Create(AOwner: TComponent);
begin
  inherited Create(AOwner, TAccordionSection);
end;

function TAccordionSections.GetItem(Index: integer): TAccordionSection;
begin
  Result := TAccordionSection(inherited Items[Index]);
end;

function TAccordionSections.Insert(Index: integer): TAccordionSection;
begin
  Result := TAccordionSection(inherited Insert(Index));
end;

procedure TAccordionSections.SetItem(Index: integer;
  const Value: TAccordionSection);
begin
  inherited Items[Index] := Value;
end;

procedure TAccordionSections.Update(Item: TCollectionItem);
begin
  inherited;
  if Assigned(OnChange) then
    OnChange(Self);
end;

{ TWebAccordion }

constructor TWebAccordion.Create(AOwner: TComponent);
begin
  inherited;
  FSections := TAccordionSections.Create(Self, TAccordionSection);
  FSections.OnChange := SectionsChanged;
  Width := 320;
  Height := 240;
end;

destructor TWebAccordion.Destroy;
begin
  FSections.Free;
  inherited;
end;

function TWebAccordion.GetDesignText: string;
begin
  if Sections.Count = 0 then
    Result := Name
  else
    Result := '';
end;

procedure TWebAccordion.InitBootstrapStyle;
begin
  inherited;
  ElementFont := efCSS;
  ElementContentClassName := 'card-body';
  ElementSectionClassName := 'card-header';
end;

procedure TWebAccordion.Paint;
var
  i,y,th: integer;
begin
  inherited;

  y := 0;
  Canvas.Font.Assign(Font);
  th := Canvas.TextHeight('gh') + 4;

  for i := 0 to Sections.Count - 1 do
  begin
    Canvas.Brush.Color := clSilver;
    Canvas.Pen.Color := clSilver;
    Canvas.Rectangle(0,y, Width, y + th);
    Canvas.TextOut(2,y + 2, Sections[i].Caption);

    Canvas.Brush.Color := clWhite;
    Canvas.Pen.Color := clWhite;
    Canvas.TextOut(2, y + 2 + th, Sections[i].Content);

    y := y + 2 * th;

    if y > Height then
      break;
  end;

end;

procedure TWebAccordion.SectionsChanged(Sender: TObject);
begin
  Invalidate;
end;

procedure TWebAccordion.SetSections(const Value: TAccordionSections);
begin
  FSections.Assign(Value);
end;

{ TAccordionSection }

procedure TAccordionSection.Assign(Source: TPersistent);
begin
  inherited;
end;

function TAccordionSection.CaptionElement: TJSHTMLElement;
begin
  Result := nil;
end;

function TAccordionSection.PanelElement: TJSHTMLElement;
begin
  Result := nil;
end;

procedure TAccordionSection.SetCaption(const Value: string);
begin
  FCaption := Value;
  (Collection as TAccordionSections).Update(Self);
end;

procedure TAccordionSection.SetContent(const Value: string);
begin
  FContent := Value;
  (Collection as TAccordionSections).Update(Self);
end;

{ TLayoutMargins }

procedure TLayoutMargins.Assign(Source: TPersistent);
begin
  if (Source is TLayoutMargins) then
  begin
    FLeft := (Source as TLayoutMargins).Left;
    FRight := (Source as TLayoutMargins).Right;
    FTop := (Source as TLayoutMargins).Top;
    FBottom := (Source as TLayoutMargins).Bottom;
  end;
end;


{ TWebImageZoomControl }

constructor TWebImageZoomControl.Create(AOwner: TComponent);
begin
  inherited;
  FAppearance := TImageZoomAppearance.Create(Self);
  FPictureZoom := TPicture.Create;
  FURLPictureZoom := TPicture.Create;
end;

destructor TWebImageZoomControl.Destroy;
begin
  FAppearance.Free;
  FPictureZoom.Free;
  FURLPictureZoom.Free;
  inherited;
end;

procedure TWebImageZoomControl.SetAppearance(const Value: TImageZoomAppearance);
begin
  FAppearance := Value;
end;

procedure TWebImageZoomControl.SetPictureZoom(const Value: TPicture);
begin
  FPictureZoom.Assign(Value);
end;

procedure TWebImageZoomControl.SetURLZoom(const Value: string);
begin
  FURLZoom := Value;
end;

{ TImageZoomAppearance }

procedure TImageZoomAppearance.Assign(Source: TPersistent);
begin
  inherited;
  if (Source is TImageZoomAppearance) then
  begin
    FHeightPercent := (Source as TImageZoomAppearance).HeightPercent;
    FWidthPercent := (Source as TImageZoomAppearance).WidthPercent;
    FResponsiveHeightPercent := (Source as TImageZoomAppearance).ResponsiveHeightPercent;
    FResponsiveWidthPercent := (Source as TImageZoomAppearance).ResponsiveWidthPercent;
  end;
end;

constructor TImageZoomAppearance.Create(AOwner: TWebImageZoomControl);
begin
  FHeightPercent := 100;
  FWidthPercent := 100;
  FResponsiveHeightPercent := 100;
  FResponsiveWidthPercent := 100;
  FResponsiveMaxWidth := 768;
  FOwner := AOwner;
end;

destructor TImageZoomAppearance.Destroy;
begin
  inherited;
end;

function TImageZoomAppearance.GetOwner: TPersistent;
begin
  Result := FOwner;
end;


{ TLookupValueItem }

procedure TLookupValueItem.Assign(Source: TPersistent);
begin
  if (Source is TLookupValueItem) then
  begin
    FValue := (Source as TLookupValueItem).Value;
    FDisplayText := (Source as TLookupValueItem).DisplayText;
  end;
end;

{ TLookupValues }

procedure TLookupValues.AddPair(AValue, ADisplayText: string);
var
  lv: TLookupValueItem;
begin
  lv := TLookupValueItem(inherited Add);
  lv.Value := AValue;
  lv.DisplayText := ADisplayText;
end;

constructor TLookupValues.Create(AOwner: TComponent);
begin
  inherited Create(AOwner, TLookupValueItem);
end;

{ TCustomWebLookupComboBox }

constructor TCustomWebLookupComboBox.Create(AOwner: TComponent);
var
  FDesignTime: boolean;
begin
  inherited;
  FWidthStyle := ssAbsolute;
  FWidthPercent := 100;
  FHeightStyle := ssAbsolute;
  FHeightPercent := 100;
  FShowFocus := true;
  FElementFont := efProperty;
  FElementPosition := epAbsolute;
  Style := TComboBoxStyle.csOwnerDrawVariable;
  FLookupValues := TLookupValues.Create(Self);

  FDesignTime := (csDesigning in ComponentState) and not
                 ((csReading in AOwner.ComponentState) or (csLoading in AOwner.ComponentState));

  if FDesignTime and (GetOwnerCSSLibrary(AOwner) = cssBootstrap) then
  begin
    ElementFont := efCSS;
    ElementClassName := 'custom-select';
  end;
end;

destructor TCustomWebLookupComboBox.Destroy;
begin
  FLookupValues.Free;
  inherited;
end;

function TCustomWebLookupComboBox.GetDisplayText: string;
begin
  Result := '';
end;

function TCustomWebLookupComboBox.GetHeightEx: integer;
begin
  Result := inherited Height;
end;

function TCustomWebLookupComboBox.GetValue: string;
begin
  Result := '';
end;

procedure TCustomWebLookupComboBox.SetDisplayText(const Value: string);
begin
//
end;

procedure TCustomWebLookupComboBox.SetHeightEx(const Value: integer);
begin
  ItemHeight := Value - 6;
  inherited Height := Value;
end;

procedure TCustomWebLookupComboBox.SetHeightPercent(const Value: TPercentSize);
begin
  if (FHeightPercent <> Value) then
  begin
    FHeightPercent := Value;
    if Assigned(Parent) then
      Parent.Realign;
  end;
end;

procedure TCustomWebLookupComboBox.SetHeightStyle(const Value: TSizeStyle);
begin
  if (FHeightStyle <> Value) then
  begin
    FHeightStyle := Value;
    if Assigned(Parent) then
      Parent.Realign;
  end;
end;

procedure TCustomWebLookupComboBox.SetLookupValues(const Value: TLookupValues);
begin
  FLookupValues.Assign(Value);
end;

procedure TCustomWebLookupComboBox.SetValue(const Value: string);
begin
//
end;

procedure TCustomWebLookupComboBox.SetWidthPercent(const Value: TPercentSize);
begin
  FWidthPercent := Value;
end;

procedure TCustomWebLookupComboBox.SetWidthStyle(const Value: TSizeStyle);
begin
  FWidthStyle := Value;
end;

{ TControlCollectionItem }

procedure TControlCollectionItem.AssignTo(ADest: TPersistent);
begin
  if ADest is TControlCollectionItem then
  begin
    TControlCollectionItem(ADest).FColumn := FColumn;
    TControlCollectionItem(ADest).FControl := FControl;
    TControlCollectionItem(ADest).FRow := FRow;
  end
  else
    inherited AssignTo(ADest);
end;

end.
