{********************************************************************}
{                                                                    }
{ written by TMS Software                                            }
{            copyright  2018                                        }
{            Email : info@tmssoftware.com                            }
{            Web : http://www.tmssoftware.com                        }
{                                                                    }
{ The source code is given as is. The author is not responsible      }
{ for any possible damage done due to the use of this code.          }
{ The complete source code remains property of the author and may    }
{ not be distributed, published, given or sold in any form as such.  }
{ No parts of the source code can be included in any other component }
{ or application without written authorization of the author.        }
{********************************************************************}

unit WEBLib.WebSocketClient;

interface

uses
  Classes, JS, Web, WEBLib.Controls;

const
  DefaultPort = 8888;

type
  TWebSocketClientDataReceivedEvent = procedure(Sender: TObject; Origin: string; SocketData: TJSObjectRecord) of object;

  {$IFNDEF LCLTMSWEB}
  [ComponentPlatforms(TMSWebPlatform)]
  {$ENDIF}
  TWebSocketClient = class(TComponent)
  private
    FPort: Integer;
    FHostName: string;
    FPathName: string;
    FOnConnect: TNotifyEvent;
    FOnDisconnect: TNotifyEvent;
    FOnDataReceived: TWebSocketClientDataReceivedEvent;
  protected
    procedure CreateWebSockets;
    procedure DoMessage(AEvent: TJSMessageEvent);
    procedure DoOpen(AEvent: TJSEvent);
    procedure DoClose(AEvent: TJSEvent);
  public
    constructor Create(AOwner: TComponent); overload; override;
    constructor Create(AOwner: TComponent; APort: Integer); reintroduce; overload; virtual;
    constructor Create(AOwner: TComponent; AHostName: string; APort: Integer); reintroduce; overload; virtual;
    constructor Create(AOwner: TComponent; AHostName: string; APathName: string; APort: Integer); reintroduce; overload; virtual;
    function GetDefaultHostName: string; virtual;
    function GetDefaultPort: Integer; virtual;
    function GetDefaultPathName: string; virtual;
    function SupportsWebSockets: Boolean; virtual;
    function GetDefaultProtocol: string; virtual;
    function GetWebSocketPath: string; virtual;
    procedure Connect;
    procedure Send(const AMessage: string); overload;
    procedure Send(AMessage: TJSArrayBuffer); overload;
    procedure Disconnect;
  published
    property Port: Integer read FPort write FPort default DefaultPort;
    property HostName: string read FHostName write FHostName;
    property PathName: string read FPathName write FPathName;
    property OnConnect: TNotifyEvent read FOnConnect write FOnConnect;
    property OnDisconnect: TNotifyEvent read FOnDisconnect write FOnDisconnect;
    property OnDataReceived: TWebSocketClientDataReceivedEvent read FOnDataReceived write FOnDataReceived;
  end;

implementation

{ TWebSocketClient }

constructor TWebSocketClient.Create(AOwner: TComponent);
begin
  Create(AOwner, GetDefaultHostName, GetDefaultPathName, GetDefaultPort);
end;

procedure TWebSocketClient.Connect;
begin
end;

constructor TWebSocketClient.Create(AOwner: TComponent; APort: Integer);
begin
  Create(AOwner, GetDefaultHostName, GetDefaultPathName, APort);
end;

constructor TWebSocketClient.Create(AOwner: TComponent; AHostName: string;
  APort: Integer);
begin
  Create(AOwner, AHostName, GetDefaultPathName, APort);
end;

constructor TWebSocketClient.Create(AOwner: TComponent; AHostName,
  APathName: string; APort: Integer);
begin
  inherited Create(AOwner);
  FPort := APort;
  FHostName := AHostName;
  FPathName := APathName;
end;

procedure TWebSocketClient.CreateWebSockets;
begin
end;

procedure TWebSocketClient.Disconnect;
begin
end;

procedure TWebSocketClient.DoClose(AEvent: TJSEvent);
begin
end;

procedure TWebSocketClient.DoMessage(AEvent: TJSMessageEvent);
begin
end;

procedure TWebSocketClient.DoOpen(AEvent: TJSEvent);
begin
end;

function TWebSocketClient.GetDefaultHostName: string;
begin
  Result := '';
end;

function TWebSocketClient.GetDefaultPathName: string;
begin
  Result := '';
end;

function TWebSocketClient.GetDefaultPort: Integer;
begin
  Result := DefaultPort;
end;

function TWebSocketClient.GetDefaultProtocol: string;
begin
  Result := '';
end;

function TWebSocketClient.GetWebSocketPath: string;
begin
  Result := '';
end;

procedure TWebSocketClient.Send(const AMessage: string);
begin
end;

procedure TWebSocketClient.Send(AMessage: TJSArrayBuffer);
begin
end;

function TWebSocketClient.SupportsWebSockets: Boolean;
begin
  Result := true;
end;

end.
