{********************************************************************}
{                                                                    }
{ written by TMS Software                                            }
{            Copyright (c) 2018 - 2020                               }
{            Email : info@tmssoftware.com                            }
{            Web : http://www.tmssoftware.com                        }
{                                                                    }
{ The source code is given as is. The author is not responsible      }
{ for any possible damage done due to the use of this code.          }
{ The complete source code remains property of the author and may    }
{ not be distributed, published, given or sold in any form as such.  }
{ No parts of the source code can be included in any other component }
{ or application without written authorization of the author.        }
{********************************************************************}

unit jsdelphisystem;

interface

uses
  SysUtils;

type
  TJSValue = record
  strict private
    FValue: Variant;
  public
    class function Create(const AValue: Boolean): TJSValue; overload; static;
    class function Create(const AValue: Double): TJSValue; overload; static;
    class function Create(const AValue: Integer): TJSValue; overload; static;
    class function Create(const AValue: string): TJSValue; overload; static;
    class function Create(const AValue: IInterface): TJSValue; overload; static;
    class function Create(const AValue: TBytes): TJSValue; overload; static;
    class function Create(const AValue: TDate): TJSValue; overload; static;
    class function Create(const AValue: TDateTime): TJSValue; overload; static;
    class function Create(const AValue: TJSValue): TJSValue; overload; static;
    class function Create(const AValue: TObject): TJSValue; overload; static;
    class function Create(const AValue: TTime): TJSValue; overload; static;
    class operator Explicit(const AValue: Boolean): TJSValue;
    class operator Explicit(const AValue: Double): TJSValue;
    class operator Explicit(const AValue: Integer): TJSValue;
    class operator Explicit(const AValue: string): TJSValue;
    class operator Explicit(const AValue: IInterface): TJSValue;
    class operator Explicit(const AValue: TBytes): TJSValue;
    class operator Explicit(const AValue: TDate): TJSValue;
    class operator Explicit(const AValue: TDateTime): TJSValue;
    class operator Explicit(const AValue: TJSValue): string;
    class operator Explicit(const AValue: TJSValue): Integer;
    class operator Explicit(const AValue: TJSValue): TObject;
    class operator Explicit(const AValue: TJSValue): Boolean;
    class operator Explicit(const AValue: TJSValue): TBytes;
    class operator Explicit(const AValue: TJSValue): Double;
    class operator Explicit(const AValue: TJSValue): IInterface;
    class operator Explicit(const AValue: TObject): TJSValue;
    class operator Explicit(const AValue: TTime): TJSValue;
    class operator Implicit(const AValue: Boolean): TJSValue;
    class operator Implicit(const AValue: Double): TJSValue;
    class operator Implicit(const AValue: Integer): TJSValue;
    class operator Implicit(const AValue: string): TJSValue;
    class operator Implicit(const AValue: IInterface): TJSValue;
    class operator Implicit(const AValue: TBytes): TJSValue;
    class operator Implicit(const AValue: TDate): TJSValue;
    class operator Implicit(const AValue: TDateTime): TJSValue;
    class operator Implicit(const AValue: TJSValue): Integer;
    class operator Implicit(const AValue: TJSValue): string;
    class operator Implicit(const AValue: TJSValue): Double;
    class operator Implicit(const AValue: TJSValue): Boolean;
    class operator Implicit(const AValue: TJSValue): TBytes;
    class operator Implicit(const AValue: TJSValue): IInterface;
    class operator Implicit(const AValue: TJSValue): TObject;
    class operator Implicit(const AValue: TObject): TJSValue;
    class operator Implicit(const AValue: TTime): TJSValue;
  end;
  JSValue = TJSValue;

  Float32 = Double;
  Float64 = Double;
  TJSValueDynArray = array of JSValue;
  TJSObjectPropertyDescriptor = TObject;

implementation

{ TJSValue }

class function TJSValue.Create(const AValue: Boolean): TJSValue;
begin
  Result.FValue := AValue;
end;

class function TJSValue.Create(const AValue: Double): TJSValue;
begin
  Result.FValue := AValue;
end;

class function TJSValue.Create(const AValue: Integer): TJSValue;
begin
  Result.FValue := AValue;
end;

class function TJSValue.Create(const AValue: string): TJSValue;
begin
  Result.FValue := AValue;
end;

class function TJSValue.Create(const AValue: IInterface): TJSValue;
begin
  Result.FValue := AValue;
end;

class function TJSValue.Create(const AValue: TBytes): TJSValue;
begin
  Result.FValue := AValue;
end;

class function TJSValue.Create(const AValue: TDate): TJSValue;
begin
  Result.FValue := AValue;
end;

class function TJSValue.Create(const AValue: TDateTime): TJSValue;
begin
  Result.FValue := AValue;
end;

class function TJSValue.Create(const AValue: TJSValue): TJSValue;
begin
  Result.FValue := AValue.FValue;
end;

class function TJSValue.Create(const AValue: TObject): TJSValue;
begin
  Result.FValue := NativeInt(AValue);
end;

class function TJSValue.Create(const AValue: TTime): TJSValue;
begin
  Result.FValue := AValue;
end;

class operator TJSValue.Explicit(const AValue: Boolean): TJSValue;
begin
  Result := Create(AValue);
end;

class operator TJSValue.Explicit(const AValue: Double): TJSValue;
begin
  Result := Create(AValue);
end;

class operator TJSValue.Explicit(const AValue: Integer): TJSValue;
begin
  Result := Create(AValue);
end;

class operator TJSValue.Explicit(const AValue: string): TJSValue;
begin
  Result := Create(AValue);
end;

class operator TJSValue.Explicit(const AValue: IInterface): TJSValue;
begin
  Result := Create(AValue);
end;

class operator TJSValue.Explicit(const AValue: TBytes): TJSValue;
begin
  Result := Create(AValue);
end;

class operator TJSValue.Explicit(const AValue: TDate): TJSValue;
begin
  Result := Create(AValue);
end;

class operator TJSValue.Explicit(const AValue: TDateTime): TJSValue;
begin
  Result := Create(AValue);
end;

class operator TJSValue.Explicit(const AValue: TJSValue): IInterface;
begin
  Result := AValue.FValue;
end;

class operator TJSValue.Explicit(const AValue: TJSValue): Integer;
begin
  Result := AValue.FValue;
end;

class operator TJSValue.Explicit(const AValue: TJSValue): Boolean;
begin
  Result := AValue.FValue;
end;

class operator TJSValue.Explicit(const AValue: TJSValue): TBytes;
begin
  Result := AValue.FValue;
end;

class operator TJSValue.Explicit(const AValue: TJSValue): TObject;
begin
  {$IFNDEF LCLTMSWEB}
  Result := TObject(NativeInt(AValue.FValue));
  {$ENDIF}
end;

class operator TJSValue.Explicit(const AValue: TJSValue): string;
begin
  Result := AValue.FValue;
end;

class operator TJSValue.Explicit(const AValue: TJSValue): Double;
begin
  Result := AValue.FValue;
end;

class operator TJSValue.Explicit(const AValue: TObject): TJSValue;
begin
  Result := Create(AValue);
end;

class operator TJSValue.Explicit(const AValue: TTime): TJSValue;
begin
  Result := Create(AValue);
end;

class operator TJSValue.Implicit(const AValue: Boolean): TJSValue;
begin
  Result := Create(AValue);
end;

class operator TJSValue.Implicit(const AValue: Double): TJSValue;
begin
  Result := Create(AValue);
end;

class operator TJSValue.Implicit(const AValue: Integer): TJSValue;
begin
  Result := Create(AValue);
end;

class operator TJSValue.Implicit(const AValue: string): TJSValue;
begin
  Result := Create(AValue);
end;

class operator TJSValue.Implicit(const AValue: IInterface): TJSValue;
begin
  Result := Create(AValue);
end;

class operator TJSValue.Implicit(const AValue: TBytes): TJSValue;
begin
  Result := Create(AValue);
end;

class operator TJSValue.Implicit(const AValue: TDate): TJSValue;
begin
  Result := Create(AValue);
end;

class operator TJSValue.Implicit(const AValue: TDateTime): TJSValue;
begin
  Result := Create(AValue);
end;

class operator TJSValue.Implicit(const AValue: TJSValue): IInterface;
begin
  Result := AValue.FValue;
end;

class operator TJSValue.Implicit(const AValue: TJSValue): Boolean;
begin
  Result := AValue.FValue;
end;

class operator TJSValue.Implicit(const AValue: TJSValue): TObject;
begin
  {$IFNDEF LCLTMSWEB}
  Result := TObject(NativeInt(AValue.FValue));
  {$ENDIF}
end;

class operator TJSValue.Implicit(const AValue: TJSValue): string;
begin
  Result := Create(AValue);
end;

class operator TJSValue.Implicit(const AValue: TJSValue): Integer;
begin
  Result := AValue.FValue;
end;

class operator TJSValue.Implicit(const AValue: TJSValue): Double;
begin
  Result := AValue.FValue;
end;

class operator TJSValue.Implicit(const AValue: TJSValue): TBytes;
begin
  Result := AValue.FValue;
end;

class operator TJSValue.Implicit(const AValue: TObject): TJSValue;
begin
  Result := Create(AValue);
end;

class operator TJSValue.Implicit(const AValue: TTime): TJSValue;
begin
  Result := Create(AValue);
end;

end.
