unit libjquery;

interface

uses jsdelphisystem, js, web;

type
  TCallbacks = class;
  TAjaxEvent = class;
  TJQueryDeferred = class;
  TJQXHR = class;
  TJSAjaxSettings = class;
  TJQuery = class;

  TJQueryTopLeft = record
    top : Integer;
    left : Integer;
  end;
  TCallback = procedure(args: JSValue);
  TCallbackEvent = procedure(args: JSValue);

  TCallbacks = class
  public
    function add(aCallBack: TCallBack): TCallbacks; overload;
    function add(aCallBack: Array of TCallBack): TCallbacks; overload;
    function add(aCallBack: TCallBackEvent): TCallbacks; overload;
    function add(aCallBack: Array of TCallBackEvent): TCallbacks; overload;
    function disable: TCallBacks;
    function disabled: Boolean;
    function empty: TCallBacks;
    function fire(arguments: JSValue): TCallbacks;
    function fired: Boolean;
    function fireWith(context: JSValue; arguments: JSValue): TCallbacks;
    function has(aCallBack: TCallBack): Boolean; overload;
    function has(aCallBack: TCallBackEvent): Boolean; overload;
    function lock: TCallBacks;
    function locked: Boolean;
    function remove(aCallBack: TCallBack): TCallbacks; overload;
    function remove(aCallBack: Array of TCallBack): TCallbacks; overload;
    function remove(aCallBack: TCallBackEvent): TCallbacks; overload;
    function remove(aCallBack: Array of TCallBackEvent): TCallbacks; overload;
  end;
  TJQueryAddClassHandler = function(aIndex: Integer; AClass: string)
                                   : string;
  TJQueryAttrHandler = function(aIndex: Integer; aAttr: string) : JSValue;
  TJQueryCSSHandler = function(aIndex: Integer; AClass: string) : JSValue;
  TJQueryEachHandler = function(aIndex: Integer; AElement: TJSElement)
                               : Boolean;
  TJQueryFilterHandler = function(aIndex: Integer; AElement: TJSElement)
                                 : Boolean;
  TJQueryHeightHandler = function(aIndex: Integer; AHeight: jsValue)
                                 : JSValue;
  TJQueryHTMLHandler = function(aIndex: Integer; aHTML: string) : string;
  TJQueryMapHandler = function(aIndex: Integer; AElement: TJSElement)
                              : TJSObject;
  TJQueryOffsetHandler = function(aIndex: Integer; 
                                aCoords: TJQueryTopLeft)
                                 : TJQueryTopLeft;
  TJQueryPropHandler = function(aIndex: Integer; oldProp: JSValue)
                               : JSValue;
  TJQueryQueueHandler = procedure;
  TJQueryTextHandler = function(aIndex: Integer; aString: string) : string;
  TJQueryToggleClassHandler = function(aIndex: Integer; 
                                     aClassName: string; 
                                     AState: Boolean) : string;
  TJQueryValHandler = function(aIndex: Integer; aValue: string) : string;
  TJQueryWidthHandler = function(aIndex: Integer; AHeight: jsValue)
                                : JSValue;
  TJQueryDeQueueFunction = procedure;
  TJQueryAddQueueHandler = procedure(aFunc: TJQueryDeQueueFunction);

  TAjaxEvent = class(TJSEvent)
  end;
  TDeferredDoneHandler = function : Boolean;

  TJQueryDeferred = class(TJSObject)
  public
    function done(aHandler: TDeferredDoneHandler): TJQueryDeferred; overload;
    function done(aHandlers: Array of TDeferredDoneHandler): TJQueryDeferred; overload;
  end;
  TJQXHRDoneHandler = function(aData: jsValue; textStatus: string; 
                             aJQXHR: TJQXHR) : Boolean;
  TJQXHRFailHandler = function(aJQXHR: TJQXHR; textStatus: string; 
                             AErrorThrown: jsValue) : Boolean;
  TJQXHRAlwaysHandler = function(arg1: TJSObject; textStatus: string; 
                               arg2: TJSObject) : Boolean;

  TJQXHR = class(TJQueryDeferred)
  private
    FReadyState: NativeInt;
    FResponse: JSValue;
    FResponseText: string;
    FresponseXML: TJSDocument;
    FUpload: TJSXMLHttpRequestUpload;
    FStatus: NativeInt;
    FStatusText: string;
  public
    function getResponseHeader(aName: string): string;
    function getAllResponseHeaders: string;
    procedure overrideMimeType(aType: string);
    procedure setRequestHeader(aName: string; AValue: string);
    procedure done(aHandler: TJQXHRDoneHandler); overload;
    procedure always(aHandler: TJQXHRAlwaysHandler); overload;
    procedure fail(aHandler: TJQXHRFailHandler); overload;
    procedure _then(aSuccess: TJQXHRDoneHandler; aFail: TJQXHRFailHandler); overload;
    procedure abort; overload;
    procedure abort(AStatusText: string); overload;
    property readyState: NativeInt read FReadyState;
    property ResponseHeaders[aName : string]: string read getResponseHeader;
    property responseXML: TJSDocument read FresponseXML;
    property responseText: string read FResponseText;
    property status: NativeInt read FStatus;
    property statusText: string read FStatusText;
  end;
  TJQueryAjaxSettingsHandler = function(aHXR: TJQXHR; 
                                      aOptions: TJSAjaxSettings)
                                       : Boolean;
  TJQueryAjaxSettingsDataFilter = function(aData: string; aType: string)
                                          : JSValue;
  TJQueryAjaxSettingsErrorHandler = function(aHXR: TJQXHR; 
                                           aOptions: TJSAjaxSettings; 
                                           aStatus: string; 
                                           aError: string) : Boolean;
  TJQueryAjaxSettingsSuccessHandler = function(data: JSValue; 
                                             aStatus: string; 
                                             aHXR: TJQXHR) : Boolean;
  TJQueryAjaxSettsingsXHRHandler = function : JSValue;

  TJSAjaxSettings = class(TJSObject)
    accepts: TJSObject;
    async: Boolean;
    beforeSend: TJQueryAjaxSettingsHandler;
    cache: Boolean;
    complete: TJQueryAjaxSettingsHandler;
    contents: TJSObject;
    contentType: string;
    context: TJSObject;
    converters: TJSObject;
    crossDomain: Boolean;
    data: JSValue;
    dataFilter: TJQueryAjaxSettingsDataFilter;
    dataType: string;
    error: TJQueryAjaxSettingsErrorHandler;
    global: Boolean;
    headers: TJSObject;
    ifModified: Boolean;
    isLocal: Boolean;
    json: string;
    jsonpCallback: string;
    method: string;
    mimeType: string;
    password: string;
    processData: Boolean;
    scriptCharset: string;
    statusCode: TJSObject;
    success: TJQueryAjaxSettingsSuccessHandler;
    timeout: NativeInt;
    traditional: Boolean;
    url: string;
    username: string;
    xhr: TJQueryAjaxSettsingsXHRHandler;
    xhrFields: TJSObject;
  end;
  TJQueryAjaxTransportCompleteHandler = function(aStatus: NativeInt; 
                                               aStatusText: string; 
                                               responses: TJSObject; 
                                               Headers: TJSObject)
                                                : Boolean;
  TJQueryAjaxTransportSendHandler = function(headers: TJSObject; 
                                           onComplete: TJQueryAjaxTransportCompleteHandler)
                                            : Boolean;
  TJQueryAjaxTransportAbortHandler = function : Boolean;
  TJQueryAjaxTransport = record
    send : TJQueryAjaxTransportSendHandler;
    abort : TJQueryAjaxTransportAbortHandler;
  end;
  TJQueryAjaxTransportHandler = function(aOptions: TJSObject; 
                                       aOriginalOptions: TJSObject; 
                                       aXHR: TJQXHR)
                                        : TJQueryAjaxTransport;
  TJQueryAjaxPrefilterHandler = procedure(aOptions: TJSObject; 
                                        aOriginalOptions: TJSObject; 
                                        aXHR: TJQXHR);
  TJQueryAjaxEventHandler = function(aEvent: TAjaxEvent; aHXR: TJQXHR; 
                                   aOptions: TJSAjaxSettings) : Boolean;
  TJQueryAjaxErrorHandler = function(aEvent: TAjaxEvent; aHXR: TJQXHR; 
                                   aOptions: TJSAjaxSettings; 
                                   aError: string) : Boolean;
  TJQueryAjaxSuccessHandler = function(aData: TJSObject; aStatus: string; 
                                     aXHR: TJQXHR) : Boolean;
  TJQueryAjaxLoadHandler = function(aResponseText: string; 
                                  aStatus: string; aXHR: TJQXHR)
                                   : Boolean;
  TJQueryAjaxScriptHandler = function(aScript: string; aStatus: string; 
                                    aXHR: TJQXHR) : Boolean;
  TJQueryAjaxHandler = procedure;

  TJQuery = class(TJSObject)
  private
    FCSSHooks: TJSObject;
    FCSSNumber: TJSObject;
    FReady: TJSPromise;
    function getEl(aIndex: Integer): TJSElement;
  public
    function add(const aSelector: string): TJQuery; overload;
    function add(const aSelector: string; AContext: TJSElement): TJQuery; overload;
    function add(const aElement: TJSElement): TJQuery; overload;
    function add(const aElement: Array of TJSElement): TJQuery; overload;
    function add(const aQuery: TJQuery): TJQuery; overload;
    function addBack(const aSelector: string): TJQuery; overload;
    function addBack: TJQuery; overload;
    function addClass(const aClass: string): TJQuery; overload;
    function addClass(const aClassFunction: TJQueryAddClassHandler): TJQuery; overload;
    function ajaxComplete(aHandler: TJQueryAjaxEventHandler): TJQuery;
    function ajaxError(aHandler: TJQueryAjaxEventHandler): TJQuery;
    function ajaxSend(aHandler: TJQueryAjaxEventHandler): TJQuery;
    function ajaxStart(aHandler: TJQueryAjaxHandler): TJQuery;
    function ajaxStop(aHandler: TJQueryAjaxHandler): TJQuery;
    function ajaxSuccess(aHandler: TJQueryAjaxEventHandler): TJQuery;
    class function ajax(aURL: string; aSettings: TJSObject): tJQXHR; overload;
    class function ajax(aSettings: TJSObject): tJQXHR; overload;
    class function ajax(aSettings: TJSAjaxSettings): tJQXHR; overload;
    class procedure ajaxPrefilter(dataTypes: string; aHandler: TJQueryAjaxPrefilterHandler); overload;
    class procedure ajaxSetup(aSettings: TJSAjaxSettings); overload;
    class procedure ajaxSetup(aSettings: TJSObject); overload;
    class procedure ajaxTransport(aDataType: string; AHandler: TJQueryAjaxTransportHandler);
    function attr(const attributeName: string): string; overload;
    function attr(const attributeName: string; const Value: string): TJQuery; overload;
    function attr(const attributes: TJSObject): TJQuery; overload;
    function attr(const attributeName: string; aHandler: TJQueryAttrHandler): TJQuery; overload;
    class function Callbacks: TCallbacks; overload;
    class function Callbacks(const aFlags: string): TCallbacks; overload;
    function children(const aSelector: string): TJQuery; overload;
    function children: TJQuery; overload;
    function clearQueue: TJQuery; overload;
    function clearQueue(const aQueueName: string): TJQuery; overload;
    function closest(const aSelector: string): TJQuery; overload;
    function closest(const aSelector: string; AContext: TJSElement): TJQuery; overload;
    function closest(const aQuery: TJQuery): TJQuery; overload;
    function closest(const aElement: TJSElement): TJQuery; overload;
    function contents: TJQuery;
    function css(const aPropertyName: TJSObject): string; overload;
    function css(const aPropertyName: string): string; overload;
    function css(const aPropertyNames: Array of string): string; overload;
    function css(const aPropertyName: string; const Avalue: string): TJQuery; overload;
    function css(const aPropertyName: string; Avalue: Integer): TJQuery; overload;
    function css(const aPropertyName: string; AHandler: TJQueryCSSHandler): TJQuery; overload;
    class function data(aElement: TJSElement; const aKey: string; aValue: jsValue): TJSObject; overload;
    class function data(aElement: TJSElement; const aKey: string): TJSObject; overload;
    class function data(aElement: TJSElement): TJSObject; overload;
    function data(aKey: string; aValue: JSValue): TJQuery; overload;
    function data(aObj: TJSObject): TJQuery; overload;
    function data(aKey: string): TJSObject; overload;
    function data: TJSObject; overload;
    function dequeue: TJQuery; overload;
    function dequeue(const aQueueName: string): TJQuery; overload;
    class function dequeue(aElement: TJSElement): TJQuery; overload;
    class function dequeue(aElement: TJSElement; const aQueueName: string): TJQuery; overload;
    function _end: TJQuery;
    function eq(AIndex: Integer): TJQuery;
    function each(aHandler: TJQueryEachHandler): TJQuery;
    class function escapeSelector(const S: string): string;
    function filter(const aSelector: string): TJQuery; overload;
    function filter(aHandler: TJQueryFilterHandler): TJQuery; overload;
    function filter(const aQuery: TJQuery): TJQuery; overload;
    function filter(const aElement: TJSElement): TJQuery; overload;
    function filter(const aElements: Array of TJSElement): TJQuery; overload;
    function find(const aSelector: string): TJQuery; overload;
    function find(const aQuery: TJQuery): TJQuery; overload;
    function find(const aElement: TJSElement): TJQuery; overload;
    function first: TJQuery;
    function get(aIndex: Integer): TJSElement; overload;
    class function get: TJQXHR; overload;
    class function get(url: string): TJQXHR; overload;
    {$IFNDEF LCLTMSWEB}
    class function get(url: string; Data: string): TJQXHR; overload;
    class function get(url: string; Data: TJSObject): TJQXHR; overload;
    class function get(url: string; Data: TJSObject; success: TJQueryAjaxSuccessHandler): TJQXHR; overload;
    class function get(url: string; Data: string; success: TJQueryAjaxSuccessHandler): TJQXHR; overload;
    class function get(url: string; Data: TJSObject; success: TJQueryAjaxSuccessHandler; aDataType: string): TJQXHR; overload;
    class function get(url: string; Data: string; success: TJQueryAjaxSuccessHandler; aDataType: string): TJQXHR; overload;
    {$ENDIF}
    class function get(aSettings: TJSAjaxSettings): TJQXHR; overload;
    class function get(aSettings: TJSObject): TJQXHR; overload;
    class function getJSON(url: string): TJQXHR; overload;
    {$IFNDEF LCLTMSWEB}
    class function getJSON(url: string; Data: string): TJQXHR; overload;
    class function getJSON(url: string; Data: TJSObject): TJQXHR; overload;
    class function getJSON(url: string; Data: TJSObject; success: TJQueryAjaxSuccessHandler): TJQXHR; overload;
    class function getJSON(url: string; Data: string; success: TJQueryAjaxSuccessHandler): TJQXHR; overload;
    class function getJSON(url: string; Data: TJSObject; success: TJQueryAjaxSuccessHandler; aDataType: string): TJQXHR; overload;
    class function getJSON(url: string; Data: string; success: TJQueryAjaxSuccessHandler; aDataType: string): TJQXHR; overload;
    {$ENDIF}
    class function getScript(url: string): TJQXHR; overload;
    class function getScript(url: string; aSuccess: TJQueryAjaxScriptHandler): TJQXHR; overload;
    function has(const aSelector: string): TJQuery; overload;
    function has(const aQuery: TJQuery): TJQuery; overload;
    function hasClass(const aClassName: string): Boolean;
    class function hasData(aElement: TJSElement): Boolean;
    function height: Integer; overload;
    function height(aValue: Integer): TJQuery; overload;
    function height(aValue: string): TJQuery; overload;
    function height(aHandler: TJQueryHeightHandler): TJQuery; overload;
    function html: string; overload;
    function html(const aHTML: string): TJQuery; overload;
    function html(const aHandler: TJQueryHTMLHandler): TJQuery; overload;
    function innerHeight: Integer; overload;
    function innerHeight(aValue: Integer): TJQuery; overload;
    function innerHeight(aValue: string): TJQuery; overload;
    function innerHeight(aHandler: TJQueryHeightHandler): TJQuery; overload;
    function innerWidth: Integer; overload;
    function innerWidth(aValue: Integer): TJQuery; overload;
    function innerWidth(aValue: string): TJQuery; overload;
    function innerWidth(aHandler: TJQueryWidthHandler): TJQuery; overload;
    function _is(const aSelector: string): TJQuery; overload;
    function _is(const aQuery: TJQuery): TJQuery; overload;
    function _is(aHandler: TJQueryFilterHandler): TJQuery; overload;
    function _is(const aElement: TJSElement): TJQuery; overload;
    function _is(const aElements: Array of TJSElement): TJQuery; overload;
    function last: TJQuery;
    class function load(url: string): TJQXHR; overload;
    {$IFNDEF LCLTMSWEB}
    class function load(url: string; Data: string): TJQXHR; overload;
    class function load(url: string; Data: TJSObject): TJQXHR; overload;
    class function load(url: string; Data: TJSObject; success: TJQueryAjaxLoadHandler): TJQXHR; overload;
    class function load(url: string; Data: string; success: TJQueryAjaxLoadHandler): TJQXHR; overload;
    {$ENDIF}
    function map(aHandler: TJQueryMapHandler): TJQuery;
    function next: TJQuery; overload;
    function next(const aSelector: string): TJQuery; overload;
    function nextAll: TJQuery; overload;
    function nextAll(const aSelector: string): TJQuery; overload;
    function nextUntil: TJQuery; overload;
    function nextUntil(const aSelector: string): TJQuery; overload;
    function nextUntil(const aSelector: string; const aFilter: string): TJQuery; overload;
    function nextUntil(const aElement: TJSElement): TJQuery; overload;
    function nextUntil(const aElement: TJSElement; aFilter: string): TJQuery; overload;
    function nextUntil(const aQuery: TJQuery): TJQuery; overload;
    function nextUntil(const aQuery: TJQuery; aFilter: string): TJQuery; overload;
    function _not(const aSelector: string): TJQuery; overload;
    function _not(const aSelector: TJSElement): TJQuery; overload;
    function _not(const aSelector: Array of TJSElement): TJQuery; overload;
    function _not(const aSelector: TJQuery): TJQuery; overload;
    function _not(const aSelector: TJQueryFilterHandler): TJQuery; overload;
    function noConflict: TJSObject; overload;
    function noConflict(removeAll: Boolean): TJSObject; overload;
    function offSet: Integer; overload;
    function offSet(const aOffset: TJQueryTopLeft): TJQuery; overload;
    function offSet(aHandler: TJQueryOffsetHandler): TJQuery; overload;
    function offsetParent: TJQuery;
    function outerHeight(IncludeMargin: Boolean): Integer; overload;
    function outerHeight: Integer; overload;
    function outerHeight(aValue: Integer): TJQuery; overload;
    function outerHeight(aValue: string): TJQuery; overload;
    function outerHeight(aHandler: TJQueryHeightHandler): TJQuery; overload;
    function outerWidth(IncludeMargin: Boolean): Integer; overload;
    function outerWidth: Integer; overload;
    function outerWidth(aValue: Integer): TJQuery; overload;
    function outerWidth(aValue: string): TJQuery; overload;
    function outerWidth(aHandler: TJQueryWidthHandler): TJQuery; overload;
    class function param(aObject: string): string; overload;
    class function param(aObject: TJSObject): string; overload;
    class function param(aObject: TJQuery): string; overload;
    class function param(aObject: string; traditional: Boolean): string; overload;
    class function param(aObject: TJSObject; traditional: Boolean): string; overload;
    class function param(aObject: TJQuery; traditional: Boolean): string; overload;
    function parent: TJQuery; overload;
    function parent(const ASelector: string): TJQuery; overload;
    function parents: TJQuery; overload;
    function parents(const ASelector: string): TJQuery; overload;
    function parentsUntil: TJQuery; overload;
    function parentsUntil(const aSelector: string): TJQuery; overload;
    function parentsUntil(const aSelector: string; const aFilter: string): TJQuery; overload;
    function parentsUntil(const aElement: TJSElement): TJQuery; overload;
    function parentsUntil(const aElement: TJSElement; aFilter: string): TJQuery; overload;
    function parentsUntil(const aQuery: TJQuery): TJQuery; overload;
    function parentsUntil(const aQuery: TJQuery; aFilter: string): TJQuery; overload;
    function position: TJQueryTopLeft;
    class function post(url: string): TJQXHR; overload;
    {$IFNDEF LCLTMSWEB}
    class function post(url: string; Data: string): TJQXHR; overload;
    class function post(url: string; Data: TJSObject): TJQXHR; overload;
    class function post(url: string; Data: TJSObject; success: TJQueryAjaxSuccessHandler): TJQXHR; overload;
    class function post(url: string; Data: string; success: TJQueryAjaxSuccessHandler): TJQXHR; overload;
    class function post(url: string; Data: TJSObject; success: TJQueryAjaxSuccessHandler; aDataType: string): TJQXHR; overload;
    class function post(url: string; Data: string; success: TJQueryAjaxSuccessHandler; aDataType: string): TJQXHR; overload;
    {$ENDIF}
    class function post(aSettings: TJSAjaxSettings): TJQXHR; overload;
    class function post(aSettings: TJSObject): TJQXHR; overload;
    function prev: TJQuery; overload;
    function prev(const aSelector: string): TJQuery; overload;
    function prevAll: TJQuery; overload;
    function prevAll(const aSelector: string): TJQuery; overload;
    function prevUntil: TJQuery; overload;
    function prevUntil(const aSelector: string): TJQuery; overload;
    function prevUntil(const aSelector: string; const aFilter: string): TJQuery; overload;
    function prevUntil(const aElement: TJSElement): TJQuery; overload;
    function prevUntil(const aElement: TJSElement; aFilter: string): TJQuery; overload;
    function prevUntil(const aQuery: TJQuery): TJQuery; overload;
    function prevUntil(const aQuery: TJQuery; aFilter: string): TJQuery; overload;
    function prop(const aPropertyName: string): JSValue; overload;
    function prop(const aPropertyName: string; AValue: JSValue): TJQuery; overload;
    function prop(const TJSObject): TJQuery; overload;
    function prop(const aPropertyName: string; aHandler: TJQueryPropHandler): TJQuery; overload;
    class function queue(element: TJSElement): TJSarray; overload;
    class function queue(element: TJSElement; const aQueueName: string): TJSarray; overload;
    class function queue(element: TJSElement; const aQueueName: string; anewQueue: TJSarray): TJQuery; overload;
    class function queue(element: TJSElement; const aQueueName: string; aHandler: TJQueryQueueHandler): TJQuery; overload;
    function queue: TJSarray; overload;
    function queue(aQueueName: string): TJSarray; overload;
    function queue(anArray: TJSArray): TJQuery; overload;
    function queue(aQueueName: string; anArray: TJSarray): TJQuery; overload;
    function queue(aQueueName: string; aHandler: TJQueryAddQueueHandler): TJQuery; overload;
    function removeAttr(const attributeName: string): TJQuery;
    function removeClass(const aClass: string): TJQuery; overload;
    function removeClass(const aClassFunction: TJQueryAddClassHandler): TJQuery; overload;
    class function removeData(aElement: TJSElement; const aName: string): TJQuery; overload;
    class function removeData(aElement: TJSElement): TJQuery; overload;
    function removeData(const aName: string): TJQuery; overload;
    function removeData(const aNames: Array of string): TJQuery; overload;
    function removeData: TJQuery; overload;
    function removeProp(const aPropertyName: string): TJQuery;
    function scrollLeft: Integer; overload;
    function scrollLeft(aValue: Integer): TJQuery; overload;
    function scrollTop: Integer; overload;
    function scrollTop(aValue: Integer): TJQuery; overload;
    function serialize: string;
    function serializeArray: TJSObjectDynArrayArray;
    function siblings: TJQuery; overload;
    function siblings(const aSelector: string): TJQuery; overload;
    function slice(aStart: Integer): TJQuery; overload;
    function slice(aStart: Integer; aEnd: Integer): TJQuery; overload;
    function sub: TJQuery;
    function text: string; overload;
    function text(const aText: string): TJQuery; overload;
    function text(const aText: Integer): TJQuery; overload;
    function text(const aText: Double): TJQuery; overload;
    function text(const aText: Boolean): TJQuery; overload;
    function text(aHandler: TJQueryTextHandler): TJQuery; overload;
    function toggleClass(const aClass: string): TJQuery; overload;
    function toggleClass(const aClass: string; aState: Boolean): TJQuery; overload;
    function toggleClass(const aHandler: TJQueryToggleClassHandler): TJQuery; overload;
    function toggleClass(const aHandler: TJQueryToggleClassHandler; AState: Boolean): TJQuery; overload;
    function val: JSValue; overload;
    function val(const aValue: string): TJQuery; overload;
    function val(const aValue: Integer): TJQuery; overload;
    function val(const aValue: Array of string): TJQuery; overload;
    function val(aHandler: TJQueryValHandler): TJQuery; overload;
    function when(APromise: TJSPromise): TJSPromise; overload;
    function when: TJSPromise; overload;
    function Width: Integer; overload;
    function Width(aValue: Integer): TJQuery; overload;
    function Width(aValue: string): TJQuery; overload;
    function Width(aHandler: TJQueryWidthHandler): TJQuery; overload;
    property ready: TJSPromise read FReady;
    property cssHooks: TJSObject read FCSSHooks;
    property cssNumber: TJSObject read FCSSNumber;
    property Elements[AIndex : Integer]: TJSElement read getEl; default;
  end;
  function JQuery(const aSelector: string): TJQuery; overload;
function JQuery(const aSelector: string; Context: TJSElement): TJQuery; overload;
function JQuery(const aElement: TJSElement): TJQuery; overload;
function JQuery(const aElement: Array of TJSElement): TJQuery; overload;
function JQuery(const aElement: TJSObject): TJQuery; overload;
function JQuery(const aQuery: TJQuery): TJQuery; overload;
function JQuery: TJQuery; overload;
var
  gJQuery: TJQuery;


implementation
{$HINTS OFF}
{$WARNINGS OFF}

// Dummy implementations for externals

function TCallbacks.add(aCallBack: TCallBack): TCallbacks;

begin
end;

function TCallbacks.add(aCallBack: Array of TCallBack): TCallbacks;

begin
end;

function TCallbacks.add(aCallBack: TCallBackEvent): TCallbacks;

begin
end;

function TCallbacks.add(aCallBack: Array of TCallBackEvent): TCallbacks;

begin
end;

function TCallbacks.disable: TCallBacks;

begin
end;

function TCallbacks.disabled: Boolean;

begin
end;

function TCallbacks.empty: TCallBacks;

begin
end;

function TCallbacks.fire(arguments: JSValue): TCallbacks;

begin
end;

function TCallbacks.fired: Boolean;

begin
end;

function TCallbacks.fireWith(context: JSValue; arguments: JSValue): TCallbacks;

begin
end;

function TCallbacks.has(aCallBack: TCallBack): Boolean;

begin
end;

function TCallbacks.has(aCallBack: TCallBackEvent): Boolean;

begin
end;

function TCallbacks.lock: TCallBacks;

begin
end;

function TCallbacks.locked: Boolean;

begin
end;

function TCallbacks.remove(aCallBack: TCallBack): TCallbacks;

begin
end;

function TCallbacks.remove(aCallBack: Array of TCallBack): TCallbacks;

begin
end;

function TCallbacks.remove(aCallBack: TCallBackEvent): TCallbacks;

begin
end;

function TCallbacks.remove(aCallBack: Array of TCallBackEvent): TCallbacks;

begin
end;

function TJQueryDeferred.done(aHandler: TDeferredDoneHandler): TJQueryDeferred;

begin
end;

function TJQueryDeferred.done(aHandlers: Array of TDeferredDoneHandler): TJQueryDeferred;

begin
end;

function TJQXHR.getResponseHeader(aName: string): string;

begin
end;

function TJQXHR.getAllResponseHeaders: string;

begin
end;

procedure TJQXHR.overrideMimeType(aType: string);

begin
end;

procedure TJQXHR.setRequestHeader(aName: string; AValue: string);

begin
end;

procedure TJQXHR.done(aHandler: TJQXHRDoneHandler);

begin
end;

procedure TJQXHR.always(aHandler: TJQXHRAlwaysHandler);

begin
end;

procedure TJQXHR.fail(aHandler: TJQXHRFailHandler);

begin
end;

procedure TJQXHR._then(aSuccess: TJQXHRDoneHandler; aFail: TJQXHRFailHandler);

begin
end;

procedure TJQXHR.abort;

begin
end;

procedure TJQXHR.abort(AStatusText: string);

begin
end;

function TJQuery.getEl(aIndex: Integer): TJSElement;

begin
end;

function TJQuery.add(const aSelector: string): TJQuery;

begin
end;

function TJQuery.add(const aSelector: string; AContext: TJSElement): TJQuery;

begin
end;

function TJQuery.add(const aElement: TJSElement): TJQuery;

begin
end;

function TJQuery.add(const aElement: Array of TJSElement): TJQuery;

begin
end;

function TJQuery.add(const aQuery: TJQuery): TJQuery;

begin
end;

function TJQuery.addBack(const aSelector: string): TJQuery;

begin
end;

function TJQuery.addBack: TJQuery;

begin
end;

function TJQuery.addClass(const aClass: string): TJQuery;

begin
end;

function TJQuery.addClass(const aClassFunction: TJQueryAddClassHandler): TJQuery;

begin
end;

function TJQuery.ajaxComplete(aHandler: TJQueryAjaxEventHandler): TJQuery;

begin
end;

function TJQuery.ajaxError(aHandler: TJQueryAjaxEventHandler): TJQuery;

begin
end;

function TJQuery.ajaxSend(aHandler: TJQueryAjaxEventHandler): TJQuery;

begin
end;

function TJQuery.ajaxStart(aHandler: TJQueryAjaxHandler): TJQuery;

begin
end;

function TJQuery.ajaxStop(aHandler: TJQueryAjaxHandler): TJQuery;

begin
end;

function TJQuery.ajaxSuccess(aHandler: TJQueryAjaxEventHandler): TJQuery;

begin
end;

class function TJQuery.ajax(aURL: string; aSettings: TJSObject): tJQXHR;

begin
end;

class function TJQuery.ajax(aSettings: TJSObject): tJQXHR;

begin
end;

class function TJQuery.ajax(aSettings: TJSAjaxSettings): tJQXHR;

begin
end;

class procedure TJQuery.ajaxPrefilter(dataTypes: string; aHandler: TJQueryAjaxPrefilterHandler);

begin
end;

class procedure TJQuery.ajaxSetup(aSettings: TJSAjaxSettings);

begin
end;

class procedure TJQuery.ajaxSetup(aSettings: TJSObject);

begin
end;

class procedure TJQuery.ajaxTransport(aDataType: string; AHandler: TJQueryAjaxTransportHandler);

begin
end;

function TJQuery.attr(const attributeName: string): string;

begin
end;

function TJQuery.attr(const attributeName: string; const Value: string): TJQuery;

begin
end;

function TJQuery.attr(const attributes: TJSObject): TJQuery;

begin
end;

function TJQuery.attr(const attributeName: string; aHandler: TJQueryAttrHandler): TJQuery;

begin
end;

class function TJQuery.Callbacks: TCallbacks;

begin
end;

class function TJQuery.Callbacks(const aFlags: string): TCallbacks;

begin
end;

function TJQuery.children(const aSelector: string): TJQuery;

begin
end;

function TJQuery.children: TJQuery;

begin
end;

function TJQuery.clearQueue: TJQuery;

begin
end;

function TJQuery.clearQueue(const aQueueName: string): TJQuery;

begin
end;

function TJQuery.closest(const aSelector: string): TJQuery;

begin
end;

function TJQuery.closest(const aSelector: string; AContext: TJSElement): TJQuery;

begin
end;

function TJQuery.closest(const aQuery: TJQuery): TJQuery;

begin
end;

function TJQuery.closest(const aElement: TJSElement): TJQuery;

begin
end;

function TJQuery.contents: TJQuery;

begin
end;

function TJQuery.css(const aPropertyName: TJSObject): string;

begin
end;

function TJQuery.css(const aPropertyName: string): string;

begin
end;

function TJQuery.css(const aPropertyNames: Array of string): string;

begin
end;

function TJQuery.css(const aPropertyName: string; const Avalue: string): TJQuery;

begin
end;

function TJQuery.css(const aPropertyName: string; Avalue: Integer): TJQuery;

begin
end;

function TJQuery.css(const aPropertyName: string; AHandler: TJQueryCSSHandler): TJQuery;

begin
end;

class function TJQuery.data(aElement: TJSElement; const aKey: string; aValue: jsValue): TJSObject;

begin
end;

class function TJQuery.data(aElement: TJSElement; const aKey: string): TJSObject;

begin
end;

class function TJQuery.data(aElement: TJSElement): TJSObject;

begin
end;

function TJQuery.data(aKey: string; aValue: JSValue): TJQuery;

begin
end;

function TJQuery.data(aObj: TJSObject): TJQuery;

begin
end;

function TJQuery.data(aKey: string): TJSObject;

begin
end;

function TJQuery.data: TJSObject;

begin
end;

function TJQuery.dequeue: TJQuery;

begin
end;

function TJQuery.dequeue(const aQueueName: string): TJQuery;

begin
end;

class function TJQuery.dequeue(aElement: TJSElement): TJQuery;

begin
end;

class function TJQuery.dequeue(aElement: TJSElement; const aQueueName: string): TJQuery;

begin
end;

function TJQuery._end: TJQuery;

begin
end;

function TJQuery.eq(AIndex: Integer): TJQuery;

begin
end;

function TJQuery.each(aHandler: TJQueryEachHandler): TJQuery;

begin
end;

class function TJQuery.escapeSelector(const S: string): string;

begin
end;

function TJQuery.filter(const aSelector: string): TJQuery;

begin
end;

function TJQuery.filter(aHandler: TJQueryFilterHandler): TJQuery;

begin
end;

function TJQuery.filter(const aQuery: TJQuery): TJQuery;

begin
end;

function TJQuery.filter(const aElement: TJSElement): TJQuery;

begin
end;

function TJQuery.filter(const aElements: Array of TJSElement): TJQuery;

begin
end;

function TJQuery.find(const aSelector: string): TJQuery;

begin
end;

function TJQuery.find(const aQuery: TJQuery): TJQuery;

begin
end;

function TJQuery.find(const aElement: TJSElement): TJQuery;

begin
end;

function TJQuery.first: TJQuery;

begin
end;

function TJQuery.get(aIndex: Integer): TJSElement;

begin
end;

class function TJQuery.get: TJQXHR;

begin
end;

class function TJQuery.get(url: string): TJQXHR;

begin
end;

{$IFNDEF LCLTMSWEB}

class function TJQuery.get(url: string; Data: string): TJQXHR;

begin
end;

class function TJQuery.get(url: string; Data: TJSObject): TJQXHR;

begin
end;

class function TJQuery.get(url: string; Data: TJSObject; success: TJQueryAjaxSuccessHandler): TJQXHR;

begin
end;

class function TJQuery.get(url: string; Data: string; success: TJQueryAjaxSuccessHandler): TJQXHR;

begin
end;

class function TJQuery.get(url: string; Data: TJSObject; success: TJQueryAjaxSuccessHandler; aDataType: string): TJQXHR;

begin
end;

class function TJQuery.get(url: string; Data: string; success: TJQueryAjaxSuccessHandler; aDataType: string): TJQXHR;

begin
end;

{$ENDIF}

class function TJQuery.get(aSettings: TJSAjaxSettings): TJQXHR;

begin
end;

class function TJQuery.get(aSettings: TJSObject): TJQXHR;

begin
end;

class function TJQuery.getJSON(url: string): TJQXHR;

begin
end;

{$IFNDEF LCLTMSWEB}

class function TJQuery.getJSON(url: string; Data: string): TJQXHR;

begin
end;

class function TJQuery.getJSON(url: string; Data: TJSObject): TJQXHR;

begin
end;

class function TJQuery.getJSON(url: string; Data: TJSObject; success: TJQueryAjaxSuccessHandler): TJQXHR;

begin
end;

class function TJQuery.getJSON(url: string; Data: string; success: TJQueryAjaxSuccessHandler): TJQXHR;

begin
end;

class function TJQuery.getJSON(url: string; Data: TJSObject; success: TJQueryAjaxSuccessHandler; aDataType: string): TJQXHR;

begin
end;

class function TJQuery.getJSON(url: string; Data: string; success: TJQueryAjaxSuccessHandler; aDataType: string): TJQXHR;

begin
end;

{$ENDIF}

class function TJQuery.getScript(url: string): TJQXHR;

begin
end;

class function TJQuery.getScript(url: string; aSuccess: TJQueryAjaxScriptHandler): TJQXHR;

begin
end;

function TJQuery.has(const aSelector: string): TJQuery;

begin
end;

function TJQuery.has(const aQuery: TJQuery): TJQuery;

begin
end;

function TJQuery.hasClass(const aClassName: string): Boolean;

begin
end;

class function TJQuery.hasData(aElement: TJSElement): Boolean;

begin
end;

function TJQuery.height: Integer;

begin
end;

function TJQuery.height(aValue: Integer): TJQuery;

begin
end;

function TJQuery.height(aValue: string): TJQuery;

begin
end;

function TJQuery.height(aHandler: TJQueryHeightHandler): TJQuery;

begin
end;

function TJQuery.html: string;

begin
end;

function TJQuery.html(const aHTML: string): TJQuery;

begin
end;

function TJQuery.html(const aHandler: TJQueryHTMLHandler): TJQuery;

begin
end;

function TJQuery.innerHeight: Integer;

begin
end;

function TJQuery.innerHeight(aValue: Integer): TJQuery;

begin
end;

function TJQuery.innerHeight(aValue: string): TJQuery;

begin
end;

function TJQuery.innerHeight(aHandler: TJQueryHeightHandler): TJQuery;

begin
end;

function TJQuery.innerWidth: Integer;

begin
end;

function TJQuery.innerWidth(aValue: Integer): TJQuery;

begin
end;

function TJQuery.innerWidth(aValue: string): TJQuery;

begin
end;

function TJQuery.innerWidth(aHandler: TJQueryWidthHandler): TJQuery;

begin
end;

function TJQuery._is(const aSelector: string): TJQuery;

begin
end;

function TJQuery._is(const aQuery: TJQuery): TJQuery;

begin
end;

function TJQuery._is(aHandler: TJQueryFilterHandler): TJQuery;

begin
end;

function TJQuery._is(const aElement: TJSElement): TJQuery;

begin
end;

function TJQuery._is(const aElements: Array of TJSElement): TJQuery;

begin
end;

function TJQuery.last: TJQuery;

begin
end;

class function TJQuery.load(url: string): TJQXHR;

begin
end;

{$IFNDEF LCLTMSWEB}

class function TJQuery.load(url: string; Data: string): TJQXHR;

begin
end;

class function TJQuery.load(url: string; Data: TJSObject): TJQXHR;

begin
end;

class function TJQuery.load(url: string; Data: TJSObject; success: TJQueryAjaxLoadHandler): TJQXHR;

begin
end;

class function TJQuery.load(url: string; Data: string; success: TJQueryAjaxLoadHandler): TJQXHR;

begin
end;

{$ENDIF}

function TJQuery.map(aHandler: TJQueryMapHandler): TJQuery;

begin
end;

function TJQuery.next: TJQuery;

begin
end;

function TJQuery.next(const aSelector: string): TJQuery;

begin
end;

function TJQuery.nextAll: TJQuery;

begin
end;

function TJQuery.nextAll(const aSelector: string): TJQuery;

begin
end;

function TJQuery.nextUntil: TJQuery;

begin
end;

function TJQuery.nextUntil(const aSelector: string): TJQuery;

begin
end;

function TJQuery.nextUntil(const aSelector: string; const aFilter: string): TJQuery;

begin
end;

function TJQuery.nextUntil(const aElement: TJSElement): TJQuery;

begin
end;

function TJQuery.nextUntil(const aElement: TJSElement; aFilter: string): TJQuery;

begin
end;

function TJQuery.nextUntil(const aQuery: TJQuery): TJQuery;

begin
end;

function TJQuery.nextUntil(const aQuery: TJQuery; aFilter: string): TJQuery;

begin
end;

function TJQuery._not(const aSelector: string): TJQuery;

begin
end;

function TJQuery._not(const aSelector: TJSElement): TJQuery;

begin
end;

function TJQuery._not(const aSelector: Array of TJSElement): TJQuery;

begin
end;

function TJQuery._not(const aSelector: TJQuery): TJQuery;

begin
end;

function TJQuery._not(const aSelector: TJQueryFilterHandler): TJQuery;

begin
end;

function TJQuery.noConflict: TJSObject;

begin
end;

function TJQuery.noConflict(removeAll: Boolean): TJSObject;

begin
end;

function TJQuery.offSet: Integer;

begin
end;

function TJQuery.offSet(const aOffset: TJQueryTopLeft): TJQuery;

begin
end;

function TJQuery.offSet(aHandler: TJQueryOffsetHandler): TJQuery;

begin
end;

function TJQuery.offsetParent: TJQuery;

begin
end;

function TJQuery.outerHeight(IncludeMargin: Boolean): Integer;

begin
end;

function TJQuery.outerHeight: Integer;

begin
end;

function TJQuery.outerHeight(aValue: Integer): TJQuery;

begin
end;

function TJQuery.outerHeight(aValue: string): TJQuery;

begin
end;

function TJQuery.outerHeight(aHandler: TJQueryHeightHandler): TJQuery;

begin
end;

function TJQuery.outerWidth(IncludeMargin: Boolean): Integer;

begin
end;

function TJQuery.outerWidth: Integer;

begin
end;

function TJQuery.outerWidth(aValue: Integer): TJQuery;

begin
end;

function TJQuery.outerWidth(aValue: string): TJQuery;

begin
end;

function TJQuery.outerWidth(aHandler: TJQueryWidthHandler): TJQuery;

begin
end;

class function TJQuery.param(aObject: string): string;

begin
end;

class function TJQuery.param(aObject: TJSObject): string;

begin
end;

class function TJQuery.param(aObject: TJQuery): string;

begin
end;

class function TJQuery.param(aObject: string; traditional: Boolean): string;

begin
end;

class function TJQuery.param(aObject: TJSObject; traditional: Boolean): string;

begin
end;

class function TJQuery.param(aObject: TJQuery; traditional: Boolean): string;

begin
end;

function TJQuery.parent: TJQuery;

begin
end;

function TJQuery.parent(const ASelector: string): TJQuery;

begin
end;

function TJQuery.parents: TJQuery;

begin
end;

function TJQuery.parents(const ASelector: string): TJQuery;

begin
end;

function TJQuery.parentsUntil: TJQuery;

begin
end;

function TJQuery.parentsUntil(const aSelector: string): TJQuery;

begin
end;

function TJQuery.parentsUntil(const aSelector: string; const aFilter: string): TJQuery;

begin
end;

function TJQuery.parentsUntil(const aElement: TJSElement): TJQuery;

begin
end;

function TJQuery.parentsUntil(const aElement: TJSElement; aFilter: string): TJQuery;

begin
end;

function TJQuery.parentsUntil(const aQuery: TJQuery): TJQuery;

begin
end;

function TJQuery.parentsUntil(const aQuery: TJQuery; aFilter: string): TJQuery;

begin
end;

function TJQuery.position: TJQueryTopLeft;

begin
end;

class function TJQuery.post(url: string): TJQXHR;

begin
end;

{$IFNDEF LCLTMSWEB}

class function TJQuery.post(url: string; Data: string): TJQXHR;

begin
end;

class function TJQuery.post(url: string; Data: TJSObject): TJQXHR;

begin
end;

class function TJQuery.post(url: string; Data: TJSObject; success: TJQueryAjaxSuccessHandler): TJQXHR;

begin
end;

class function TJQuery.post(url: string; Data: string; success: TJQueryAjaxSuccessHandler): TJQXHR;

begin
end;

class function TJQuery.post(url: string; Data: TJSObject; success: TJQueryAjaxSuccessHandler; aDataType: string): TJQXHR;

begin
end;

class function TJQuery.post(url: string; Data: string; success: TJQueryAjaxSuccessHandler; aDataType: string): TJQXHR;

begin
end;

{$ENDIF}

class function TJQuery.post(aSettings: TJSAjaxSettings): TJQXHR;

begin
end;

class function TJQuery.post(aSettings: TJSObject): TJQXHR;

begin
end;

function TJQuery.prev: TJQuery;

begin
end;

function TJQuery.prev(const aSelector: string): TJQuery;

begin
end;

function TJQuery.prevAll: TJQuery;

begin
end;

function TJQuery.prevAll(const aSelector: string): TJQuery;

begin
end;

function TJQuery.prevUntil: TJQuery;

begin
end;

function TJQuery.prevUntil(const aSelector: string): TJQuery;

begin
end;

function TJQuery.prevUntil(const aSelector: string; const aFilter: string): TJQuery;

begin
end;

function TJQuery.prevUntil(const aElement: TJSElement): TJQuery;

begin
end;

function TJQuery.prevUntil(const aElement: TJSElement; aFilter: string): TJQuery;

begin
end;

function TJQuery.prevUntil(const aQuery: TJQuery): TJQuery;

begin
end;

function TJQuery.prevUntil(const aQuery: TJQuery; aFilter: string): TJQuery;

begin
end;

function TJQuery.prop(const aPropertyName: string): JSValue;

begin
end;

function TJQuery.prop(const aPropertyName: string; AValue: JSValue): TJQuery;

begin
end;

function TJQuery.prop(const TJSObject): TJQuery;

begin
end;

function TJQuery.prop(const aPropertyName: string; aHandler: TJQueryPropHandler): TJQuery;

begin
end;

class function TJQuery.queue(element: TJSElement): TJSarray;

begin
end;

class function TJQuery.queue(element: TJSElement; const aQueueName: string): TJSarray;

begin
end;

class function TJQuery.queue(element: TJSElement; const aQueueName: string; anewQueue: TJSarray): TJQuery;

begin
end;

class function TJQuery.queue(element: TJSElement; const aQueueName: string; aHandler: TJQueryQueueHandler): TJQuery;

begin
end;

function TJQuery.queue: TJSarray;

begin
end;

function TJQuery.queue(aQueueName: string): TJSarray;

begin
end;

function TJQuery.queue(anArray: TJSArray): TJQuery;

begin
end;

function TJQuery.queue(aQueueName: string; anArray: TJSarray): TJQuery;

begin
end;

function TJQuery.queue(aQueueName: string; aHandler: TJQueryAddQueueHandler): TJQuery;

begin
end;

function TJQuery.removeAttr(const attributeName: string): TJQuery;

begin
end;

function TJQuery.removeClass(const aClass: string): TJQuery;

begin
end;

function TJQuery.removeClass(const aClassFunction: TJQueryAddClassHandler): TJQuery;

begin
end;

class function TJQuery.removeData(aElement: TJSElement; const aName: string): TJQuery;

begin
end;

class function TJQuery.removeData(aElement: TJSElement): TJQuery;

begin
end;

function TJQuery.removeData(const aName: string): TJQuery;

begin
end;

function TJQuery.removeData(const aNames: Array of string): TJQuery;

begin
end;

function TJQuery.removeData: TJQuery;

begin
end;

function TJQuery.removeProp(const aPropertyName: string): TJQuery;

begin
end;

function TJQuery.scrollLeft: Integer;

begin
end;

function TJQuery.scrollLeft(aValue: Integer): TJQuery;

begin
end;

function TJQuery.scrollTop: Integer;

begin
end;

function TJQuery.scrollTop(aValue: Integer): TJQuery;

begin
end;

function TJQuery.serialize: string;

begin
end;

function TJQuery.serializeArray: TJSObjectDynArrayArray;

begin
end;

function TJQuery.siblings: TJQuery;

begin
end;

function TJQuery.siblings(const aSelector: string): TJQuery;

begin
end;

function TJQuery.slice(aStart: Integer): TJQuery;

begin
end;

function TJQuery.slice(aStart: Integer; aEnd: Integer): TJQuery;

begin
end;

function TJQuery.sub: TJQuery;

begin
end;

function TJQuery.text: string;

begin
end;

function TJQuery.text(const aText: string): TJQuery;

begin
end;

function TJQuery.text(const aText: Integer): TJQuery;

begin
end;

function TJQuery.text(const aText: Double): TJQuery;

begin
end;

function TJQuery.text(const aText: Boolean): TJQuery;

begin
end;

function TJQuery.text(aHandler: TJQueryTextHandler): TJQuery;

begin
end;

function TJQuery.toggleClass(const aClass: string): TJQuery;

begin
end;

function TJQuery.toggleClass(const aClass: string; aState: Boolean): TJQuery;

begin
end;

function TJQuery.toggleClass(const aHandler: TJQueryToggleClassHandler): TJQuery;

begin
end;

function TJQuery.toggleClass(const aHandler: TJQueryToggleClassHandler; AState: Boolean): TJQuery;

begin
end;

function TJQuery.val: JSValue;

begin
end;

function TJQuery.val(const aValue: string): TJQuery;

begin
end;

function TJQuery.val(const aValue: Integer): TJQuery;

begin
end;

function TJQuery.val(const aValue: Array of string): TJQuery;

begin
end;

function TJQuery.val(aHandler: TJQueryValHandler): TJQuery;

begin
end;

function TJQuery.when(APromise: TJSPromise): TJSPromise;

begin
end;

function TJQuery.when: TJSPromise;

begin
end;

function TJQuery.Width: Integer;

begin
end;

function TJQuery.Width(aValue: Integer): TJQuery;

begin
end;

function TJQuery.Width(aValue: string): TJQuery;

begin
end;

function TJQuery.Width(aHandler: TJQueryWidthHandler): TJQuery;

begin
end;

function JQuery(const aSelector: string): TJQuery;

begin
end;

function JQuery(const aSelector: string; Context: TJSElement): TJQuery;

begin
end;

function JQuery(const aElement: TJSElement): TJQuery;

begin
end;

function JQuery(const aElement: Array of TJSElement): TJQuery;

begin
end;

function JQuery(const aElement: TJSObject): TJQuery;

begin
end;

function JQuery(const aQuery: TJQuery): TJQuery;

begin
end;

function JQuery: TJQuery;

begin
end;


// end of dummy implementations




end.
