{$Error This is a closed source unit. You cannot compile it, it was already compiled.}
{$Error Probably the IDE has cleaned up and you have to unpack the zip again.}
{********************************************************************}
{                                                                    }
{ written by TMS Software                                            }
{            Copyright (c) 2019 - 2020                               }
{            Email : info@tmssoftware.com                            }
{            Web : http://www.tmssoftware.com                        }
{                                                                    }
{ The source code is given as is. The author is not responsible      }
{ for any possible damage done due to the use of this code.          }
{ The complete source code remains property of the author and may    }
{ not be distributed, published, given or sold in any form as such.  }
{ No parts of the source code can be included in any other component }
{ or application without written authorization of the author.        }
{********************************************************************}

unit WEBLib.Devices;

{$DEFINE NOPP}

interface

uses
  Classes, SysUtils, WEBLib.Controls, Web, JS,
  WEBLib.ExtCtrls, WEBLib.Graphics, libfilereader, libglmatrix, libmedia;

type
  TCameraType = (ctFront, ctRear, ctSelected);
  TCameraBaseFormat = (cbfJPEG, cbfPNG);

  TCameraDevice = class(TCollectionItem)
  public
    procedure Assign(Source: TPersistent); override;
    property Id: string read FId write FId;
    property Name: string read FName write FName;
  end;

  TCameraDevices = class(TCollection)
  public
    constructor Create;
    function Add: TCameraDevice; reintroduce;
    function GetDeviceById(Id: string): TCameraDevice;
    function GetDeviceByName(Name: string): TCameraDevice;
    function Insert(Index: Integer): TCameraDevice; reintroduce;
    property Items[Index: integer]: TCameraDevice read GetItem write SetItem;
  end;

  TCameraEvent = procedure(Sender: TObject; ACamera: TCameraDevice) of object;
  TCameraBeforeStartEvent = procedure(Sender: TObject; var AConstraints: TJSObjectRecord) of object;

  TCamera = class(TCustomControl)
  protected
    procedure BindEvents; override;
    function CreateElement: TJSElement; override;
    procedure UpdateElementData; override;
    procedure UpdateElementVisual; override;
    function GetVideoTrack: TWebMediaStreamTrack;
    function HandleStreamPlay(aEvent : TJSPointerEvent): Boolean;
    function HandleEnumerateDevices(aMediaDevices: JSValue): JSValue;
    function HandleUserMedia(aMediaStream: JSValue): JSValue;
    function HandleLoadedMetaData(Event: TEventListenerEvent): Boolean;
    procedure UnbindEvents; override;
  public
    destructor Destroy; override;
    procedure CreateInitialize; override;
    function BrowserSupportedConstraints: TStringList;
    procedure Pause;
    procedure Resume;
    procedure SetSelectedCameraDevice(aDevice: TCameraDevice);
    procedure Start;
    procedure Stop;
    property CameraDevices: TCameraDevices read FCameraDevices;
    property Paused: Boolean read FPaused;
    property SnapShotAsBase64: string read GetSnapShotBase64;
    property SnapShotAsImageData: TJSImageData read GetSnapShotImgData;
    property SnapShotAsUint8Array: TJSUint8Array read GetSnapShotUint8Array;
  published
    property Align;
    property AlignWithMargins;
    property Anchors;
    property CameraType: TCameraType read FCameraType write SetCameraType default ctFront;
    property BaseFormat: TCameraBaseFormat read FCameraBaseFormat write FCameraBaseFormat default cbfJPEG;
    property HeightPercent;
    property HeightStyle;
    property Margins;
    property WidthPercent;
    property WidthStyle;
    property OnBeforeStart: TCameraBeforeStartEvent read FOnBeforeStart write FOnBeforeStart;
    property OnCameraDevicesInitialized: TNotifyEvent read FOnCameraDevicesInitialized write FOnCameraDevicesInitialized;
    property OnCameraPause: TCameraEvent read FOnCameraPause write FOnCameraPause;
    property OnCameraResume: TCameraEvent read FOnCameraResume write FOnCameraResume;
    property OnCameraStreamPlay: TCameraEvent read FOnCameraStreamPlay write FOnCameraStreamPlay;
    property OnCameraStop: TCameraEvent read FOnCameraStop write FOnCameraStop;
    property OnClick;
    property OnDblClick;
    property OnMouseDown;
    property OnMouseEnter;
    property OnMouseLeave;
    property OnMouseMove;
    property OnMouseUp;
    property OnSwitchCamera: TCameraEvent read FOnSwitchCamera write FOnSwitchCamera;
  end;

  TWebCamera = class(TCamera);

  TMediaCaptureType = (mctVideo, mctAudio, mctBoth);
  TMediaCaptureRecordingMode = (mrmManual, mrmAuto);
  TMediaCaptureFFTSize = (mfs32, mfs64, mfs128, mfs256, mfs512, mfs1024, mfs2048, mfs4096, mfs8192, mfs16384, mfs32768);
  TMediaCaptureCloseEvent = procedure(Sender: TObject; ABinary: TJSUint8Array; ABase: string) of object;

  TMediaCapture = class(TComponent)
  protected
    procedure CreateAnalyser(AStream: TWebMediaStream);
    procedure DoAutoProcess(aTime: TJSDOMHighResTimeStamp);
    procedure DoHandleCameraStart(Sender: TObject; ACamera: TCameraDevice);
    procedure DoHandleCaptureStop;
    function DoHandleDataAvailable(Event: TEventListenerEvent): Boolean;
    procedure GetAudioTrack;
    function GetFFTSize(ASize: TMediaCaptureFFTSize): Integer;
    function GetMinDecibels(ASensitivity: Integer): Integer;
    function GetTimeConstant(AConstant: Double): Double;
    procedure GetVideoTrack;
    function HandleAudioUserMedia(aMediaStream: JSValue): JSValue;
    function HandleFileReaderLoad(Event: TEventListenerEvent): Boolean;
    function HandleVideoUserMedia(aMediaStream: JSValue): JSValue;
    procedure InitializeContext(AStream: TWebMediaStream);
    procedure InitializeRecording;
    procedure Notification(AComponent: TComponent; Operation: TOperation); override;
    procedure RequestDevices;
    procedure RequestVideoAccess;
    procedure StartRecording;
    procedure StopAuto;
    procedure StopManual;
  public
    constructor Create(AOwner: TComponent); override;
    procedure Start;
    procedure Stop;
  published
    property Capture: TMediaCaptureType read FCapture write SetCapture default mctBoth;
    property Camera: TCamera read FCamera write SetCamera;
    property FFTSize: TMediaCaptureFFTSize read FFFTSize write SetFFTSize default mfs2048;
    property RecordingMode: TMediaCaptureRecordingMode read FRecordingMode write SetRecordingMode default mrmManual;
    property Sensitivity: Integer read FSensitivity write SetSensitivity default 100;
    property SmoothingTimeConstant: Double read FTimeConstant write SetTimeConstant; //default 0.9
    property OnStartCapture: TNotifyEvent read FOnStartCapture write FOnStartCapture;
    property OnStopCapture: TMediaCaptureCloseEvent read FOnStopCapture write FOnStopCapture;
  end;

  TWebMediaCapture = class(TMediaCapture);

  TDecoderDecodedEvent = procedure(Sender: TObject; ADecoded: string) of object;

  TDecoder = class(TComponent)
  protected
    procedure DecodeURL(AURL: string); virtual;
    procedure DoDecoded; virtual;
    procedure DoDecodeTimer(Sender: TObject);
    procedure Notification(AComponent: TComponent; Operation: TOperation); override;
    property Camera: TCamera read FCamera write SetCamera;
    property ImageControl: TImageControl read FImageControl write FImageControl;
    property EnableTimer: Boolean read FEnableTimer write SetEnableTimer default False;
    property OnDecoded: TDecoderDecodedEvent read FOnDecoded write FOnDecoded;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  end;

  TQRDecoder = class(TDecoder)
  protected
    procedure DoDecoded; override;
    function HandleImageLoad(Event: TEventListenerEvent): boolean;
  public
    procedure DecodeURL(AURL: string); override;
  published
    property Camera;
    property EnableTimer;
    property OnDecoded;
  end;

  TWebQRDecoder = class(TQRDecoder);

  TBarcodeDecoderReader = (bcvCode128, bcvEAN, bcvEAN8, bcvCode39, bcvCode39VIN, bcvCodabar, bcvUPC, bcvUPCE, bcvI2OF5, bcv2OF5, bcvCode93);
  TBarcodeDecoderReaders = set of TBarcodeDecoderReader;

  TBarcodeDecoder = class(TDecoder)
    FTempDisable: Boolean;
    FInternalTimer: Boolean;
    FReader: TBarcodeDecoderReaders;
    FObj: TWebQuaggaJSResultObject;
    FConfigObj: TWebQuaggaJSConfigObject;
  protected
    procedure DoDecoded; override;
    procedure DoDecodeCallback(aResult: TWebQuaggaJSResultObject);
    procedure InitConfigObj;
  public
    constructor Create(AOwner: TComponent); override;
    procedure DecodeURL(AURL: string); override;
  published
    property Camera;
    property EnableTimer;
    property Reader: TBarcodeDecoderReaders read FReader write SetReader default [bcvCode128];
    property OnDecoded;
  end;

  TWebBarcodeDecoder = class(TBarcodeDecoder);

function Uint8ToBase64(AArray: TJSUint8Array): string;
function IsSafari: Boolean;

var
  TBarcodeDecoderValue: array[TBarcodeDecoderReader] of string = ('code_128_reader', 'ean_reader', 'ean_8_reader', 'code_39_reader', 'code_39_vin_reader',
    'codabar_reader', 'upc_reader', 'upc_e_reader', 'i2of5_reader', '2of5_reader', 'code_93_reader');

implementation
end.
