{$Error This is a closed source unit. You cannot compile it, it was already compiled.}
{$Error Probably the IDE has cleaned up and you have to unpack the zip again.}
{********************************************************************}
{                                                                    }
{ written by TMS Software                                            }
{            copyright (c) 2019 - 2020                               }
{            Email : info@tmssoftware.com                            }
{            Web : http://www.tmssoftware.com                        }
{                                                                    }
{ The source code is given as is. The author is not responsible      }
{ for any possible damage done due to the use of this code.          }
{ The complete source code remains property of the author and may    }
{ not be distributed, published, given or sold in any form as such.  }
{ No parts of the source code can be included in any other component }
{ or application without written authorization of the author.        }
{********************************************************************}

unit WEBLib.IndexedDb;

//{$define idbdebug}

{
  For soft error testing: look at the
  comments for "soft error testing"

  Hard error testing: Any Delphi Exception will be
  hard error and should cause red alert.
  1) Try sending a usage error from the app, for example
     set the KeyFieldName to nonexistent field say 'idx'
  2) Look for comment with Hard error testing and simulate such
     an error from base dataset class
}


{$DEFINE NOPP}

interface

uses
   Classes, JS, Web, SysUtils, WEBLib.CDS, JSONDataSet,
   DB, libindexeddb;

type
  TIndexedDbOpCode = (opOpen, opAdd, opPut, opDelete, opGet, opGetAllKeys, opGetAllObjs,
                      opGetAllIndexKeys, opGetAllObjsByIndex);

  TIndexedDbResultEvent = procedure(success: Boolean; opCode: TIndexedDbOpCode; data: JSValue; sequenceId: JSValue; errorName, errorMsg: String) of object;

  TIndexedDb = Class(TComponent)
  protected
    function ObjectStoreExists: Boolean;
    function GetObjectStore(theReq: TMIDBOpenDBRequest): TMIDBObjectStore;
    function DoUpgradeNeeded(Event: TEventListenerEvent): Boolean;
    function VersionChangeNeeded: Boolean;
    function DoSuccess(Event: TEventListenerEvent): Boolean;
    function DoFail(Event: TEventListenerEvent): Boolean;
    function DoVersionChange(Event: TEventListenerEvent): Boolean;
    function DoBlocked(Event: TEventListenerEvent): Boolean;
    function GetAutoIncrementDynamic: Boolean;
    procedure CorrectPrimaryKey(dataObj: TJSObject);
    procedure SetIndexFields(aFields: TJSArray);
  public
    constructor Create(AOwner: TComponent); overload; override;
    destructor Destroy; override;

    // Add Indexes before opening a "new" database. If database exists then
    // AddIndex will be ignored.
    procedure AddIndex(anIndexName: String; fields: String; isUnique: Boolean=False);

    { Explanation of parameters:
     Pass a DBName and ObjectStoreName. These will be created if absent.
     KeyFieldName: specify if you want to use inline-key (data contains key). In that case, don't pass key separately to PutData below.
                   The key field value will then be taken from that named property from the object passed to PutData.
     AutoIncrement: This will be effective only if db does not exist so that the db is created with autoincrement.
                    In case of autoincrement, you need not pass key to addData. The auto-created key is returned
                    in its success result.
    }
    procedure Open(aDbName: String; aObjectStoreName: String; KeyFieldName: String = ''; autoIncrement: Boolean = False; sequenceID: JSValue = 0);

    procedure PutData(akey: JSValue; data: JSValue; sequenceID: JSValue = 0); overload;
    procedure PutData(data: JSValue; sequenceID: JSValue = 0); overload;
    procedure AddData(data: JSValue; sequenceID: JSValue = 0);
    procedure DeleteData(AKey: JSValue; sequenceID: JSValue = 0);
    procedure GetData(akey: JSValue; sequenceID: JSValue = 0);
    procedure GetKeys(sequenceID: JSValue = 0);
    procedure GetAllObjs(sequenceID: JSValue = 0);
    procedure GetIndexKeys(indexPropertyName: String; sequenceID: JSValue = 0);
    // Gets objects by sequence set in properties ActiveIndex, IndexDescending
    // If ActiveIndex is empty then gets by primary index.
    procedure GetAllObjsByIndex(sequenceID: JSValue = 0);
    procedure GetIndexData(indexPropertyName: String; akey: JSValue; sequenceID: JSValue = 0);

    property DatabaseName: String read FDatabaseName;
    property ObjectStoreName: String read FObjectStoreName;
    property KeyFieldName: String read FKeyFieldName;
    property AutoIncrement: Boolean read FAutoIncrement;
    property ActiveIndex: String read FActiveIndex write FActiveIndex;
    property IndexDescending: Boolean read FIndexDescending write FIndexDescending;

    property OnResult: TIndexedDbResultEvent read FOnResult write FOnResult;
  end;

  TWebIndexedDB = Class(TIndexedDB);

  TIndexedDbClientDataProxy = Class(TDataProxy)
  protected
    function GetDataRequest(aOptions: TLoadOptions; aAfterRequest: TDataRequestEvent; aAfterLoad: TDatasetLoadEvent) : TDataRequest; override;
    function DoGetData(aRequest : TDataRequest) : Boolean; override;
    procedure DoOnError(opCode: TIndexedDbOpCode; errorName, errorMsg: String);
    procedure DoOnSuccess(success: Boolean; opCode: TIndexedDbOpCode; data: JSValue; sequenceID: JSValue; errorName, errorMsg: String);
    procedure ProcessUpdate(desc: TRecordUpdateDescriptor);
    procedure Close;
    // Dummy but necessary only as it is an Abstract method
    function ProcessUpdateBatch(aBatch: TRecordUpdateBatch): Boolean; override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  end;

  { TIndexedDbClientDataset }

  TIDBErrorEvent = procedure(DataSet: TDataSet; opCode: TIndexedDbOpCode; errorName, errorMsg: String) of object;
  TIDBInitSuccessEvent = procedure(Sender: TObject) of Object;
  TIDBProc = reference to procedure;

  TKeyId = record
    value: JSValue;
  end;
  TIDBAfterUpdateEvent = procedure(success: Boolean; opCode: TIndexedDbOpCode; keyId: TKeyId; errorName, errorMsg: String) of object;

  TIndexedDbClientDataset = Class(TClientDataSet)
  protected
    procedure InternalClose; override;
    Function AddToChangeList(aChange : TUpdateStatus) : TRecordUpdateDescriptor ; override;
    procedure SetActive(Value: Boolean); override;
    procedure AfterLoadDFMValues; override;
    procedure DoAfterOpen; override;
    procedure DoOnError(opCode: TIndexedDbOpCode; errorName, errorMsg: String);
    procedure UpdateStarts;
    procedure UpdateEnds;
    procedure FixKeyFieldCase;
    function IsKeyFieldValid: Boolean;
    function IsKeyFieldInteger: Boolean;
    procedure DoInitSuccess(success: Boolean; opCode: TIndexedDbOpCode; data: JSValue; sequenceID: JSValue; errorName, errorMsg: String);
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure Refresh; reintroduce;

    // Calling Init is necessary only if you have nultiple CDS
    // components using the same database.
    // In such a case, before making those CDS components Active
    // * call Init on the first CDS component.
    // * Pass an event handler to wait for success event before calling
    //   Init on the next CDS component and so on.
    // * Also, if you are using AddIDBIndex,
    //   please make sure that it is called before Init.
    function Init(ProcInitSuccess: TIDBProc=nil): Boolean;

    // Call AddIDBIndex before Init and Set Active calls.
    // Index will be added only for a new object store. If
    // the object store already exists then AddIDBIndex will be
    // ignored.
    procedure AddIDBIndex(anIndexName: String; fields: String; isUnique: Boolean=False);
    property IDBDataLoaded: Boolean read FIDBDataLoaded write FIDBDataLoaded;
    property IDBActiveIndex: String read FIDBActiveIndex write FIDBActiveIndex;
    property IDBIndexDescending: Boolean read FIDBIndexDescending write FIDBIndexDescending;
  published
    property Active;

    // Database and ObjectStore will be created if absent
    property IDBDatabaseName: String read FIDBDatabaseName write FIDBDatabaseName;
    property IDBObjectStoreName: String read FIDBObjectStoreName write FIDBObjectStoreName;
    // Specify primary key here
    property IDBKeyFieldName: String read FIDBKeyFieldName write FIDBKeyFieldName;

    // Specify AutoIncrement for above key.
    // Only effective when creating a new db. This can not be used
    // to change AutoIncrement property of an existing DB.
    property IDBAutoIncrement: Boolean read FIDBAutoIncrement write FIDBAutoIncrement;
    property OnIDBError: TIDBErrorEvent read FOnIDBError write FOnIDBError;
    property ONIDBAfterUpdate: TIDBAfterUpdateEvent read FONIDBAfterUpdate write FONIDBAfterUpdate;
    property OnInitSuccess: TIDBInitSuccessEvent read FOnInitSuccess write FOnInitSuccess;
  end;

  TWebIndexedDbClientDataset = class(TIndexedDBClientDataSet);

implementation
end.
