{$Error This is a closed source unit. You cannot compile it, it was already compiled.}
{$Error Probably the IDE has cleaned up and you have to unpack the zip again.}
unit WEBLib.Miletus;

interface

uses
  Classes, js, web, WEBLib.JSON, WEBLib.Forms, WEBLib.Menus, Types,
  WEBLib.Graphics, WEBLib.CDS, db;

type
  TMiletusObserverChangeEvent = procedure(Sender: TObject; AMessageID: Integer; AMessageData: TJSONValue) of object;

  TMiletusObserver = class(TComponent)
  protected
    procedure ObserverChange(AMessageID: Integer; AMessageData: TJSONValue); virtual;
    property ObserverEnabled: Boolean read FObserverEnabled write FObserverEnabled default True;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    property OnObserverChange: TMiletusObserverChangeEvent read FOnObserverChange write FOnObserverChange;
  end;

  //TMiletusStringListLoadFromFileProc = reference to procedure;
  TMiletusLoadFileProc = reference to procedure;

  TMiletusStringList = class(TStringList)
  protected
    procedure InternalSendToFile(const FileName: string; ASync: Boolean);
    procedure DoStringListResponse(Sender: TObject; AMessageID: Integer; AData: TJSONValue);
  public
    constructor Create; reintroduce;
    destructor Destroy; override;
    procedure LoadFromFile(const FileName: string); virtual;
    procedure LoadFromFileAsync(const FileName: string; AProc: TMiletusLoadFileProc); virtual; reintroduce;
    procedure SaveToFile(const FileName: string); override;
    procedure SaveToFileAsync(const FileName: string);
  end;

  TMiletusBinaryDataStream = class(TPersistent)
  protected
    procedure InternalSaveToFile(const FileName: string; ASync: Boolean);
    procedure DoStringListResponse(Sender: TObject; AMessageID: Integer; AData: TJSONValue);
    function HandleResponse(Event: TEventListenerEvent): Boolean;
    function HandleAbort(Event: TEventListenerEvent): Boolean;
  public
    constructor Create;
    destructor Destroy; override;
    procedure LoadFromFile(const FileName: string); virtual;
    procedure LoadFromFileAsync(const FileName: string; AProc: TMiletusLoadFileProc); virtual;
    procedure LoadFromFileRequest(const FileName: string; AProc: TMiletusLoadFileProc); virtual;
    procedure SaveToFile(const FileName: string); virtual;
    procedure SaveToFileAsync(const FileName: string); virtual;
    procedure SaveToFileRequest(const FileName: string); virtual;
    property AsArrayBuffer: TJSArrayBuffer read GetArrayBuffer write SetArrayBuffer;
    property Base64: string read FBase64 write FBase64;
    property Text: string read GetText write SetText;
  end;

  TMiletusOpenDialogOption = (ofReadOnly, ofOverwritePrompt, ofHideReadOnly,
    ofNoChangeDir, ofShowHelp, ofNoValidate, ofAllowMultiSelect,
    ofExtensionDifferent, ofPathMustExist, ofFileMustExist, ofCreatePrompt,
    ofShareAware, ofNoReadOnlyReturn, ofNoTestFileCreate, ofNoNetworkButton,
    ofNoLongNames, ofOldStyleDialog, ofNoDereferenceLinks, ofEnableIncludeNotify,
    ofEnableSizing, ofDontAddToRecent, ofForceShowHidden);

  TMiletusOpenDialogOptions = set of TMiletusOpenDialogOption;

  TMiletusDialogProc = reference to procedure(FileName: string);

  TMiletusOpenDialog = class(TMiletusObserver)
  protected
    FOpenID: Integer;
    procedure ObserverChange(AMessageID: Integer; AMessageData: TJSONValue); override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure Execute(AProc: TMiletusDialogProc); overload; virtual;
    function Execute: Boolean; overload; virtual;
    property Files: TStrings read FFiles write FFiles;
  published
    property DefaultExt: string read FDefaultExt write FDefaultExt;
    property FileName: string read FFileName write FFileName;
    property Filter: string read FFilter write FFilter;
    property FilterIndex: Integer read FFilterIndex write FFilterIndex;
    property InitialDir: string read FInitialDir write FInitialDir;
    property Options: TMiletusOpenDialogOptions read FOptions write FOptions;
    property Title: string read FTitle write FTitle;
  end;

  TMiletusSaveDialog = class(TMiletusOpenDialog)
  public
    constructor Create(AOwner: TComponent); override;
  end;

  TMiletusMessageBoxProc = reference to procedure(ACheckBoxChecked: Boolean);
  TMiletusMessageBoxType = (mbtNone, mbtInfo, mbtError, mbtQuestion, mbtWarning);

  TMiletusMessageBox = class(TMiletusObserver)
  protected
    procedure ObserverChange(AMessageID: Integer; AMessageData: TJSONValue); override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure Execute(AProc: TMiletusMessageBoxProc = nil);
    procedure ExecuteSync;
  published
    property Buttons: TStringList read FButtons write SetButtons;
    property VerificationChecked: Boolean read FVerificationChecked write FVerificationChecked;
    property VerificationText: string read FVerificationText write FVerificationText;
    property Title: string read FTitle write FTitle;
    property DialogType: TMiletusMessageBoxType read FType write FType;
    property Text: string read FText write FText;
    property NoLink: Boolean read FNoLink write FNoLink;
    property Response: NativeInt read FResponse write FResponse;
    property Caption: string read FCaption write FCaption;
  end;

  TMiletusErrorBox = class(TComponent)
  public
    procedure Execute;
  published
    property Content: string read FContent write FContent;
    property Title: string read FTitle write FTitle;
  end;

  TMiletusMainMenu = class(TCustomMainMenu)
  protected
    procedure GenerateMainMenu;
    procedure MenuItemClick(AIndexes: string; AChecked: Boolean = False);
    procedure DoReceiveMenuResponse(Sender: TObject; AMessageID: Integer; AData: TJSONValue);
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure EndUpdate; override;
  end;

  TMiletusPopupMenu = class(TCustomMainMenu)
  protected
    function GeneratePopupMenu(AX, AY: Integer): TJSONObject;
    procedure MenuItemClick(AIndexes: string; AChecked: Boolean = False);
    procedure DoReceivePopupMenuResponse(Sender: TObject; AMessageID: Integer; AData: TJSONValue);
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure CloseMenu;
    procedure Popup(X, Y: Integer);
    property OnPopup: TNotifyEvent read FOnPopup write FOnPopup;
  end;

  TMiletusFileWatch = class(TCollectionItem)
  public
    constructor Create(ACollection: TCollection); override;
  published
    property FileName: string read FFileName write SetFileName;
    property OnChange: TNotifyEvent read FOnChange write FOnChange;
  end;

  TMiletusFileWatches = class(TOwnedCollection)
  public
    function Add: TMiletusFileWatch; reintroduce;
    property Items[Index: Integer]: TMiletusFileWatch read GetItem write SetItem;
  end;

  TMiletusFileWatcher = class(TMiletusObserver)
  protected
    procedure ObserverChange(AMessageID: Integer; AMessageData: TJSONValue); override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  published
    property Files: TMiletusFileWatches read FFiles write FFiles;
  end;

  TMiletusShortcut = class(TCollectionItem)
  public
    destructor Destroy; override;
  published
    property Shortcut: TShortcut read FShortcut write SetShortcut;
    property OnChange: TNotifyEvent read FOnChange write FOnChange;
  end;

  TMiletusShortcuts = class(TOwnedCollection)
  public
    function Add: TMiletusShortcut; reintroduce;
    property Items[Index: Integer]: TMiletusShortcut read GetItem write SetItem;
  end;

  TMiletusGlobalShortcuts = class(TMiletusObserver)
  protected
    procedure ObserverChange(AMessageID: Integer; AMessageData: TJSONValue); override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  published
    property Shortcuts: TMiletusShortcuts read FShortcuts write FShortcuts;
  end;

  TMiletusNotification = class(TPersistent)
  public
    Name: string;
    Title: string;
    AlertBody: string;
  end;

  TMiletusNotificationCenter = class(TComponent)
  public
    function CreateNotification: TMiletusNotification; overload;
    function CreateNotification(const AName, AAlertBody: string): TMiletusNotification; overload;
    procedure PresentNotification(const ANotification: TMiletusNotification);
  end;

  TMiletusTrayIcon = class(TMiletusObserver)
  protected
    procedure Notification(AComponent: TComponent; Operation: TOperation); override;
    procedure DoCreateTrayIcon;
    procedure ObserverChange(AMessageID: Integer; AMessageData: TJSONValue); override;
  public
    constructor Create(AOwner: TComponent); override;
    procedure EndUpdate; override;
  published
    property Hint: string read FHint write SetHint;
    property IconURL: string read FIconURL write FIconURL;
    property PopupMenu: TMiletusPopupMenu read FPopupMenu write FPopupMenu;
    property OnClick: TNotifyEvent read FOnClick write FOnClick;
  end;

  TMiletusClipboard = class(TPersistent)
  public
    function HasFormat(AFormat: Word): Boolean;
    property AsText: string read GetAsText write SetAsText;
  end;

  TMiletusShell = class(TPersistent)
  public
    procedure Beep;
    procedure OpenExternal(URL: string);
    procedure OpenItem(FullPath: string);
    procedure MoveItemToTrash(FullPath: string);
    procedure ShowItemInFolder(FullPath: string);
  end;

  TMiletusWindow = class(TMiletusObserver)
  protected
    procedure ObserverChange(AMessageID: Integer; AMessageData: TJSONValue); override;
  public
    procedure Close;
    procedure Hide;
    procedure LoadFromURL(URL: string);
    procedure Show;
    procedure ShowModal;
    property FormClass: TFormClass read FFormClass write FFormClass;
  published
    property OnClose: TNotifyEvent read FOnClose write FOnClose;
    property OnShow: TNotifyEvent read FOnShow write FOnShow;
    property OnHide: TNotifyEvent read FOnHide write FOnHide;
    property OnMinimize: TNotifyEvent read FOnMinimize write FOnMinimize;
    property OnMaximize: TNotifyEvent read FOnMaximize write FOnMaximize;
    property OnResize: TNotifyEvent read FOnResize write FOnResize;
  end;

  TMiletusClientDataSet = class;

  TMiletusDBError = class
  public
    property message : string read FMessage;
    property name : string read FName;
  end;

  TMiletusDataProxy = class(TDataProxy)
  protected
    procedure DoOnError(AErrorCode: string);
    function GetConnectionObject: TJSONObject;
    function GetDataRequest(aOptions: TLoadOptions; aAfterRequest: TDataRequestEvent; aAfterLoad: TDatasetLoadEvent) : TDataRequest; override;
    function DoGetData(aRequest : TDataRequest): Boolean; override;
    procedure ProcessUpdate(desc: TRecordUpdateDescriptor);
    // Dummy Abstract method
    function ProcessUpdateBatch(aBatch: TRecordUpdateBatch): Boolean; override;
    procedure DoDBObserverMessage(Sender: TObject; AMessageID: Integer; AData: TJSONValue);
  public
    constructor Create(AOwner: TComponent); override;
  end;

  TMiletusDBDriver = class;

  TMiletusClientDataSetErrorEvent = procedure(AErrorMessage: string) of object;

  TMiletusClientDataSet = class(TClientDataSet)
  protected
    procedure DoAfterOpen; override;
    procedure DoError(AErrorCode: string);
    procedure SetActive(Value: Boolean); override;
    procedure AfterLoadDFMValues; override;
    procedure UpdateStarts;
    procedure UpdateEnds;
    procedure InternalClose; override;
    Function AddToChangeList(aChange : TUpdateStatus) : TRecordUpdateDescriptor ; override;
    property DataLoaded: Boolean read FDataLoaded write FDataLoaded;
    property DBProxy: TMiletusDataProxy read FDBProxy write FDBProxy;
    procedure Notification(AComponent: TComponent; Operation: TOperation); override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure Refresh(Force: Boolean=False); reintroduce;
  published
    property DBDriver: TMiletusDBDriver read FDBDriver write FDBDriver;
    property QueryText: string read FQueryText write FQueryText;
    property OnError: TMiletusClientDataSetErrorEvent read FOnError write FOnError;
  end;

  TMiletusDBDriver = class(TComponent)
  protected
    FDriverID: string;
  public
    constructor Create(AOwner: TComponent); override;
    function SettingsAsJSON: TJSONObject; virtual;
    property DriverID: string read FDriverID;
  published
    property DataBase: string read FDataBase write FDataBase;
    property UserName: string read FUserName write FUserName;
    property Password: string read FPassword write FPassword;
    property IndexName: string read FIndexName write FIndexName;
    //MonitorBy
    property MonitorByInitial: Boolean read FMonitorByInitial write FMonitorByInitial;
    property PoolCleanupTimeout: Integer read FPoolCleanupTimeout write FPoolCleanupTimeout;
    property Pooled: Boolean read FPooled write FPooled;
    property PoolExpireTimeout: Integer read FPoolExpireTimeout write FPoolExpireTimeout;
    property PoolMaximumItems: Integer read FPoolMaximumItems write FPoolMaximumItems;
  end;

  TMiletusAccessDBDriver = class(TMiletusDBDriver)
  public
    constructor Create(AOwner: TComponent); override;
    function SettingsAsJSON: TJSONObject; override;
  published
    property LoginTimeout: Integer read FLoginTimeout write FLoginTimeout;
    property ReadOnly: Boolean read FReadOnly write FReadOnly;
    //property StringFormat
    property SystemDB: string read FSystemDB write FSystemDB;
  end;

  TMiletusMySQLDBDriver = class(TMiletusDBDriver)
  public
    constructor Create(AOwner: TComponent); override;
    function SettingsAsJSON: TJSONObject; override;
  published
    //Characterset
    property Compress: Boolean read FCompress write FCompress;
    property LoginTimeout: Integer read FLoginTimeout write FLoginTimeout;
    property MetaCurCatalog: string read FMetaCurCatalog write FMetaCurCatalog;
    property MetaDefCatalog: string read FMetaDefCatalog write FMetaDefCatalog;
    property Port: Integer read FPort write FPort;
    property ReadTimeout: Integer read FReadTimeout write FReadTimeout;
    //ResultMode
    property Server: string read FServer write FServer;
    property SSL_ca: string read FSSL_ca write FSSL_ca;
    property SSL_capath: string read FSSL_capath write FSSL_capath;
    property SSL_cert: string read FSSL_cert write FSSL_cert;
    property SSL_cipher: string read FSSL_cipher write FSSL_cipher;
    property SSL_key: string read FSSL_key write FSSL_key;
    //TinyIntFormat
    property UseSSL: Boolean read FUseSSL write FUseSSL;
    property WriteTimeout: Integer read FWriteTimeout write FWriteTimeout;
  end;

  TMiletusSQLiteDBDriver = class(TMiletusDBDriver)
  public
    constructor Create(AOwner: TComponent); override;
    function SettingsAsJSON: TJSONObject; override;
  published
    property BusyTimeout: Integer read FBusyTimeout write FBusyTimeout;
    property CacheSize: Integer read FCacheSize write FCacheSize;
    //property DateTimeFormat
    //property Encrypt
    property Extensions: string read FExtensions write FExtensions;
    //property ForeignKeys
    //property GUIDFormat
    //property JournalMode
    //property LockingMode
    property MetaCurCatalog: string read FMetaCurCatalog write FMetaCurCatalog;
    property MetaDefCatalog: string read FMetaDefCatalog write FMetaDefCatalog;
    //property OpenMode
    property SharedCache: Boolean read FSharedCache write FSharedCache;
    property SQLiteAdvanced: string read FSQLiteAdvanced write FSQLiteAdvanced;
    //property StringFormat
    //property Synchronous
  end;

  TMiletusFormMessageEvent = procedure(Sender: TObject; FormID, AMessage: string) of object;

  TMiletusFormBorderStyle = (bsNoneBorder, bsSingleBorder, bsSizeableBorder, bsDialogBorder, bsToolWindowBorder, bsSizeToolWinBorder);
  TDefaultMonitor = (dmDesktop, dmPrimary, dmMainForm, dmActiveForm);
  TWindowState = (wsNormal, wsMinimized, wsMaximized);
  TBorderIcon = (biSystemMenu, biMinimize, biMaximize, biHelp);
  TBorderIcons = set of TBorderIcon;

  TMiletusForm = class(TWebForm)
  protected
    procedure DoReceiveFormMessage(Sender: TObject; AMessageID: Integer; AData: TJSONValue);
    procedure Loaded; override;
  public
    procedure CreateInitialize; override;
    procedure Close; override;
    procedure EndUpdate; override;
    procedure RegisterForm(FormID: string);
    procedure SendMessage(FormID, AMessage: string);
  published
    property BorderStyle: TMiletusFormBorderStyle read FBorderStyle write SetBorderStyle;
    property BorderIcons: TBorderIcons read FBorderIcons write SetBorderIcons;
    property BorderWidth;
    property DefaultMonitor: TDefaultMonitor read FDefaultMonitor write SetDefaultMonitor;
    property FormStyle;
    property Position;
    property ScreenSnap: Boolean read FScreenSnap write SetScreenSnap;
    property SnapBuffer: Integer read FSnapBuffer write SetSnapBuffer;
    property WindowState: TWindowState read FWindowState write SetWindowState;
    property OnFormMessage: TMiletusFormMessageEvent read FOnFormMessage write FOnFormMessage;
  end;

  TMiletusCommunicationReceiveMessageEvent = procedure(AMessageID: Integer; AMessageData: TJSONValue) of object;

  TMiletusCommunication = class(TPersistent)
  protected
    procedure Change(AMessageID: Integer; AMessageData: TJSONValue);
    procedure DoHandleVersionInfo(AMessageData: TJSONValue);
  public
    constructor Create;
    destructor Destroy; override;
    procedure RegisterObserver(Observer: TMiletusObserver);
    procedure UnregisterObserver(Observer: TMiletusObserver);
    procedure ReceiveMessageFromMiletus(const AMessage: string);
  end;

  TMiletusClientCommunicationMessageEvent = procedure (Sender: TObject; AMessageID: Integer; AMessageData: TJSONObject) of object;

  TMiletusClientCommunication = class(TMiletusObserver)
  protected
    procedure ObserverChange(AMessageID: Integer; AMessageData: TJSONValue); override;
  public
    procedure SendMessage(AMessageID: Integer; AMessageData: TJSONObject);
  published
    property OnMessage: TMiletusClientCommunicationMessageEvent read FOnMessage write FOnMessage;
  end;

function GetCursorPos: TPoint;
procedure GetMiletusPath(APathType: Integer; var APath: string);
procedure StartFileDrag(APath: string);
function MiletusShell: TMiletusShell;
function MiletusClipboard: TMiletusClipboard;
procedure OpenDevTools;

var
  MiletusCommunication: TMiletusCommunication;
  MiletusOpenDialogOption: array[TMiletusOpenDialogOption] of string = (
    'ofReadOnly', 'ofOverwritePrompt', 'ofHideReadOnly',
    'ofNoChangeDir', 'ofShowHelp', 'ofNoValidate', 'ofAllowMultiSelect',
    'ofExtensionDifferent', 'ofPathMustExist', 'ofFileMustExist', 'ofCreatePrompt',
    'ofShareAware', 'ofNoReadOnlyReturn', 'ofNoTestFileCreate', 'ofNoNetworkButton',
    'ofNoLongNames', 'ofOldStyleDialog', 'ofNoDereferenceLinks', 'ofEnableIncludeNotify',
    'ofEnableSizing', 'ofDontAddToRecent', 'ofForceShowHidden');

const
  {Component messages}
  NM_WINDOW_SHOW = $001;
  NM_WINDOW_CLOSE = $002;
  NM_WINDOW_SHOWMODAL = $003;
  NM_WINDOW_HIDE = $004;
  NM_WINDOW_LOADURL = $005;
  NM_WINDOW_RESIZE = $006;
  NM_WINDOW_MINIMIZE = $007;
  NM_WINDOW_MAXIMIZE = $008;
  NM_WINDOW_MSGREG = $009;
  NM_WINDOW_MSGSEND = $00A;
  NM_WINDOW_MSGGET = $00B;
  NM_WINDOW_UPDATE = $00C;
  NM_OPENDEVTOOLS = $00D;
  NM_VERSION_INFO = $00E;
  NM_VERSION_GET = $00F;
  NM_OPENDIALOG_OPEN = $020;
  NM_SAVEDIALOG_OPEN = $021;
  NM_ERRORDLG = $022;
  NM_MESSAGEBOX_OPEN = $023;
  NM_STRINGLIST_LOAD = $024;
  NM_STRINGLIST_SAVE = $025;
  NM_MAINMENU_UPDATE = $026;
  NM_MAINMENU_ITEMCLICK = $027;
  NM_POPUPMENU_OPEN = $028;
  NM_POPUPMENU_CLOSE = $029;
  NM_POPUPMENU_ITEMCLICK = $02A;
  NM_FILEWATCH_ADD = $02B;
  NM_FILEWATCH_REMOVE = $02C;
  NM_FILEWATCH_CHANGE = $02D;
  NM_SHORTCUT_ADD = $02E;
  NM_SHORTCUT_REMOVE = $02F;
  NM_SHORTCUT_CHANGE = $030;
  NM_TRAYICON_UPDATE = $031;
  NM_TRAYICON_CLICK = $032;
  NM_TRAYICON_MENUCLICK = $033;
  NM_DB_CONNECT = $034;
  NM_DB_GETROWS = $035;
  NM_DB_SETROWS = $036;
  NM_DB_GETFIELDS = $037;
  NM_DB_SETFIELDS = $038;
  NM_DB_INSERT = $039;
  NM_DB_MODIFY = $03A;
  NM_DB_DELETE = $03B;
  NM_DB_SETAUTOINC = $03C;
  NM_DB_ERROR = $03D;

  {Shell/function messages}
  NM_NOTIFICATION = $100;
  NM_BEEP = $101;
  NM_SHOWFILE = $102;
  NM_OPENPATH = $103;
  NM_OPENURL = $104;
  NM_DELETEFILE = $105;
  NM_STARTDRAG = $106;
  NM_GETCURSOR = $107;
  NM_GETPATH = $108;
  NM_CLIPBOARD_SET = $109;
  NM_CLIPBOARD_GET = $10A;
  NM_CLIPBOARD_FORMAT = $10B;

  {User defined}
  NM_USER = $400;

  {Path values}
  NP_APPDATA = $01;
  NP_APPPATH = $02;
  NP_DESKTOP = $03;
  NP_DOCUMENTS = $04;
  NP_DOWNLOADS = $05;
  NP_EXE = $06;
  NP_HOME = $07;
  NP_MUSIC = $08;
  NP_USERDATA = $09;
  NP_PICTURES = $0A;
  NP_TEMP = $0B;
  NP_VIDEOS = $0C;

  {Clipboard formats}
  NC_TEXT = $01;
  NC_BITMAP = $02;
  NC_METAFILEPICT = $03;
  NC_PICTURE = $04;
  NC_COMPONENT = $05;

implementation
initialization
end.
