{$Error This is a closed source unit. You cannot compile it, it was already compiled.}
{$Error Probably the IDE has cleaned up and you have to unpack the zip again.}
{********************************************************************}
{                                                                    }
{ written by TMS Software                                            }
{            copyright (c) 2018 - 2020                               }
{            Email : info@tmssoftware.com                            }
{            Web : http://www.tmssoftware.com                        }
{                                                                    }
{ The source code is given as is. The author is not responsible      }
{ for any possible damage done due to the use of this code.          }
{ The complete source code remains property of the author and may    }
{ not be distributed, published, given or sold in any form as such.  }
{ No parts of the source code can be included in any other component }
{ or application without written authorization of the author.        }
{********************************************************************}

unit WEBLib.PayPal;

{$DEFINE NOPP}

interface

uses
  Classes, Web, JS, WEBLib.Controls, WEBLib.Graphics, WEBLib.JSON;

const
  MAJ_VER = 2; // Major version nr.
  MIN_VER = 0; // Minor version nr.
  REL_VER = 0; // Release nr.
  BLD_VER = 0; // Build nr.

  // version history
  // 1.0.0.0 : First release
  // 1.0.0.1 : Fixed: Issue when TPayPalItem.Tax <> 0
  // 1.0.0.2 : Fixed: Issue with events in Release mode
  // 2.0.0.0 : New: Migration to PayPal JavaScript SDK

type
  TPayPal = class;
  TPayPalPayment = class;
  TPayPalItems = class;

  TPayPalPaymentEventArgs = class(TPersistent)
  public
    property Description: string read FDescription write FDescription;
    property PaymentID: string read FPaymentID write FPaymentID;
    property PayerID: string read FPayerID write FPayerID;
    property OrderID: string read FOrderID write FOrderID;
    property SaleID: string read FSaleID write FSaleID;
    property PaymentState: string read FPaymentState write FPaymentState;
    property Email: string read FEmail write FEmail;
    property Phone: string read FPhone write FPhone;
    property CustomText: string read FCustomText write FCustomText;
    property InvoiceNumber: string read FInvoiceNumber write FInvoiceNumber;
    property FirstName: string read FFirstName write FFirstName;
    property LastName: string read FLastName write FLastName;
    property Address1: string read FAddress1 write FAddress1;
    property Address2: string read FAddress2 write FAddress2;
    property City: string read FCity write FCity;
    property PostalCode: string read FPostalCode write FPostalCode;
    property CountryCode: string read FCountryCode write FCountryCode;
    property State: string read FState write FState;
    property RecipientName: string read FRecipientName write FRecipientName;
    property Total: string read FTotal write FTotal;
    property Currency: string read FCurrency write FCurrency;
  end;

  TPayPalPaymentEvent = procedure(Sender: TObject; Args: TPayPalPaymentEventArgs) of object;

  TPayPalErrorEventArgs = class(TPersistent)
  public
    property ErrorName: string read FErrorName write FErrorName;
    property ErrorDetails: TStringList read FErrorDetails write FErrorDetails;
  end;

  TPayPalErrorEvent = procedure(Sender: TObject; Args: TPayPalErrorEventArgs) of object;

  TPayPalCurrency = (pcAUD, pcBRL, pcCAD, pcCZK, pcDKK, pcEUR, pcHKD, pcHUF,
    pcILS, pcJPY, pcMYR, pcMXN, pcTWD, pcNZD, pcNOK, pcPHP, pcPLN, pcGBP, pcRUB,
    pcSGD, pcSEK, pcCHF, pcTHB, pcUSD);
  TPayPalLocale = (plDefault, plEN_US, plDE_DE, plNL_NL, plFR_FR, plPT_BR, plFR_CA, plZH_CN,
    plDA_DK, plRU_RU, plIT_IT, plNO_NO, plPL_PL, plPT_PT, plES_ES, plSV_SE,
    plEN_GB, plJA_JP);

  TPayPalItem = class(TCollectionItem)
  public
    constructor Create(Collection: TCollection); override;
    destructor Destroy; override;
  published
    property Name: string read FName write SetName;
    property Price: Double read FPrice write SetPrice;
    property Quantity: Integer read FQuantity write SetQuantity default 1;
    property Description: string read FDescription write SetDescription;
    property Tax: Double read FTax write SetTax default 0;
    property SKU: String read FSKU write SetSKU;
    property Tag: Integer read FTag write SetTag default 0;
    property TagObject: TObject read FTagObject write SetTagObject;
  end;

  TPayPalItems = class(TOwnedCollection)
  public
    constructor Create(AOwner: TPayPalPayment); overload;
    function GetOwner: TPersistent; override;
    function Add: TPayPalItem; reintroduce;
    function Insert(Index: Integer):TPayPalItem; reintroduce;
    property Items[Index: integer]: TPayPalItem read GetItems write SetItems; default;
    property OnChange: TNotifyEvent read FOnChange write FOnChange;
  end;

  TPayPalPayment = class(TPersistent)
  public
    constructor Create(AOwner: TPayPal); overload;
    destructor Destroy; override;
    property OnChange: TNotifyEvent read FOnChange write FOnChange;
    procedure Assign(Source: TPersistent); override;
  published
    property Tax: Double read FTax write SetTax default 0;
    property Shipping: Double read FShipping write SetShipping default 0;
    property ShippingDiscount: Double read FShippingDiscount write SetShippingDiscount default 0;
    property HandlingFee: Double read FHandlingFee write SetHandlingFee default 0;
    property Insurance: Double read FInsurance write SetInsurance default 0;
    property CustomText: string read FCustomText write SetCustomText;
    property InvoiceNumber: string read FInvoiceNumber write SetInvoiceNumber;
    property Description: string read FDescription write SetDescription;
    property Currency: TPayPalCurrency read FCurrency write SetCurrency default pcUSD;
    property Locale: TPayPalLocale read FLocale write SetLocale default plDefault;
    property Items: TPayPalItems read FItems write SetItems;
    property Address1: string read FAddress1 write SetAddress1;
    property Address2: string read FAddress2 write SetAddress2;
    property City: string read FCity write SetCity;
    property PostalCode: string read FPostalCode write SetPostalCode;
    property CountryCode: string read FCountryCode write SetCountryCode;
    property State: string read FState write SetState;
    property Phone: string read FPhone write SetPhone;
    property RecipientName: string read FRecipientName write SetRecipientName;
  end;

  TPayPal = class(TCustomControl)
  protected
    function CreateElement: TJSElement; override;
    procedure UpdateElement; override;
    procedure UpdateScript;
    procedure AddScript(Script: string);
    function CurrencyToStr(ACurrency: TPayPalCurrency): string;
    procedure AddScriptLinkLoaded(const link: string);
    procedure ScriptLinkLoaded(Event: TJSEvent);
    function HandlePaymentDone(Event, PaymentDetails: TJSObject): Boolean;
    function HandlePaymentCancelled(): Boolean;
    function HandlePaymentError(Error: TJSObject): Boolean;
  public
    destructor Destroy; override;
    procedure CreateInitialize; override;
  published
    property Enabled;
    property ElementClassName;
    property ElementID;

    property APIKey: string read FAPIKey write SetAPIKey;
    property Payment: TPayPalPayment read FPayment write SetPayment;

    property OnPaymentDone: TPayPalPaymentEvent read FOnPaymentDone write FOnPaymentDone;
    property OnPaymentCancelled: TNotifyEvent read FOnPaymentCancelled write FOnPaymentCancelled;
    property OnPaymentError: TPayPalErrorEvent read FOnPaymentError write FOnPaymentError;
  end;

  TWebPayPal = class(TPayPal);

implementation
end.

