{$Error This is a closed source unit. You cannot compile it, it was already compiled.}
{$Error Probably the IDE has cleaned up and you have to unpack the zip again.}
{********************************************************************}
{                                                                    }
{ written by TMS Software                                            }
{            copyright (c) 2018 - 2020                               }
{            Email : info@tmssoftware.com                            }
{            Web : http://www.tmssoftware.com                        }
{                                                                    }
{ The source code is given as is. The author is not responsible      }
{ for any possible damage done due to the use of this code.          }
{ The complete source code remains property of the author and may    }
{ not be distributed, published, given or sold in any form as such.  }
{ No parts of the source code can be included in any other component }
{ or application without written authorization of the author.        }
{********************************************************************}

unit WEBLib.ThreeJsControls;

{$DEFINE NOPP}

interface


uses classes, Sysutils, WEBLib.Controls, WEBLib.webtools,
   WEBLib.graphics, web, js, libthreejs, math, strutils;

const
   // Use this prefix on object names that you don't want to
   // clear when the scene is cleared
   BuiltinPrefix: string = 'builtin-';

type

  TThreeJsTextStyle = class
  public
    constructor create;
    property FontFace: string read FFontFace write FFontFace;
    property FontHeight: double read FFontHeight write FFontHeight;
    property IsBold: boolean read FIsBold write FIsBold;
    property IsTransparentFill: boolean read FIsTransparentFill write FIsTransparentFill;
    property TextColor: TColor read FTextColor write FTextColor;
    property FillColor: TColor read FFillColor write FFillColor;
    property BorderWidth: integer read FBorderWidth write FBorderWidth;
    property BorderColor: TColor read FBorderColor write FBorderColor;
  end;

  TThreeJsCustomControl = class(TCustomControl)
    // hidden from subclasses
  protected
    procedure BindEvents; override;
    procedure UnbindEvents; override;
    function CreateElement: TJSElement; override;
    procedure Loaded; override;
    procedure SetParent(AValue: TControl); override;

    // This is executed only the first time form is loaded
    // containing the component. So defaults can be set in
    // this.
    procedure InitThreeJs; virtual;

    // Override these to bind new control specific events
    // Sample code on how to do it is in BindEvents of TControl
    procedure BindThreeJsEvents; virtual;
    procedure UnbindThreeJsEvents; virtual;
    procedure Paint; override;

    // Click on non-object area
    procedure HandleClick; virtual;
    procedure HandleDblClick; virtual;
    // click for only the target objects provided by the above
    // raycaster target objects
    function HandleObjectClick(anObject: TThreeJsObject3D; intersection: TThreeJsIntersection): boolean; virtual;
    procedure HandleObjectExit(anObject: TThreeJsObject3D); virtual;
    procedure HandleObjectDblClick(anObject: TThreeJsObject3D); virtual;
    procedure HandleObjectMouseMove(anObject: TThreeJsObject3D; intersection: TThreeJsIntersection); virtual;
    procedure HandleObjectMouseEnter(anObject: TThreeJsObject3D); virtual;
    procedure HandleObjectMouseLeave(anObject: TThreeJsObject3D); virtual;
    // base class overrides that call the above virtual methods
    function GetRaycasterTargetObjects: TJSArray; virtual;
    function GetObjectAt(X, Y: double;
              var intersects: TJSArray;
              var intersection: TThreeJsIntersection): TThreeJsObject3D;
    procedure MouseDown(Button: TMouseButton; Shift: TShiftState; X,Y: Integer); override;
    procedure MouseMove(Shift: TShiftState; X, Y: Integer); override;
    procedure DblClick; override;
    procedure KeyDown(var AKey: Word; Shift: TShiftState); override;

    // ThreeJs control is created or not
    property IsInitialized: boolean read FIsInitialized;
    // Disallow event binding to the enclosing DIV, for example in list which has its own events completely
    property IsDefaultEventBinding: boolean read FisDefaultEventBinding write FisDefaultEventBinding;
  public
    procedure CreateInitialize; override;

    procedure SaveState; reintroduce;
    procedure ResetState;
    procedure ResetCameraPosition;
    procedure ResetSceneAndCamera;

    function GetObjectByName(aname: string): TThreeJsObject3D;
    procedure RemoveObject(anObject: TThreeJsObject3D); overload;
    procedure RemoveObject(aname: string); overload;
    procedure RemoveObjectFromCamera(aname: string);

    function AddAxes(aLength: double): TThreeJsObject3D;

    function AddChartGridXY(aWidth, aHeight: double;
      numDivisionsHorizontal, numDivisionsVertical: integer;
      aGridColor: TColor=$ffffff; aLineColor: TColor=$000000;
        isThickLines:boolean=false; isTransparent:boolean=true): TThreeJsObject3D;

    function AddChartGridYZ(aWidth, aHeight: double;
      numDivisionsHorizontal, numDivisionsVertical: integer;
      aGridColor: TColor=$ffffff; aLineColor: TColor=$000000;
        isThickLines:boolean=false; isTransparent:boolean=true): TThreeJsObject3D;


    function AddPlane(
                aWidth: double; aHeight: double;
                aColor: TColor=$ffffff
              ): TThreeJsObject3D;
    function AddFloorPlane(
                xWidth: double; zWidth: double;
                aColor: TColor=$E6D8AD
              ): TThreeJsObject3D;

    function AddLabelZ(aText: string; aZValue: double; floorPlaneWidth: double; isFixed: boolean=false;
        valign: TTXVerticalAlign = txvCenter;
        halign: TTXHorizontalAlign = txhLeft;
        textColor: TColor=$000000;
        fontFace: string = 'Arial';
        fontHeight: double = 1;
        isBold: boolean = true;
        isTransparentFill: boolean = true;
        fillColor: TColor = $eeeeee;
        borderWidth: double = 0;
        borderColor: TColor = $E6D8AD;
        marginPixels: double = 24;
        radiusPixels: double = 6
        ): TThreeJsObject3D;
    function AddLabelX(aText: string; anXValue: double; floorPlaneDepth: double;
        isFixed: boolean=false;
        valign: TTXVerticalAlign = txvCenter;
        halign: TTXHorizontalAlign = txhLeft;
        textColor: TColor = $000000;
        fontFace: string = 'Arial';
        fontHeight: double = 1;
        isBold: boolean = true;
        isTransparentFill: boolean = true;
        fillColor: TColor = $eeeeee;
        borderWidth: double = 0;
        borderColor: TColor = $E6D8AD;
        marginPixels: double = 24;
        radiusPixels: double = 6
        ): TThreeJsObject3D;
    function AddLabelY(aText: string; aYValue: double; floorPlaneDepth: double; isFixed: boolean=false;
        valign: TTXVerticalAlign = txvCenter;
        halign: TTXHorizontalAlign = txhLeft;
        textColor: TColor = $000000;
        fontFace: string = 'Arial';
        fontHeight: double = 1;
        isBold: boolean = true;
        isTransparentFill: boolean = true;
        fillColor: TColor = $eeeeee;
        borderWidth: double = 0;
        borderColor: TColor = $E6D8AD;
        marginPixels: double = 24;
        radiusPixels: double = 6
        ): TThreeJsObject3D;
    function addText(aText: string;
        X, Y, Z: double;
        isFixed: boolean=false;
        isDrawTopmost: boolean=false;
        // alignment of whole sprite with respect to x , y, z point
        valign: TTXVerticalAlign = txvCenter;
        halign: TTXHorizontalAlign = txhLeft;
        // alignment of internal text in case of Multi-line text
        halignMultiLineText: TTXHorizontalAlign = txhLeft;
        textColor: TColor = $000000;
        fontFace: string = 'Arial';
        fontHeight: double = 1;
        isBold: boolean = true;
        isTransparentFill: boolean = true;
        fillColor: TColor = $ffffff;
        borderWidth: double = 0;
        borderColor: TColor = $E6D8AD): TThreeJsObject3D;

    function AddBar(width, height, depth: double;
                aColor: TColor;
                positionX, positionY, positionZ: Double;
                castShadow: boolean = true;
                isTransparent: boolean = false;
                opacity: double=0.6
              ): TThreeJsObject3D;
    function AddCube(
                length: double; aColor: TColor;
                positionX, positionY, positionZ: Double;
                castShadow: boolean = true;
                isTransparent: boolean = false;
                opacity: double=0.6
              ): TThreeJsObject3D;
    function AddSphere(radiusPixels: Double;
                aColor: TColor;
                positionX, positionY, positionZ: Double;
                castShadow: boolean = true;
                isTransparent: boolean = false;
                opacity: double=0.6;
                widthSegments: integer=32;
                heightSegments: integer=32
              ): TThreeJsObject3D;
    function AddCylinder(radiusTop, radisBottom, height: Double;
                aColor: TColor;
                positionX, positionY, positionZ: Double;
                castShadow: boolean = true;
                isTransparent: boolean = false;
                opacity: double=0.6;
                radialSegments: integer=32
              ): TThreeJsObject3D;

    function AddLine(points: TJSArray;
         aColor: TColor; aLineWidth: double;
         castShadow: boolean = true;
         isTransparent: boolean = false;
         opacity: double=0.6
         ): TThreeJsObject3D;

    function Add2dShape(
                geo: TThreeJsGeometry; aColor: TColor;
                aWidth: double;
                castShadow: boolean = true;
                isTransparent: boolean = false;
                opacity: double=0.6
              ): TThreeJsObject3D;
    // If you want lines without shadow and click features
    // such as on our chart walls, use this.
    function Add2dLine(aGeometry: TThreeJsGeometry;
         aColor: TColor; aLineWidth: double;
         isTransparent: boolean = false;
         opacity: double=0.6
         ): TThreeJsObject3D;

    procedure SetTargetViewVector(aVector: TThreeJsVector3);
    procedure RotateLeft(deltaAngleDeg: double);
    procedure RotateUp(deltaAngleDeg: double);
    procedure ZoomIn(deltaScale: double);
    procedure PanLeft;
    procedure PanRight;
    procedure PanUp;
    procedure PanDown;
    procedure DumpOrbitControlsPosition;
    procedure ShowDebugArrow(origin, terminus: TThreeJsVector3;
                        aColor: TColor=$0000ff; isShow: boolean=true);

    property Renderer: TThreeJsRenderer read FRenderer;
    property Scene: TThreeJsScene read FScene;
    property Camera: TThreeJsCamera read FCamera;

    property SpotLight: TThreeJsSpotLight read FSpotLight;
    property AmbientLight: TThreeJsAmbientLight read FAmbientLight;
    property DefaultAmbientLightColor: TColor read FDefaultAmbientLightColor write FDefaultAmbientLightColor;
    property DefaultAmbientLightIntensity: double read FDefaultAmbientLightIntensity write FDefaultAmbientLightIntensity;
    property DefaultSpotLightColor: TColor read FDefaultSpotLightColor write FDefaultSpotLightColor;
    property DefaultSpotLightIntensity: double read FDefaultSpotLightIntensity write FDefaultSpotLightIntensity;

    property OrbitControls: TThreeJsOrbitControls read FOrbitControls;
    property Axes: TThreeJsAxes read FAxes;
    property CenterPointTarget: TThreeJsVector3 read FCenterPointTarget;
    property InitialCameraPosition: TThreeJsVector3 read FInitialCameraPosition write FInitialCameraPosition;
    property OnClick: TNotifyEvent read FOnClick write FOnClick;
    property OnDblClick: TNotifyEvent read FOnDblClick write FOnDblClick;
  end;

  TThreeJsLoadObjectEvent = procedure(Sender: TObject; success: boolean; anObject: TThreeJsObject3D; errorMsg: string) of object;
  TThreeJsObjectEvent = procedure(Sender: TObject;
        frontObject: TThreeJsObject3D; intersection: TThreeJsIntersection) of object;
  TThreeJsObjectNotifyEvent = procedure(Sender: TObject; frontObject: TThreeJsObject3D) of object;

  TThreeJsPaintBox = class(TThreeJsCustomControl)
  protected
    function HandleObjectClick(anObject: TThreeJsObject3D; intersection: TThreeJsIntersection): boolean; override;
    procedure HandleObjectExit(anObject: TThreeJsObject3D); override;
    procedure HandleObjectDblClick(anObject: TThreeJsObject3D); override;
    procedure HandleObjectMouseMove(anObject: TThreeJsObject3D; intersection: TThreeJsIntersection); override;
    procedure HandleObjectMouseEnter(anObject: TThreeJsObject3D); override;
    procedure HandleObjectMouseLeave(anObject: TThreeJsObject3D); override;
  public
    procedure CreateInitialize; override;
    function ShowAnnotation(aname: string; x, y, z: double; aText: string;
        targetObject: TThreeJsObject3D = nil;
        forcedProportionalHeight: double = 0;
        // alignment of whole sprite with respect to x , y, z point
        valign: TTXVerticalAlign = txvBottom;
        halign: TTXHorizontalAlign = txhCenter;
        // alignment of internal text in case of Multi-line text
        halignMultiLineText: TTXHorizontalAlign = txhCenter
       ): TThreeJsObject3D;

    property AnnotationTextStyle: TThreeJsTextStyle read FAnnotationTextStyle write FAnnotationTextStyle;
  published
    property OnClick;
    property OnDblClick;

    property OnItemClick: TThreeJsObjectEvent read FOnItemClick write FOnItemClick;
    property OnItemExit: TThreeJsObjectNotifyEvent read FOnItemExit write FOnItemExit;
    property OnItemDblClick: TThreeJsObjectNotifyEvent read FOnItemDblClick write FOnItemDblClick;
    property OnItemMouseMove: TThreeJsObjectEvent read FOnItemMouseMove write FOnItemMouseMove;
    property OnItemMouseEnter: TThreeJsObjectNotifyEvent read FOnItemMouseEnter write FOnItemMouseEnter;
    property OnItemMouseLeave: TThreeJsObjectNotifyEvent read FOnItemMouseLeave write FOnItemMouseLeave;
  end;

  TWebThreeJsPaintBox = class(TThreeJsPaintBox);

  TThreeJsModelBox = class(TThreeJsPaintBox)
  protected
    procedure InitThreeJs; override;
    procedure InitBeforeLoad; virtual;
    procedure DoOnDoneObjectLoader(success: boolean; anObject: TJSObject); virtual;
    procedure DoOnObjectLoad(success: boolean; anObject: TThreeJsObject3D; errorMsg: string); virtual;
    procedure DoOnLoadDefaultLoadingManager; virtual;
  public
    procedure CreateInitialize; override;
    procedure AddObjectMtl(aName: string;
         objFile, objPath, mtlFile, mtlPath, texturePath: string;
         positionX: double=0; positionY: double=0; positionZ: Double=0);
    procedure AddObjectGltf(aName: string;
         gltfFile, gltfPath: string;
         positionX: double=0; positionY: double=0; positionZ: Double=0);
    // useful for above addObject functions
    procedure BringObjectInFullView(anObject: TThreeJsObject3d);
    procedure AddLights; virtual;
    procedure RemoveLights; virtual;
    procedure ExportSceneGltf(gltfFileName: string);

    property DefaultLightColor: TColor read FDefaultLightColor write FDefaultLightColor;
    property DefaultLightIntensity: double read FDefaultLightIntensity write FDefaultLightIntensity;
    property UseGammaCorrection: boolean read FUseGammaCorrection write FUseGammaCorrection;
    property AdditionalLights: TJSArray read FAdditionalLights;
  published
    property OnObjectLoad: TThreeJsLoadObjectEvent read FOnObjectLoad write FOnObjectLoad;
    property OnClick;
    property OnDblClick;
    property OnItemClick;
    property OnItemDblClick;
    property OnItemExit;
    property OnItemMouseLeave;
    property OnItemMouseEnter;
    property OnItemMouseMove;
  end;

  TWebThreeJsModelBox = class(TThreeJsModelBox);

implementation
end.
