unit frmlicensegenerator;

{$mode ObjFPC}{$H+}

interface

uses
  Classes, SysUtils,
  {$IFDEF Unix}
  BaseUnix,
  {$ENDIF}
  Forms, Controls, Graphics, Dialogs, StdCtrls, ButtonPanel, LazUTF8Classes,
  LazFileUtils, LazLoggerBase, IDEDialogs, TMSWebCoreIDEOptions;

type

  { TTMSLicenseGeneratorForm }

  TTMSLicenseGeneratorForm = class(TForm)
    ButtonPanel1: TButtonPanel;
    CodeEdit: TEdit;
    CodeLabel: TLabel;
    EmailEdit: TEdit;
    EmailLabel: TLabel;
    NoteLabel: TLabel;
    procedure FormCreate(Sender: TObject);
  private
    FGeneratorPath: string;
    FDstPath: string;
    procedure ButtonPanel1OKButtonClick(Sender: TObject);
    procedure SetDstPath(const AValue: string);
  public
    property GeneratorPath: string read FGeneratorPath write FGeneratorPath;
    property DstPath: string read FDstPath write SetDstPath;
  end;

function ShowLicenseGenerator(GeneratorPath, DstPath: string): TModalResult;

implementation

function ShowLicenseGenerator(GeneratorPath, DstPath: string): TModalResult;
var
  TMSLicenseGeneratorForm: TTMSLicenseGeneratorForm;
begin
  TMSLicenseGeneratorForm:=TTMSLicenseGeneratorForm.Create(nil);
  try
    TMSLicenseGeneratorForm.GeneratorPath:=GeneratorPath;
    TMSLicenseGeneratorForm.DstPath:=DstPath;
    Result:=TMSLicenseGeneratorForm.ShowModal;
  finally
    TMSLicenseGeneratorForm.Free;
  end;
end;

{$R *.lfm}

{ TTMSLicenseGeneratorForm }

procedure TTMSLicenseGeneratorForm.FormCreate(Sender: TObject);
begin
  EmailEdit.Text:='';
  CodeEdit.Text:='';
  ButtonPanel1.OKButton.OnClick:=@ButtonPanel1OKButtonClick;
  ButtonPanel1.OKButton.ModalResult:=mrNone;
end;

procedure TTMSLicenseGeneratorForm.ButtonPanel1OKButtonClick(Sender: TObject);
var
  Params, TheOutput, TheErrors: TStringList;
  Email, Code: TCaption;
  TheExitCode, r: integer;
  LicenseFile: String;
begin
  LicenseFile:=DstPath+PathDelim+'License.lic';

  if not FileExistsUTF8(GeneratorPath) then begin
    IDEMessageDialog('Generator missing',
      'File not found:'+sLineBreak
      +GeneratorPath,mtError,[mbOk],'');
    ModalResult:=mrCancel;
    exit;
  end;

  Email:=Trim(EmailEdit.Text);
  Code:=Trim(CodeEdit.Text);

  // run license generator
  Params:=TStringList.Create;
  TheOutput:=TStringList.Create;
  TheErrors:=TStringList.Create;
  try
    Params.Add('-code:'+Code);
    Params.Add('-destpath:'+DstPath);
    Params.Add('-email:'+Email);
    Params.Add('-lazarus');
    {$IFDEF Unix}
    if not FileIsExecutable(GeneratorPath) then
    begin
      // fix executable perms on pas2js_OS_CPU
      FpChmod(GeneratorPath,&775);
      InvalidateFileStateCache(GeneratorPath);
    end;
    {$ENDIF}

    WebCore_RunTool(GeneratorPath,Params,DstPath,TheOutput,TheErrors,TheExitCode,[]);
    if FileExistsUTF8(LicenseFile) then
    begin
      ModalResult:=mrOk;
      exit;
    end;
    if TheOutput.Count>0 then
      debugln(['  Output={',TheOutput.Text,'}']);
    if TheErrors.Count>0 then
      debugln(['  Errors={',TheErrors.Text,'}']);

    r:=IDEMessageDialog('License not found',
      'The generator did not create the file:'+sLineBreak
      +LicenseFile,mtError,[mbRetry,mbIgnore],'');
    if r<>mrRetry then begin
      ModalResult:=mrCancel;
      exit;
    end;

  finally
    TheErrors.Free;
    TheOutput.Free;
    Params.Free;
  end;
end;

procedure TTMSLicenseGeneratorForm.SetDstPath(const AValue: string);
var
  NewValue: String;
begin
  NewValue:=ChompPathDelim(ExpandFileNameUTF8(AValue));
  if FDstPath=NewValue then Exit;
  FDstPath:=NewValue;
end;

initialization
  OnRunLicenseGenerator:=@ShowLicenseGenerator;

end.

