{********************************************************************}
{                                                                    }
{ written by TMS Software                                            }
{            Copyright (c) 2018 - 2020                               }
{            Email : info@tmssoftware.com                            }
{            Web : http://www.tmssoftware.com                        }
{                                                                    }
{ The source code is given as is. The author is not responsible      }
{ for any possible damage done due to the use of this code.          }
{ The complete source code remains property of the author and may    }
{ not be distributed, published, given or sold in any form as such.  }
{ No parts of the source code can be included in any other component }
{ or application without written authorization of the author.        }
{********************************************************************}

unit WEBLib.ComCtrls;

interface

uses
  {$IFNDEF LCLTMSWEB}
  System.Classes, Winapi.Windows, VCL.Graphics, VCL.Controls, SysUtils, VCL.Forms,
  Types, VCL.ComCtrls
  , WEBLib.Controls, WEBLib.Graphics, WEBLib.StdCtrls, WEBLib.ExtCtrls, Web;
  {$ENDIF}
  {$IFDEF LCLTMSWEB}
  Classes, SysUtils, Types, LCLType, LCLIntf, Graphics, Controls, ComCtrls,
  Forms, WEBLib.Controls, WEBLib.Graphics, WEBLib.StdCtrls, WEBLib.ExtCtrls,
  Web;
  {$ENDIF}

type
  TWebCustomRichEdit = class;

  TTextAttributes = class(TPersistent)
  strict private
    FAlignment: TAlignment;
    FBackColor: TColor;
    FColor: TColor;
    FHeight: Integer;
    FName: string;
    FOrderedList: Boolean;
    FSize: Integer;
    FStyle: TFontStyles;
    FUnOrderedList: Boolean;
  public
    constructor Create(AOwner: TWebCustomRichEdit);
    procedure Assign(Source: TPersistent); override;
    procedure Copy;
    procedure Cut;
    procedure Image;
    procedure Link;
    procedure Paste;
    property Alignment: TAlignment read FAlignment write FAlignment;
    property BackColor: TColor read FBackColor write FBackColor;
    property Color: TColor read FColor write FColor;
    property Height: Integer read FHeight write FHeight;
    property Name: string read FName write FName;
    property OrderedList: Boolean read FOrderedList write FOrderedList;
    property Size: Integer read FSize write FSize;
    property Style: TFontStyles read FStyle write FStyle;
    property UnOrderedList: Boolean read FUnOrderedList write FUnOrderedList;
  end;

  TWebCustomRichEdit = class(TWebCustomControl)
  strict private
    FLines: TStrings;
    FOnSelectionChange: TNotifyEvent;
    FPlainText: string;
    FSelAttributes: TTextAttributes;
    procedure SetLines(const Value: TStrings);
  private
    FShowFocus: boolean;
    FInsertLinebreaks: boolean;
    FWantTabs: boolean;
    FReadOnly: boolean;
    FOnChange: TNotifyEvent;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure DoImageAction;
    procedure DoLinkAction;
    procedure DoSelectAction(StartPosition, EndPosition: Integer);
    procedure InsertHTML(HTML: string);
    function GetContent: string;
    property InsertLinebreaks: boolean read FInsertLinebreaks write FInsertLineBreaks default false;
    property Lines: TStrings read FLines write SetLines;
    property PlainText: string read FPlainText write FPlainText;
    property ReadOnly: boolean read FReadOnly write FReadOnly default false;
    property SelAttributes: TTextAttributes read FSelAttributes write FSelAttributes;
    property ShowFocus: boolean read FShowFocus write FShowFocus default true;
    property WantTabs: boolean read FWantTabs write FWantTabs default false;
    property OnChange: TNotifyEvent read FOnChange write FOnChange;
    property OnSelectionChange: TNotifyEvent read FOnSelectionChange write FOnSelectionChange;
  end;

  {$IFNDEF LCLTMSWEB}
  [ComponentPlatforms(TMSWebPlatform)]
  {$ENDIF}
  TWebRichEdit = class(TWebCustomRichEdit)
  private
    FEventStopPropagation: TEventPropagation;
  public
    property EventStopPropagation: TEventPropagation read FEventStopPropagation write FEventStopPropagation;
    property PlainText;
  published
    property Align;
    property Anchors;
    property AutoSize;
    property BorderStyle;
    property ChildOrder;
    property Color;
    property DragMode;
    property Font;
    property InsertLinebreaks;
    property Lines;
    property ReadOnly;
    property Role;
    property ShowHint;
    property Visible;
    property WantTabs;
    property OnChange;
    property OnClick;
    property OnDblClick;
    property OnSelectionChange;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;
    property OnMouseDown;
    property OnMouseUp;
    property OnMouseMove;
    property OnMouseLeave;
    property OnMouseEnter;
    property OnEnter;
    property OnExit;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnStartDrag;
  end;

  TWebCustomTabSet = class(TWebCustomControl)
  strict private
    FItemIndex: Integer;
    FItems: TStringList;
    FOnChange: TNotifyEvent;
    FSelectedColor: TColor;
  private
    FSelectedTextColor: TColor;
    procedure SetItems(const Value: TStringList);
    procedure SetItemIndex(const Value: Integer);
  protected
    procedure ItemsChanged(Sender: TObject);
    procedure Paint; override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure Clear;
    procedure SelectNextTab(GoForward: boolean);
  published
    property Align;
    {$IFNDEF LCLTMSWEB}
    property AlignWithMargins;
    {$ENDIF}
    property ChildOrder;
    property Color;
    property DragMode;
    property ElementClassName;
    property ElementID;
    property Enabled;
    property Font;
    property Height;
    property Hint;
    property ItemIndex: Integer read FItemIndex write SetItemIndex;
    property Items: TStringList read FItems write SetItems;
    property Left;
    property ParentFont;
    property Role;
    property SelectedColor: TColor read FSelectedColor write FSelectedColor default clSilver;
    property SelectedTextColor: TColor read FSelectedTextColor write FSelectedTextColor default clBlack;
    property ShowHint;
    property TabOrder;
    property TabStop;
    property Top;
    property Visible;
    property Width;
    property OnChange: TNotifyEvent read FOnChange write FOnChange;
    property OnClick;
    property OnDblClick;
    property OnEnter;
    property OnExit;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;
    property OnMouseDown;
    property OnMouseEnter;
    property OnMouseLeave;
    property OnMouseMove;
    property OnMouseUp;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnStartDrag;
  end;

  {$IFNDEF LCLTMSWEB}
  [ComponentPlatforms(TMSWebPlatform)]
  {$ENDIF}
  TWebTabSet = class(TWebCustomTabSet)
  end;

  {$IFNDEF LCLTMSWEB}
  [ComponentPlatforms(TMSWebPlatform)]
  {$ENDIF}
  TWebTabSheet = class(TWebCustomControl)
  private
    FCaption: string;
    FColor: TColor;
    FTabVisible: Boolean;
    FElementBodyClassName: TElementClassName;
    FShowCaption: boolean;
    FOnShow: TNotifyEvent;
    FOnHide: TNotifyEvent;
    procedure SetCaption(const Value: string);
    procedure SetTabVisible(const Value: Boolean);
  protected
    procedure Paint; override;
    procedure SetParent(AParent: TWinControl); override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  published
    property Align;
    property Anchors;
    property AutoSize;
    property Caption: string read FCaption write SetCaption;
    property ChildOrder;
    property Color: TColor read FColor write FColor default clWhite;
    property DragMode;
    property ElementBodyClassName: TElementClassName read FElementBodyClassName write FElementBodyClassName;
    property ElementFont;
    property ElementPosition;
    property Font;
    property ParentFont;
    property ShowCaption: boolean read FShowCaption write FShowCaption default True;
    property TabVisible: Boolean read FTabVisible write SetTabVisible default True;
    property Visible;
    property OnClick;
    property OnDblClick;
    property OnHide: TNotifyEvent read FOnHide write FOnHide;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnShow: TNotifyEvent read FOnShow write FOnShow;
    property OnStartDrag;
  end;

  {$IFNDEF LCLTMSWEB}
  [ComponentPlatforms(TMSWebPlatform)]
  {$ENDIF}
  TWebPageControl = class(TWebCustomControl)
  private
    FSelectedColor: TColor;
    FTabIndex: integer;
    FItems: TStringList;
    FOnChange: TNotifyEvent;
    FList: TList;
    FDesignTime: boolean;
    FSelectedTextColor: TColor;
    FShowTabs: boolean;
    procedure CMDesignHitTest(var Message: TCMDesignHitTest); message CM_DESIGNHITTEST;
    procedure SetItems(const Value: TStringList);
    function GetPage(index: integer): TWebTabSheet;
    function GetBorderStyle: TBorderStyle;
    procedure SetBorderStyleEx(const Value: TBorderStyle);
    function GetActivePage: TWebTabSheet;
    function GetActivePageIndex: integer;
  protected
    procedure Loaded; override;
    procedure ItemsChanged(Sender: TObject);
    procedure SetTabIndex(const Value: integer);
    procedure AlignControls(AControl: TControl; var ARect: TRect); override;
    procedure Paint; override;
    procedure AddSheet(ASheet: TWebTabSheet);
    procedure RemoveSheet(ASheet: TWebTabSheet);
    procedure CreateWnd; override;
    procedure Notification(AComponent: TComponent; AOperation: TOperation); override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure GetChildren(Proc: TGetChildProc; Root: TComponent); override;
    property Items: TStringList read FItems write SetItems;
    procedure SelectNextPage(GoForward: boolean);
    function PageCount: integer;
    function PageIndexFromElement(AElement: TJSElement): Integer;
    property Pages[index: integer]: TWebTabSheet read GetPage;
    property ActivePage: TWebTabSheet read GetActivePage;
    property ActivePageIndex: integer read GetActivePageIndex;
  published
    property Align;
    {$IFNDEF LCLTMSWEB}
    property AlignWithMargins;
    {$ENDIF}
    property Anchors;
    property BorderStyle: TBorderStyle read GetBorderStyle write SetBorderStyleEx default bsNone;
    property ChildOrder;
    property DragMode;
    property ElementClassName;
    property ElementFont;
    property ElementID;
    property Enabled;
    property Font;
    property Height;
    property Hint;
    property TabIndex: integer read FTabIndex write SetTabIndex default -1;
    property Left;
    property ParentFont;
    property Role;
    property ShowHint;
    property ShowTabs: boolean read FShowTabs write FShowTabs default true;
    property SelectedColor: TColor read FSelectedColor write FSelectedColor default clSilver;
    property SelectedTextColor: TColor read FSelectedTextColor write FSelectedTextColor default clBlack;
    property TabOrder;
    property TabStop;
    property Top;
    property Visible;
    property Width;
    property OnChange: TNotifyEvent read FOnChange write FOnChange;
    property OnClick;
    property OnDblClick;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;
    property OnMouseDown;
    property OnMouseUp;
    property OnMouseMove;
    property OnMouseLeave;
    property OnMouseEnter;
    property OnEnter;
    property OnExit;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnStartDrag;
  end;

  TProgressBarStyle = (pbstNormal, pbstMarquee, pbstDiv);
  TProgressBarValue = (pbvNone, pbvPercentage, pbvAbsolute);

  {$IFNDEF LCLTMSWEB}
  [ComponentPlatforms(TMSWebPlatform)]
  {$ENDIF}
  TWebProgressBar = class(TWebCustomControl)
  private
    FMax: integer;
    FMin: integer;
    FPosition: integer;
    FStyle: TProgressBarStyle;
    FElementBarClassName: string;
    FValue: TProgressBarValue;
    procedure SetMax(const Value: integer);
    procedure SetMin(const Value: integer);
    procedure SetPosition(const Value: integer);
  protected
    function GetDesignText: string; override;
    procedure Paint; override;
    procedure InitBootstrapStyle; override;
  public
    constructor Create(AOwner: TComponent); override;
  published
    property Align;
    {$IFNDEF LCLTMSWEB}
    property AlignWithMargins;
    {$ENDIF}
    property Anchors;
    property ChildOrder;
    property DragMode;
    property ElementBarClassName: string read FElementBarClassName write FElementBarClassName;
    property ElementClassName;
    property ElementPosition;
    property ElementID;
    property Enabled;
    property Height;
    property Hint;
    property Left;
    property Min: integer read FMin write SetMin default 0;
    property Max: integer read FMax write SetMax default 100;
    property Position: integer read FPosition write SetPosition default 0;
    property Role;
    property Style: TProgressBarStyle read FStyle write FStyle default pbstNormal;
    property Top;
    property Value: TProgressBarValue read FValue write FValue default pbvNone;
    property Visible;
    property Width;
    property OnClick;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnStartDrag;
  end;

  TTreeViewNodeEvent = procedure(Sender: TObject; ANode: TTreeNode) of object;
  TTreeViewNodeAllowEvent = procedure(Sender: TObject; ANode: TTreeNode; var Allow: boolean) of object;

  TTreeViewNodeRenderEvent = procedure(Sender: TObject; Node: TTreeNode; AElement: TJSHTMLElementRecord) of object;

  TTreeNodeHelper = class helper for TTreeNode
    function AddChild(const S: string; Data: TObject): TTreeNode; overload;
    function AddFirstChild(const S: string; Data: TObject): TTreeNode; overload;
  end;

  TTreeNodesHelper = class helper for TTreeNodes
    function Add(const S: string): TTreeNode; overload;
    function AddChild(Parent: TTreeNode; const S: string; Data: TObject): TTreeNode; overload;
    function AddChildFirst(Parent: TTreeNode; const S: string; Data: TObject): TTreeNode; overload;
  end;

  {$IFNDEF LCLTMSWEB}
  [ComponentPlatforms(TMSWebPlatform)]
  {$ENDIF}
  TWebTreeView = class(TCustomTreeView)
  private
    FElementID: TElementID;
    FElementClassName: TElementClassName;
    FOnClickNode: TTreeViewNodeEvent;
    FOnDblClickNode: TTreeViewNodeEvent;
    FOnRenderNode: TTreeViewNodeRenderEvent;
    FAutoExpand: boolean;
    FWidthStyle: TSizeStyle;
    FHeightPercent: TPercentSize;
    FHeightStyle: TSizeStyle;
    FWidthPercent: TPercentSize;
    FSelected: TTreeNode;
    FChildOrder: integer;
    FRole: string;
    FEventStopPropagation: TEventPropagation;
    FElementNodeSelectedClassName: TElementClassName;
    FElementNodeClassName: TElementClassName;
    FElementFont: TElementFont;
    FElementPosition: TElementPosition;
    procedure SetHeightPercent(const Value: TPercentSize);
    procedure SetHeightStyle(const Value: TSizeStyle);
    procedure SetWidthPercent(const Value: TPercentSize);
    procedure SetWidthStyle(const Value: TSizeStyle);
  public
    constructor Create(AOwner: TComponent); override;
    property Items;
    procedure BeginUpdate;
    procedure EndUpdate;
    property EventStopPropagation: TEventPropagation read FEventStopPropagation write  FEventStopPropagation;
    function GetNodeFromID(AId: string): TTreeNode;
    function GetNodeElement(ANode: TTreeNode): TJSHTMLElement;
    property Selected: TTreeNode read FSelected write FSelected;
  published
    property Align;
    property Anchors;
    {$IFNDEF LCLTMSWEB}
    property AlignWithMargins;
    {$ENDIF}
    property AutoExpand: boolean read FAutoExpand write FAutoExpand default false;
    property ChildOrder: integer read FChildOrder write FChildOrder default 0;
    property Color;
    property DragMode;
    property ElementClassName: TElementClassName read FElementClassName write FElementClassName;
    property ElementNodeClassName: TElementClassName read FElementNodeClassName write FElementNodeClassName;
    property ElementNodeSelectedClassName: TElementClassName read FElementNodeSelectedClassName write FElementNodeSelectedClassName;

    property ElementPosition: TElementPosition read FElementPosition write FElementPosition default epAbsolute;
    property ElementFont: TElementFont read FElementFont write FElementFont default efProperty;
    property ElementID: TElementID read FElementID write FElementID;
    property Enabled;
    property Font;

    property HeightStyle: TSizeStyle read FHeightStyle write SetHeightStyle default ssAbsolute;
    property WidthStyle: TSizeStyle read FWidthStyle write SetWidthStyle default ssAbsolute;
    property HeightPercent: TPercentSize read FHeightPercent write SetHeightPercent;
    property WidthPercent: TPercentSize read FWidthPercent write SetWidthPercent;

    property ParentFont;
    property Role: string read FRole write FRole;
    property ShowHint;
    property Visible;
    property OnChange;
    property OnChanging;

    property OnClick;
    property OnCollapsed;
    property OnCollapsing;
    property OnDblClick;
    property OnClickNode: TTreeViewNodeEvent read FOnClickNode write FOnClickNode;
    property OnDblClickNode: TTreeViewNodeEvent read FOnDblClickNode write FOnDblClickNode;
    property OnEnter;
    property OnExit;
    property OnExpanded;
    property OnExpanding;
    property OnKeyDown;
    property OnKeyUp;
    property OnKeyPress;
    property OnMouseMove;
    property OnMouseDown;
    property OnMouseUp;
    property OnMouseLeave;
    property OnMouseEnter;
    property OnRenderNode: TTreeViewNodeRenderEvent read FOnRenderNode write FOnRenderNode;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnStartDrag;
  end;

implementation

{$IFNDEF LCLTMSWEB}
uses
  System.UITypes;
{$ENDIF}

{ TTextAttributes }

constructor TTextAttributes.Create(AOwner: TWebCustomRichEdit);
begin
  inherited Create;
end;

procedure TTextAttributes.Assign(Source: TPersistent);
begin
end;

procedure TTextAttributes.Copy;
begin
end;

procedure TTextAttributes.Cut;
begin
end;

procedure TTextAttributes.Image;
begin
end;

procedure TTextAttributes.Link;
begin
end;

procedure TTextAttributes.Paste;
begin
end;

{ TWebCustomRichEdit }

constructor TWebCustomRichEdit.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FLines := TStringList.Create;
  Width := 400;
  Height := 300;
  FShowFocus := true;
end;

destructor TWebCustomRichEdit.Destroy;
begin
  FLines.Free;
  inherited;
end;

procedure TWebCustomRichEdit.DoImageAction;
begin
end;

procedure TWebCustomRichEdit.DoLinkAction;
begin
end;

procedure TWebCustomRichEdit.DoSelectAction(StartPosition, EndPosition: Integer);
begin
end;

function TWebCustomRichEdit.GetContent: string;
begin
  Result := '';
end;

procedure TWebCustomRichEdit.InsertHTML(HTML: string);
begin
  //
end;

procedure TWebCustomRichEdit.SetLines(const Value: TStrings);
begin
  FLines.Assign(Value);
end;


{ TWebCustomTabSet }

constructor TWebCustomTabSet.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FItems := TStringList.Create;
  TStringList(FItems).OnChange := ItemsChanged;

  FItemIndex := 0;
  FSelectedColor := clSilver;
  FSelectedTextColor := clBlack;
end;

destructor TWebCustomTabSet.Destroy;
begin
  FItems.Free;
  inherited Destroy;
end;

procedure TWebCustomTabSet.ItemsChanged(Sender: TObject);
begin
  Invalidate;
end;

procedure TWebCustomTabSet.Paint;
var
  i,x,w: integer;
  s: string;
  r: TRect;
begin

  x := 0;

  Canvas.Brush.Color := Color;
  Canvas.Brush.Style := bsSolid;
  Canvas.Pen.Color := clBlack;
  Canvas.Pen.Width := 1;
  Canvas.Pen.Style := psSolid;

  Canvas.Rectangle(ClientRect);
  Canvas.Font.Assign(Font);

  for i := 0 to Items.Count - 1 do
  begin
    s := Items[i];

    w := Canvas.TextWidth(s);

    if i = ItemIndex then
    begin
      Canvas.Brush.Color := clSilver;
      Canvas.Brush.Style := bsSolid;
      Canvas.Pen.Color := clSilver;
      r := Rect(x,0,x + w + 8 + 1, Height);
      InflateRect(r, -1, -1);
      Canvas.Rectangle(r);
    end;

    Canvas.Brush.Style := bsClear;
    Canvas.TextOut(x + 4, 4, s);

    Canvas.Pen.Color := clBlack;

    x := x + w + 8;

    if i < Items.Count - 1 then
    begin
      Canvas.MoveTo(x,0);
      Canvas.LineTo(x,Height);
    end;
  end;
end;

procedure TWebCustomTabSet.SetItemIndex(const Value: Integer);
begin
  FItemIndex := Value;
  if HandleAllocated then
    Invalidate;
end;

procedure TWebCustomTabSet.SetItems(const Value: TStringList);
begin
  FItems.Assign(Value);
end;

procedure TWebCustomTabSet.SelectNextTab(GoForward: boolean);
begin
  if GoForward then
  begin
    if ItemIndex < Items.Count - 1 then
      ItemIndex := ItemIndex + 1
    else
      ItemIndex := 0;
  end
  else
  begin
    if ItemIndex > 0 then
      ItemIndex := ItemIndex - 1
    else
      ItemIndex := Items.Count - 1;
  end;
end;


procedure TWebCustomTabSet.Clear;
begin
end;


{ TWebTabSheet }

constructor TWebTabSheet.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  ControlStyle := ControlStyle + [csAcceptsControls];
  Color := clWhite;
  FShowCaption := True;
  FTabVisible := True;
end;

destructor TWebTabSheet.Destroy;
begin
  if Assigned(Parent) and (Parent is TWebPageControl) then
    (Parent as TWebPageControl).RemoveSheet(Self);

  inherited;
end;

procedure TWebTabSheet.Paint;
var
  lRect: TRect;
begin
  if Assigned(Parent) and (Parent is TWebPageControl) then
  begin
    if (Parent as TWebPageControl).BorderStyle = bsSingle then
      Canvas.Pen.Color := BorderColor
    else
      Canvas.Pen.Color := Canvas.Brush.Color;

    lRect := ClientRect;
    Canvas.Rectangle(lRect);
  end;
end;

procedure TWebTabSheet.SetCaption(const Value: string);
begin
  FCaption := Value;
  if Assigned(Parent) then
    Parent.Invalidate;
end;

procedure TWebTabSheet.SetParent(AParent: TWinControl);
begin
  inherited;

  if Assigned(AParent) and (AParent is TWebPageControl) then
    (AParent as TWebPageControl).AddSheet(Self);
end;

procedure TWebTabSheet.SetTabVisible(const Value: Boolean);
begin
  FTabVisible := Value;
  if Assigned(Parent) then
    Parent.Invalidate;
end;

{ TWebPageControl }

procedure TWebPageControl.AddSheet(ASheet: TWebTabSheet);
var
  i: integer;
  found: boolean;
begin
  found := false;
  for i := 0 to FList.Count - 1 do
  begin
    if TWebTabSheet(FList.Items[i]) = ASheet then
      found := true;
  end;

  if not found then
    FList.Add(ASheet);

  Invalidate;
end;

procedure TWebPageControl.AlignControls(AControl: TControl; var ARect: TRect);
var
  i: integer;
begin
  inherited;

  for I := 0 to ControlCount - 1 do
  begin
    if Controls[i] is TWebTabSheet then
    begin
      Controls[i].SetBounds(0, 20, Width, Height - 20);
    end;
  end;
end;

procedure TWebPageControl.CMDesignHitTest(var Message: TCMDesignHitTest);
var
  p: TPoint;
  Tab, i, x, bw: integer;
begin
  Tab := -1;

  if (csDesigning in ComponentState) then
  begin
    GetCursorPos(P);
    P := ScreenToClient(P);
    x := 0;

    if (p.Y < 20)
      and ({$IFDEF LCLTMSWEB}GetKeyState{$ELSE}GetAsyncKeyState{$ENDIF}(VK_LBUTTON) <> 0) then
    begin
      for i := 0 to FList.Count - 1 do
      begin
        bw := Canvas.TextWidth(TWebTabSheet(FList.Items[i]).Caption) + 5;

        if (p.X > x) and (p.X < x + BW) then
        begin
          TabIndex := i;
          break;
        end;
        x := x + BW;
      end;
    end;
  end;

  if Tab = -1 then
    inherited;
end;

constructor TWebPageControl.Create(AOwner: TComponent);
begin
  inherited;
  ControlStyle := ControlStyle + [csAcceptsControls];
  FItems := TStringList.Create;
  FItems.OnChange := ItemsChanged;
  SelectedColor := clSilver;
  SelectedTextColor := clBlack;
  BorderStyle := bsNone;
  FList := TList.Create;
  FShowTabs := true;
  Width := 400;
  Height := 300;
  FTabIndex := -1;
  FDesignTime := (csDesigning in ComponentState) and not
                 ((csReading in Owner.ComponentState) or (csLoading in Owner.ComponentState));
end;

procedure TWebPageControl.CreateWnd;
var
  TS: TWebTabSheet;
begin
  inherited;

  if FDesignTime then
  begin
    if ControlCount = 0 then
    begin
      FDesignTime := false;
      TS := TWebTabSheet.Create(Owner);
      TS.Parent := Self;
      TS.Caption := Name + 'Sheet1';
      TS.Name := TS.Caption;

      TS := TWebTabSheet.Create(Owner);
      TS.Parent := Self;
      TS.Caption := Name + 'Sheet2';
      TS.Name := TS.Caption;

      TS := TWebTabSheet.Create(Owner);
      TS.Parent := Self;
      TS.Caption := Name + 'Sheet3';
      TS.Name := TS.Caption;
    end;
  end;
end;

destructor TWebPageControl.Destroy;
begin
  FList.Free;
  FItems.Free;
  inherited;
end;

function TWebPageControl.GetActivePage: TWebTabSheet;
begin
  Result := nil;
end;

function TWebPageControl.GetActivePageIndex: integer;
begin
  Result := -1;
end;

function TWebPageControl.GetBorderStyle: TBorderStyle;
begin
  Result := inherited BorderStyle;
end;

procedure TWebPageControl.GetChildren(Proc: TGetChildProc; Root: TComponent);
var
  I: Integer;
  Control: TControl;
begin
  for I := 0 to FList.Count - 1 do Proc(TComponent(FList[I]));

  for I := 0 to ControlCount - 1 do
  begin
    Control := Controls[I];
    if (Control.Owner = Root) and (FList.IndexOf(Control) < 0) then Proc(Control);
  end;
end;

function TWebPageControl.GetPage(index: integer): TWebTabSheet;
begin
  Result := nil;

  if (index >= 0) and (index < FList.Count) then
    Result := TWebTabSheet(FList.Items[index]);
end;

procedure TWebPageControl.ItemsChanged(Sender: TObject);
begin
  if Assigned(Onchange) then
    OnChange(Self);
end;

procedure TWebPageControl.Loaded;
begin
  inherited Loaded;
  if TabIndex < 0 then
    TabIndex := 0;
end;

procedure TWebPageControl.Notification(AComponent: TComponent;
  AOperation: TOperation);
var
  i: integer;
begin
  inherited;
  if (AOperation = opRemove) and not (csDestroying in ComponentState) then
  begin
    for i := 0 to FList.Count - 1 do
    begin
      {$IFNDEF LCLTMSWEB}
      if FList.Items[i] = AComponent then
      {$ENDIF}
      {$IFDEF LCLTMSWEB}
      if FList.Items[i] = @AComponent then
      {$ENDIF}
      begin
        FList.Delete(i);
        Invalidate;
        break;
      end;
    end;
  end;
end;

function TWebPageControl.PageCount: integer;
begin
  Result := FList.Count;
end;

function TWebPageControl.PageIndexFromElement(AElement: TJSElement): Integer;
begin
  Result := -1;
end;

procedure TWebPageControl.Paint;
var
  i: integer;
  x: integer;
  s: string;
  bw: integer;
  TS: TWebTabSheet;
begin
  Canvas.Brush.Color := clBtnFace;
  Canvas.Brush.Style := bsSolid;
  Canvas.Pen.Color := clBtnFace;
  Canvas.Pen.Style := psSolid;
  Canvas.Rectangle(ClientRect);

  x := 2;
  Canvas.Brush.Style := bsClear;

  Canvas.Font.Assign(Font);
  for i := 0 to FList.Count - 1 do
  begin
    TS := TWebTabSheet(FList.Items[i]);
    s := TS.Caption;
    bw := Canvas.TextWidth(s);
    if i = TabIndex then
    begin
      Canvas.Brush.Color := clWhite;
      Canvas.Brush.Style := bsSolid;
      Canvas.Pen.Color := clWhite;
      Canvas.Pen.Style := psSolid;
      Canvas.Rectangle(x, 0, x + bw + 4, 20);
    end;

    Canvas.Brush.Style := bsClear;
    Canvas.TextOut(x,2, s);
    x := x + bw + 10;
  end;
end;

procedure TWebPageControl.RemoveSheet(ASheet: TWebTabSheet);
var
  i: integer;
begin
  for i := 0 to FList.Count - 1 do
  begin
    if TWebTabSheet(FList.Items[i]) = ASheet then
    begin
      FList.Delete(i);
      Break;
    end;
  end;

  if FList.Count > 0 then
    TabIndex := 0
  else
    TabIndex := -1;

  Invalidate;
end;

procedure TWebPageControl.SelectNextPage(GoForward: boolean);
begin
  if GoForward then
  begin
    if TabIndex < PageCount - 1 then
      TabIndex := TabIndex + 1
    else
      TabIndex := 0;
  end
  else
  begin
    if TabIndex > 0 then
      TabIndex := TabIndex - 1
    else
      TabIndex := PageCount - 1;
  end;
end;

procedure TWebPageControl.SetBorderStyleEx(const Value: TBorderStyle);
begin
  inherited BorderStyle := Value;
end;

procedure TWebPageControl.SetItems(const Value: TStringList);
begin
  FItems.Assign(Value);
end;

procedure TWebPageControl.SetTabIndex(const Value: integer);
var
  TS: TWebTabSheet;
begin
  if (Value >= 0) and (Value < FList.Count) then
  begin
    TS := TWebTabSheet(FList.Items[Value]);
    TS.BringToFront;
    FTabIndex := Value;
    Invalidate;
  end;
end;


{ TWebProgressBar }

constructor TWebProgressBar.Create(AOwner: TComponent);
begin
  FStyle := pbstNormal;
  inherited;
  FMin := 0;
  FMax := 100;
  FPosition := 0;
  Width := 300;
  Height := 24;
end;


function TWebProgressBar.GetDesignText: string;
begin
  Result := '';
end;

procedure TWebProgressBar.InitBootstrapStyle;
begin
  inherited;
  ElementFont := efCSS;
  ElementClassName := 'progress';
  ElementBarClassName := 'progress-bar';
  Style := pbstDiv;
end;

procedure TWebProgressBar.Paint;
var
  dx: integer;
begin
  inherited;

  if (FMax - FMin > 0) then
  begin
    dx := Round(Width * FPosition / (FMax - FMin));

    Canvas.Brush.Color := clHighlight;
    Canvas.Brush.Style := bsSolid;
    Canvas.Pen.Color := clHighlight;
    Canvas.Pen.Style := psSolid;
    Canvas.Rectangle(0,0,dx,Height);
  end;
end;

procedure TWebProgressBar.SetMax(const Value: integer);
begin
  if (Value >= 0) and (Value >= FMin) then
  begin
    FMax := Value;
    Invalidate;
  end;
end;

procedure TWebProgressBar.SetMin(const Value: integer);
begin
  if (FMin >= 0) and (FMin <= FMax) then
  begin
    FMin := Value;
    Invalidate;
  end;
end;

procedure TWebProgressBar.SetPosition(const Value: integer);
begin
  if (FPosition >= FMin) and (FPosition <= FMax) then
  begin
    FPosition := Value;
    Invalidate;
  end;
end;

{ TWebTreeView }

procedure TWebTreeView.BeginUpdate;
begin
  //
end;

constructor TWebTreeView.Create(AOwner: TComponent);
begin
  inherited;
  FWidthStyle := ssAbsolute;
  FWidthPercent := 100;
  FHeightStyle := ssAbsolute;
  FHeightPercent := 100;
  FElementPosition := epABsolute;
  FElementFont := efProperty;
end;

procedure TWebTreeView.EndUpdate;
begin
  //
end;

function TWebTreeView.GetNodeElement(ANode: TTreeNode): TJSHTMLElement;
begin
  Result := nil;
end;

function TWebTreeView.GetNodeFromID(AId: string): TTreeNode;
begin
  Result := nil;
end;

procedure TWebTreeView.SetHeightPercent(const Value: TPercentSize);
begin
  FHeightPercent := Value;
end;

procedure TWebTreeView.SetHeightStyle(const Value: TSizeStyle);
begin
  FHeightStyle := Value;
end;

procedure TWebTreeView.SetWidthPercent(const Value: TPercentSize);
begin
  FWidthPercent := Value;
end;

procedure TWebTreeView.SetWidthStyle(const Value: TSizeStyle);
begin
  FWidthStyle := Value;
end;

{ TTreeNodesHelper }

function TTreeNodesHelper.Add(const S: string): TTreeNode;
begin
  Result := nil;
end;

function TTreeNodesHelper.AddChild(Parent: TTreeNode; const S: string; Data: TObject): TTreeNode;
begin
  Result := nil;
end;

function TTreeNodesHelper.AddChildFirst(Parent: TTreeNode; const S: string; Data: TObject): TTreeNode;
begin
  Result := nil;
end;

function TTreeNodeHelper.AddChild(const S: string; Data: TObject): TTreeNode;
begin
  Result := nil;
end;

function TTreeNodeHelper.AddFirstChild(const S: string; Data: TObject): TTreeNode;
begin
  Result := nil;
end;

end.
