{********************************************************************}
{                                                                    }
{ written by TMS Software                                            }
{            copyright (c) 2019 - 2020                               }
{            Email : info@tmssoftware.com                            }
{            Web : http://www.tmssoftware.com                        }
{                                                                    }
{ The source code is given as is. The author is not responsible      }
{ for any possible damage done due to the use of this code.          }
{ The complete source code remains property of the author and may    }
{ not be distributed, published, given or sold in any form as such.  }
{ No parts of the source code can be included in any other component }
{ or application without written authorization of the author.        }
{********************************************************************}

unit WEBLib.FirestoreCDS;

interface

uses
  {$IFNDEF LCLTMSWEB}
  System.Classes, DB, DBClient, WEBLib.Controls, Web, jsdelphisystem, js;
  {$ENDIF}
  {$IFDEF LCLTMSWEB}
  Classes, DB, WEBLib.Controls, Web, jsdelphisystem;
  {$ENDIF}

type
  TFireStoreRangeFilterType =  (frStartAt, frEndAt, frStartAfter, frEndBefore);
  TFirestoreErrorEvent = procedure(Sender: TObject; errorName, errorMsg: String) of object;
  TFirestoreSignInChangeEvent = procedure(isSignIn: Boolean; UserName: String; UserEmail: String) of object;

  {$IFNDEF LCLTMSWEB}
  [ComponentPlatforms(TMSWebPlatform)]
  [JSLibReference('https://www.gstatic.com/firebasejs/6.3/firebase-app.js;https://www.gstatic.com/firebasejs/6.3/firebase-auth.js;https://www.gstatic.com/firebasejs/6.3/firebase-firestore.js')]
  {$ENDIF}
  TWebFirestoreClientDataset = Class(TDataSet)
  private
    FApiKey, FCollectionName, FAuthDomain, FProjectId: String;
    FKeyFieldName: String;
    FSignInRequired, FAutoGenerateKeys: Boolean;
    FOnError: TFirestoreErrorEvent;
    FOnSignInChange: TFirestoreSignInChangeEvent;
    FParams: TParams;
    procedure SetParams(const Value: TParams);
  protected
    procedure InternalClose; override;
    procedure InternalHandleException; override;
    procedure InternalInitFieldDefs; override;
    procedure InternalOpen; override;
    function IsCursorOpen: Boolean; override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure Refresh(Force: Boolean=False); reintroduce;

    // Query related
    procedure AddSortFieldDef(aField: String; isAscending: Boolean);
    procedure ClearSortFieldDefs;

    // Filters
    // Important! The following work on current sorted order
    // The results may be different or even not allowed based 
	// on the field you specified in AddSortFieldDef above
    procedure AddServiceFilterCondition(aField: String; anOperator: String; aValue: JSValue);

    // The value refers to the value of active sort field specified
	// by AddSortFieldDef above
    procedure AddServiceRangeFilter(aRangeFilterType: TFireStoreRangeFilterType; aValue: JSValue);

    procedure AddServiceFilterCount(numRecords: Integer);
    procedure AddServiceFilterContinueAfterLast;
    procedure ClearServiceFilters;

    procedure SubscribeToSignInEvents(doSubscribe: Boolean);
  published
    property ApiKey: String read FApiKey write FApiKey;
    property CollectionName: String read FCollectionName write FCollectionName;
    property KeyFieldName: String read FKeyFieldName write FKeyFieldName;
    property AuthDomain: String read FAuthDomain write FAuthDomain;
    property Params: TParams read FParams write SetParams;
    property ProjectId: String read FProjectId write FProjectId;
    property SignInRequired: Boolean read FSignInRequired write FSignInRequired;
    property AutoGenerateKeys: Boolean read FAutoGenerateKeys write FAutoGenerateKeys;
    property OnError: TFirestoreErrorEvent read FOnError write FOnError;
    property OnSignInChange: TFirestoreSignInChangeEvent read FOnSignInChange write FOnSignInChange;

    // Other
    property Active;
    property BeforeOpen;
    property AfterOpen;
    property BeforeClose;
    property AfterClose;
    property BeforeInsert;
    property AfterInsert;
    property BeforeEdit;
    property AfterEdit;
    property BeforePost;
    property AfterPost;
    property BeforeCancel;
    property AfterCancel;
    property BeforeDelete;
    property AfterDelete;
    property BeforeScroll;
    property AfterScroll;
    property OnCalcFields;
    property OnDeleteError;
    property OnEditError;
    property OnFilterRecord;
    property OnNewRecord;
    property OnPostError;
  end;

implementation

procedure TWebFirestoreClientDataset.InternalClose;
begin
  //
end;

procedure TWebFirestoreClientDataset.InternalHandleException;
begin
  //
end;

procedure TWebFirestoreClientDataset.InternalInitFieldDefs;
begin
  //
end;

procedure TWebFirestoreClientDataset.InternalOpen;
begin
  //
end;

function TWebFirestoreClientDataset.IsCursorOpen: Boolean;
begin
  Result := false;
end;

constructor TWebFirestoreClientDataset.Create(AOwner: TComponent);
begin
  inherited;
  FApiKey := '';
  FCollectionName := ''; 
  FAuthDomain := '';
  FProjectId := '';
  FSignInRequired := True;
  FKeyFieldName := '';
  FAutoGenerateKeys := True;
  FParams := TParams.Create(Self);
end;

destructor TWebFirestoreClientDataset.Destroy;
begin
  FParams.Free;
  inherited;
end;

procedure TWebFirestoreClientDataset.Refresh;
begin
end;

procedure TWebFirestoreClientDataset.AddSortFieldDef(aField: String; isAscending: Boolean);
begin
end;

procedure TWebFirestoreClientDataset.ClearSortFieldDefs;
begin
end;

procedure TWebFirestoreClientDataset.SetParams(const Value: TParams);
begin
  FParams.Assign(Value);
end;

procedure TWebFirestoreClientDataset.AddServiceFilterCondition(aField: String; anOperator: String; aValue: JSValue);
begin
end;

procedure TWebFirestoreClientDataset.AddServiceRangeFilter(aRangeFilterType: TFireStoreRangeFilterType; aValue: JSValue);
begin
end;

procedure TWebFirestoreClientDataset.AddServiceFilterCount(numRecords: Integer);
begin
end;

procedure TWebFirestoreClientDataset.AddServiceFilterContinueAfterLast;
begin
end;

procedure TWebFirestoreClientDataset.ClearServiceFilters;
begin
end;

procedure TWebFirestoreClientDataset.SubscribeToSignInEvents(doSubscribe: Boolean);
begin
end;

end.
