{********************************************************************}
{                                                                    }
{ written by TMS Software                                            }
{            Copyright (c) 2018 - 2021                               }
{            Email : info@tmssoftware.com                            }
{            Web : http://www.tmssoftware.com                        }
{                                                                    }
{ The source code is given as is. The author is not responsible      }
{ for any possible damage done due to the use of this code.          }
{ The complete source code remains property of the author and may    }
{ not be distributed, published, given or sold in any form as such.  }
{ No parts of the source code can be included in any other component }
{ or application without written authorization of the author.        }
{********************************************************************}

unit WEBLib.Grids;

interface

uses
  {$IFNDEF LCLTMSWEB}
  System.Classes, WinApi.Windows, Vcl.Graphics, Vcl.Grids, Vcl.Controls, Vcl.Forms,
  {$ENDIF}
  {$IFDEF LCLTMSWEB}
  Classes, Graphics, Grids, Controls, Forms,
  {$ENDIF}
  WEBLib.Controls, Web, JS, DB, jsDelphiSystem;


type
  TInitMethod = (imRandom, imLinear);

  TGridGetCellDataEvent = procedure(Sender: TObject; ACol,ARow: integer; AField: TField; var AValue: string) of object;

  TGridGetCellClassEvent = procedure(Sender: TObject; ACol,ARow: integer; AField: TField; AValue: string; var AClassName: string) of object;

  TGridGetCellChildrenEvent = procedure(Sender: TObject; ACol,ARow: integer; AField: TField; AValue: string; AElement: TJSHTMLElementRecord) of object;

  TGridRequestEvent = procedure(Sender: TObject; ARequest: TJSXMLHttpRequestRecord) of object;

  TCellCheckEvent = procedure(Sender: TObject; ACol,ARow: integer; Checked: boolean) of object;

  TCellCanEditEvent = procedure(Sender: TObject; ACol,ARow: integer; var CanEdit: boolean) of object;

  TColumnSizedEvent = procedure(Sender: TObject; ACol: integer) of object;

  {$IFNDEF LCLTMSWEB}
  [ComponentPlatforms(TMSWebPlatform)]
  {$ENDIF}
  TWebStringGrid = class(TStringGrid)
  private
    FOnHttpRequestError: TNotifyEvent;
    FOnHttpRequestSuccess: TNotifyEvent;
    FElementFont: TElementFont;
    FElementID: TElementID;
    FElementPosition: TElementPosition;
    FElementClassName: TElementClassName;
    FOnGetCellClass: TGridGetCellClassEvent;
    FOnGetCellData: TGridGetCellDataEvent;
    FOnGetCellChildren: TGridGetCellChildrenEvent;
    FParentElementID: TJSElement;
    FParentElement: TJSElement;
    FOnHttpRequest: TGridRequestEvent;
    FOnCheckClick: TCellCheckEvent;
    FWidthStyle: TSizeStyle;
    FHeightPercent: TPercentSize;
    FHeightStyle: TSizeStyle;
    FWidthPercent: TPercentSize;
    FChildOrder: integer;
    FOnCanEditCell: TCellCanEditEvent;
    FFixedTextColor: TColor;
    FOnColumnSized: TColumnSizedEvent;
    function GetCellElements(ACol, ARow: integer): TJSHTMLElement;
    procedure SetHeightPercent(const Value: TPercentSize);
    procedure SetHeightStyle(const Value: TSizeStyle);
    procedure SetWidthPercent(const Value: TPercentSize);
    procedure SetWidthStyle(const Value: TSizeStyle);
  public
    constructor Create(AOwner: TComponent); override;
    procedure LoadFromJSON(const AURL: string; ADataNode: string = ''); overload;
    procedure LoadFromJSON(AJSON: TJSObject; ADataNode: string = ''); overload;
    procedure LoadFromCSV(const AURL: string; Delimiter: char = ';'; LoadFixed: boolean = false);
    procedure SaveToCSV(const AFileName: string; Delimiter: char = ';'; SaveFixed: boolean = false);
    procedure InitSample(AInitMethod: TInitMethod);
    property ParentElement: TJSElement read FParentElement write FParentElement;
    property ParentElementID: TJSElement read FParentElementID write FParentElementID;
    property CellElements[ACol,ARow: integer]: TJSHTMLElement read GetCellElements;
    procedure InsertRow(Index: integer);
    procedure RemoveRow(Index: integer);
    procedure ShowEdit;
    procedure HideEdit;
  published
    property ChildOrder: integer read FChildOrder write FChildOrder default 0;
    property DragMode;
    property ElementClassName: TElementClassName read FElementClassName write FElementClassName;
    property ElementID: TElementID read FElementID write FElementID;
    property ElementFont: TElementFont read FElementFont write FElementFont default efProperty;
    property ElementPosition: TElementPosition read FElementPosition write FElementPosition default epAbsolute;
    property FixedTextColor: TColor read FFixedTextColor write FFixedTextColor default clBlack;
    property HeightStyle: TSizeStyle read FHeightStyle write SetHeightStyle default ssAbsolute;
    property WidthStyle: TSizeStyle read FWidthStyle write SetWidthStyle default ssAbsolute;
    property HeightPercent: TPercentSize read FHeightPercent write SetHeightPercent;
    property WidthPercent: TPercentSize read FWidthPercent write SetWidthPercent;
    property OnCanEditCell: TCellCanEditEvent read FOnCanEditCell write FOnCanEditCell;
    property OnCheckClick: TCellCheckEvent read FOnCheckClick write FOnCheckClick;
    property OnColumnSized: TColumnSizedEvent read FOnColumnSized write FOnColumnSized;
    property OnGetCellData: TGridGetCellDataEvent read FOnGetCellData write FOnGetCellData;
    property OnGetCellChildren: TGridGetCellChildrenEvent read FOnGetCellChildren write FOnGetCellChildren;
    property OnGetCellClass: TGridGetCellClassEvent read FOnGetCellClass write FOnGetCellClass;
    property OnHttpRequestError: TNotifyEvent read FOnHttpRequestError write FOnHttpRequestError;
    property OnHttpRequestSuccess: TNotifyEvent read FOnHttpRequestSuccess write FOnHttpRequestSuccess;
    property OnHttpRequest: TGridRequestEvent read FOnHttpRequest write FOnHttpRequest;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnStartDrag;
  end;

  TCellEvent = procedure(Sender: TObject; ACol, ARow: integer) of object;

  TTableControlOptions = class(TPersistent)
  private
    FScrollVertical: boolean;
    FResizeColumns: boolean;
  public
    procedure Assign(Source: TPersistent); override;
  published
    property ScrollVertical: boolean read FScrollVertical write FScrollVertical default false;
    property ResizeColumns: boolean read FResizeColumns write FResizeColumns default false;
  end;

  TWebCustomTableControl = class(TWebCustomControl)
  private
    FRowCount: integer;
    FColCount: integer;
    FRowHeader: boolean;
    FColHeader: boolean;
    FElementHeaderClassName: TElementClassName;
    FElementTableClassName: TElementClassName;
    FOnHttpRequestError: TNotifyEvent;
    FOnHttpRequestSuccess: TNotifyEvent;
    FOnDblClickCell: TCellEvent;
    FOnClickCell: TCellEvent;
    FOptions: TTableControlOptions;
    FOnGetCellClass: TGridGetCellClassEvent;
    FOnGetCellData: TGridGetCellDataEvent;
    FOnGetCellChildren: TGridGetCellChildrenEvent;
    FOnHttpRequest: TGridRequestEvent;
    FOnCheckClick: TCellCheckEvent;
    function GetCell(col, row: integer): string;
    procedure SetCell(col, row: integer; const Value: string);
    function GetRowClassName(row: integer): string;
    procedure SetRowClassName(row: integer; const Value: string);
    function GetCellElement(col, row: integer): TJSElement;
    procedure SetColCount(const Value: integer);
    procedure SetRowCount(const Value: integer);
    procedure SetOptions(const Value: TTableControlOptions);
  protected
    procedure Paint; override;
    property OnHttpRequestError: TNotifyEvent read FOnHttpRequestError write FOnHttpRequestError;
    property OnHttpRequestSuccess: TNotifyEvent read FOnHttpRequestSuccess write FOnHttpRequestSuccess;
    property OnHttpRequest: TGridRequestEvent read FOnHttpRequest write FOnHttpRequest;
    property ColCount: integer read FColCount write SetColCount default 5;
    property RowCount: integer read FRowCount write SetRowCount default 10;
    procedure InitBootstrapStyle; override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure LoadFromJSON(const AURL: string; ADataNode: string = '');
    procedure LoadFromCSV(const AURL: string; Delimiter: char = ';'; LoadFixed: boolean = false);
    property Cells[col,row: integer]: string read GetCell write SetCell;
    property CellElements[col,row: integer]: TJSElement read GetCellElement;
    property RowClassName[row: integer]: string read GetRowClassName write SetRowClassName;
    procedure InitSample(AInitMethod: TInitMethod);
  published
    property Align;
    {$IFNDEF LCLTMSWEB}
    property AlignWithMargins;
    {$ENDIF}
    property Anchors;
    property BiDiMode;
    property BorderStyle default bsSingle;
    property BorderColor;
    property ChildOrder;
    property Color default clWindow;
    property ColHeader: boolean read FColHeader write FColHeader default true;
    property DragMode;
    property ElementFont;
    property ElementHeaderClassName: TElementClassName read FElementHeaderClassName write FElementHeaderClassName;
    property ElementPosition;
    property ElementTableClassName: TElementClassName read FElementTableClassName write FElementTableClassName;
    property Font;
    property Hint;
    property Options: TTableControlOptions read FOptions write SetOptions;
    property RowHeader: boolean read FRowHeader write FRowHeader default true;
    property ShowHint;
    property Visible;
    property OnCheckClick: TCellCheckEvent read FOnCheckClick write FOnCheckClick;
    property OnClick;
    property OnClickCell: TCellEvent read FOnClickCell write FOnClickCell;
    property OnDblClickCell: TCellEvent read FOnDblClickCell write FOnDblClickCell;
    property OnDblClick;
    property OnGetCellData: TGridGetCellDataEvent read FOnGetCellData write FOnGetCellData;
    property OnGetCellChildren: TGridGetCellChildrenEvent read FOnGetCellChildren write FOnGetCellChildren;
    property OnGetCellClass: TGridGetCellClassEvent read FOnGetCellClass write FOnGetCellClass;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnStartDrag;
  end;

  {$IFNDEF LCLTMSWEB}
  [ComponentPlatforms(TMSWebPlatform)]
  {$ENDIF}
  TWebTableControl = class(TWebCustomTableControl)
  published
    property ColCount;
    property RowCount;
    property OnHttpRequestError;
    property OnHttpRequestSuccess;
    property OnHttpRequest;
  end;


  TWebResponsiveGridItem = class(TCollectionItem)
  private
    FHTML: string;
    FElementHandle: TJSHTMLElement;
    FItemObject: TObject;
    FTag: integer;
    FVisible: boolean;
    FJSONObject: JSValue;
    procedure SetHTML(const Value: string);
    function GetJSONElementValue(id: string): string;
  public
    constructor Create(ACollection: TCollection); override;
    procedure Assign(Source: TPersistent); override;
    property ElementHandle: TJSHTMLElement read FElementHandle;
    property ItemObject: TObject read FItemObject write FItemObject;
    property JSONElement: JSValue read FJSONObject write FJSONObject;
    property JSONElementValue[id: string]: string read GetJSONElementValue;
  published
    property HTML: string read FHTML write SetHTML;
    property Tag: integer read FTag write FTag default 0;
    property Visible: boolean read FVisible write FVisible default True;
  end;

  TResponsiveGridItem = class(TWebResponsiveGridItem);

  TWebResponsiveGridItems = class(TOwnedCollection)
  private
    function GetItem(Index: integer): TWebResponsiveGridItem;
    procedure SetItem(Index: integer; const Value: TWebResponsiveGridItem);
  public
    constructor Create(AOwner: TComponent);
    function Add: TWebResponsiveGridItem;
    function Insert(Index: integer): TWebResponsiveGridItem;
    property Items[Index: integer]: TWebResponsiveGridItem read GetItem write SetItem; default;
  end;

  TResponsiveGridItems = class(TWebResponsiveGridItems);

  TWebResponsiveGridOptions = class(TPersistent)
  private
    FItemWidthMin: integer;
    FItemHeight: integer;
    FItemGap: integer;
    FScrollVertical: boolean;
    FItemHoverColor: TColor;
    FItemBorderColor: TColor;
    FItemHoverBorderColor: TColor;
    FItemClassName: TElementClassName;
    FItemColor: TColor;
    FItemPadding: integer;
    FItemTemplate: string;
    FOnChange: TNotifyEvent;
    FItemSelectedBorderColor: TColor;
    FItemSelectedTextColor: TColor;
    FItemSelectedColor: TColor;
    procedure SetItemBorderColor(const Value: TColor);
    procedure SetItemColor(const Value: TColor);
    procedure SetItemGap(const Value: integer);
    procedure SetItemHeight(const Value: integer);
    procedure SetItemWidthMin(const Value: integer);
    procedure SetItemSelectedBorderColor(const Value: TColor);
    procedure SetItemSelectedColor(const Value: TColor);
    procedure SetItemSelectedTextColor(const Value: TColor);
    procedure SetItemTemplate(const Value: string);
  protected
    procedure Changed;
  public
    constructor Create;
    procedure Assign(Source: TPersistent); override;
    property OnChange: TNotifyEvent read FOnChange write FOnChange;
  published
    property ItemClassName: TElementClassName read FItemClassName write FItemClassName;
    property ItemColor: TColor read FItemColor write SetItemColor default clNone;
    property ItemBorderColor: TColor read FItemBorderColor write SetItemBorderColor default clNone;
    property ItemGap: integer read FItemGap write SetItemGap default 10;
    property ItemHeight: integer read FItemHeight write SetItemHeight default 100;
    property ItemHoverColor: TColor read FItemHoverColor write FItemHoverColor default clNone;
    property ItemHoverBorderColor: TColor read FItemHoverBorderColor write FItemHoverBorderColor default clNone;
    property ItemSelectedColor: TColor read FItemSelectedColor write SetItemSelectedColor default clNone;
    property ItemSelectedBorderColor: TColor read FItemSelectedBorderColor write SetItemSelectedBorderColor default clNone;
    property ItemSelectedTextColor: TColor read FItemSelectedTextColor write SetItemSelectedTextColor default clNone;
    property ItemPadding: integer read FItemPadding write FItemPadding default 0;
    property ItemTemplate: string read FItemTemplate write SetItemTemplate;
    property ItemWidthMin: integer read FItemWidthMin write SetItemWidthMin default 100;
    property ScrollVertical: boolean read FScrollVertical write FScrollVertical default true;
  end;

  TResponsiveGridOptions = class(TWebResponsiveGridOptions);

  TResponsiveGridItemEvent = procedure(Sender: TObject; Index: integer) of object;

  TResponsiveGridItemFieldValueEvent = procedure(Sender: TObject; Index: integer; AFieldName: string; var AValue: string) of object;

  {$IFNDEF LCLTMSWEB}
  [ComponentPlatforms(TMSWebPlatform)]
  {$ENDIF}
  TWebResponsiveGrid = class(TWebCustomControl)
  private
    FItemIndex: integer;
    FOptions: TWebResponsiveGridOptions;
    FItems: TWebResponsiveGridItems;
    FOnItemDblClick: TResponsiveGridItemEvent;
    FOnItemClick: TResponsiveGridItemEvent;
    FOnItemCreated: TResponsiveGridItemEvent;
    FOnItemGetFieldValue: TResponsiveGridItemFieldValueEvent;
    FOnHttpRequestError: TNotifyEvent;
    FOnHttpRequestSuccess: TNotifyEvent;
    FOnHttpRequest: TGridRequestEvent;
    procedure SetOptions(const Value: TWebResponsiveGridOptions);
    procedure SetItems(const Value: TWebResponsiveGridItems);
  protected
    procedure Paint; override;
    procedure OptionsChanged(Sender: TObject);

  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure LoadFromJSON(const AURL: string; ADataNode: string = '');
    procedure LoadFromCSV(const AURL: string; Delimiter: char = ';');
    function ListElementHandle: TJSElement;
    function ItemByTag(ATag: integer): TResponsiveGridItem;
  published
    property Align;
    {$IFNDEF LCLTMSWEB}
    property AlignWithMargins;
    {$ENDIF}
    property Anchors;
    property ChildOrder;
    property ElementClassName;
    property ElementID;
    property ElementFont;
    property ElementPosition;
    property Font;
    property Height;
    property HeightStyle;
    property HeightPercent;
    property ItemIndex: integer read FItemIndex write FItemIndex;
    property Items: TWebResponsiveGridItems read FItems write SetItems;
    property Options: TWebResponsiveGridOptions read FOptions write SetOptions;
    property ParentFont;
    property WidthPercent;
    property WidthStyle;
    property Visible;

    property OnClick;
    property OnDblClick;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property OnMouseLeave;
    property OnMouseEnter;

    property OnItemCreated: TResponsiveGridItemEvent read FOnItemCreated write FOnItemCreated;
    property OnItemGetFieldValue: TResponsiveGridItemFieldValueEvent read FOnItemGetFieldValue write FOnItemGetFieldValue;
    property OnItemClick: TResponsiveGridItemEvent read FOnItemClick write FOnItemClick;
    property OnItemDblClick: TResponsiveGridItemEvent read FOnItemDblClick write FOnItemDblClick;
    property OnHttpRequestError: TNotifyEvent read FOnHttpRequestError write FOnHttpRequestError;
    property OnHttpRequestSuccess: TNotifyEvent read FOnHttpRequestSuccess write FOnHttpRequestSuccess;
    property OnHttpRequest: TGridRequestEvent read FOnHttpRequest write FOnHttpRequest;
  end;

function GridRect(Left,Top,Right,Bottom: integer): TGridRect;

implementation

uses
  SysUtils, Types, Weblib.HTML;

function GridRect(Left,Top,Right,Bottom: integer): TGridRect;
begin
  Result.Left := 0;
end;

{ TWebCustomTableControl }

constructor TWebCustomTableControl.Create(AOwner: TComponent);
begin
  inherited;

  Color := clWindow;
  ColCount := 5;
  RowCount := 10;
  RowHeader := true;
  ColHeader := true;
  Width := 300;
  Height := 200;
  FOptions := TTableControlOptions.Create;
end;

destructor TWebCustomTableControl.Destroy;
begin
  FOptions.Free;
  inherited;
end;

function TWebCustomTableControl.GetCell(col, row: integer): string;
begin
  Result := '';
end;

function TWebCustomTableControl.GetCellElement(col, row: integer): TJSElement;
begin
  Result := nil;
end;

function TWebCustomTableControl.GetRowClassName(row: integer): string;
begin
  Result := '';
end;

procedure TWebCustomTableControl.InitBootstrapStyle;
begin
  inherited;
  ElementFont := efCSS;
  ElementHeaderClassName := 'thead-light';
  ElementTableClassName := 'table table-striped table-bordered  table-hover';
end;

procedure TWebCustomTableControl.InitSample(AInitMethod: TInitMethod);
begin
  //
end;

procedure TWebCustomTableControl.LoadFromCSV(const AURL: string; Delimiter: char = ';'; LoadFixed: boolean = false);
begin
  //
end;

procedure TWebCustomTableControl.LoadFromJSON(const AURL: string; ADataNode: string);
begin
  //
end;

procedure TWebCustomTableControl.Paint;
var
  i,w,x: integer;

begin
  inherited;

  if ColCount > 0 then
  begin
    w := Width div ColCount;

    for i := 1 to ColCount - 1 do
    begin
      Canvas.Pen.Style := psDot;
      Canvas.Pen.Color := clGray;
      Canvas.Pen.Width := 1;

      x := w * I;
      Canvas.MoveTo(x,0);
      Canvas.LineTo(x,Height);
    end;
  end;

  w := 0;
  for i := 0 to RowCount - 1 do
  begin
    Canvas.TextOut(2,2 + w, IntToStr(I));
    w := w + Canvas.TextHeight('gh') + 2;
  end;

end;

procedure TWebCustomTableControl.SetCell(col, row: integer; const Value: string);
begin
  //
end;

procedure TWebCustomTableControl.SetColCount(const Value: integer);
begin
  if (FColCount <> Value) and (FColCount >= 0) then
  begin
    FColCount := Value;
    Invalidate;
  end;
end;

procedure TWebCustomTableControl.SetOptions(const Value: TTableControlOptions);
begin
  FOptions.Assign(Value);
end;

procedure TWebCustomTableControl.SetRowClassName(row: integer; const Value: string);
begin

end;

procedure TWebCustomTableControl.SetRowCount(const Value: integer);
begin
  if (FRowCount <> Value) and (Value >= 0) then
  begin
    FRowCount := Value;
    Invalidate;
  end;
end;

{ TWebStringGrid }

constructor TWebStringGrid.Create(AOwner: TComponent);
begin
  inherited;
  FElementFont := efProperty;
  FElementPosition := epAbsolute;
  FWidthStyle := ssAbsolute;
  FWidthPercent := 100;
  FHeightStyle := ssAbsolute;
  FHeightPercent := 100;
  FFixedTextColor := clBlack;
end;

function TWebStringGrid.GetCellElements(ACol, ARow: integer): TJSHTMLElement;
begin
  Result := nil;
end;

procedure TWebStringGrid.HideEdit;
begin
  //
end;

procedure TWebStringGrid.InitSample(AInitMethod: TInitMethod);
begin
  //
end;

procedure TWebStringGrid.InsertRow(Index: integer);
begin
//
end;

procedure TWebStringGrid.LoadFromCSV(const AURL: string; Delimiter: char = ';'; LoadFixed: boolean = false);
begin
  //
end;

procedure TWebStringGrid.LoadFromJSON(AJSON: TJSObject; ADataNode: string);
begin

end;

procedure TWebStringGrid.RemoveRow(Index: integer);
begin
//
end;

procedure TWebStringGrid.LoadFromJSON(const AURL: string; ADataNode: string);
begin
  //
end;

procedure TWebStringGrid.SaveToCSV(const AFileName: string; Delimiter: char;
  SaveFixed: boolean);
begin
  //
end;

procedure TWebStringGrid.SetHeightPercent(const Value: TPercentSize);
begin
  FHeightPercent := Value;
end;

procedure TWebStringGrid.SetHeightStyle(const Value: TSizeStyle);
begin
  FHeightStyle := Value;
end;

procedure TWebStringGrid.SetWidthPercent(const Value: TPercentSize);
begin
  FWidthPercent := Value;
end;

procedure TWebStringGrid.SetWidthStyle(const Value: TSizeStyle);
begin
  FWidthStyle := Value;
end;

procedure TWebStringGrid.ShowEdit;
begin
  //
end;

{ TWebResponsiveGridItem }

procedure TWebResponsiveGridItem.Assign(Source: TPersistent);
begin
  if (Source is TWebResponsiveGridItem) then
  begin
    FHTML := (Source as TWebResponsiveGridItem).HTML;
    FTag := (Source as TWebResponsiveGridItem).Tag;
  end;
end;

constructor TWebResponsiveGridItem.Create(ACollection: TCollection);
begin
  inherited;
  FVisible := true;
  FTag := 0;
end;

function TWebResponsiveGridItem.GetJSONElementValue(id: string): string;
begin
  Result := '';
end;

procedure TWebResponsiveGridItem.SetHTML(const Value: string);
begin
  FHTML := Value;
end;

{ TWebResponsiveGridItems }

function TWebResponsiveGridItems.Add: TWebResponsiveGridItem;
begin
  Result := TWebResponsiveGridItem(inherited Add);
end;

constructor TWebResponsiveGridItems.Create(AOwner: TComponent);
begin
  inherited Create(AOwner, TWebResponsiveGridItem);
end;

function TWebResponsiveGridItems.GetItem(Index: integer): TWebResponsiveGridItem;
begin
  Result := TWebResponsiveGridItem(inherited Items[Index]);
end;

function TWebResponsiveGridItems.Insert(Index: integer): TWebResponsiveGridItem;
begin
  Result := TWebResponsiveGridItem(inherited Insert(Index));
end;

procedure TWebResponsiveGridItems.SetItem(Index: integer;
  const Value: TWebResponsiveGridItem);
begin
  inherited Items[Index] := Value;
end;

{ TWebResponsiveGrid }

constructor TWebResponsiveGrid.Create(AOwner: TComponent);
begin
  inherited;
  FOptions := TWebResponsiveGridOptions.Create;
  FOptions.OnChange := OptionsChanged;
  FItems := TWebResponsiveGridItems.Create(Self);
  Width := 300;
  Height := 200;
end;

destructor TWebResponsiveGrid.Destroy;
begin
  FOptions.Free;
  FItems.Free;
  inherited;
end;

function TWebResponsiveGrid.ItemByTag(ATag: integer): TResponsiveGridItem;
begin
  Result := nil;
end;

function TWebResponsiveGrid.ListElementHandle: TJSElement;
begin
  Result := nil;
end;

procedure TWebResponsiveGrid.LoadFromCSV(const AURL: string; Delimiter: char = ';');
begin
  //
end;

procedure TWebResponsiveGrid.LoadFromJSON(const AURL: string;
  ADataNode: string);
begin
  //
end;

procedure TWebResponsiveGrid.OptionsChanged(Sender: TObject);
begin
  Invalidate;
end;

procedure TWebResponsiveGrid.Paint;
var
  cols, colw, i: integer;
  r: TRect;
  av,sv,fa,ah,cid,cv,ct: string;
  xs,ys,ml,hl: integer;
  hr,cr: TRect;
  {$IFNDEF LCLTMSWEB}
  LFont: TLogFont;
  hOldFont,hNewFont: HFont;
  {$ENDIF}
begin
  inherited;

  colw := Options.ItemWidthMin + Options.ItemPadding;

  i := 1;
  Canvas.Pen.Color := clRed;
  Canvas.Pen.Width := 1;
  Canvas.Pen.Style := psSolid;

  {$IFNDEF LCLTMSWEB}
  Canvas.Font.Name := WEBDEFAULTFONT;
  Canvas.Font.Size := 7;
  Canvas.Font.Color := clRed;
  GetObject(Canvas.Font.Handle, SizeOf(LFont), Addr(LFont));
  LFont.lfEscapement := +900;
  LFont.lfOrientation := +900;
  hNewFont := CreateFontIndirect(LFont);
  hOldFont := SelectObject(Canvas.Handle,hNewFont);
  {$ENDIF}

  if colw > 0 then
  begin
    while colw * i < Width do
    begin
      Canvas.MoveTo(colw * i, Height - 6);
      Canvas.LineTo(colw * i, Height);
      {$IFNDEF LCLTMSWEB}
      Canvas.TextOut(colw * i, Height - 6, '(' + inttostr(i)+')' + inttostr(colw * i));
      {$ENDIF}
      inc(i);
    end;
  end;

  {$IFNDEF LCLTMSWEB}
  hNewFont := SelectObject(Canvas.Handle,hOldFont);
  DeleteObject(hNewFont);
  Canvas.Font.Assign(Font);
  {$ENDIF}

  if colw > 0 then
    cols := Width div colw
  else
    cols := 1;

  colw := Round((Width - (cols * Options.ItemGap))/cols);

  Canvas.Pen.Width := 1;

  if Options.ItemBorderColor = clNone then
  begin
    Canvas.Pen.Color := clGray;
    Canvas.Pen.Style := psDot;
  end
  else
  begin
    Canvas.Pen.Color := Options.ItemBorderColor;
    Canvas.Pen.Style := psSolid;
  end;

  if Options.ItemColor = clNone then
    Canvas.Brush.Style := bsClear
  else
  begin
    Canvas.Brush.Style := bsSolid;
    Canvas.Brush.Color := Options.ItemColor;
  end;

  for i := 1 to cols do
  begin
    r.Top := 1;
    r.Bottom := 1 + Options.ItemHeight;
    r.Left := (colw + Options.ItemGap) * (i - 1);
    r.Right := r.Left + colw;

    Canvas.Rectangle(r);

    if (i = 1) and (Options.ItemTemplate <> '') then
    begin
      HtmlDrawEx(Canvas, Options.ItemTemplate, r, nil, -1,-1,-1,-1,-1,
        false,false,false,false,false,false,true,false,'',1.0,clBlue,clNone,clNone,clGray,av,sv,fa,ah,xs,ys,hl,ml,hr,cr,cid,cv,ct);
    end;
  end;
end;

procedure TWebResponsiveGrid.SetItems(const Value: TWebResponsiveGridItems);
begin
  FItems.Assign(Value);
end;

procedure TWebResponsiveGrid.SetOptions(const Value: TWebResponsiveGridOptions);
begin
  FOptions.Assign(Value);
end;

{ TWebResponsiveGridOptions }

procedure TWebResponsiveGridOptions.Assign(Source: TPersistent);
begin
  if (Source is TWebResponsiveGridOptions) then
  begin
    FItemGap := (Source as TWebResponsiveGridOptions).ItemGap;
    FItemHeight := (Source as TWebResponsiveGridOptions).ItemHeight;
    FItemWidthMin := (Source as TWebResponsiveGridOptions).ItemWidthMin;
  end;
end;

procedure TWebResponsiveGridOptions.Changed;
begin
  if Assigned(OnChange) then
    OnChange(Self);
end;

constructor TWebResponsiveGridOptions.Create;
begin
  inherited;
  FItemWidthMin := 100;
  FItemHeight := 100;
  FItemGap := 10;
  FScrollVertical := true;
  FItemColor := clNone;
  FItemBorderColor := clNone;
  FItemHoverColor := clNone;
  FItemHoverBorderColor := clNone;
  FItemSelectedColor := clNone;
  FItemSelectedBorderColor := clNone;
  FItemSelectedTextColor := clNone;
end;

procedure TWebResponsiveGridOptions.SetItemBorderColor(const Value: TColor);
begin
  if (FItemBorderColor <> Value) then
  begin
    FItemBorderColor := Value;
    Changed;
  end;
end;

procedure TWebResponsiveGridOptions.SetItemColor(const Value: TColor);
begin
  if (FItemColor <> Value) then
  begin
    FItemColor := Value;
    Changed;
  end;
end;

procedure TWebResponsiveGridOptions.SetItemGap(const Value: integer);
begin
  if (FItemGap <> Value) then
  begin
    FItemGap := Value;
    Changed;
  end;
end;

procedure TWebResponsiveGridOptions.SetItemHeight(const Value: integer);
begin
  if (FItemHeight <> Value) then
  begin
    FItemHeight := Value;
    Changed;
  end;
end;

procedure TWebResponsiveGridOptions.SetItemSelectedBorderColor(
  const Value: TColor);
begin
  if (FItemSelectedBorderColor <> Value) then
  begin
    FItemSelectedBorderColor := Value;
    Changed;
  end;
end;

procedure TWebResponsiveGridOptions.SetItemSelectedColor(const Value: TColor);
begin
  if (FItemSelectedColor <> Value) then
  begin
    FItemSelectedColor := Value;
    Changed;
  end;
end;

procedure TWebResponsiveGridOptions.SetItemSelectedTextColor(
  const Value: TColor);
begin
  if (FItemSelectedTextColor <> Value) then
  begin
    FItemSelectedTextColor := Value;
    Changed;
  end;
end;

procedure TWebResponsiveGridOptions.SetItemTemplate(const Value: string);
begin
  if (FItemTemplate <> Value) then
  begin
    FItemTemplate := Value;
    Changed;
  end;
end;

procedure TWebResponsiveGridOptions.SetItemWidthMin(const Value: integer);
begin
  if (FItemWidthMin <> Value) then
  begin
    FItemWidthMin := Value;
    Changed;
  end;
end;

{ TTableControlOptions }

procedure TTableControlOptions.Assign(Source: TPersistent);
begin
  if (Source is TTableControlOptions) then
  begin
    FScrollVertical := (Source as TTableControlOptions).ScrollVertical;
    FResizeColumns := (Source as TTableControlOptions).ResizeColumns;
  end;

end;

end.
