{********************************************************************}
{                                                                    }
{ written by TMS Software                                            }
{            copyright  2016                                        }
{            Email : info@tmssoftware.com                            }
{            Web : http://www.tmssoftware.com                        }
{                                                                    }
{ The source code is given as is. The author is not responsible      }
{ for any possible damage done due to the use of this code.          }
{ The complete source code remains property of the author and may    }
{ not be distributed, published, given or sold in any form as such.  }
{ No parts of the source code can be included in any other component }
{ or application without written authorization of the author.        }
{********************************************************************}

unit WEBLib.JQGridDatabaseAdapter;

interface

uses
  Classes, WEBLib.JQGrid, WEBLib.Controls, DB;

const
  MAJ_VER = 1; // Major version nr.
  MIN_VER = 0; // Minor version nr.
  REL_VER = 0; // Release nr.
  BLD_VER = 0; // Build nr.

  // version history
  // v1.0.0.0 : first release

resourcestring
  sWebJQXgridDatabaseAdapterUnidirectionalDataSet = 'Unidirectional DataSet.';
  sWebJQXgridDatabaseAdapterNoGridAssigned = 'No grid assigned.';
  sWebJQXgridDatabaseAdapterNoFieldsSpecified = 'No fields specified.';
  sWebJQXgridDatabaseAdapterCannotAddFieldsDataSource = 'Cannot add fields, no datasource assigned.';
  sWebJQXgridDatabaseAdapterCannotAddFieldsDataSet = 'Cannot add fields, no DataSet specified for datasource.';
  sWebJQXgridDatabaseAdapterCheckBoxField = 'Cannot assign true when CheckBoxField or ProgressField is also true.';
  sWebJQXgridDatabaseAdapterPictureField = 'Cannot assign true when PictureField or ProgressField is also true.';
  sWebJQXgridDatabaseAdapterCheckPictureField = 'Cannot assign true when CheckBoxField or PictureField is also true.';

type
  TWebJQXGridDatabaseAdapter = class;

  TWebJQXGridDatabaseAdapterItemDataLink = class(TDataLink)
  private
    FModified: Boolean;
    FAdapter: TWebJQXGridDatabaseAdapter;
  protected
    procedure ActiveChanged; override;
    procedure DataSetChanged; override;
    procedure DataSetScrolled(Distance: Integer); override;
    procedure RecordChanged(Field: TField); override;
    procedure UpdateData; override;
    function Adapter: TWebJQXGridAdapter;
  public
    constructor Create(AAdapter: TWebJQXGridDatabaseAdapter);
    destructor Destroy; override;
    procedure Modified;
    procedure Reset;
  end;

  TWebJQXGridDatabaseAdapterColumn = class(TCollectionItem)
  private
    FAdapter: TWebJQXGridDatabaseAdapter;
    FField: TField;
    FFieldName: string;
    FAutoCreated: Boolean;
    FHeader: string;
    FCheckBoxField: Boolean;
    FPictureField: Boolean;
    FProgressField: Boolean;
    FCheckTrue: string;
    FCheckFalse: string;
    FUseLookupEditor: boolean;
    FHTMLTemplate: string;
    FUseColumnEditor: Boolean;
    procedure SetFieldName(const Value: string);
    function GetField: TField;
    procedure SetField(Value: TField);
    procedure SetHeader(const Value: string);
    procedure SetCheckBoxField(const Value: Boolean);
    procedure SetPictureField(const Value: Boolean);
    procedure SetProgressField(const Value: Boolean);
    procedure SetHTMLTemplate(const Value: string);
  protected
    function GetDisplayName: string; override;
    property AutoCreated: Boolean read FAutoCreated write FAutoCreated;
  public
    constructor Create(ACollection: TCollection); override;
    procedure Assign(Source: TPersistent); override;
    function Adapter: TWebJQXGridDatabaseAdapter;
    property Field: TField read GetField write SetField;
  published
    property FieldName: string read FFieldName write SetFieldName;
    property Header: string read FHeader write SetHeader;
    property CheckBoxField: Boolean read FCheckBoxField write SetCheckBoxField default false;
    property ProgressField: Boolean read FProgressField write SetProgressField default false;
    property PictureField: Boolean read FPictureField write SetPictureField default false;
    property CheckFalse: string read FCheckFalse write FCheckFalse;
    property UseLookupEditor: Boolean read FUseLookupEditor write FUseLookupEditor default True;
    property UseColumnEditor: Boolean read FUseColumnEditor write FUseColumnEditor default False;
    property CheckTrue: string read FCheckTrue write FCheckTrue;
    property HTMLTemplate: string read FHTMLTemplate write SetHTMLTemplate;
  end;

  TWebJQXGridDatabaseAdapterColumns = class(TOwnedCollection)
  private
    FAdapter: TWebJQXGridDatabaseAdapter;
    function GetItem(Index: Integer): TWebJQXGridDatabaseAdapterColumn;
    procedure SetItem(Index: Integer; const Value: TWebJQXGridDatabaseAdapterColumn);
  public
    constructor Create(AAdapter: TWebJQXGridDatabaseAdapter);
    function HasFieldsDefined: boolean;
    function Add: TWebJQXGridDatabaseAdapterColumn;
    function Insert(index: Integer): TWebJQXGridDatabaseAdapterColumn;
    function Adapter: TWebJQXGridDatabaseAdapter;
    property Items[Index: Integer]: TWebJQXGridDatabaseAdapterColumn read GetItem write SetItem; default;
  end;

  TWebJQXGridDatabaseAdapterDataSetType = (adsSequenced, adsNonSequenced);
  TWebJQXGridDatabaseAdapterPostMode = (apmNone, apmCell, apmRow);

  TWebJQXGridDatabaseAdapterGetRecordCountEvent = procedure(Sender: TObject; var ARecordCount: Integer) of object;
  TWebJQXGridDatabaseAdapterGetHTMLTemplateDataEvent = procedure(Sender: TObject; ACol, ARow: integer; AFieldname: string; var AData: string) of object;
  TWebJQXGridDatabaseAdapterGetHTMLTemplateEvent = procedure(Sender: TObject; ACol, ARow: integer; var AHTMLTemplate: string; AFields: TFields) of object;

  TWebJQXGridDatabaseAdapterLookupItem = class
  private
    FIndex: Integer;
  public
    constructor Create(AIndex: Integer);
    property Index: Integer read FIndex;
  end;

  [ComponentPlatformsAttribute(TMSWebPlatform)]
  TWebJQXGridDatabaseAdapter = class(TWebJQXGridAdapter)
  private
    FExportStartRow: Integer;
    FExportRow: Integer;
    FOldAR: Integer;
    FOldTopRow: Integer;
    FEditText: string;
    FLookupKeys: TStringList;
    FPrevRecNo: integer;
    FRecordChanged: Boolean;
    FOldDatasetType: TWebJQXGridDatabaseAdapterDataSetType;
    FEmptyDataSet: Boolean;
    FShowDefinedFields: Boolean;
    FDataLink: TWebJQXGridDatabaseAdapterItemDataLink;
    FColumns: TWebJQXGridDatabaseAdapterColumns;
    FAutoRemoveColumns: Boolean;
    FAutoCreateColumns: Boolean;
    FOnGetRecordCount: TWebJQXGridDatabaseAdapterGetRecordCountEvent;
    FDataSetType: TWebJQXGridDatabaseAdapterDataSetType;
    FDataSetTypeAuto: Boolean;
    FShowPictureFields: Boolean;
    FShowBooleanFields: Boolean;
    FPostMode: TWebJQXGridDatabaseAdapterPostMode;
    FShowMemoFields: Boolean;
    FOnGetHTMLTemplateData: TWebJQXGridDatabaseAdapterGetHTMLTemplateDataEvent;
    FOnGetHTMLTemplate: TWebJQXGridDatabaseAdapterGetHTMLTemplateEvent;
    function GetDataSource: TDataSource;
    procedure SetDataSource(const Value: TDataSource);
    procedure SetColumns(const Value: TWebJQXGridDatabaseAdapterColumns);
    function GetFieldAtColumn(ACol: Integer): TField;
    function GetFieldIndexAtColumn(ACol: Integer): Integer;
    procedure SetDataSetType(
      const Value: TWebJQXGridDatabaseAdapterDataSetType);
  protected
    procedure ClearOldPosition;
    procedure Notification(AComponent: TComponent; AOperation: TOperation); override;
    procedure RecordChanged({%H-}Field: TField);
    procedure UpdateData;
    procedure ActiveChange;
    procedure DataSetChanged;
    procedure UpdateVisibleFields;
    procedure UpdateGrid;
    procedure Scroll(Distance: Integer);
    procedure UpdateRowCount;
    procedure UpdateBounds; override;
    procedure Initialize; override;
    procedure DoAutoRemoveColumns(ARemoveDefined: Boolean); virtual;
    procedure DoGetRecordCount(var ARecordCount: Integer); virtual;
    procedure SetActiveRecord(ARow: Integer);
    procedure ExportNotification(AState: TJQXGridExportState; ARow: Integer); override;
    procedure LoadLookupList(AField: TField; AList: TStrings);
    procedure GetCellData(Col, Row: Integer; DataField: string; var ACellData: String); override;
    procedure SelectCell(ACell: TJQXGridCellRec); override;
    procedure CellEditValidateData(ACol, ARow: Integer; var CellString: String; var Allow: Boolean); override;
    procedure CellEditGetData(ACol, ARow: Integer; var CellString: String); override;
    function CheckDatasetType: Boolean;
    function GetBufferCount: Integer;
    function GetRecordCount: Integer;
    function GetRecordNo: Integer;
    function GetLookupKey(AIndex: Integer): string;
    function HTMLDBReplace(AValue: string; ADataset: TDataSet; ACol, ARow: Integer): string;
  public
    procedure Assign(Source: TPersistent); override;
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    function CheckDataSet: Boolean;
    property DataLink: TWebJQXGridDatabaseAdapterItemDataLink read FDataLink;
    property FieldAtColumn[ACol: Integer]: TField read GetFieldAtColumn;
    property FieldIndexAtColumn[ACol: Integer]: Integer read GetFieldIndexAtColumn;
    procedure AddAllFields;
    procedure RemoveAllFields;
    procedure RemoveAllColumns;
    procedure LoadAllDataAndDisconnect;
  published
    property DataSource: TDataSource read GetDataSource write SetDataSource;
    property Columns: TWebJQXGridDatabaseAdapterColumns read FColumns write SetColumns;
    property AutoCreateColumns: Boolean read FAutoCreateColumns write FAutoCreateColumns default True;
    property AutoRemoveColumns: Boolean read FAutoRemoveColumns write FAutoRemoveColumns default True;
    property OnGetRecordCount: TWebJQXGridDatabaseAdapterGetRecordCountEvent read FOnGetRecordCount write FOnGetRecordCount;
    property OnGetHTMLTemplateData: TWebJQXGridDatabaseAdapterGetHTMLTemplateDataEvent read FOnGetHTMLTemplateData write FOnGetHTMLTemplateData;
    property OnGetHTMLTemplate: TWebJQXGridDatabaseAdapterGetHTMLTemplateEvent read FOnGetHTMLTemplate write FOnGetHTMLTemplate;
    property DataSetType: TWebJQXGridDatabaseAdapterDataSetType read FDataSetType write SetDataSetType default adsSequenced;
    property DataSetTypeAuto: Boolean read FDataSetTypeAuto write FDataSetTypeAuto default True;
    property ShowBooleanFields: Boolean read FShowBooleanFields write FShowBooleanFields default True;
    property ShowPictureFields: Boolean read FShowPictureFields write FShowPictureFields default True;
    property ShowMemoFields: Boolean read FShowMemoFields write FShowMemoFields default False;
    property PostMode: TWebJQXGridDatabaseAdapterPostMode read FPostMode write FPostMode default apmRow;
  end;

implementation

uses
  WEBLib.Dialogs, SysUtils, Math;

procedure TWebJQXGridDatabaseAdapter.ActiveChange;
begin

end;

procedure TWebJQXGridDatabaseAdapter.AddAllFields;
begin
end;

procedure TWebJQXGridDatabaseAdapter.Assign(Source: TPersistent);
begin
  if Source is TWebJQXGridDatabaseAdapter then
  begin
    FColumns.Assign((Source as TWebJQXGridDatabaseAdapter).Columns);
    FAutoRemoveColumns := (Source as TWebJQXGridDatabaseAdapter).AutoRemoveColumns;
    FAutoCreateColumns := (Source as TWebJQXGridDatabaseAdapter).AutoCreateColumns;
    FDataSetType := (Source as TWebJQXGridDatabaseAdapter).DataSetType;
    FDataSetTypeAuto := (Source as TWebJQXGridDatabaseAdapter).DataSetTypeAuto;
    FShowBooleanFields := (Source as TWebJQXGridDatabaseAdapter).ShowBooleanFields;
    FShowPictureFields := (Source as TWebJQXGridDatabaseAdapter).ShowPictureFields;
    FShowMemoFields := (Source as TWebJQXGridDatabaseAdapter).ShowMemoFields;
    FPostMode := (Source as TWebJQXGridDatabaseAdapter).PostMode;
  end;
end;

procedure TWebJQXGridDatabaseAdapter.UpdateBounds;
begin
  inherited;
end;

procedure TWebJQXGridDatabaseAdapter.UpdateGrid;
begin
end;

procedure TWebJQXGridDatabaseAdapter.CellEditGetData(ACol, ARow: Integer;
  var CellString: String);
begin
  inherited;

end;

procedure TWebJQXGridDatabaseAdapter.CellEditValidateData(ACol, ARow: Integer;
  var CellString: String; var Allow: Boolean);
begin
  inherited;

end;

function TWebJQXGridDatabaseAdapter.CheckDataSet: Boolean;
begin
  Result := Active and Assigned(FDataLink) and Assigned(DataLink.DataSet) and DataLink.DataSet.Active;
end;

function TWebJQXGridDatabaseAdapter.CheckDatasetType: Boolean;
begin
  if DataSetTypeAuto then
  begin
    if not FDataLink.DataSet.IsSequenced then
      FDataSetType := adsNonSequenced
    else
      FDataSetType := adsSequenced;
  end;

  Result := (FOldDatasetType <> FDataSetType) and (DataLink.DataSet.State = dsBrowse);
  FOldDatasetType := FDataSetType;
end;

procedure TWebJQXGridDatabaseAdapter.ClearOldPosition;
begin
end;

constructor TWebJQXGridDatabaseAdapter.Create(AOwner: TComponent);
begin
  inherited;
  FEditText := '';
  FOldAR := -1;
  FExportRow := -1;
  FExportStartRow := -1;
  ClearOldPosition;
  FOldTopRow := -1;
  FLookupKeys := TStringList.Create;
  FDataSetTypeAuto := True;
  FEmptyDataSet := False;
  FDataSetType := adsSequenced;
  FDataLink := TWebJQXGridDatabaseAdapterItemDataLink.Create(Self);
  FColumns := TWebJQXGridDatabaseAdapterColumns.Create(Self);
  FAutoRemoveColumns := True;
  FAutoCreateColumns := True;
  FShowPictureFields := True;
  FShowBooleanFields := True;
  FShowDefinedFields := False;
  FPostMode := apmRow;
end;

procedure TWebJQXGridDatabaseAdapter.DataSetChanged;
begin

end;

procedure TWebJQXGridDatabaseAdapter.UpdateData;
begin
end;

destructor TWebJQXGridDatabaseAdapter.Destroy;
begin
  FLookupKeys.Free;
  FColumns.Free;
  FDataLink.Free;
  inherited;
end;

procedure TWebJQXGridDatabaseAdapter.DoAutoRemoveColumns(
  ARemoveDefined: Boolean);
begin
end;

procedure TWebJQXGridDatabaseAdapter.DoGetRecordCount(
  var ARecordCount: Integer);
begin
  if Assigned(OnGetRecordCount) then
    OnGetRecordCount(Self, ARecordCount);
end;

procedure TWebJQXGridDatabaseAdapter.ExportNotification(
  AState: TJQXGridExportState; ARow: Integer);
begin
  inherited;

end;

function TWebJQXGridDatabaseAdapter.GetBufferCount: Integer;
begin
  Result := 0;
end;

procedure TWebJQXGridDatabaseAdapter.GetCellData(Col, Row: Integer;
  DataField: string; var ACellData: String);
begin
  inherited;

end;

function TWebJQXGridDatabaseAdapter.GetDataSource: TDataSource;
begin
  Result := DataLink.DataSource;
end;

function TWebJQXGridDatabaseAdapter.GetFieldAtColumn(ACol: Integer): TField;
var
  j: integer;
begin
  Result := nil;
  if (ACol >= Columns.Count) or not CheckDataSet then
    Exit;

  if (Columns[ACol].HTMLTemplate <> '') then
    Exit;

  if FShowDefinedFields then
  begin
    if Columns[ACol].FieldName <> '' then
      Result := DataLink.DataSet.FieldByName(Columns[ACol].FieldName);
  end
  else
  begin
    if Columns[ACol].FieldName <> '' then
      Result := DataLink.DataSet.FieldByName(Columns[ACol].FieldName)
    else
    begin
      j := GetFieldIndexAtColumn(ACol);
      if (j <> -1) then
        Result := DataLink.DataSet.Fields[j]
      else
        Result := nil;
    end;
  end;
end;

function TWebJQXGridDatabaseAdapter.GetFieldIndexAtColumn(
  ACol: Integer): Integer;
var
  j, i, fieldIdx: Integer;
  f: TField;
begin
  Result := -1;
  if not CheckDataSet or (ACol >= Columns.Count) then
    Exit;

  if (Columns[ACol].HTMLTemplate <> '') then
    Exit;

  if FShowDefinedFields then
  begin
    if Columns[ACol].FieldName <> '' then
    begin
      f := DataLink.DataSet.FieldByName(Columns[ACol].FieldName);
      if Assigned(f) then
        Result := f.Index;
    end;
  end
  else
  begin
    if Columns[ACol].FieldName <> '' then
    begin
      f := DataLink.DataSet.FieldByName(Columns[ACol].FieldName);
      if Assigned(f) then
        Result := f.Index;
    end
    else
    begin
      fieldIdx := 0;
      j := 0;
      for i := 0 to DataLink.DataSet.FieldCount - 1 do
      begin
        if DataLink.DataSet.Fields[i].Visible then
        begin
          if fieldIdx >= (ACol - j) then
          begin
            Result := i;
            break;
          end;
          fieldIdx := fieldIdx + 1;
        end;
      end;
    end;
  end;
end;

function TWebJQXGridDatabaseAdapter.GetLookupKey(AIndex: Integer): string;
var
  I: Integer;
begin
  Result := '';
  for I := 0 to FLookupKeys.Count - 1 do
  begin
    if (TWebJQXGridDatabaseAdapterLookupItem(FLookupKeys.Objects[I]).Index = AIndex) then
    begin
      Result := FLookupKeys[I];
      Break;
    end;
  end;
end;

function TWebJQXGridDatabaseAdapter.GetRecordCount: Integer;

  function RowsInDataSet: Integer;
  var
    cb: TBookMark;
    iseof, isbof: Boolean;
  begin
    Result := -1;
    if not CheckDataSet then
      Exit;

    iseof := DataLink.DataSet.Eof;
    isbof := DataLink.DataSet.Bof;

    if isbof and iseof then
      Result := 0
    else
      with DataLink.DataSet do
      begin
        cb := GetBookMark;

        if BookmarkValid(cb) then
        begin
          First;
          if (csDesigning in ComponentState) then
            Result := MoveBy(100)
          else
            Result := MoveBy($7FFFFFFF) + 1;

          GotoBookMark(cb);
        end;

        FreeBookMark(cb);
      end;

    if iseof then
      DataLink.DataSet.Next;

    if isbof then
      DataLink.DataSet.Prior;

    if DataLink.DataSet.State = dsInsert then
      Result := Result + 1;
  end;
begin
  Result := 0;
  if Assigned(FDataLink) then
  begin
    if Assigned(DataLink.DataSet) then
    begin
      if DataLink.DataSet.Active then
      begin
        DataLink.DataSet.DisableControls;
        if Assigned(OnGetRecordCount) then
          OnGetRecordCount(Self, Result)
        else
        begin
          Result := RowsInDataSet;
          if Result = -1 then
            Result := DataLink.RecordCount;
        end;
        FEmptyDataSet := Result = 0;
        DataLink.DataSet.EnableControls;
      end;
    end;
  end;
end;

function TWebJQXGridDatabaseAdapter.GetRecordNo: Integer;
begin
  if DataLink.DataSet.State = dsInsert then
    Result := FPrevRecNo + 1
  else
    Result := DataLink.DataSet.RecNo;
end;

function TWebJQXGridDatabaseAdapter.HTMLDBReplace(AValue: string;
  ADataset: TDataSet; ACol, ARow: Integer): string;
var
  beforetag, aftertag, fld, dbfld: string;
  i, j: integer;
  AField: TField;
begin
  beforetag := '';

  while Pos('<#', AValue) > 0 do
  begin
    i := pos('<#', AValue);
    beforetag := beforetag + copy(AValue, 1, i - 1); //part prior to the tag
    aftertag := copy(AValue, i, length(AValue)); //part after the tag
    j := pos('>', aftertag);
    fld := copy(aftertag, 1, j - 1);
    Delete(fld, 1, 2);
    Delete(AValue, 1, i + j - 1);

    dbfld := '';
    if Assigned(ADataSet) then
    begin
      if ADataSet.Active then
      begin
        AField := ADataSet.FindField(fld);
        if Assigned(AField) then
        begin
          if AField.IsBlob then
          begin
            if not ShowMemoFields then
            begin
              dbfld := '(MEMO)'
            end
            else
            begin
              if Assigned(AField.OnGetText) then
                dbfld := AField.DisplayText
              else
                dbfld := AField.AsString;
            end;
          end
          else
           dbfld := AField.DisplayText;
        end;

        if Assigned(FOnGetHTMLTemplateData) then
          FOnGetHTMLTemplateData(Self, ACol, ARow, fld, dbfld);
      end
      else
        dbfld := '(' + fld + ')';
    end
    else
      dbfld := '(' + fld + ')';

    beforetag := beforetag + dbfld;
  end;

  Result := beforetag + AValue;
end;

procedure TWebJQXGridDatabaseAdapter.Initialize;
begin
  inherited;
  ActiveChange;
end;

procedure TWebJQXGridDatabaseAdapter.LoadAllDataAndDisconnect;
begin
end;

procedure TWebJQXGridDatabaseAdapter.LoadLookupList(AField: TField;
  AList: TStrings);
var
  s, k: string;
  i: Integer;
begin
  if not CheckDataset then
    Exit;

  if Assigned(AField) and Assigned(AField.LookupDataSet) then
  begin
    AField.LookupDataSet.DisableControls;
    AField.LookupDataSet.First;
    i := 1;
    FLookupKeys.Clear;
    AList.Clear;

    while not AField.LookupDataSet.Eof do
    begin
      s := AField.LookupDataSet.FieldByName(AField.LookupResultField).AsString;
      k := AField.LookupDataSet.FieldByName(AField.LookupKeyFields).AsString;
      FLookupKeys.AddObject(k, TWebJQXGridDatabaseAdapterLookupItem.Create(i));
      AList.AddObject(s, TWebJQXGridDatabaseAdapterLookupItem.Create(i));
      inc(i);
      AField.LookupDataSet.Next;
    end;
    AField.LookupDataSet.EnableControls;
  end;
end;

procedure TWebJQXGridDatabaseAdapter.Notification(AComponent: TComponent;
  AOperation: TOperation);
begin
  inherited;
  if (AOperation = opRemove) and (AComponent = DataSource) then
  begin
    DataSource := nil;
    Grid := nil;
    Initialize;
  end;
end;

procedure TWebJQXGridDatabaseAdapter.RecordChanged(Field: TField);
begin
  FRecordChanged := True;
  UpdateGrid;
end;

procedure TWebJQXGridDatabaseAdapter.RemoveAllColumns;
begin
end;

procedure TWebJQXGridDatabaseAdapter.RemoveAllFields;
begin
end;

procedure TWebJQXGridDatabaseAdapter.Scroll(Distance: Integer);
begin
end;

procedure TWebJQXGridDatabaseAdapter.SelectCell(ACell: TJQXGridCellRec);
begin
  inherited;

end;

procedure TWebJQXGridDatabaseAdapter.SetActiveRecord(ARow: Integer);
begin
end;

procedure TWebJQXGridDatabaseAdapter.SetColumns(
  const Value: TWebJQXGridDatabaseAdapterColumns);
begin
  FColumns.Assign(Value);
end;

procedure TWebJQXGridDatabaseAdapter.SetDataSetType(
  const Value: TWebJQXGridDatabaseAdapterDataSetType);
begin
  if Value <> FDataSetType then
  begin
    FDataSetType := Value;
    if CheckDataSet then
    begin
      DataLink.DataSet.Active := false;
      DataLink.DataSet.Active := true;
    end;
  end;
end;

procedure TWebJQXGridDatabaseAdapter.SetDataSource(const Value: TDataSource);
begin
  if Value = DataLink.Datasource then
    Exit;

  DataLink.DataSource := Value;

  if Value <> nil then
    Value.FreeNotification(Self);
end;

procedure TWebJQXGridDatabaseAdapter.UpdateRowCount;
begin

end;

procedure TWebJQXGridDatabaseAdapter.UpdateVisibleFields;
begin

end;

{ TWebJQXGridDatabaseAdapterItemDataLink }

procedure TWebJQXGridDatabaseAdapterItemDataLink.ActiveChanged;
begin
  if Active and Assigned(DataSource) then
    if Assigned(DataSource.DataSet) then
      if DataSource.DataSet.IsUnidirectional then
        raise Exception.Create(sWebJQXgridDatabaseAdapterUnidirectionalDataSet);

  FAdapter.ActiveChange;
  FModified := False;
end;

function TWebJQXGridDatabaseAdapterItemDataLink.Adapter: TWebJQXGridAdapter;
begin
  Result := FAdapter;
end;

constructor TWebJQXGridDatabaseAdapterItemDataLink.Create(
  AAdapter: TWebJQXGridDatabaseAdapter);
begin
  FAdapter := AAdapter;
  VisualControl := True;
end;

procedure TWebJQXGridDatabaseAdapterItemDataLink.DataSetChanged;
begin
  FAdapter.DataSetChanged;
  FModified := False;
end;

procedure TWebJQXGridDatabaseAdapterItemDataLink.DataSetScrolled(
  Distance: Integer);
begin
  FAdapter.Scroll(Distance);
end;

destructor TWebJQXGridDatabaseAdapterItemDataLink.Destroy;
begin

  inherited;
end;

procedure TWebJQXGridDatabaseAdapterItemDataLink.Modified;
begin
  FModified := True;
end;

procedure TWebJQXGridDatabaseAdapterItemDataLink.RecordChanged(Field: TField);
begin
  FAdapter.RecordChanged(Field);
end;

procedure TWebJQXGridDatabaseAdapterItemDataLink.Reset;
begin
  if FModified then
    RecordChanged(nil)
  else
    Dataset.Cancel;
end;

procedure TWebJQXGridDatabaseAdapterItemDataLink.UpdateData;
begin
  if FModified then
    FAdapter.UpdateData;
  FModified := False;
end;

{ TWebJQXGridDatabaseAdapterColumn }

function TWebJQXGridDatabaseAdapterColumn.Adapter: TWebJQXGridDatabaseAdapter;
begin
  Result := FAdapter;
end;

procedure TWebJQXGridDatabaseAdapterColumn.Assign(Source: TPersistent);
begin
  if Source is TWebJQXGridDatabaseAdapterColumn then
  begin
  end;
end;

constructor TWebJQXGridDatabaseAdapterColumn.Create(ACollection: TCollection);
begin
  inherited;
  FUseLookupEditor := True;
  FCheckTrue := 'True';
  FCheckFalse := 'False';
  FCheckBoxField := False;
  FProgressField := False;
  FPictureField := False;
  FAdapter := nil;
  if Assigned(Collection) and (Collection is TWebJQXGridDatabaseAdapterColumns) then
    FAdapter := (Collection as TWebJQXGridDatabaseAdapterColumns).Adapter;
end;

function TWebJQXGridDatabaseAdapterColumn.GetDisplayName: string;
begin
  if Header = '' then
  begin
    if FieldName = '' then
      Result := 'Column ' + Inttostr(Index)
    else
      Result := FieldName;
  end
  else
    Result := Header;
end;

function TWebJQXGridDatabaseAdapterColumn.GetField: TField;
var
  ap: TWebJQXGridDatabaseAdapter;
begin
  ap := Adapter;
  if (FField = nil) and (Length(FFieldName) > 0) and Assigned(ap) and
    Assigned(ap.DataLink.DataSet) then
    with ap.Datalink.DataSet do
      if Active {$IFNDEF LCLWEBLIB}or not (lcAutomatic in Fields.LifeCycles){$ENDIF} then
        SetField(FindField(FieldName));
  Result := FField;
end;

procedure TWebJQXGridDatabaseAdapterColumn.SetCheckBoxField(
  const Value: Boolean);
begin
  if Value and (FPictureField or FProgressField) then
    raise Exception.Create(sWebJQXgridDatabaseAdapterPictureField);

  if FCheckBoxField <> Value then
  begin
    FCheckBoxField := Value;

    if Assigned(FAdapter) then
      FAdapter.UpdateGrid;
  end;
end;

procedure TWebJQXGridDatabaseAdapterColumn.SetField(Value: TField);
var
  ap: TWebJQXGridDatabaseAdapter;
begin
  if FField = Value then
    Exit;

  if Assigned(Value) and (csDestroying in Value.ComponentState) then
    Value := nil;

  ap := Adapter;

  if Assigned(FField) and Assigned(ap) then
    FField.RemoveFreeNotification(ap);

  FField := Value;
  if Assigned(Value) then
  begin
    if Assigned(ap) then
      FField.FreeNotification(ap);
    FFieldName := Value.FieldName;
  end;

  if Assigned(FAdapter) then
    FAdapter.UpdateGrid;
end;

procedure TWebJQXGridDatabaseAdapterColumn.SetFieldName(const Value: string);
var
  f: TField;
  ap: TWebJQXGridDatabaseAdapter;
begin
  f := nil;
  ap := Adapter;
  if Assigned(ap) and Assigned(ap.DataLink.DataSet) and
    not (csLoading in ap.ComponentState) and (Length(Value) > 0) then
      f := ap.DataLink.DataSet.FindField(Value);
  FFieldName := Value;
  if Value <> '' then
    Field := f;

  if Assigned(FAdapter) then
    FAdapter.UpdateGrid;
end;

procedure TWebJQXGridDatabaseAdapterColumn.SetHeader(const Value: string);
begin
  FHeader := Value;
end;

procedure TWebJQXGridDatabaseAdapterColumn.SetHTMLTemplate(const Value: string);
begin
  FHTMLTemplate := Value;
  if assigned(FAdapter) then
    FAdapter.UpdateGrid;
end;

procedure TWebJQXGridDatabaseAdapterColumn.SetPictureField(
  const Value: Boolean);
begin
  if Value and (FCheckBoxField or FProgressField) then
    raise Exception.Create(sWebJQXgridDatabaseAdapterCheckBoxField);

  if FPictureField <> Value then
  begin
    FPictureField := Value;
    if Assigned(FAdapter) then
      FAdapter.UpdateGrid;
  end;
end;

procedure TWebJQXGridDatabaseAdapterColumn.SetProgressField(
  const Value: Boolean);
begin
  if Value and (FCheckBoxField or FPictureField) then
    raise Exception.Create(sWebJQXgridDatabaseAdapterCheckPictureField);

  if FProgressField <> Value then
  begin
    FProgressField := Value;
    if Assigned(FAdapter) then
      FAdapter.UpdateGrid;
  end;
end;

{ TWebJQXGridDatabaseAdapterColumns }

function TWebJQXGridDatabaseAdapterColumns.Adapter: TWebJQXGridDatabaseAdapter;
begin
  Result := FAdapter;
end;

function TWebJQXGridDatabaseAdapterColumns.Add: TWebJQXGridDatabaseAdapterColumn;
begin
  Result := TWebJQXGridDatabaseAdapterColumn(inherited Add);
end;

constructor TWebJQXGridDatabaseAdapterColumns.Create(AAdapter: TWebJQXGridDatabaseAdapter);
begin
  inherited Create(AAdapter, TWebJQXGridDatabaseAdapterColumn);
  FAdapter := AAdapter;
end;

function TWebJQXGridDatabaseAdapterColumns.GetItem(Index: Integer): TWebJQXGridDatabaseAdapterColumn;
begin
  Result := TWebJQXGridDatabaseAdapterColumn(inherited GetItem(Index));
end;

function TWebJQXGridDatabaseAdapterColumns.HasFieldsDefined: boolean;
var
  i: Integer;
begin
  Result := False;
  for i := 0 to Count - 1 do
  begin
    if (Items[i].FieldName <> '') and not (Items[i].AutoCreated) then
    begin
      Result := True;
      Break;
    end;
  end;
end;

function TWebJQXGridDatabaseAdapterColumns.Insert(index: Integer): TWebJQXGridDatabaseAdapterColumn;
begin
  Result := TWebJQXGridDatabaseAdapterColumn(inherited Insert(Index));
end;

procedure TWebJQXGridDatabaseAdapterColumns.SetItem(Index: Integer; const Value: TWebJQXGridDatabaseAdapterColumn);
begin
  inherited SetItem(Index, Value);
end;

{ TWebJQXGridDatabaseAdapterLookupItem }

constructor TWebJQXGridDatabaseAdapterLookupItem.Create(AIndex: Integer);
begin
  FIndex := AIndex;
end;

end.
