unit WEBLib.Miletus;

interface

uses
  Classes, js, web, WEBLib.JSON, WEBLib.Forms, WEBLib.Menus, Types, WEBLib.Graphics,
  WEBLib.Controls, Vcl.Forms, WEBLib.CDS;

type
  TMiletusObserverChangeEvent = procedure(Sender: TObject; AMessageID: Integer; AMessageData: TJSONValue) of object;

  TMiletusObserver = class(TComponent)
  private
    FObserverEnabled: Boolean;
    FOnObserverChange: TMiletusObserverChangeEvent;
  protected
    procedure ObserverChange(AMessageID: Integer; AMessageData: TJSONValue); virtual;
  public
    constructor Create(AOwner: TComponent); override;
    property ObserverEnabled: Boolean read FObserverEnabled write FObserverEnabled default True;
    property OnObserverChange: TMiletusObserverChangeEvent read FOnObserverChange write FOnObserverChange;
  end;

  //TMiletusStringListLoadFromFileProc = reference to procedure;
  TMiletusLoadFileProc = reference to procedure;

  TMiletusStringList = class(TStringList)
  public
    procedure LoadFromFile(const FileName: string); override;
    procedure LoadFromFileAsync(const FileName: string; AProc: TMiletusLoadFileProc); virtual;
    procedure SaveToFile(const FileName: string); override;
    procedure SaveToFileAsync(const FileName: string);
  end;

  TMiletusBinaryDataStream = class(TPersistent)
  private
    FBase64: string;
    FArrayBuffer: TJSArrayBuffer;
    FText: string;
  public
    procedure LoadFromFile(const FileName: string); virtual;
    procedure LoadFromFileAsync(const FileName: string; AProc: TMiletusLoadFileProc); virtual;
    procedure LoadFromFileRequest(const FileName: string; AProc: TMiletusLoadFileProc); virtual;
    procedure SaveToFile(const FileName: string); virtual;
    procedure SaveToFileAsync(const FileName: string); virtual;
    procedure SaveToFileRequest(const FileName: string); virtual;
    property AsArrayBuffer: TJSArrayBuffer read FArrayBuffer write FArrayBuffer;
    property Base64: string read FBase64 write FBase64;
    property Text: string read FText write FText;
  end;

  TMiletusOpenDialogOption = (ofReadOnly, ofOverwritePrompt, ofHideReadOnly,
    ofNoChangeDir, ofShowHelp, ofNoValidate, ofAllowMultiSelect,
    ofExtensionDifferent, ofPathMustExist, ofFileMustExist, ofCreatePrompt,
    ofShareAware, ofNoReadOnlyReturn, ofNoTestFileCreate, ofNoNetworkButton,
    ofNoLongNames, ofOldStyleDialog, ofNoDereferenceLinks, ofEnableIncludeNotify,
    ofEnableSizing, ofDontAddToRecent, ofForceShowHidden);

  TMiletusOpenDialogOptions = set of TMiletusOpenDialogOption;

  TMiletusDialogProc = reference to procedure(FileName: string);

  {$IFNDEF LCLTMSWEB}
  [ComponentPlatforms(TMSWebPlatform)]
  {$ENDIF}
  TMiletusOpenDialog = class(TComponent)
  private
    FFileName: string;
    FFilterIndex: Integer;
    FDefaultExt: string;
    FTitle: string;
    FFilter: string;
    FInitialDir: string;
    FFiles: TStrings;
    FOptions: TMiletusOpenDialogOptions;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure Execute(AProc: TMiletusDialogProc); overload; virtual;
    function Execute: Boolean; overload; virtual;
    property Files: TStrings read FFiles write FFiles;
  published
    property DefaultExt: string read FDefaultExt write FDefaultExt;
    property FileName: string read FFileName write FFileName;
    property Filter: string read FFilter write FFilter;
    property FilterIndex: Integer read FFilterIndex write FFilterIndex default 1;
    property InitialDir: string read FInitialDir write FInitialDir;
    property Options: TMiletusOpenDialogOptions read FOptions write FOptions;
    property Title: string read FTitle write FTitle;
  end;

  {$IFNDEF LCLTMSWEB}
  [ComponentPlatforms(TMSWebPlatform)]
  {$ENDIF}
  TMiletusSaveDialog = class(TMiletusOpenDialog);

  TMiletusMessageBoxProc = reference to procedure(ACheckBoxChecked: Boolean);
  TMiletusMessageBoxType = (mbtNone, mbtInfo, mbtError, mbtQuestion, mbtWarning);

  {$IFNDEF LCLTMSWEB}
  [ComponentPlatforms(TMSWebPlatform)]
  {$ENDIF}
  TMiletusMessageBox = class(TComponent)
  private
    FButtons: TStringList;
    FVerificationChecked: Boolean;
    FVerificationText: string;
    FTitle: string;
    FText: string;
    FNoLink: Boolean;
    FResponse: NativeInt;
    FCaption: string;
    FType: TMiletusMessageBoxType;
    procedure SetButtons(const Value: TStringList);
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure Execute(AProc: TMiletusMessageBoxProc = nil);
  published
    property Buttons: TStringList read FButtons write SetButtons;
    property VerificationChecked: Boolean read FVerificationChecked write FVerificationChecked default False;
    property VerificationText: string read FVerificationText write FVerificationText;
    property Title: string read FTitle write FTitle;
    property DialogType: TMiletusMessageBoxType read FType write FType default mbtNone;
    property Text: string read FText write FText;
    property NoLink: Boolean read FNoLink write FNoLink default True;
    property Response: NativeInt read FResponse write FResponse default -1;
    property Caption: string read FCaption write FCaption;
  end;

  {$IFNDEF LCLTMSWEB}
  [ComponentPlatforms(TMSWebPlatform)]
  {$ENDIF}
  TMiletusErrorBox = class(TComponent)
  private
    FContent: string;
    FTitle: string;
  public
    constructor Create(AOwner: TComponent); override;
    procedure Execute;
  published
    property Content: string read FContent write FContent;
    property Title: string read FTitle write FTitle;
  end;

  {$IFNDEF LCLTMSWEB}
  [ComponentPlatforms(TMSWebPlatform)]
  {$ENDIF}
  TMiletusMainMenu = class(TWebCustomMainMenu);

  {$IFNDEF LCLTMSWEB}
  [ComponentPlatforms(TMSWebPlatform)]
  {$ENDIF}
  TMiletusPopupMenu = class(TWebPopupMenu)
  private
    FOnPopup: TNotifyEvent;
  public
    procedure CloseMenu;
    procedure Popup(X, Y: Integer); reintroduce;
    property OnPopup: TNotifyEvent read FOnPopup write FOnPopup;
  end;

  TMiletusFileWatch = class(TCollectionItem)
  private
    FFileName: string;
    FOnChange: TNotifyEvent;
  public
    constructor Create(ACollection: TCollection); override;
  published
    property FileName: string read FFileName write FFileName;
    property OnChange: TNotifyEvent read FOnChange write FOnChange;
  end;

  TMiletusFileWatches = class(TOwnedCollection)
  private
    function GetItem(Index: Integer): TMiletusFileWatch; reintroduce;
    procedure SetItem(Index: Integer; const Value: TMiletusFileWatch); reintroduce;
  public
    function Add: TMiletusFileWatch; reintroduce;
    property Items[Index: Integer]: TMiletusFileWatch read GetItem write SetItem;
  end;

  {$IFNDEF LCLTMSWEB}
  [ComponentPlatforms(TMSWebPlatform)]
  {$ENDIF}
  TMiletusFileWatcher = class(TComponent)
  private
    FFiles: TMiletusFileWatches;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  published
    property Files: TMiletusFileWatches read FFiles write FFiles;
  end;

  TMiletusShortcut = class(TCollectionItem)
  private
    FShortcut: TShortcut;
    FOnChange: TNotifyEvent;
  published
    property Shortcut: TShortcut read FShortcut write FShortcut;
    property OnChange: TNotifyEvent read FOnChange write FOnChange;
  end;

  TMiletusShortcuts = class(TOwnedCollection)
  private
    function GetItem(Index: Integer): TMiletusShortcut; reintroduce;
    procedure SetItem(Index: Integer; const Value: TMiletusShortcut); reintroduce;
  public
    function Add: TMiletusShortcut; reintroduce;
    property Items[Index: Integer]: TMiletusShortcut read GetItem write SetItem;
  end;

  {$IFNDEF LCLTMSWEB}
  [ComponentPlatforms(TMSWebPlatform)]
  {$ENDIF}
  TMiletusGlobalShortcuts = class(TComponent)
  private
    FShortcuts: TMiletusShortcuts;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  published
    property Shortcuts: TMiletusShortcuts read FShortcuts write FShortcuts;
  end;

  TMiletusNotification = class(TPersistent)
  public
    Name: string;
    Title: string;
    AlertBody: string;
  end;

  {$IFNDEF LCLTMSWEB}
  [ComponentPlatforms(TMSWebPlatform)]
  {$ENDIF}
  TMiletusNotificationCenter = class(TComponent)
  public
    function CreateNotification: TMiletusNotification; overload;
    function CreateNotification(const AName, AAlertBody: string): TMiletusNotification; overload;
    procedure PresentNotification(const ANotification: TMiletusNotification);
  end;

  {$IFNDEF LCLTMSWEB}
  [ComponentPlatforms(TMSWebPlatform)]
  {$ENDIF}
  TMiletusTrayIcon = class(TComponent)
  private
    FPopupMenu: TMiletusPopupMenu;
    FHint: string;
    FOnClick: TNotifyEvent;
    FIconURL: string;
  protected
    procedure Notification(AComponent: TComponent; Operation: TOperation); override;
  public
    constructor Create(AOwner: TComponent); override;
  published
    property Hint: string read FHint write FHint;
    property IconURL: string read FIconURL write FIconURL;
    property PopupMenu: TMiletusPopupMenu read FPopupMenu write FPopupMenu;
    property OnClick: TNotifyEvent read FOnClick write FOnClick;
  end;

  TMiletusClipboard = class(TPersistent)
  private
    FAsText: string;
  public
    function HasFormat(AFormat: Word): Boolean;
    property AsText: string read FAsText write FAsText;
  end;

  TMiletusShell = class(TPersistent)
  public
    procedure Beep;
    procedure OpenExternal(URL: string);
    procedure OpenItem(FullPath: string);
    procedure MoveItemToTrash(FullPath: string);
    procedure ShowItemInFolder(FullPath: string);
  end;

  TFormClass = class of TForm;

  {$IFNDEF LCLTMSWEB}
  [ComponentPlatforms(TMSWebPlatform)]
  {$ENDIF}
  TMiletusWindow = class(TComponent)
  private
    FFormClass: TFormClass;
    FOnShow: TNotifyEvent;
    FOnHide: TNotifyEvent;
    FOnMaximize: TNotifyEvent;
    FOnMinimize: TNotifyEvent;
    FOnClose: TNotifyEvent;
    FOnResize: TNotifyEvent;
  public
    procedure Close;
    procedure Hide;
    procedure LoadFromURL(URL: string);
    procedure Show;
    procedure ShowModal;
    property FormClass: TFormClass read FFormClass write FFormClass;
  published
    property OnClose: TNotifyEvent read FOnClose write FOnClose;
    property OnShow: TNotifyEvent read FOnShow write FOnShow;
    property OnHide: TNotifyEvent read FOnHide write FOnHide;
    property OnMinimize: TNotifyEvent read FOnMinimize write FOnMinimize;
    property OnMaximize: TNotifyEvent read FOnMaximize write FOnMaximize;
    property OnResize: TNotifyEvent read FOnResize write FOnResize;
  end;

  TMiletusDBDriver = class;

  TMiletusClientDataSetErrorEvent = procedure(AErrorMessage: string) of object;
  
  {$IFNDEF LCLTMSWEB}
  [ComponentPlatforms(TMSWebPlatform)]
  {$ENDIF}
  TMiletusClientDataSet = class(TWebClientDataSet)
  private
    FQueryText: string;
    FDBDriver: TMiletusDBDriver;
    FOnError: TMiletusClientDataSetErrorEvent;
  protected
    procedure Notification(AComponent: TComponent; Operation: TOperation); override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure Refresh(Force: Boolean=False); reintroduce;
  published
    property DBDriver: TMiletusDBDriver read FDBDriver write FDBDriver;
    property QueryText: string read FQueryText write FQueryText;
    property OnError: TMiletusClientDataSetErrorEvent read FOnError write FOnError;
  end;

  TMiletusDBDriver = class(TComponent)
  private
    FDataBase: string;
    FPassword: string;
    FUserName: string;
    FPooled: Boolean;
    FMonitorByInitial: Boolean;
    FPoolExpireTimeout: Integer;
    FPoolCleanupTimeout: Integer;
    FPoolMaximumItems: Integer;
    FIndexName: string;
  protected
  public
    constructor Create(AOwner: TComponent); override;
    function SettingsAsJSON: TJSONObject; virtual;
  published
    property DataBase: string read FDataBase write FDataBase;
    property UserName: string read FUserName write FUserName;
    property Password: string read FPassword write FPassword;
    property IndexName: string read FIndexName write FIndexName;
    //MonitorBy
    property MonitorByInitial: Boolean read FMonitorByInitial write FMonitorByInitial default False;
    property PoolCleanupTimeout: Integer read FPoolCleanupTimeout write FPoolCleanupTimeout default 30000;
    property Pooled: Boolean read FPooled write FPooled default False;
    property PoolExpireTimeout: Integer read FPoolExpireTimeout write FPoolExpireTimeout default 90000;
    property PoolMaximumItems: Integer read FPoolMaximumItems write FPoolMaximumItems default 50;
  end;

  {$IFNDEF LCLTMSWEB}
  [ComponentPlatforms(TMSWebPlatform)]
  {$ENDIF}
  TMiletusAccessDBDriver = class(TMiletusDBDriver)
  private
    FSystemDB: string;
    FLoginTimeout: Integer;
    FReadOnly: Boolean;
    function IsSystemDBStored: Boolean;
  public
    constructor Create(AOwner: TComponent); override;
  published
    property LoginTimeout: Integer read FLoginTimeout write FLoginTimeout default 0;
    property ReadOnly: Boolean read FReadOnly write FReadOnly default False;
    //property StringFormat
    property SystemDB: string read FSystemDB write FSystemDB stored IsSystemDBStored;
  end;

  {$IFNDEF LCLTMSWEB}
  [ComponentPlatforms(TMSWebPlatform)]
  {$ENDIF}
  TMiletusMySQLDBDriver = class(TMiletusDBDriver)
  private
    FCompress: Boolean;
    FMetaCurCatalog: string;
    FSSL_key: string;
    FLoginTimeout: Integer;
    FReadTimeout: Integer;
    FMetaDefCatalog: string;
    FPort: Integer;
    FWriteTimeout: Integer;
    FSSL_cipher: string;
    FSSL_cert: string;
    FUseSSL: Boolean;
    FSSL_ca: string;
    FSSL_capath: string;
    FServer: string;
    function IsMetaCurCatalogStored: Boolean;
    function IsMetaDefCatalogStored: Boolean;
    function IsServerStored: Boolean;
    function IsSSLCapathStored: Boolean;
    function IsSSLCaStored: Boolean;
    function IsSSLCertStored: Boolean;
    function IsSSLCipherStored: Boolean;
    function IsSSLKeyStored: Boolean;
  public
    constructor Create(AOwner: TComponent); override;
  published
    //Characterset
    property Compress: Boolean read FCompress write FCompress default True;
    property LoginTimeout: Integer read FLoginTimeout write FLoginTimeout default 0;
    property MetaCurCatalog: string read FMetaCurCatalog write FMetaCurCatalog stored IsMetaCurCatalogStored;
    property MetaDefCatalog: string read FMetaDefCatalog write FMetaDefCatalog stored IsMetaDefCatalogStored;
    property Port: Integer read FPort write FPort default 3306;
    property ReadTimeout: Integer read FReadTimeout write FReadTimeout default 0;
    //ResultMode
    property Server: string read FServer write FServer stored IsServerStored;
    property SSL_ca: string read FSSL_ca write FSSL_ca stored IsSSLCaStored;
    property SSL_capath: string read FSSL_capath write FSSL_capath stored IsSSLCapathStored;
    property SSL_cert: string read FSSL_cert write FSSL_cert stored IsSSLCertStored;
    property SSL_cipher: string read FSSL_cipher write FSSL_cipher stored IsSSLCipherStored;
    property SSL_key: string read FSSL_key write FSSL_key stored IsSSLKeyStored;
    //TinyIntFormat
    property UseSSL: Boolean read FUseSSL write FUseSSL default False;
    property WriteTimeout: Integer read FWriteTimeout write FWriteTimeout default 0;
  end;

  {$IFNDEF LCLTMSWEB}
  [ComponentPlatforms(TMSWebPlatform)]
  {$ENDIF}
  TMiletusSQLiteDBDriver = class(TMiletusDBDriver)
  private
    FCacheSize: Integer;
    FBusyTimeout: Integer;
    FSQLiteAdvanced: string;
    FMetaCurCatalog: string;
    FExtensions: string;
    FMetaDefCatalog: string;
    FSharedCache: Boolean;
    function IsExtensionsStored: Boolean;
    function IsMetaCurCatalogStored: Boolean;
    function IsMetaDefCatalogStored: Boolean;
    function IsSQLiteAdvancedStored: Boolean;
  public
    constructor Create(AOwner: TComponent); override;
  published
    property BusyTimeout: Integer read FBusyTimeout write FBusyTimeout default 10000;
    property CacheSize: Integer read FCacheSize write FCacheSize default 10000;
    //property DateTimeFormat
    //property Encrypt
    property Extensions: string read FExtensions write FExtensions stored IsExtensionsStored;
    //property ForeignKeys
    //property GUIDFormat
    //property JournalMode
    //property LockingMode
    property MetaCurCatalog: string read FMetaCurCatalog write FMetaCurCatalog stored IsMetaCurCatalogStored;
    property MetaDefCatalog: string read FMetaDefCatalog write FMetaDefCatalog stored IsMetaDefCatalogStored;
    //property OpenMode
    property SharedCache: Boolean read FSharedCache write FSharedCache default True;
    property SQLiteAdvanced: string read FSQLiteAdvanced write FSQLiteAdvanced stored IsSQLiteAdvancedStored;
    //property StringFormat
    //property Synchronous
  end;

  TMiletusFormMessageEvent = procedure(Sender: TObject; FormID, AMessage: string) of object;
  TMiletusFormBorderStyle = (bsNoneBorder, bsSingleBorder, bsSizeableBorder, bsDialogBorder, bsToolWindowBorder, bsSizeToolWinBorder);

  TMiletusForm = class(TWebForm)
  private
    FOnFormMessage: TMiletusFormMessageEvent;
    FBorderStyle: TMiletusFormBorderStyle;
    procedure SetBorderStyle(const Value: TMiletusFormBorderStyle);
  public
    constructor Create(AOwner: TComponent); override;
    procedure RegisterForm(FormID: string);
    procedure SendMessage(FormID, AMessage: string);
  published
    property BorderStyle: TMiletusFormBorderStyle read FBorderStyle write SetBorderStyle default bsSizeableBorder;
    property BorderWidth default 0;
    property FormStyle default fsNormal;
    property Position default poDefaultPosOnly;
    property BorderIcons default [biSystemMenu, biMinimize, biMaximize];
    property DefaultMonitor default dmActiveForm;
    property ScreenSnap;
    property SnapBuffer default 10;
    property WindowState default wsNormal;
    property OnFormMessage: TMiletusFormMessageEvent read FOnFormMessage write FOnFormMessage;
  end;

  TMiletusCommunicationReceiveMessageEvent = procedure(AMessageID: Integer; AMessageData: TJSONValue) of object;

  TMiletusCommunication = class(TPersistent)
  public
    procedure RegisterObserver(Observer: TMiletusObserver);
    procedure UnregisterObserver(Observer: TMiletusObserver);
    procedure ReceiveMessageFromMiletus(const AMessage: string);
  end;

  TMiletusClientCommunicationMessageEvent = procedure (Sender: TObject; AMessageID: Integer; AMessageData: TJSONObject) of object;

  {$IFNDEF LCLTMSWEB}
  [ComponentPlatforms(TMSWebPlatform)]
  {$ENDIF}
  TMiletusClientCommunication = class(TComponent)
  private
    FOnMessage: TMiletusClientCommunicationMessageEvent;
  public
    procedure SendMessage(AMessageID: Integer; AMessageData: TJSONObject);
  published
    property OnMessage: TMiletusClientCommunicationMessageEvent read FOnMessage write FOnMessage;
  end;

function GetCursorPos: TPoint;
procedure GetMiletusPath(APathType: Integer; var APath: string);
procedure StartFileDrag(APath: string);
function MiletusShell: TMiletusShell;
function MiletusClipboard: TMiletusClipboard;
procedure OpenDevTools;

var
  MiletusCommunication: TMiletusCommunication;
  MiletusOpenDialogOption: array[TMiletusOpenDialogOption] of string = (
    'ofReadOnly', 'ofOverwritePrompt', 'ofHideReadOnly',
    'ofNoChangeDir', 'ofShowHelp', 'ofNoValidate', 'ofAllowMultiSelect',
    'ofExtensionDifferent', 'ofPathMustExist', 'ofFileMustExist', 'ofCreatePrompt',
    'ofShareAware', 'ofNoReadOnlyReturn', 'ofNoTestFileCreate', 'ofNoNetworkButton',
    'ofNoLongNames', 'ofOldStyleDialog', 'ofNoDereferenceLinks', 'ofEnableIncludeNotify',
    'ofEnableSizing', 'ofDontAddToRecent', 'ofForceShowHidden');

const
  {Component messages}
  NM_WINDOW_SHOW = $001;
  NM_WINDOW_CLOSE = $002;
  NM_WINDOW_SHOWMODAL = $003;
  NM_WINDOW_HIDE = $004;
  NM_WINDOW_LOADURL = $005;
  NM_WINDOW_RESIZE = $006;
  NM_WINDOW_MINIMIZE = $007;
  NM_WINDOW_MAXIMIZE = $008;
  NM_WINDOW_MSGREG = $009;
  NM_WINDOW_MSGSEND = $00A;
  NM_WINDOW_MSGGET = $00B;
  NM_WINDOW_UPDATE = $00C;
  NM_OPENDEVTOOLS = $00D;
  NM_VERSION_INFO = $00E;
  NM_VERSION_GET = $00F;
  NM_OPENDIALOG_OPEN = $020;
  NM_SAVEDIALOG_OPEN = $021;
  NM_ERRORDLG = $022;
  NM_MESSAGEBOX_OPEN = $023;
  NM_STRINGLIST_LOAD = $024;
  NM_STRINGLIST_SAVE = $025;
  NM_MAINMENU_UPDATE = $026;
  NM_MAINMENU_ITEMCLICK = $027;
  NM_POPUPMENU_OPEN = $028;
  NM_POPUPMENU_CLOSE = $029;
  NM_POPUPMENU_ITEMCLICK = $02A;
  NM_FILEWATCH_ADD = $02B;
  NM_FILEWATCH_REMOVE = $02C;
  NM_FILEWATCH_CHANGE = $02D;
  NM_SHORTCUT_ADD = $02E;
  NM_SHORTCUT_REMOVE = $02F;
  NM_SHORTCUT_CHANGE = $030;
  NM_TRAYICON_UPDATE = $031;
  NM_TRAYICON_CLICK = $032;
  NM_TRAYICON_MENUCLICK = $033;
  NM_DB_CONNECT = $034;
  NM_DB_GETROWS = $035;
  NM_DB_SETROWS = $036;
  NM_DB_GETFIELDS = $037;
  NM_DB_SETFIELDS = $038;
  NM_DB_INSERT = $039;
  NM_DB_MODIFY = $03A;
  NM_DB_DELETE = $03B;
  NM_DB_SETAUTOINC = $03C;
  NM_DB_ERROR = $03D;

  {Shell/function messages}
  NM_NOTIFICATION = $100;
  NM_BEEP = $101;
  NM_SHOWFILE = $102;
  NM_OPENPATH = $103;
  NM_OPENURL = $104;
  NM_DELETEFILE = $105;
  NM_STARTDRAG = $106;
  NM_GETCURSOR = $107;
  NM_GETPATH = $108;
  NM_CLIPBOARD_SET = $109;
  NM_CLIPBOARD_GET = $10A;
  NM_CLIPBOARD_FORMAT = $10B;

  {User defined}
  NM_USER = $400;

  {Path values}
  NP_APPDATA = $01;
  NP_APPPATH = $02;
  NP_DESKTOP = $03;
  NP_DOCUMENTS = $04;
  NP_DOWNLOADS = $05;
  NP_EXE = $06;
  NP_HOME = $07;
  NP_MUSIC = $08;
  NP_USERDATA = $09;
  NP_PICTURES = $0A;
  NP_TEMP = $0B;
  NP_VIDEOS = $0C;

  {Clipboard formats}
  NC_TEXT = $01;
  NC_BITMAP = $02;
  NC_METAFILEPICT = $03;
  NC_PICTURE = $04;
  NC_COMPONENT = $05;

implementation

function GetCursorPos: TPoint;
begin

end;

procedure GetMiletusPath(APathType: Integer; var APath: string);
begin

end;

procedure StartFileDrag(APath: string);
begin

end;

function MiletusShell: TMiletusShell;
begin
  Result := nil;
end;

function MiletusClipboard: TMiletusClipboard;
begin
  Result := nil;
end;

procedure OpenDevTools;
begin

end;

{ TMiletusStringList }

procedure TMiletusStringList.LoadFromFile(const FileName: string);
begin

end;

procedure TMiletusStringList.LoadFromFileAsync(const FileName: string;
  AProc: TMiletusLoadFileProc);
begin

end;

procedure TMiletusStringList.SaveToFile(const FileName: string);
begin
  inherited;

end;

procedure TMiletusStringList.SaveToFileAsync(const FileName: string);
begin

end;

{ TMiletusOpenDialog }

constructor TMiletusOpenDialog.Create(AOwner: TComponent);
begin
  inherited;
  FFiles := TStringList.Create;
  FDefaultExt := '';
  FFileName := '';
  FFilter := '';
  FInitialDir := '';
  FTitle := '';
  FFilterIndex := 1;
end;

destructor TMiletusOpenDialog.Destroy;
begin
  FFiles.Free;
  inherited;
end;

function TMiletusOpenDialog.Execute: Boolean;
begin
  Result := False;
end;

procedure TMiletusOpenDialog.Execute(AProc: TMiletusDialogProc);
begin

end;

{ TMiletusMessageBox }

constructor TMiletusMessageBox.Create(AOwner: TComponent);
begin
  inherited;
  FButtons := TStringList.Create;
  FVerificationChecked := False;
  FVerificationText := '';
  FTitle := '';
  FType := mbtNone;
  FText := '';
  FNoLink := True;
  FResponse := -1;
  FCaption := '';
end;

destructor TMiletusMessageBox.Destroy;
begin
  FButtons.Free;
  inherited;
end;

procedure TMiletusMessageBox.Execute(AProc: TMiletusMessageBoxProc);
begin

end;

procedure TMiletusMessageBox.SetButtons(const Value: TStringList);
begin
  FButtons.Assign(Value);
end;

{ TMiletusErrorBox }

constructor TMiletusErrorBox.Create(AOwner: TComponent);
begin
  inherited;
  FContent := '';
  FTitle := '';
end;

procedure TMiletusErrorBox.Execute;
begin

end;

{ TMiletusPopupMenu }

procedure TMiletusPopupMenu.CloseMenu;
begin

end;

procedure TMiletusPopupMenu.Popup(X, Y: Integer);
begin

end;

{ TMiletusFileWatch }

constructor TMiletusFileWatch.Create(ACollection: TCollection);
begin
  inherited;
  FFileName := '';
end;

{ TMiletusFileWatches }

function TMiletusFileWatches.Add: TMiletusFileWatch;
begin
  Result := TMiletusFileWatch(inherited Add);
end;

function TMiletusFileWatches.GetItem(Index: Integer): TMiletusFileWatch;
begin
  Result := TMiletusFileWatch(inherited Items[Index]);
end;

procedure TMiletusFileWatches.SetItem(Index: Integer;
  const Value: TMiletusFileWatch);
begin
  inherited Items[Index] := Value;
end;

{ TMiletusFileWatcher }

constructor TMiletusFileWatcher.Create(AOwner: TComponent);
begin
  inherited;
  FFiles := TMiletusFileWatches.Create(Self, TMiletusFileWatch);
end;

destructor TMiletusFileWatcher.Destroy;
begin
  FFiles.Free;
  inherited;
end;

{ TMiletusShortcuts }

function TMiletusShortcuts.Add: TMiletusShortcut;
begin
  Result := TMiletusShortcut(inherited Add);
end;

function TMiletusShortcuts.GetItem(Index: Integer): TMiletusShortcut;
begin
  Result := TMiletusShortcut(inherited Items[Index]);
end;

procedure TMiletusShortcuts.SetItem(Index: Integer;
  const Value: TMiletusShortcut);
begin
  inherited Items[Index] := Value;
end;

{ TMiletusGlobalShortcuts }

constructor TMiletusGlobalShortcuts.Create(AOwner: TComponent);
begin
  inherited;
  FShortcuts := TMiletusShortCuts.Create(Self, TMiletusShortcut);
end;

destructor TMiletusGlobalShortcuts.Destroy;
begin
  FShortcuts.Free;
  inherited;
end;

{ TMiletusNotificationCenter }

function TMiletusNotificationCenter.CreateNotification: TMiletusNotification;
begin
  Result := nil;
end;

function TMiletusNotificationCenter.CreateNotification(const AName,
  AAlertBody: string): TMiletusNotification;
begin
  Result := nil;
end;

procedure TMiletusNotificationCenter.PresentNotification(
  const ANotification: TMiletusNotification);
begin

end;

{ TMiletusTrayIcon }

constructor TMiletusTrayIcon.Create(AOwner: TComponent);
begin
  inherited;
  FHint := '';
  FIconURL := '';
end;

procedure TMiletusTrayIcon.Notification(AComponent: TComponent;
  Operation: TOperation);
begin
  inherited;
  if not (csDestroying in ComponentState) then
  begin
    if (Operation = opRemove) then
    begin
      if (AComponent = FPopupMenu) then
        FPopupMenu := nil;
    end;
  end;
end;

{ TMiletusShell }

procedure TMiletusShell.Beep;
begin

end;

procedure TMiletusShell.MoveItemToTrash(FullPath: string);
begin

end;

procedure TMiletusShell.OpenExternal(URL: string);
begin

end;

procedure TMiletusShell.OpenItem(FullPath: string);
begin

end;

procedure TMiletusShell.ShowItemInFolder(FullPath: string);
begin

end;

{ TMiletusWindow }

procedure TMiletusWindow.Close;
begin

end;

procedure TMiletusWindow.Hide;
begin

end;

procedure TMiletusWindow.LoadFromURL(URL: string);
begin

end;

procedure TMiletusWindow.Show;
begin

end;

procedure TMiletusWindow.ShowModal;
begin

end;

{ TMiletusForm }

constructor TMiletusForm.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FBorderStyle := bsSizeableBorder;
  inherited BorderStyle := bsSizeable;
  BorderIcons := [biSystemMenu, biMinimize, biMaximize];
  SnapBuffer := 10;
  BorderWidth := 0;
  FormStyle := fsNormal;
  Position := poDefaultPosOnly;
  DefaultMonitor := dmActiveForm;
  ScreenSnap := True;
  WindowState := wsNormal;
end;

procedure TMiletusForm.RegisterForm(FormID: string);
begin

end;

procedure TMiletusForm.SendMessage(FormID, AMessage: string);
begin

end;

procedure TMiletusForm.SetBorderStyle(const Value: TMiletusFormBorderStyle);
begin
  FBorderStyle := Value;
  case Value of
    bsNoneBorder: inherited BorderStyle := bsNone;
    bsSingleBorder: inherited BorderStyle := bsSingle;
    bsSizeableBorder: inherited BorderStyle := bsSizeable;
    bsDialogBorder: inherited BorderStyle := bsDialog;
    bsToolWindowBorder: inherited BorderStyle := bsToolWindow;
    bsSizeToolWinBorder: inherited BorderStyle := bsSizeToolWin;
  end;
end;

{ TMiletusCommunication }

procedure TMiletusCommunication.ReceiveMessageFromMiletus(const AMessage: string);
begin

end;

procedure TMiletusCommunication.RegisterObserver(Observer: TMiletusObserver);
begin

end;

procedure TMiletusCommunication.UnregisterObserver(Observer: TMiletusObserver);
begin

end;

{ TMiletusClientCommunication }

procedure TMiletusClientCommunication.SendMessage(AMessageID: Integer;
  AMessageData: TJSONObject);
begin

end;

{ TMiletusObserver }

constructor TMiletusObserver.Create(AOwner: TComponent);
begin
  inherited;
  FObserverEnabled := True;
end;

procedure TMiletusObserver.ObserverChange(AMessageID: Integer;
  AMessageData: TJSONValue);
begin
  //
end;

{ TMiletusClientDataSet }

constructor TMiletusClientDataSet.Create(AOwner: TComponent);
begin
  inherited;
  //
end;

destructor TMiletusClientDataSet.Destroy;
begin

  inherited;
end;

procedure TMiletusClientDataSet.Notification(AComponent: TComponent;
  Operation: TOperation);
begin
  inherited;
  if not (csDestroying in ComponentState) then
  begin
    if (Operation = opRemove) then
    begin
      if (AComponent = FDBDriver) then
        FDBDriver := nil;
    end;
  end;
end;

procedure TMiletusClientDataSet.Refresh(Force: Boolean);
begin

end;

{ TMiletusDBDriver }

constructor TMiletusDBDriver.Create(AOwner: TComponent);
begin
  inherited;
  FDataBase := '';
  FUserName := '';
  FPassword := '';
  //MonitorBy...
  FMonitorByInitial := False;
  FPoolCleanupTimeout := 30000;
  FPooled := False;
  FPoolExpireTimeout := 90000;
  FPoolMaximumItems := 50;
end;

function TMiletusDBDriver.SettingsAsJSON: TJSONObject;
begin
  Result := nil;
end;

{ TMiletusAccessDBDriver }

constructor TMiletusAccessDBDriver.Create(AOwner: TComponent);
begin
  inherited;
  FLoginTimeout := 0;
  FReadOnly := False;
  //StringFormat
  FSystemDB := '';
end;

function TMiletusAccessDBDriver.IsSystemDBStored: Boolean;
begin
  Result := FSystemDB <> '';
end;

{ TMiletusMySQLDBDriver }

constructor TMiletusMySQLDBDriver.Create(AOwner: TComponent);
begin
  inherited;
  FCompress := True;
  FLoginTimeout := 0;
  FMetaCurCatalog := '';
  FMetaDefCatalog := '';
  FPort := 3306;
  FReadTimeout := 0;
  //ResultMode
  FServer := '';
  FSSL_ca := '';
  FSSL_capath := '';
  FSSL_cert := '';
  FSSL_cipher := '';
  FSSL_key := '';
  //TinyIntFormat
  FUseSSL := False;
  FWriteTimeout := 0;
end;

function TMiletusMySQLDBDriver.IsMetaCurCatalogStored: Boolean;
begin
  Result := FMetaCurCatalog <> '';
end;

function TMiletusMySQLDBDriver.IsMetaDefCatalogStored: Boolean;
begin
  Result := FMetaDefCatalog <> '';
end;

function TMiletusMySQLDBDriver.IsServerStored: Boolean;
begin
  Result := FServer <> '';
end;

function TMiletusMySQLDBDriver.IsSSLCapathStored: Boolean;
begin
  Result := FSSL_capath <> '';
end;

function TMiletusMySQLDBDriver.IsSSLCaStored: Boolean;
begin
  Result := FSSL_ca <> '';
end;

function TMiletusMySQLDBDriver.IsSSLCertStored: Boolean;
begin
  Result := FSSL_cert <> '';
end;

function TMiletusMySQLDBDriver.IsSSLCipherStored: Boolean;
begin
  Result := FSSL_cipher <> '';
end;

function TMiletusMySQLDBDriver.IsSSLKeyStored: Boolean;
begin
  Result := FSSL_key <> '';
end;

{ TMiletusSQLiteDBDriver }

constructor TMiletusSQLiteDBDriver.Create(AOwner: TComponent);
begin
  inherited;
  FCacheSize := 10000;
  FBusyTimeout := 10000;
  FSQLiteAdvanced := '';
  FMetaCurCatalog := '';
  FExtensions := '';
  FMetaDefCatalog := '';
  FSharedCache := True;
end;

function TMiletusSQLiteDBDriver.IsExtensionsStored: Boolean;
begin
  Result := FExtensions <> '';
end;

function TMiletusSQLiteDBDriver.IsMetaCurCatalogStored: Boolean;
begin
  Result := FMetaCurCatalog <> '';
end;

function TMiletusSQLiteDBDriver.IsMetaDefCatalogStored: Boolean;
begin
  Result := FMetaDefCatalog <> '';
end;

function TMiletusSQLiteDBDriver.IsSQLiteAdvancedStored: Boolean;
begin
  Result := FSQLiteAdvanced <> '';
end;

{ TMiletusBinaryDataStream }

procedure TMiletusBinaryDataStream.LoadFromFile(const FileName: string);
begin

end;

procedure TMiletusBinaryDataStream.LoadFromFileAsync(const FileName: string;
  AProc: TMiletusLoadFileProc);
begin

end;

procedure TMiletusBinaryDataStream.LoadFromFileRequest(const FileName: string;
  AProc: TMiletusLoadFileProc);
begin

end;

procedure TMiletusBinaryDataStream.SaveToFile(const FileName: string);
begin

end;

procedure TMiletusBinaryDataStream.SaveToFileAsync(const FileName: string);
begin

end;

procedure TMiletusBinaryDataStream.SaveToFileRequest(const FileName: string);
begin

end;

{ TMiletusClipboard }

function TMiletusClipboard.HasFormat(AFormat: Word): Boolean;
begin
  Result := False;
end;

end.
