{********************************************************************}
{                                                                    }
{ written by TMS Software                                            }
{            copyright (c) 2019 - 2020                               }
{            Email : info@tmssoftware.com                            }
{            Web : http://www.tmssoftware.com                        }
{                                                                    }
{ The source code is given as is. The author is not responsible      }
{ for any possible damage done due to the use of this code.          }
{ The complete source code remains property of the author and may    }
{ not be distributed, published, given or sold in any form as such.  }
{ No parts of the source code can be included in any other component }
{ or application without written authorization of the author.        }
{********************************************************************}

unit WEBLib.Slider;

interface

uses
  {$IFNDEF LCLTMSWEB}
  Classes, VCL.Graphics, WEBLib.Controls, Web, jsdelphisystem, js,
  libswiper;
  {$ENDIF}
  {$IFDEF LCLTMSWEB}
  Classes, Graphics, WEBLib.Controls, Web, jsdelphisystem, js,
  libswiper;
  {$ENDIF}

type
  TWebImageSlider = class;
  TSliderAppearance = class;

  TNavigationBullets = class(TPersistent)
  private
    FOwner: TSliderAppearance;
    FOpacity: double;
    FColor, FColorActive: TColor;
    FSize, FSpaceBetween, FSpaceEdge: Integer;
  public
    constructor Create(AOwner: TSliderAppearance); overload;
    destructor Destroy; override;
    procedure Assign(Source: TPersistent); override;
  published
    property Color: TColor read FColor write FColor default clLtGray;
    property ColorActive: TColor read FColorActive write FColorActive default clBlue;
    property Size: Integer read FSize write FSize default 12;
    property SpaceBetween: Integer read FSpaceBetween write FSpaceBetween default 12;
    property SpaceEdge: Integer read FSpaceEdge write FSpaceEdge default 8;
    property Opacity: double read FOpacity write FOpacity;
  end;

  TNavigationThumbnails = class(TPersistent)
  private
    FOwner: TSliderAppearance;
    FSizePercent, FSpaceBetween, FNumDisplayed: Integer;
    FSizeAspectRatio: Boolean;
    FOpacity, FOpacityActive: double;

    FWidthActiveBorder: Integer;
    FColorActiveBorder: TColor;
  public
    constructor Create(AOwner: TSliderAppearance); overload;
    destructor Destroy; override;
    procedure Assign(Source: TPersistent); override;
  published
    property SizePercent: Integer read FSizePercent write FSizePercent default 20;
    property NumDisplayed: Integer read FNumDisplayed write FNumDisplayed default 6;
    property SizeAspectRatio: Boolean read FSizeAspectRatio write FSizeAspectRatio default False;
    property SpaceBetween: Integer read FSpaceBetween write FSpaceBetween default 8;
    property Opacity: double read FOpacity write FOpacity;
    property ColorActiveBorder: TColor read FColorActiveBorder write FColorActiveBorder default clBlack;
    property WidthActiveBorder: Integer read FWidthActiveBorder write FWidthActiveBorder default 3;
    property OpacityActive: double read FOpacityActive write FOpacityActive;
  end;

  TNavigationButtons = class(TPersistent)
  private
    FOwner: TSliderAppearance;
    FColor: TColor;
    FVisible: Boolean;
  public
    constructor Create(AOwner: TSliderAppearance); overload;
    destructor Destroy; override;
    procedure Assign(Source: TPersistent); override;
  published
    property Color: TColor read FColor write FColor default clNone;
    property Visible: Boolean read FVisible write FVisible default True;
  end;

  TNavigationStyle = (navsNone, navsThumbnails, navsBullets);
  TTransitionEffect = (tefSlide, tefFade, tefFlip);

  TSliderAppearance = class(TPersistent)
  private
    FOwner: TWebImageSlider;
    FNavigationStyle: TNavigationStyle;
    FTransitionEffect: TTransitionEffect;
    FBullets: TNavigationBullets;
    FThumbnails: TNavigationThumbnails;
    FButtons: TNavigationButtons;
    procedure SetBullets(const Value: TNavigationBullets);
    procedure SetThumbnails(const Value: TNavigationThumbnails);
    procedure SetButtons(const Value: TNavigationButtons);
  public
    constructor Create(AOwner: TWebImageSlider); overload;
    destructor Destroy; override;
    procedure Assign(Source: TPersistent); override;
  published
    property NavigationStyle: TNavigationStyle read FNavigationStyle write FNavigationStyle;
    property TransitionEffect: TTransitionEffect read FTransitionEffect write FTransitionEffect;
    property Bullets: TNavigationBullets read FBullets write SetBullets;
    property Thumbnails: TNavigationThumbnails read FThumbnails write SetThumbnails;
    property Buttons: TNavigationButtons read FButtons write SetButtons;
  end;

  TImageSliderImageEvent = procedure(Sender: TObject; ImageIndex: integer) of object;
  TImageSliderThumbnailEvent = procedure(Sender: TObject; ImageIndex: integer) of object;

  {$IFNDEF LCLTMSWEB}
  [ComponentPlatforms(TMSWebPlatform)]
  [JSLibReference('https://download.tmssoftware.com/tmsweb/slider/swiper.min.js', 'https://download.tmssoftware.com/tmsweb/slider/swiper.min.css')]
  {$ENDIF}
  TWebImageSlider = class(TWebCustomControl)
  private
    FImageSwiper: TJSSwiper;
    FThumbnailsSwiper: TJSSwiper;
    FAppearance: TSliderAppearance;
    FImageURLs: TStringList;
    FActiveImageIndex, FPreviousActiveIndex,
    FLastClickedIndex: Integer;

    FOnImageChange: TImageSliderImageEvent;
    FOnImageClick: TImageSliderImageEvent;
    FOnThumbnailClick: TImageSliderThumbnailEvent;

  protected
    procedure SetImageURLs(const Value: TStringList);
  public
    constructor Create(AOwner: TComponent); overload; override;
    destructor Destroy; override;

    procedure RefreshImages;
    procedure SlideTo(anIndex: Integer; delaymsecs: Integer=0);
    procedure SlideNext(delaymsecs: Integer=0);
    procedure SlidePrev(delaymsecs: Integer=0);
    procedure SlideFirst(delaymsecs: Integer = 0);
    procedure SlideLast(delaymsecs: Integer = 0);

    property ImageSwiper: TJSSwiper read FImageSwiper;
    property ThumbnailsSwiper: TJSSwiper read FThumbnailsSwiper;
    property ActiveImageIndex: Integer read FActiveImageIndex write FActiveImageIndex;
    property PreviousActiveIndex: Integer read FPreviousActiveIndex;
    property LastClickedIndex: Integer read FLastClickedIndex;

  published
    property ImageURLs: TStringList read FImageURLs write SetImageURLs;
    property Appearance: TSliderAppearance read FAppearance write FAppearance;
    property OnImageChange: TImageSliderImageEvent read FOnImageChange write FOnImageChange;
    property OnImageClick: TImageSliderImageEvent read FOnImageClick write FOnImageClick;
    property OnThumbnailClick: TImageSliderThumbnailEvent read FOnThumbnailClick write FOnThumbnailClick;

    property Align;
    {$IFNDEF LCLTMSWEB}
    property AlignWithMargins;
    {$ENDIF}
    property Anchors;
    property Color;
    property Height;
    property Left;
    {$IFNDEF LCLTMSWEB}
    property Margins;
    {$ENDIF}
    property Top;
    property Visible;
    property Width;

    property OnClick;
    property OnDblClick;
    property OnMouseDown;
    property OnMouseUp;
    property OnMouseMove;
  end;

implementation

const
  ThumbsClassName: String = 'slider-thumbs';

{ TNavigationBullets }

procedure TNavigationBullets.Assign(Source: TPersistent);
begin
  inherited;
  if (Source is TNavigationBullets) then
  begin
    FOpacity := (Source as TNavigationBullets).FOpacity;
    FColor := (Source as TNavigationBullets).FColor;
    FColorActive := (Source as TNavigationBullets).FColorActive;
    FSize := (Source as TNavigationBullets).FSize;
    FSpaceBetween := (Source as TNavigationBullets).FSpaceBetween;
    FSpaceEdge := (Source as TNavigationBullets).FSpaceEdge;
  end;
end;

constructor TNavigationBullets.Create(AOwner: TSliderAppearance);
begin
  FOwner := AOwner;
  FOpacity := 1;
  FColor := clLtGray;
  FColorActive := clBlue;
  FSize := 12;
  FSpaceBetween := 12;
  FSpaceEdge := 8;
end;

destructor TNavigationBullets.Destroy;
begin
  inherited;
end;

{ TNavigationThumbnails }

procedure TNavigationThumbnails.Assign(Source: TPersistent);
begin
  inherited;
  if (Source is TNavigationThumbnails) then
  begin
    FOpacity := (Source as TNavigationThumbnails).FOpacity;
    FColorActiveBorder := (Source as TNavigationThumbnails).FColorActiveBorder;
    FSizePercent := (Source as TNavigationThumbnails).FSizePercent;
    FSpaceBetween := (Source as TNavigationThumbnails).FSpaceBetween;
    FWidthActiveBorder := (Source as TNavigationThumbnails).FWidthActiveBorder;
    FNumDisplayed := (Source as TNavigationThumbnails).FNumDisplayed;
    FSizeAspectRatio := (Source as TNavigationThumbnails).FSizeAspectRatio;
    FOpacityActive := (Source as TNavigationThumbnails).FOpacityActive;
  end;
end;

constructor TNavigationThumbnails.Create(AOwner: TSliderAppearance);
begin
  FOwner := AOwner;
  FOpacity := 0.5;
  FOpacityActive := 1;
  FColorActiveBorder := clBlack;
  FWidthActiveBorder := 3;
  FSizePercent := 20;
  FSpaceBetween := 8;
  FNumDisplayed := 6;
  FSizeAspectRatio := False;
end;

destructor TNavigationThumbnails.Destroy;
begin
  inherited;
end;

{ TNavigationButtons }

procedure TNavigationButtons.Assign(Source: TPersistent);
begin
  inherited;
  if (Source is TNavigationButtons) then
  begin
    FColor := (Source as TNavigationButtons).FColor;
    FVisible := (Source as TNavigationButtons).FVisible;
  end;
end;

constructor TNavigationButtons.Create(AOwner: TSliderAppearance);
begin
  FOwner := AOwner;
  FColor := clNone;
  FVisible := True;
end;

destructor TNavigationButtons.Destroy;
begin
  inherited;
end;

{ TSliderAppearance }

procedure TSliderAppearance.Assign(Source: TPersistent);
begin
  inherited;
  if (Source is TSliderAppearance) then
  begin
    FBullets := (Source as TSliderAppearance).Bullets;
    FThumbnails := (Source as TSliderAppearance).Thumbnails;
    FButtons := (Source as TSliderAppearance).Buttons;
  end;
end;

constructor TSliderAppearance.Create(AOwner: TWebImageSlider);
begin
  inherited Create;
  FOwner := AOwner;
  FBullets := TNavigationBullets.Create(Self);
  FThumbnails := TNavigationThumbnails.Create(Self);
  FButtons := TNavigationButtons.Create(Self);
  FNavigationStyle := navsThumbnails;
end;

destructor TSliderAppearance.Destroy;
begin
  FBullets.Free;
  FThumbnails.Free;
  FButtons.Free;
  inherited;
end;

procedure TSliderAppearance.SetBullets(const Value: TNavigationBullets);
begin
  FBullets.Assign(Value);
end;

procedure TSliderAppearance.SetThumbnails(const Value: TNavigationThumbnails);
begin
  FThumbnails.Assign(Value);
end;

procedure TSliderAppearance.SetButtons(const Value: TNavigationButtons);
begin
  FButtons.Assign(Value);
end;

{ TWebImageSlider }


procedure TWebImageSlider.SetImageURLs(const Value: TStringList);
begin
  FImageURLs.Assign(Value);
end;

constructor TWebImageSlider.Create(AOwner: TComponent);
begin
  inherited;
  FAppearance := TSliderAppearance.Create(Self);
  FImageURLs := TStringList.Create;
  Width := 600;
  Height := 400;
end;

destructor TWebImageSlider.Destroy;
begin
  FAppearance.Free;
  FImageURLs.Free;
  inherited;
end;

procedure TWebImageSlider.RefreshImages;
begin
end;

procedure TWebImageSlider.SlideTo(anIndex: Integer; delaymsecs: Integer=0);
begin
  //
end;

procedure TWebImageSlider.SlideFirst(delaymsecs: Integer);
begin
  //
end;

procedure TWebImageSlider.SlideLast(delaymsecs: Integer);
begin
  //
end;

procedure TWebImageSlider.SlideNext(delaymsecs: Integer=0);
begin
  //
end;

procedure TWebImageSlider.SlidePrev(delaymsecs: Integer=0);
begin
  //
end;

end.
