{********************************************************************}
{                                                                    }
{ written by TMS Software                                            }
{            copyright (c) 2018                                      }
{            Email : info@tmssoftware.com                            }
{            Web : http://www.tmssoftware.com                        }
{                                                                    }
{ The source code is given as is. The author is not responsible      }
{ for any possible damage done due to the use of this code.          }
{ The complete source code remains property of the author and may    }
{ not be distributed, published, given or sold in any form as such.  }
{ No parts of the source code can be included in any other component }
{ or application without written authorization of the author.        }
{********************************************************************}
unit WEBLib.myCloudData;

interface

uses
  Classes, WEBLib.Controls, WEBLib.REST;

type
  TmyCloudDataMetaData = class;
  TmyCloudDataEntity = class;
  TmyCloudDataEntities = class;
  TmyCloudDataTable = class;

  TmyCloudDataTable = class(TCollectionItem)
  private
    FTableName: string;
    FTableID: string;
    FEntities: TmyCloudDataEntities;
    FMetaData: TmyCloudDataMetaData;
    FOnMetaData: TNotifyEvent;
    FOnEntities: TNotifyEvent;

  public
    constructor Create(ACollection: TCollection);  override;
    destructor Destroy; override;
    property MetaData: TmyCloudDataMetaData read FMetaData;
    property Entities: TmyCloudDataEntities read FEntities;
    procedure GetMetaData;
    procedure GetEntities;
  published
    property TableName: string read FTableName write FTableName;
    property TableID: string read FTableID write FTableID;
    property OnMetaData: TNotifyEvent read FOnMetaData write FOnMetaData;
    property OnEntities: TNotifyEvent read FOnEntities write FOnEntities;

  end;

  TmyCloudDataTables = class(TOwnedCollection)
  private
    function GetItem(Index: integer): TmyCloudDataTable;
    procedure SetItem(Index: integer; const Value: TmyCloudDataTable);

  public
    constructor Create(AOwner: TPersistent); reintroduce;
    function Add: TmyCloudDataTable;
    function Insert(Index: integer): TmyCloudDataTable;
    property Items[Index: integer]: TmyCloudDataTable read GetItem write SetItem; default;
  end;

  TFieldType = (ftUnknown, ftString, ftSmallint, ftInteger, ftWord, // 0..4
    ftBoolean, ftFloat, ftCurrency, ftBCD, ftDate, ftTime, ftDateTime, // 5..11
    ftBytes, ftVarBytes, ftAutoInc, ftBlob, ftMemo, ftGraphic, ftFmtMemo, // 12..18
    ftParadoxOle, ftDBaseOle, ftTypedBinary, ftCursor, ftFixedChar, ftWideString, // 19..24
    ftLargeint, ftADT, ftArray, ftReference, ftDataSet, ftOraBlob, ftOraClob, // 25..31
    ftVariant, ftInterface, ftIDispatch, ftGuid, ftTimeStamp, ftFMTBcd, // 32..37
    ftFixedWideChar, ftWideMemo, ftOraTimeStamp, ftOraInterval, // 38..41
    ftLongWord, ftShortint, ftByte, ftExtended, ftConnection, ftParams, ftStream, //42..48
    ftTimeStampOffset, ftObject, ftSingle); //49..51
  TFieldTypes = set of TFieldType;


  TMyCloudDataMetaDataTypedField = (tfNone, tfRadioButton, tfComboBox, tfCheckBox);

  TMyCloudDataMetaDataItem = class(TCollectionItem)
  private
    FFieldName: string;
    FDataType: TFieldType;
    FSize: integer;
    FEnabled: boolean;
    FLabelText: string;
    FLookupTable: int64;
    FMask: string;
    FWidth: integer;
    FMaximum: double;
    FOrder: integer;
    FMaximumDate: TDatetime;
    FLookupKeyField: string;
    FLookupField: string;
    FVisible: boolean;
    FDescription: string;
    FMinimum: double;
    FMinimumDate: TDatetime;
    FTypedField: TMyCloudDataMetaDataTypedField;
    FDefaultValue: string;
    FRequired: boolean;
  public
    constructor Create(ACollection: TCollection);  override;
    destructor Destroy; override;
  published
    property FieldName: string read FFieldName write FFieldName;
    property LabelText: string read FLabelText write FLabelText;
    property DefaultValue: string read FDefaultValue write FDefaultValue;
    property Width: integer read FWidth write FWidth;
    property Order: integer read FOrder write FOrder;
    property Mask: string read FMask write FMask;
    property Minimum: double read FMinimum write FMinimum;
    property Maximum: double read FMaximum write FMaximum;
    property MinimumDate: TDateTime read FMinimumDate write FMinimumDate;
    property MaximumDate: TDateTime read FMaximumDate write FMaximumDate;
    property Visible: boolean read FVisible write FVisible default true;
    property Enabled: boolean read FEnabled write FEnabled default true;
    property Required: boolean read FRequired write FRequired default false;
    property Description: string read FDescription write FDescription;
    property LookupTable: int64 read FLookupTable write FLookupTable;
    property LookupField: string read FLookupField write FLookupField;
    property LookupKeyField: string read FLookupKeyField write FLookupKeyField;
    property TypedField: TMyCloudDataMetaDataTypedField read FTypedField write FTypedField;
  end;

  TMyCloudDataMetaData = class(TOwnedCollection)
  private
    function GetItems(Index: Integer): TMyCloudDataMetaDataItem;
    procedure SetItems(Index: Integer; const Value: TMyCloudDataMetaDataItem);
  protected
    //function Compare(Item1, Item2 : TCollectionItem): Integer; virtual;
    //procedure QuickSort(L, R: Integer);
  public
    constructor Create(AOwner: TPersistent); reintroduce;
    function Add: TMyCloudDataMetaDataItem; overload;
    function Add(PropertyName: string; DataType: TFieldType; Size: integer = 0): TMyCloudDataMetaDataItem; overload;
    function Insert(Index: Integer): TMyCloudDataMetaDataItem;
    property Items[Index: Integer]: TMyCloudDataMetaDataItem read GetItems write SetItems; default;
    function GetMetaDataItemByName(AName: string): TMyCloudDataMetaDataItem;
    //procedure Sort;
  end;


  TMyCloudDataBlob = class(TPersistent)
  private
    //FDataStore: TAdvCustomMyCloudData;
    FTable: TMyCloudDataTable;
    FEntity: TMyCloudDataEntity;
    FField: string;
  public
    //constructor Create(ADataStore: TCustomDataStore);
    property Table: TMyCloudDataTable read FTable write FTable;
    property Entity: TMyCloudDataEntity read FEntity write FEntity;
    property Field: string read FField write FField;

    //function LoadFromFile(AFileName: string): boolean;
    //function SaveToFile(AFileName: string): boolean;
    //function LoadFromStream(AStream: TStream): boolean;
    //function SaveToStream(AStream: TStream): boolean;
  end;

  TmyCloudDataEntity = class(TCollectionItem)
  private
    //FBlob: TmyCloudDataBlob;
    FTable: TmyCloudDataTable;
    FID: string;
    FOnDelete: TNotifyEvent;
    FOnUpdate: TNotifyEvent;
    FOnInsert: TNotifyEvent;
    function GetValue(AName: string): Variant;
    procedure SetValue(AName: string; const Value: Variant);
    function GetBlob(AName: string): TMyCloudDataBlob;
    function GetIntID: int64;
  protected
    property Table: TmyCloudDataTable read FTable;
  public
    constructor Create(ACollection: TCollection);  override;
    destructor Destroy; override;
    procedure Assign(Source: TPersistent); override;
    //procedure FromJSON(jo: TJSONObject); override;
    property Value[AName: string]: Variant read GetValue write SetValue;
    property Blob[AName: string]: TMyCloudDataBlob read GetBlob;
    function Update: boolean;
    function Insert: boolean;
    function Delete: boolean;
    property IntID: int64 read GetIntID;
    property ID: string read FID write FID;

  published
    property OnInsert: TNotifyEvent read FOnInsert write FOnInsert;
    property OnUpdate: TNotifyEvent read FOnUpdate write FOnUpdate;
    property OnDelete: TNotifyEvent read FOnDelete write FOnDelete;
  end;

  TmyCloudDataEntities = class(TOwnedCollection)
  private
    FTable: TMyCloudDataTable;
    function GetItems(Index: Integer): TMyCloudDataEntity;
    procedure SetItems(Index: Integer; const Value: TMyCloudDataEntity);
  protected
    property Table: TMyCloudDataTable read FTable write FTable;
  public
    constructor Create(AOwner: TPersistent); reintroduce;
    function Add: TMyCloudDataEntity;
    function Insert(Index: Integer): TMyCloudDataEntity;
    property Items[Index: Integer]: TMyCloudDataEntity read GetItems write SetItems; default;
  end;

  {$IFNDEF LCLTMSWEB}
  [ComponentPlatforms(TMSWebPlatform)]
  {$ENDIF}
  TWebmyCloudData = class(TWebRESTClient)
  private
    FTables: TmyCloudDataTables;
    FOnTables: TNotifyEvent;
    procedure SetTables(const Value: TmyCloudDataTables);
  protected
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure TestTokens; override;
    procedure GetTables;
    property Tables: TmyCloudDataTables read FTables write SetTables;
    property OnTables: TNotifyEvent read FOnTables write FOnTables;
  end;

implementation

uses
  WEBLib.Dialogs, SysUtils;

{ TmyCloudData }

constructor TWebmyCloudData.Create(AOwner: TComponent);
begin
  inherited;
  FTables := TmyCloudDataTables.Create(Self);
end;

destructor TWebmyCloudData.Destroy;
begin
  FTables.Free;
  inherited;
end;

procedure TWebmyCloudData.GetTables;
begin
  //
end;

procedure TWebmyCloudData.SetTables(const Value: TmyCloudDataTables);
begin
  FTables := Value;
end;

procedure TWebmyCloudData.TestTokens;
begin
  //
end;

{ TmyCloudDataTables }

function TmyCloudDataTables.Add: TmyCloudDataTable;
begin
  Result := TmyCloudDataTable(inherited Add);
end;

constructor TmyCloudDataTables.Create(AOwner: TPersistent);
begin
  inherited Create(AOwner, TmyCloudDataTable);
end;

function TmyCloudDataTables.GetItem(Index: integer): TmyCloudDataTable;
begin
  Result := TmyCloudDataTable(inherited Items[Index]);
end;

function TmyCloudDataTables.Insert(Index: integer): TmyCloudDataTable;
begin
  Result := TmyCloudDataTable(inherited Insert(Index));
end;

procedure TmyCloudDataTables.SetItem(Index: integer;
  const Value: TmyCloudDataTable);
begin
  inherited Items[Index] := Value;
end;

{ TmyCloudDataTable }

constructor TmyCloudDataTable.Create(ACollection: TCollection);
begin
  inherited;
  FMetaData := TmyCloudDataMetaData.Create(Self);
  FEntities := TmyCloudDataEntities.Create(Self);
end;

destructor TmyCloudDataTable.Destroy;
begin
  FMetaData.Free;
  FEntities.Free;
  inherited;
end;

procedure TmyCloudDataTable.GetEntities;
begin
  //
end;

procedure TmyCloudDataTable.GetMetaData;
begin
  //
end;

{ TMyCloudDataMetaData }

function TMyCloudDataMetaData.Add(PropertyName: string; DataType: TFieldType;
  Size: integer): TMyCloudDataMetaDataItem;
begin
  Result := TMyCloudDataMetaDataItem(inherited Add);
  Result.FFieldName := PropertyName;
  Result.FDataType := DataType;
  Result.FSize := Size;
end;

function TMyCloudDataMetaData.Add: TMyCloudDataMetaDataItem;
begin
  Result := TMyCloudDataMetaDataItem(inherited Add);
end;

constructor TMyCloudDataMetaData.Create(AOwner: TPersistent);
begin
  inherited Create(AOwner, TMyCloudDataMetaDataItem);
end;

function TMyCloudDataMetaData.GetItems(
  Index: Integer): TMyCloudDataMetaDataItem;
begin
  Result := TMyCloudDataMetaDataItem(inherited Items[Index]);
end;

function TMyCloudDataMetaData.GetMetaDataItemByName(
  AName: string): TMyCloudDataMetaDataItem;
begin
  //
  Result := nil;
end;

function TMyCloudDataMetaData.Insert(Index: Integer): TMyCloudDataMetaDataItem;
begin
  Result := TMyCloudDataMetaDataItem(inherited Insert(Index));
end;

procedure TMyCloudDataMetaData.SetItems(Index: Integer;
  const Value: TMyCloudDataMetaDataItem);
begin
  inherited Items[Index] := Value;
end;

{ TmyCloudDataEntity }

procedure TmyCloudDataEntity.Assign(Source: TPersistent);
begin
  inherited;

end;

constructor TmyCloudDataEntity.Create(ACollection: TCollection);
begin
  inherited;
  FTable := TmyCloudDataEntities(ACollection).FTable;
end;

function TmyCloudDataEntity.Delete: boolean;
begin
  Result := true;
end;

destructor TmyCloudDataEntity.Destroy;
begin

  inherited;
end;

function TmyCloudDataEntity.GetBlob(AName: string): TMyCloudDataBlob;
begin
  Result := nil;
end;

function TmyCloudDataEntity.GetIntID: int64;
begin
  Result := StrToInt('');
end;

function TmyCloudDataEntity.GetValue(AName: string): Variant;
begin

end;

function TmyCloudDataEntity.Insert: boolean;
begin
  Result := true;
end;

procedure TmyCloudDataEntity.SetValue(AName: string; const Value: Variant);
begin

end;

function TmyCloudDataEntity.Update: boolean;
begin
  Result := true;
end;

{ TmyCloudDataEntities }

function TmyCloudDataEntities.Add: TMyCloudDataEntity;
begin
  Result := TMyCloudDataEntity(inherited Add);
end;

constructor TmyCloudDataEntities.Create(AOwner: TPersistent);
begin
  inherited Create(AOwner, TmyCloudDataEntity);

end;

function TmyCloudDataEntities.GetItems(Index: Integer): TMyCloudDataEntity;
begin
  Result := TMyCloudDataEntity(inherited Items[Index]);
end;

function TmyCloudDataEntities.Insert(Index: Integer): TMyCloudDataEntity;
begin
  Result := TMyCloudDataEntity(inherited Insert(Index));
end;

procedure TmyCloudDataEntities.SetItems(Index: Integer;
  const Value: TMyCloudDataEntity);
begin
  inherited Items[index] := Value;
end;

{ TMyCloudDataMetaDataItem }

constructor TMyCloudDataMetaDataItem.Create(ACollection: TCollection);
begin
  inherited;

end;

destructor TMyCloudDataMetaDataItem.Destroy;
begin

  inherited;
end;

end.
