unit JS;

{$IFDEF LCLTMSWEB}
{$mode delphi}
{$ENDIF}

interface

uses
  SysUtils, jsdelphisystem, Types;

type
  TJSArray = class;
  TJSTypedArray = class;

  {$IFNDEF LCLTMSWEB}
  AsyncAttribute = class(TCustomAttribute);
  {$ENDIF}

  EJS = class(TObject)
  private
    FMessage: string;
  public
    constructor Create(const Msg: string);
    property Message: string read FMessage write FMessage;
  end;
  TJSObjectPropertyDescriptor = JSValue;
  Float32 = Double;
  Float64 = Double;

  TJSObject = class
  private
    function GetProperties(Name: string): JSValue;
    procedure SetProperties(Name: string; const AValue: JSValue);
  public
    constructor new;
    class function create(const proto: TJSObject): TJSObject; overload;
    class function create(const proto: TJSObject; const propertiesObject: TJSObject): TJSObject; overload;
    class function assign(const Target: TJSObject; const Source1: TJSObject): TJSObject;
    class procedure defineProperty(const obj: TJSObject; propname: string; const descriptor: TJSObjectPropertyDescriptor);
    class function freeze(const obj: TJSObject): TJSObject;
    class function getOwnPropertyDescriptor(const obj: TJSObject; propname: string): TJSObjectPropertyDescriptor;
    class function getOwnPropertyNames(const obj: TJSObject): TStringDynArray;
    class function getPrototypeOf(const obj: TJSObject): TJSObject;
    class function isExtensible(const obj: TJSObject): Boolean;
    class function isFrozen(const obj: TJSObject): Boolean;
    class function isSealed(const obj: TJSObject): Boolean;
    class function keys(const obj: TJSObject): TStringDynArray;
    class function preventExtensions(const obj: TJSObject): TJSObject;
    class function seal(const obj: TJSObject): TJSObject;
    class function setPrototypeOf(const obj: TJSObject; const prototype: TJSObject): TJSObject;
    function hasOwnProperty(prop: string): Boolean;
    function isPrototypeOf(const obj: TJSObject): Boolean;
    function propertyIsEnumerable(propname: string): Boolean;
    function toLocaleString: string;
    function toString: string; reintroduce;
    function valueOf: JSValue;
    property Properties[Name : string]: JSValue read GetProperties write SetProperties; default;
  end;
  TJSObjectDynArray = Array of TJSObject;
  TJSObjectDynArrayArray = Array of TJSObjectDynArray;
  TJSStringDynArray = Array of String;

  TJSFunction = class(TJSObject)
  private
    Flength: NativeInt;
    Fprototyp: TJSFunction;
  public
    name: string;
    property prototyp: TJSFunction read Fprototyp;
    property length: NativeInt read Flength;
    function apply(thisArg: TJSObject; const ArgArray: TJSValueDynArray): JSValue;
    function bind(thisArg: TJSObject): JSValue;
    function call(thisArg: TJSObject): JSValue;
  end;

  TJSDate = class(TJSFunction)
  private
    function getDate: NativeInt;
    function getFullYear: NativeInt;
    function getHours: NativeInt;
    function getMilliseconds: NativeInt;
    function getMinutes: NativeInt;
    function getMonth: NativeInt;
    function getSeconds: NativeInt;
    function getYear: NativeInt;
    function getTime: NativeInt;
    function getUTCDate: NativeInt;
    function getUTCFullYear: NativeInt;
    function getUTCHours: NativeInt;
    function getUTCMilliseconds: NativeInt;
    function getUTCMinutes: NativeInt;
    function getUTCMonth: NativeInt;
    function getUTCSeconds: NativeInt;
    procedure setDate(const AValue: NativeInt);
    procedure setFullYear(const AValue: NativeInt);
    procedure setHours(const AValue: NativeInt);
    procedure setMilliseconds(const AValue: NativeInt);
    procedure setMinutes(const AValue: NativeInt);
    procedure setMonth(const AValue: NativeInt);
    procedure setSeconds(const AValue: NativeInt);
    procedure setYear(const AValue: NativeInt);
    procedure setTime(const AValue: NativeInt);
    procedure setUTCDate(const AValue: NativeInt);
    procedure setUTCFullYear(const AValue: NativeInt);
    procedure setUTCHours(const AValue: NativeInt);
    procedure setUTCMilliseconds(const AValue: NativeInt);
    procedure setUTCMinutes(const AValue: NativeInt);
    procedure setUTCMonth(const AValue: NativeInt);
    procedure setUTCSeconds(const AValue: NativeInt);
  public
    constructor New(const MilliSecsSince1970: NativeInt); overload;
    constructor New(const aDateString: string); overload;
    constructor New(aYear: NativeInt; aMonth: NativeInt; aDayOfMonth: NativeInt = 1; TheHours: NativeInt = 0; TheMinutes: NativeInt = 0; TheSeconds: NativeInt = 0; TheMilliseconds: NativeInt = 0); overload;
    class function now: NativeInt;
    class function parse(const aDateString: string): NativeInt;
    class function UTC(aYear: NativeInt; aMonth: NativeInt = 0; aDayOfMonth: NativeInt = 1; TheHours: NativeInt = 0; TheMinutes: NativeInt = 0; TheSeconds: NativeInt = 0; TheMilliseconds: NativeInt = 0): NativeInt;
    function getDay: NativeInt;
    function getTimezoneOffset: NativeInt;
    function getUTCDay: NativeInt;
    function toDateString: string;
    function toISOString: string;
    function toJSON: string;
    function toGMTString: string;
    function toLocaleDateString: string;
    function toLocaleString: string; reintroduce;
    function toLocaleTimeString: string;
    function toTimeString: string;
    function toUTCString: string;
    property Year: NativeInt read getYear write setYear;
    property Time: NativeInt read getTime write setTime;
    property FullYear: NativeInt read getFullYear write setFullYear;
    property UTCDate: NativeInt read getUTCDate write setUTCDate;
    property UTCFullYear: NativeInt read getUTCFullYear write setUTCFullYear;
    property UTCHours: NativeInt read getUTCHours write setUTCHours;
    property UTCMilliseconds: NativeInt read getUTCMilliseconds write setUTCMilliseconds;
    property UTCMinutes: NativeInt read getUTCMinutes write setUTCMinutes;
    property UTCMonth: NativeInt read getUTCMonth write setUTCMonth;
    property UTCSeconds: NativeInt read getUTCSeconds write setUTCSeconds;
    property Month: NativeInt read getMonth write setMonth;
    property Date: NativeInt read getDate write setDate;
    property Hours: NativeInt read getHours write setHours;
    property Minutes: NativeInt read getMinutes write setMinutes;
    property Seconds: NativeInt read getSeconds write setSeconds;
    property Milliseconds: NativeInt read getMilliseconds write setMilliseconds;
  end;
  TLocaleCompareOptions = record
    localematched : string;
    usage : string;
    sensitivity : string;
    ignorePunctuation : Boolean;
    numeric : Boolean;
    caseFirst : string;
  end;

  TJSRegexp = class
  private
    fglobal: Boolean;
    fignoreCase: Boolean;
    fmultiline: Boolean;
    fsource: string;
    funicode: Boolean;
  public
    constructor New(Pattern: string); overload;
    constructor New(Pattern: string; Flags: string); overload;
  public
    lastIndex: NativeInt;
    function exec(aString: string): TStringDynArray;
    function test(aString: string): Boolean;
    function toString: string; reintroduce;
    property Global: Boolean read fglobal;
    property IgnoreCase: Boolean read FIgnoreCase;
    property Multiline: Boolean read FMultiLine;
    property Source: string read FSource;
    property Unicode: Boolean read FUnicode;
  end;
  TReplaceCallBack = function : string;

  TJSString = class
  private
    flength: NativeInt;
  public
    constructor New(const S: string); overload;
    constructor New(const I: NativeInt); overload;
    constructor New(const D: Double); overload;
    property length: NativeInt read flength;
    class function fromCharCode: string;
    class function fromCodePoint(const AValue: string): string; overload;
    class function fromCodePoint(const AValue1, AValue2: string): string; overload;
    class function fromCodePoint(const AValue1, AValue2, AValue3: string): string; overload;
    class function fromCodePoint(const AValue1, AValue2, AValue3, AValue4: string): string; overload;
    class function fromCodePoint(const AValue1, AValue2, AValue3, AValue4, AValue5: string): string; overload;
    function anchor(const aName: string): string;
    function charAt(aIndex: NativeInt): string;
    function charCodeAt(aIndex: NativeInt): NativeInt;
    function codePointAt(aIndex: NativeInt): NativeInt;
    function concat(s: string): string;
    function endsWith(aSearchString: string; Pos: NativeInt = 0): Boolean;
    function includes(aSearchString: string; Pos: NativeInt = 0): Boolean;
    function indexOf(aSearchString: string; Pos: NativeInt = 0): Integer;
    function lastIndexOf(aSearchString: string): NativeInt; overload;
    function lastIndexOf(aSearchString: string; Pos: NativeInt): Integer; overload;
    function link(aUrl: string): string;
    function localeCompare(aCompareString: string): NativeInt; overload;
    function localeCompare(aCompareString: string; aLocales: string): Integer; overload;
    function localeCompare(compareString: string; locales: string; Options: TlocaleCompareOptions): Integer; overload;
    function match(aRegexp: TJSRegexp): TStringDynArray; overload;
    function match(aRegexp: string): TStringDynArray; overload;
    function _repeat(aCount: NativeInt): Integer;
    function replace(aRegexp: string; NewString: string): string; overload;
    function replace(aRegexp: TJSRegexp; NewString: string): string; overload;
    function replace(Regexp: string; aCallback: TReplaceCallBack): string; overload;
    function replace(Regexp: TJSRegexp; aCallback: TReplaceCallBack): string; overload;
    function search(Regexp: TJSRegexp): NativeInt; overload;
    function search(Regexp: JSValue): NativeInt; overload;
    function slice(aBeginIndex: NativeInt): string; overload;
    function slice(aBeginIndex: NativeInt; aEndIndex: NativeInt): string; overload;
    function split: TStringDynArray; overload;
    function split(aSeparator: string): TStringDynArray; overload;
    function split(aSeparator: string; aLimit: NativeInt): TStringDynArray; overload;
    function startsWith(aSearchString: string): Boolean; overload;
    function startsWith(aSearchString: string; aPosition: NativeInt): Boolean; overload;
    function substr(aStartIndex: NativeInt): string; overload;
    function substr(aStartIndex: NativeInt; aLength: NativeInt): string; overload;
    function subString(aStartIndex: NativeInt): string; overload;
    function subString(aStartIndex: NativeInt; aEndIndex: NativeInt): string; overload;
    function toLocaleLowerCase: string;
    function toLocaleUpperCase: string;
    function toLowerCase: string;
    function toString: string; reintroduce;
    function toUpperCase: string;
    function trim: string;
    function valueOf: string;
  end;
  TJSArrayCallBack = function(element: JSValue; index: NativeInt; 
                            anArray: TJSArray) : Boolean;
  TJSArrayEvent = function(element: JSValue; index: NativeInt; 
                         anArray: TJSArray) : Boolean of object;
  TJSArrayMapCallBack = function(element: JSValue; index: NativeInt; 
                               anArray: TJSArray) : JSValue;
  TJSArrayMapEvent = function(element: JSValue; index: NativeInt; 
                            anArray: TJSArray) : JSValue of object;
  TJSArrayReduceCallBack = function(accumulator: JSValue; 
                                  currentValue: JSValue; 
                                  currentIndex: NativeInt; 
                                  anArray: TJSArray) : JSValue;
  TJSArrayCompareCallBack = function(a: JSValue; b: JSValue) : NativeInt;

  TJSArray = class
  private
    function GetElements(Index: NativeInt): JSValue;
    procedure SetElements(Index: NativeInt; const AValue: JSValue);
  public
    flength: Integer;
    constructor new; overload;
    constructor new(aLength: NativeInt); overload;
    constructor new(aElement1: JSValue); overload;
    class function _of: TJSArray;
    class function isArray(a: JSValue): Boolean;
    function concat(el: JSValue): TJSArray;
    function copyWithin(aTarget: NativeInt): TJSarray; overload;
    function copyWithin(aTarget: NativeInt; aStart: NativeInt): TJSarray; overload;
    function copyWithin(aTarget: NativeInt; aStart: NativeInt; aEnd: NativeInt): TJSarray; overload;
    function every(const aCallback: TJSArrayCallBack): Boolean; overload;
    function every(const aCallback: TJSArrayEvent; aThis: TObject): Boolean; overload;
    function filter(const aCallBack: TJSArrayCallBack): TJSArray; overload;
    function filter(const aCallBack: TJSArrayEvent; aThis: TObject): TJSArray; overload;
    function fill(aValue: JSValue): TJSArray; overload;
    function fill(aValue: JSValue; aStartIndex: NativeInt): TJSArray; overload;
    function fill(aValue: JSValue; aStartIndex: NativeInt; aEndIndex: NativeInt): TJSArray; overload;
    function find(const aCallBack: TJSArrayCallBack): JSValue; overload;
    function find(const aCallBack: TJSArrayEvent; aThis: TObject): JSValue; overload;
    function findIndex(const aCallBack: TJSArrayCallBack): NativeInt; overload;
    function findIndex(const aCallBack: TJSArrayEvent; aThis: TObject): NativeInt; overload;
    procedure forEach(const aCallBack: TJSArrayCallBack); overload;
    procedure forEach(const aCallBack: TJSArrayEvent; aThis: TObject); overload;
    function includes(aElement: JSValue): Boolean; overload;
    function includes(aElement: JSValue; FromIndex: NativeInt): Boolean; overload;
    function indexOf(aElement: JSValue): NativeInt; overload;
    function indexOf(aElement: JSValue; FromIndex: NativeInt): NativeInt; overload;
    function join: string; overload;
    function join(aSeparator: string): string; overload;
    function lastIndexOf(aElement: JSValue): NativeInt; overload;
    function lastIndexOf(aElement: JSValue; FromIndex: NativeInt): NativeInt; overload;
    function map(const aCallBack: TJSArrayCallBack): TJSArray; overload;
    function map(const aCallBack: TJSArrayEvent; aThis: TObject): TJSArray; overload;
    function pop: JSValue;
    function push(aElement: JSValue): NativeInt;
    function reduce(const aCallBack: TJSArrayReduceCallBack): JSValue; overload;
    function reduce(const aCallBack: TJSArrayReduceCallBack; initialValue: JSValue): JSValue; overload;
    function reduceRight(const aCallBack: TJSArrayReduceCallBack): JSValue; overload;
    function reduceRight(const aCallBack: TJSArrayReduceCallBack; initialValue: JSValue): JSValue; overload;
    function reverse: TJSArray;
    function shift: JSValue;
    function slice: TJSArray; overload;
    function slice(aBegin: NativeInt): TJSArray; overload;
    function slice(aBegin: NativeInt; aEnd: NativeInt): TJSArray; overload;
    function some(const aCallback: TJSArrayCallBack): Boolean; overload;
    function some(const aCallback: TJSArrayEvent; aThis: TObject): Boolean; overload;
    function sort(const aCallback: TJSArrayCompareCallBack): TJSArray; overload;
    function sort: TJSArray; overload;
    function splice(aStart: NativeInt): TJSArray; overload;
    function splice(aStart: NativeInt; aDeleteCount: NativeInt): TJSArray; overload;
    function toLocaleString: string; overload;
    function toLocaleString(locales: string): string; overload;
    function toLocaleString(locales: string; const Options: TLocaleCompareOptions): string; overload;
    function toString: string; reintroduce;
    function unshift: NativeInt;
    property Length: Integer read FLength;
    property Elements[Index : NativeInt]: JSValue read GetElements write SetElements; default;
  end;

  TJSArrayBuffer = class
  private
    fLength: NativeInt;
  public
    constructor new(aByteLength: NativeInt);
    class function isView(aValue: JSValue): Boolean;
    function slice(aBegin: NativeInt): TJSArrayBuffer; overload;
    function slice(aBegin: NativeInt; aEnd: NativeInt): TJSArrayBuffer; overload;
    property byteLength: NativeInt read fLength;
  end;

  TJSBufferSource = class
  end;
  TJSTypedArrayCallBack = function(element: JSValue; index: NativeInt; 
                                 anArray: TJSTypedArray) : Boolean;
  TJSTypedArrayEvent = function(element: JSValue; index: NativeInt; 
                              anArray: TJSTypedArray) : Boolean
                               of object;
  TJSTypedArrayMapCallBack = function(element: JSValue; index: NativeInt; 
                                    anArray: TJSTypedArray) : JSValue;
  TJSTypedArrayMapEvent = function(element: JSValue; index: NativeInt; 
                                 anArray: TJSTypedArray) : JSValue
                                  of object;
  TJSTypedArrayReduceCallBack = function(accumulator: JSValue; 
                                       currentValue: JSValue; 
                                       currentIndex: NativeInt; 
                                       anArray: TJSTypedArray) : JSValue;
  TJSTypedArrayCompareCallBack = function(a: JSValue; b: JSValue)
                                         : NativeInt;

  TJSTypedArray = class(TJSBufferSource)
  private
    FBuffer: TJSArrayBuffer;
    FByteLength: NativeInt;
    FLength: NativeInt;
    FByteOffset: NativeInt;
    function getValue(Index: NativeInt): JSValue;
    procedure setValue(Index: NativeInt; AValue: JSValue);
  public
  class var
    BYTES_PER_ELEMENT: NativeInt;
    name: string;
    class function from(aValue: jsValue): TJSTypedArray; overload;
    class function from(aValue: jsValue; Map: TJSTypedArrayMapCallBack): TJSTypedArray; overload;
    class function from(aValue: jsValue; aMap: TJSTypedArrayMapEvent): TJSTypedArray; overload;
    class function _of(aValue: jsValue): TJSTypedArray;
    function copyWithin(aTarget: NativeInt): TJSTypedArray; overload;
    function copyWithin(aTarget: NativeInt; aStart: NativeInt): TJSTypedArray; overload;
    function copyWithin(aTarget: NativeInt; aStart: NativeInt; aEnd: NativeInt): TJSTypedArray; overload;
    function every(const aCallback: TJSTypedArrayCallBack): Boolean; overload;
    function every(const aCallback: TJSTypedArrayEvent; aThis: TObject): Boolean; overload;
    function fill(aValue: JSValue): TJSTypedArray; overload;
    function fill(aValue: JSValue; aStartIndex: NativeInt): TJSTypedArray; overload;
    function fill(aValue: JSValue; aStartIndex: NativeInt; aEndIndex: NativeInt): TJSTypedArray; overload;
    function filter(const aCallBack: TJSTypedArrayCallBack): TJSTypedArray; overload;
    function filter(const aCallBack: TJSTypedArrayEvent; aThis: TObject): TJSTypedArray; overload;
    function find(const aCallBack: TJSTypedArrayCallBack): JSValue; overload;
    function find(const aCallBack: TJSTypedArrayEvent; aThis: TObject): JSValue; overload;
    function findIndex(const aCallBack: TJSTypedArrayCallBack): NativeInt; overload;
    function findIndex(const aCallBack: TJSTypedArrayEvent; aThis: TObject): NativeInt; overload;
    procedure forEach(const aCallBack: TJSTypedArrayCallBack); overload;
    procedure forEach(const aCallBack: TJSTypedArrayEvent; aThis: TObject); overload;
    function includes(aElement: JSValue): Boolean; overload;
    function includes(aElement: JSValue; FromIndex: NativeInt): Boolean; overload;
    function indexOf(aElement: JSValue): NativeInt; overload;
    function indexOf(aElement: JSValue; FromIndex: NativeInt): NativeInt; overload;
    function join: string; overload;
    function join(aSeparator: string): string; overload;
    function lastIndexOf(aElement: JSValue): NativeInt; overload;
    function lastIndexOf(aElement: JSValue; FromIndex: NativeInt): NativeInt; overload;
    function map(const aCallBack: TJSTypedArrayCallBack): TJSTypedArray; overload;
    function map(const aCallBack: TJSTypedArrayEvent; aThis: TObject): TJSTypedArray; overload;
    function reduce(const aCallBack: TJSTypedArrayReduceCallBack): JSValue; overload;
    function reduce(const aCallBack: TJSTypedArrayReduceCallBack; initialValue: JSValue): JSValue; overload;
    function reduceRight(const aCallBack: TJSTypedArrayReduceCallBack): JSValue; overload;
    function reduceRight(const aCallBack: TJSTypedArrayReduceCallBack; initialValue: JSValue): JSValue; overload;
    function reverse: TJSTypedArray;
    procedure _set(anArray: TJSTypedArray); overload;
    procedure _set(anArray: TJSTypedArray; anOffset: NativeInt); overload;
    function slice: TJSTypedArray; overload;
    function slice(aBegin: NativeInt): TJSTypedArray; overload;
    function slice(aBegin: NativeInt; aEnd: NativeInt): TJSTypedArray; overload;
    function some(const aCallback: TJSTypedArrayCallBack): Boolean; overload;
    function some(const aCallback: TJSTypedArrayEvent; aThis: TObject): Boolean; overload;
    function sort(const aCallback: TJSTypedArrayCompareCallBack): TJSTypedArray; overload;
    function sort: TJSTypedArray; overload;
    function splice(aStart: NativeInt): TJSTypedArray; overload;
    function splice(aStart: NativeInt; aDeleteCount: NativeInt): TJSTypedArray; overload;
    function toLocaleString: string; overload;
    function toLocaleString(locales: string): string; overload;
    function toLocaleString(locales: string; const Options: TLocaleCompareOptions): string; overload;
    function toString: string; reintroduce;
    function unshift: NativeInt;
    property buffer: TJSArrayBuffer read FBuffer;
    property byteLength: NativeInt read FByteLength;
    property byteOffset: NativeInt read FByteOffset;
    property length: NativeInt read FLength;
    property values[Index : NativeInt]: JSValue read getValue write SetValue; default;
  end;

  TJSInt8Array = class(TJSTypedArray)
  private
    function getTypedValue(Index: Integer): ShortInt;
    procedure setTypedValue(Index: Integer; AValue: ShortInt);
  public
    property values[Index : Integer]: ShortInt read getTypedValue write setTypedValue; default;
  end;

  TJSUint8Array = class(TJSTypedArray)
  private
    function getTypedValue(Index: Integer): Byte;
    procedure setTypedValue(Index: Integer; AValue: Byte);
  public
    property values[Index : Integer]: Byte read getTypedValue write setTypedValue; default;
  end;

  TJSUint8ClampedArray = class(TJSTypedArray)
  private
    function getTypedValue(Index: Integer): Byte;
    procedure setTypedValue(Index: Integer; AValue: Byte);
  public
    property values[Index : Integer]: Byte read getTypedValue write setTypedValue; default;
  end;

  TJSInt16Array = class(TJSTypedArray)
  private
    function getTypedValue(Index: Integer): SmallInt;
    procedure setTypedValue(Index: Integer; AValue: SmallInt);
  public
    property values[Index : Integer]: SmallInt read getTypedValue write setTypedValue; default;
  end;

  TJSUint16Array = class(TJSTypedArray)
  private
    function getTypedValue(Index: Integer): Word;
    procedure setTypedValue(Index: Integer; AValue: Word);
  public
    property values[Index : Integer]: Word read getTypedValue write setTypedValue; default;
  end;

  TJSInt32Array = class(TJSTypedArray)
  private
    function getTypedValue(Index: Integer): LongInt;
    procedure setTypedValue(Index: Integer; AValue: LongInt);
  public
    property values[Index : Integer]: LongInt read getTypedValue write setTypedValue; default;
  end;

  TJSUint32Array = class(TJSTypedArray)
  private
    function getTypedValue(Index: Integer): LongWord;
    procedure setTypedValue(Index: Integer; AValue: LongWord);
  public
    property values[Index : Integer]: LongWord read getTypedValue write setTypedValue; default;
  end;

  TJSFloat32Array = class(TJSTypedArray)
  private
    function getTypedValue(Index: Integer): Float32;
    procedure setTypedValue(Index: Integer; AValue: Float32);
  public
    property values[Index : Integer]: Float32 read getTypedValue write setTypedValue; default;
  end;

  TJSFloat64Array = class(TJSTypedArray)
  private
    function getTypedValue(Index: Integer): Float64;
    procedure setTypedValue(Index: Integer; AValue: Float64);
  public
    property values[Index : Integer]: Float64 read getTypedValue write setTypedValue; default;
  end;

  TJSDataView = class(TJSBufferSource)
  private
    fBuffer: TJSArrayBuffer;
    fLength: NativeInt;
    fOffset: NativeInt;
  public
    constructor new(aBuffer: TJSArrayBuffer); overload;
    constructor new(aBuffer: TJSArrayBuffer; aOffset: NativeInt); overload;
    constructor new(aBuffer: TJSArrayBuffer; aOffset: NativeInt; aByteLength: NativeInt); overload;
    function getFloat32(aByteOffset: NativeInt): Double; overload;
    function getFloat32(aByteOffset: NativeInt; aLittleEndian: Boolean): Double; overload;
    function getFloat64(aByteOffset: NativeInt): Double; overload;
    function getFloat64(aByteOffset: NativeInt; aLittleEndian: Boolean): Double; overload;
    function getInt8(aByteOffset: NativeInt): ShortInt;
    function getInt16(aByteOffset: NativeInt): SmallInt; overload;
    function getInt16(aByteOffset: NativeInt; aLittleEndian: Boolean): SmallInt; overload;
    function getInt32(aByteOffset: NativeInt): LongInt; overload;
    function getInt32(aByteOffset: NativeInt; aLittleEndian: Boolean): LongInt; overload;
    function getUint8(aByteOffset: NativeInt): Byte; overload;
    function getUint16(aByteOffset: NativeInt): Word; overload;
    function getUint16(aByteOffset: NativeInt; aLittleEndian: Boolean): Word; overload;
    function getUint32(aByteOffset: NativeInt): LongWord; overload;
    function getUint32(aByteOffset: NativeInt; aLittleEndian: Boolean): LongWord; overload;
    procedure setFloat32(aByteOffset: NativeInt; aValue: Double); overload;
    procedure setFloat32(aByteOffset: NativeInt; aValue: Double; aLittleEndian: Boolean); overload;
    procedure setFloat64(aByteOffset: NativeInt; aValue: Double); overload;
    procedure setFloat64(aByteOffset: NativeInt; aValue: Double; aLittleEndian: Boolean); overload;
    procedure setInt8(aByteOffset: NativeInt; aValue: ShortInt);
    procedure setInt16(aByteOffset: NativeInt; aValue: SmallInt); overload;
    procedure setInt16(aByteOffset: NativeInt; aValue: SmallInt; aLittleEndian: Boolean); overload;
    procedure setInt32(aByteOffset: NativeInt; aValue: LongInt); overload;
    procedure setInt32(aByteOffset: NativeInt; aValue: LongInt; aLittleEndian: Boolean); overload;
    procedure setUint8(aByteOffset: NativeInt; aValue: Byte); overload;
    procedure setUint16(aByteOffset: NativeInt; aValue: Word); overload;
    procedure setUint16(aByteOffset: NativeInt; aValue: Word; aLittleEndian: Boolean); overload;
    procedure setUint32(aByteOffset: NativeInt; aValue: LongWord); overload;
    procedure setUint32(aByteOffset: NativeInt; aValue: LongWord; aLittleEndian: Boolean); overload;
    property byteLength: NativeInt read fLength;
    property byteOffset: NativeInt read fOffset;
    property buffer: TJSArrayBuffer read fBuffer;
  end;

  TJSJSON = class(TJSObject)
  public
    class function parse(aJSON: string): TJSObject;
    class function parseObject(aJSON : String) : TJSObject;
    class function stringify(aValue: JSValue): string; overload;
    class function stringify(aValue: JSValue; aReplacer: JSValue): string; overload;
    class function stringify(aValue: JSValue; aReplacer: JSValue; space: NativeInt): string; overload;
    class function stringify(aValue: JSValue; aReplacer: JSValue; space: string): string; overload;
  end;

  {$IFNDEF LCLTMSWEB}
  TJSPromiseResolver = reference to function (aValue : JSValue) : JSValue;
  TJSPromiseExecutor = reference to procedure (resolve,reject : TJSPromiseResolver);
  TJSPromiseFinallyHandler = reference to procedure;
  {$ENDIF}
  {$IFDEF LCLTMSWEB}
  TJSPromiseResolver = function (aValue : JSValue) : JSValue;
  TJSPromiseExecutor = procedure (resolve,reject : TJSPromiseResolver);
  TJSPromiseFinallyHandler = procedure;
  TProc = procedure;
  {$ENDIF}
  TJSPromise = Class;
  TJSPromiseArray = array of TJSPromise;

  TJSPromise = class(TObject)
  public
    constructor new(Executor : TJSPromiseExecutor);
    class function all(arg : Array of JSValue) : TJSPromise; overload;
    class function all(arg : JSValue) : TJSPromise; overload;
    class function all(arg : TJSPromiseArray) : TJSPromise; overload;
    class function race(arg : Array of JSValue) : TJSPromise; overload;
    class function race(arg : JSValue) : TJSPromise; overload;
    class function race(arg : TJSPromiseArray) : TJSPromise; overload;
    class function reject(reason : JSValue) : TJSPromise;
    class function resolve(value : JSValue): TJSPromise; overload;
    class function resolve : TJSPromise; overload;
    function _then (onAccepted : TJSPromiseResolver) : TJSPromise; overload;
    function _then (onAccepted,OnRejected: TJSPromiseResolver) : TJSPromise; overload;
    function catch (onRejected : TJSPromiseResolver) : TJSPromise;
    function _finally(value : TJSPromiseFinallyHandler): TJSPromise;
  end;

  TTMSMethod = procedure of object;

  {$IFNDEF LCLTMSWEB}
  TAwait = record
  public
    class function Exec<T>(const AValue: T): T; static;
    class function ExecP<T>(const AValue: TJSPromise): T; static;
  end;
  {$ENDIF}

var
  JSArguments: TJSValueDynArray;
  JSThis: TJSObject;
  JSExceptValue: JSValue;
function new(aElements: TJSValueDynArray): TJSObject;
function decodeURIComponent(encodedURI: string): string;
function encodeURIComponent(str: string): string;
function parseInt(s: string; Radix: NativeInt): NativeInt; overload;
function parseInt(s: string): NativeInt; overload;
function parseFloat(s: string): Double; overload;
function hasString(const v: JSValue): Boolean;
function hasValue(const v: JSValue): Boolean; assembler;
function isArray(const v: JSValue): Boolean;
function isBoolean(const v: JSValue): Boolean; assembler;
function isCallback(const v: JSValue): Boolean; assembler;
function isChar(const v: JSValue): Boolean; assembler;
function isClass(const v: JSValue): Boolean; assembler;
function isClassInstance(const v: JSValue): Boolean; assembler;
function isFunction(const v: JSValue): Boolean;
function isInteger(const v: JSValue): Boolean; assembler;
function isModule(const v: JSValue): Boolean;
function isNull(const v: JSValue): Boolean; assembler;
function isNumber(const v: JSValue): Boolean;
function isObject(const v: JSValue): Boolean;
function isRecord(const v: JSValue): Boolean; assembler;
function isString(const v: JSValue): Boolean;
function isUndefined(const v: JSValue): Boolean; assembler;
function isDefined(const v: JSValue): Boolean; assembler;
function isUTF16Char(const v: JSValue): Boolean; assembler;
function isExt(const InstanceOrClass: JSValue; const aClass: JSValue): Boolean;
function jsInstanceOf(const aFunction: JSValue; const aFunctionWithPrototype: JSValue): string; assembler;
function jsTypeOf(const v: JSValue): string;
function jsIsNaN(const v: JSValue): Boolean;
function toNumber(const v: JSValue): Double; assembler;
function toString(Value: JSValue): string;
function toObject(Value: JSValue): TJSObject;

function Await(const AValue: Boolean): Boolean; overload;
function Await(const AValue: Double): Double; overload;
function Await(const AValue: Integer): Integer; overload;
function Await(const AValue: JSValue): JSValue; overload;
function Await(const AValue: Single): Single; overload;
function Await(const AValue: string): string; overload;
function Await(const AValue: TDate): TDate; overload;
function Await(const AValue: TDateTime): TDateTime; overload;
function Await(const AValue: TTime): TTime; overload;
procedure Await(const AValue: TProc); overload;
{$IFNDEF LCLTMSWEB}
procedure Await(const AValue: TTMSMethod); overload;
{$ENDIF}

//function toInteger(const v: JSValue): NativeInt; // if v is not an integer, returns 0
//function toObject(Value: JSValue): TJSObject; // If Value is not a Javascript object, returns Nil
//function toArray(Value: JSValue): TJSArray; // If Value is not a Javascript array, returns Nil


implementation

uses
  Variants;

{$HINTS OFF}
{$WARNINGS OFF}
//{$NOTES OFF}

const
  LStringEmpty = '';

// Dummy implementations for externals

function TJSObject.GetProperties(Name: string): JSValue;

begin
end;

procedure TJSObject.SetProperties(Name: string; const AValue: JSValue);

begin
end;

constructor TJSObject.new;

begin
end;

class function TJSObject.create(const proto: TJSObject): TJSObject;

begin
end;

class function TJSObject.create(const proto: TJSObject; const propertiesObject: TJSObject): TJSObject;

begin
end;

class function TJSObject.assign(const Target: TJSObject; const Source1: TJSObject): TJSObject;

begin
end;

class procedure TJSObject.defineProperty(const obj: TJSObject; propname: string; const descriptor: TJSObjectPropertyDescriptor);

begin
end;

class function TJSObject.freeze(const obj: TJSObject): TJSObject;

begin
end;

class function TJSObject.getOwnPropertyDescriptor(const obj: TJSObject; propname: string): TJSObjectPropertyDescriptor;

begin
end;

class function TJSObject.getOwnPropertyNames(const obj: TJSObject): TStringDynArray;

begin
end;

class function TJSObject.getPrototypeOf(const obj: TJSObject): TJSObject;

begin
end;

class function TJSObject.isExtensible(const obj: TJSObject): Boolean;

begin
end;

class function TJSObject.isFrozen(const obj: TJSObject): Boolean;

begin
end;

class function TJSObject.isSealed(const obj: TJSObject): Boolean;

begin
end;

class function TJSObject.keys(const obj: TJSObject): TStringDynArray;

begin
end;

class function TJSObject.preventExtensions(const obj: TJSObject): TJSObject;

begin
end;

class function TJSObject.seal(const obj: TJSObject): TJSObject;

begin
end;

class function TJSObject.setPrototypeOf(const obj: TJSObject; const prototype: TJSObject): TJSObject;

begin
end;

function TJSObject.hasOwnProperty(prop: string): Boolean;

begin
end;

function TJSObject.isPrototypeOf(const obj: TJSObject): Boolean;

begin
end;

function TJSObject.propertyIsEnumerable(propname: string): Boolean;

begin
end;

function TJSObject.toLocaleString: string;

begin
end;

function TJSObject.toString: string;

begin
end;

function TJSObject.valueOf: JSValue;

begin
end;

function TJSFunction.apply(thisArg: TJSObject; const ArgArray: TJSValueDynArray): JSValue;

begin
end;

function TJSFunction.bind(thisArg: TJSObject): JSValue;

begin
end;

function TJSFunction.call(thisArg: TJSObject): JSValue;

begin
end;

function TJSDate.getDate: NativeInt;

begin
end;

function TJSDate.getFullYear: NativeInt;

begin
end;

function TJSDate.getHours: NativeInt;

begin
end;

function TJSDate.getMilliseconds: NativeInt;

begin
end;

function TJSDate.getMinutes: NativeInt;

begin
end;

function TJSDate.getMonth: NativeInt;

begin
end;

function TJSDate.getSeconds: NativeInt;

begin
end;

function TJSDate.getYear: NativeInt;

begin
end;

function TJSDate.getTime: NativeInt;

begin
end;

function TJSDate.getUTCDate: NativeInt;

begin
end;

function TJSDate.getUTCFullYear: NativeInt;

begin
end;

function TJSDate.getUTCHours: NativeInt;

begin
end;

function TJSDate.getUTCMilliseconds: NativeInt;

begin
end;

function TJSDate.getUTCMinutes: NativeInt;

begin
end;

function TJSDate.getUTCMonth: NativeInt;

begin
end;

function TJSDate.getUTCSeconds: NativeInt;

begin
end;

procedure TJSDate.setDate(const AValue: NativeInt);

begin
end;

procedure TJSDate.setFullYear(const AValue: NativeInt);

begin
end;

procedure TJSDate.setHours(const AValue: NativeInt);

begin
end;

procedure TJSDate.setMilliseconds(const AValue: NativeInt);

begin
end;

procedure TJSDate.setMinutes(const AValue: NativeInt);

begin
end;

procedure TJSDate.setMonth(const AValue: NativeInt);

begin
end;

procedure TJSDate.setSeconds(const AValue: NativeInt);

begin
end;

procedure TJSDate.setYear(const AValue: NativeInt);

begin
end;

procedure TJSDate.setTime(const AValue: NativeInt);

begin
end;

procedure TJSDate.setUTCDate(const AValue: NativeInt);

begin
end;

procedure TJSDate.setUTCFullYear(const AValue: NativeInt);

begin
end;

procedure TJSDate.setUTCHours(const AValue: NativeInt);

begin
end;

procedure TJSDate.setUTCMilliseconds(const AValue: NativeInt);

begin
end;

procedure TJSDate.setUTCMinutes(const AValue: NativeInt);

begin
end;

procedure TJSDate.setUTCMonth(const AValue: NativeInt);

begin
end;

procedure TJSDate.setUTCSeconds(const AValue: NativeInt);

begin
end;

constructor TJSDate.New(const MilliSecsSince1970: NativeInt);

begin
end;

constructor TJSDate.New(const aDateString: string);

begin
end;

constructor TJSDate.New(aYear: NativeInt; aMonth: NativeInt; aDayOfMonth: NativeInt = 1; TheHours: NativeInt = 0; TheMinutes: NativeInt = 0; TheSeconds: NativeInt = 0; TheMilliseconds: NativeInt = 0);

begin
end;

class function TJSDate.now: NativeInt;

begin
end;

class function TJSDate.parse(const aDateString: string): NativeInt;

begin
end;

class function TJSDate.UTC(aYear: NativeInt; aMonth: NativeInt = 0; aDayOfMonth: NativeInt = 1; TheHours: NativeInt = 0; TheMinutes: NativeInt = 0; TheSeconds: NativeInt = 0; TheMilliseconds: NativeInt = 0): NativeInt;

begin
end;

function TJSDate.getDay: NativeInt;

begin
end;

function TJSDate.getTimezoneOffset: NativeInt;

begin
end;

function TJSDate.getUTCDay: NativeInt;

begin
end;

function TJSDate.toDateString: string;

begin
end;

function TJSDate.toISOString: string;

begin
end;

function TJSDate.toJSON: string;

begin
end;

function TJSDate.toGMTString: string;

begin
end;

function TJSDate.toLocaleDateString: string;

begin
end;

function TJSDate.toLocaleString: string;

begin
end;

function TJSDate.toLocaleTimeString: string;

begin
end;

function TJSDate.toTimeString: string;

begin
end;

function TJSDate.toUTCString: string;

begin
end;

constructor TJSRegexp.New(Pattern: string);

begin
end;

constructor TJSRegexp.New(Pattern: string; Flags: string);

begin
end;

function TJSRegexp.exec(aString: string): TStringDynArray;

begin
end;

function TJSRegexp.test(aString: string): Boolean;

begin
end;

function TJSRegexp.toString: string;

begin
end;

constructor TJSString.New(const S: string);

begin
end;

constructor TJSString.New(const I: NativeInt);

begin
end;

constructor TJSString.New(const D: Double);

begin
end;

class function TJSString.fromCharCode: string;

begin
end;

class function TJSString.fromCodePoint(const AValue: string): string;
begin
  Result := LStringEmpty;
end;

class function TJSString.fromCodePoint(const AValue1, AValue2: string): string;
begin
  Result := LStringEmpty;
end;

class function TJSString.fromCodePoint(const AValue1, AValue2, AValue3: string): string;
begin
  Result := LStringEmpty;
end;

class function TJSString.fromCodePoint(const AValue1, AValue2, AValue3, AValue4: string): string;
begin
  Result := LStringEmpty;
end;

class function TJSString.fromCodePoint(const AValue1, AValue2, AValue3, AValue4, AValue5: string): string;
begin
  Result := LStringEmpty;
end;

function TJSString.anchor(const aName: string): string;

begin
end;

function TJSString.charAt(aIndex: NativeInt): string;

begin
end;

function TJSString.charCodeAt(aIndex: NativeInt): NativeInt;

begin
end;

function TJSString.codePointAt(aIndex: NativeInt): NativeInt;

begin
end;

function TJSString.concat(s: string): string;

begin
end;

function TJSString.endsWith(aSearchString: string; Pos: NativeInt = 0): Boolean;

begin
end;

function TJSString.includes(aSearchString: string; Pos: NativeInt = 0): Boolean;

begin
end;

function TJSString.indexOf(aSearchString: string; Pos: NativeInt = 0): Integer;

begin
end;

function TJSString.lastIndexOf(aSearchString: string): NativeInt;

begin
end;

function TJSString.lastIndexOf(aSearchString: string; Pos: NativeInt): Integer;

begin
end;

function TJSString.link(aUrl: string): string;

begin
end;

function TJSString.localeCompare(aCompareString: string): NativeInt;

begin
end;

function TJSString.localeCompare(aCompareString: string; aLocales: string): Integer;

begin
end;

function TJSString.localeCompare(compareString: string; locales: string; Options: TlocaleCompareOptions): Integer;

begin
end;

function TJSString.match(aRegexp: TJSRegexp): TStringDynArray;

begin
end;

function TJSString.match(aRegexp: string): TStringDynArray;

begin
end;

function TJSString._repeat(aCount: NativeInt): Integer;

begin
end;

function TJSString.replace(aRegexp: string; NewString: string): string;

begin
end;

function TJSString.replace(aRegexp: TJSRegexp; NewString: string): string;

begin
end;

function TJSString.replace(Regexp: string; aCallback: TReplaceCallBack): string;

begin
end;

function TJSString.replace(Regexp: TJSRegexp; aCallback: TReplaceCallBack): string;

begin
end;

function TJSString.search(Regexp: TJSRegexp): NativeInt;

begin
end;

function TJSString.search(Regexp: JSValue): NativeInt;

begin
end;

function TJSString.slice(aBeginIndex: NativeInt): string;

begin
end;

function TJSString.slice(aBeginIndex: NativeInt; aEndIndex: NativeInt): string;

begin
end;

function TJSString.split: TStringDynArray;

begin
end;

function TJSString.split(aSeparator: string): TStringDynArray;

begin
end;

function TJSString.split(aSeparator: string; aLimit: NativeInt): TStringDynArray;

begin
end;

function TJSString.startsWith(aSearchString: string): Boolean;

begin
end;

function TJSString.startsWith(aSearchString: string; aPosition: NativeInt): Boolean;

begin
end;

function TJSString.substr(aStartIndex: NativeInt): string;

begin
end;

function TJSString.substr(aStartIndex: NativeInt; aLength: NativeInt): string;

begin
end;

function TJSString.subString(aStartIndex: NativeInt): string;

begin
end;

function TJSString.subString(aStartIndex: NativeInt; aEndIndex: NativeInt): string;

begin
end;

function TJSString.toLocaleLowerCase: string;

begin
end;

function TJSString.toLocaleUpperCase: string;

begin
end;

function TJSString.toLowerCase: string;

begin
end;

function TJSString.toString: string;

begin
end;

function TJSString.toUpperCase: string;

begin
end;

function TJSString.trim: string;

begin
end;

function TJSString.valueOf: string;

begin
end;

function TJSArray.GetElements(Index: NativeInt): JSValue;

begin
end;

procedure TJSArray.SetElements(Index: NativeInt; const AValue: JSValue);

begin
end;

constructor TJSArray.new;

begin
end;

constructor TJSArray.new(aLength: NativeInt);

begin
end;

constructor TJSArray.new(aElement1: JSValue);

begin
end;

class function TJSArray._of: TJSArray;

begin
end;

class function TJSArray.isArray(a: JSValue): Boolean;

begin
end;

function TJSArray.concat(el: JSValue): TJSArray;

begin
end;

function TJSArray.copyWithin(aTarget: NativeInt): TJSarray;

begin
end;

function TJSArray.copyWithin(aTarget: NativeInt; aStart: NativeInt): TJSarray;

begin
end;

function TJSArray.copyWithin(aTarget: NativeInt; aStart: NativeInt; aEnd: NativeInt): TJSarray;

begin
end;

function TJSArray.every(const aCallback: TJSArrayCallBack): Boolean;

begin
end;

function TJSArray.every(const aCallback: TJSArrayEvent; aThis: TObject): Boolean;

begin
end;

function TJSArray.filter(const aCallBack: TJSArrayCallBack): TJSArray;

begin
end;

function TJSArray.filter(const aCallBack: TJSArrayEvent; aThis: TObject): TJSArray;

begin
end;

function TJSArray.fill(aValue: JSValue): TJSArray;

begin
end;

function TJSArray.fill(aValue: JSValue; aStartIndex: NativeInt): TJSArray;

begin
end;

function TJSArray.fill(aValue: JSValue; aStartIndex: NativeInt; aEndIndex: NativeInt): TJSArray;

begin
end;

function TJSArray.find(const aCallBack: TJSArrayCallBack): JSValue;

begin
end;

function TJSArray.find(const aCallBack: TJSArrayEvent; aThis: TObject): JSValue;

begin
end;

function TJSArray.findIndex(const aCallBack: TJSArrayCallBack): NativeInt;

begin
end;

function TJSArray.findIndex(const aCallBack: TJSArrayEvent; aThis: TObject): NativeInt;

begin
end;

procedure TJSArray.forEach(const aCallBack: TJSArrayCallBack);

begin
end;

procedure TJSArray.forEach(const aCallBack: TJSArrayEvent; aThis: TObject);

begin
end;

function TJSArray.includes(aElement: JSValue): Boolean;

begin
end;

function TJSArray.includes(aElement: JSValue; FromIndex: NativeInt): Boolean;

begin
end;

function TJSArray.indexOf(aElement: JSValue): NativeInt;

begin
end;

function TJSArray.indexOf(aElement: JSValue; FromIndex: NativeInt): NativeInt;

begin
end;

function TJSArray.join: string;

begin
end;

function TJSArray.join(aSeparator: string): string;

begin
end;

function TJSArray.lastIndexOf(aElement: JSValue): NativeInt;

begin
end;

function TJSArray.lastIndexOf(aElement: JSValue; FromIndex: NativeInt): NativeInt;

begin
end;

function TJSArray.map(const aCallBack: TJSArrayCallBack): TJSArray;

begin
end;

function TJSArray.map(const aCallBack: TJSArrayEvent; aThis: TObject): TJSArray;

begin
end;

function TJSArray.pop: JSValue;

begin
end;

function TJSArray.push(aElement: JSValue): NativeInt;

begin
end;

function TJSArray.reduce(const aCallBack: TJSArrayReduceCallBack): JSValue;

begin
end;

function TJSArray.reduce(const aCallBack: TJSArrayReduceCallBack; initialValue: JSValue): JSValue;

begin
end;

function TJSArray.reduceRight(const aCallBack: TJSArrayReduceCallBack): JSValue;

begin
end;

function TJSArray.reduceRight(const aCallBack: TJSArrayReduceCallBack; initialValue: JSValue): JSValue;

begin
end;

function TJSArray.reverse: TJSArray;

begin
end;

function TJSArray.shift: JSValue;

begin
end;

function TJSArray.slice: TJSArray;

begin
end;

function TJSArray.slice(aBegin: NativeInt): TJSArray;

begin
end;

function TJSArray.slice(aBegin: NativeInt; aEnd: NativeInt): TJSArray;

begin
end;

function TJSArray.some(const aCallback: TJSArrayCallBack): Boolean;

begin
end;

function TJSArray.some(const aCallback: TJSArrayEvent; aThis: TObject): Boolean;

begin
end;

function TJSArray.sort(const aCallback: TJSArrayCompareCallBack): TJSArray;

begin
end;

function TJSArray.sort: TJSArray;

begin
end;

function TJSArray.splice(aStart: NativeInt): TJSArray;

begin
end;

function TJSArray.splice(aStart: NativeInt; aDeleteCount: NativeInt): TJSArray;

begin
end;

function TJSArray.toLocaleString: string;

begin
end;

function TJSArray.toLocaleString(locales: string): string;

begin
end;

function TJSArray.toLocaleString(locales: string; const Options: TLocaleCompareOptions): string;

begin
end;

function TJSArray.toString: string;

begin
end;

function TJSArray.unshift: NativeInt;

begin
end;

constructor TJSArrayBuffer.new(aByteLength: NativeInt);

begin
end;

class function TJSArrayBuffer.isView(aValue: JSValue): Boolean;

begin
end;

function TJSArrayBuffer.slice(aBegin: NativeInt): TJSArrayBuffer;

begin
end;

function TJSArrayBuffer.slice(aBegin: NativeInt; aEnd: NativeInt): TJSArrayBuffer;

begin
end;

function TJSTypedArray.getValue(Index: NativeInt): JSValue;

begin
end;

procedure TJSTypedArray.setValue(Index: NativeInt; AValue: JSValue);

begin
end;

class function TJSTypedArray.from(aValue: jsValue): TJSTypedArray;

begin
end;

class function TJSTypedArray.from(aValue: jsValue; Map: TJSTypedArrayMapCallBack): TJSTypedArray;

begin
end;

class function TJSTypedArray.from(aValue: jsValue; aMap: TJSTypedArrayMapEvent): TJSTypedArray;

begin
end;

class function TJSTypedArray._of(aValue: jsValue): TJSTypedArray;

begin
end;

function TJSTypedArray.copyWithin(aTarget: NativeInt): TJSTypedArray;

begin
end;

function TJSTypedArray.copyWithin(aTarget: NativeInt; aStart: NativeInt): TJSTypedArray;

begin
end;

function TJSTypedArray.copyWithin(aTarget: NativeInt; aStart: NativeInt; aEnd: NativeInt): TJSTypedArray;

begin
end;

function TJSTypedArray.every(const aCallback: TJSTypedArrayCallBack): Boolean;

begin
end;

function TJSTypedArray.every(const aCallback: TJSTypedArrayEvent; aThis: TObject): Boolean;

begin
end;

function TJSTypedArray.fill(aValue: JSValue): TJSTypedArray;

begin
end;

function TJSTypedArray.fill(aValue: JSValue; aStartIndex: NativeInt): TJSTypedArray;

begin
end;

function TJSTypedArray.fill(aValue: JSValue; aStartIndex: NativeInt; aEndIndex: NativeInt): TJSTypedArray;

begin
end;

function TJSTypedArray.filter(const aCallBack: TJSTypedArrayCallBack): TJSTypedArray;

begin
end;

function TJSTypedArray.filter(const aCallBack: TJSTypedArrayEvent; aThis: TObject): TJSTypedArray;

begin
end;

function TJSTypedArray.find(const aCallBack: TJSTypedArrayCallBack): JSValue;

begin
end;

function TJSTypedArray.find(const aCallBack: TJSTypedArrayEvent; aThis: TObject): JSValue;

begin
end;

function TJSTypedArray.findIndex(const aCallBack: TJSTypedArrayCallBack): NativeInt;

begin
end;

function TJSTypedArray.findIndex(const aCallBack: TJSTypedArrayEvent; aThis: TObject): NativeInt;

begin
end;

procedure TJSTypedArray.forEach(const aCallBack: TJSTypedArrayCallBack);

begin
end;

procedure TJSTypedArray.forEach(const aCallBack: TJSTypedArrayEvent; aThis: TObject);

begin
end;

function TJSTypedArray.includes(aElement: JSValue): Boolean;

begin
end;

function TJSTypedArray.includes(aElement: JSValue; FromIndex: NativeInt): Boolean;

begin
end;

function TJSTypedArray.indexOf(aElement: JSValue): NativeInt;

begin
end;

function TJSTypedArray.indexOf(aElement: JSValue; FromIndex: NativeInt): NativeInt;

begin
end;

function TJSTypedArray.join: string;

begin
end;

function TJSTypedArray.join(aSeparator: string): string;

begin
end;

function TJSTypedArray.lastIndexOf(aElement: JSValue): NativeInt;

begin
end;

function TJSTypedArray.lastIndexOf(aElement: JSValue; FromIndex: NativeInt): NativeInt;

begin
end;

function TJSTypedArray.map(const aCallBack: TJSTypedArrayCallBack): TJSTypedArray;

begin
end;

function TJSTypedArray.map(const aCallBack: TJSTypedArrayEvent; aThis: TObject): TJSTypedArray;

begin
end;

function TJSTypedArray.reduce(const aCallBack: TJSTypedArrayReduceCallBack): JSValue;

begin
end;

function TJSTypedArray.reduce(const aCallBack: TJSTypedArrayReduceCallBack; initialValue: JSValue): JSValue;

begin
end;

function TJSTypedArray.reduceRight(const aCallBack: TJSTypedArrayReduceCallBack): JSValue;

begin
end;

function TJSTypedArray.reduceRight(const aCallBack: TJSTypedArrayReduceCallBack; initialValue: JSValue): JSValue;

begin
end;

function TJSTypedArray.reverse: TJSTypedArray;

begin
end;

procedure TJSTypedArray._set(anArray: TJSTypedArray);

begin
end;

procedure TJSTypedArray._set(anArray: TJSTypedArray; anOffset: NativeInt);

begin
end;

function TJSTypedArray.slice: TJSTypedArray;

begin
end;

function TJSTypedArray.slice(aBegin: NativeInt): TJSTypedArray;

begin
end;

function TJSTypedArray.slice(aBegin: NativeInt; aEnd: NativeInt): TJSTypedArray;

begin
end;

function TJSTypedArray.some(const aCallback: TJSTypedArrayCallBack): Boolean;

begin
end;

function TJSTypedArray.some(const aCallback: TJSTypedArrayEvent; aThis: TObject): Boolean;

begin
end;

function TJSTypedArray.sort(const aCallback: TJSTypedArrayCompareCallBack): TJSTypedArray;

begin
end;

function TJSTypedArray.sort: TJSTypedArray;

begin
end;

function TJSTypedArray.splice(aStart: NativeInt): TJSTypedArray;

begin
end;

function TJSTypedArray.splice(aStart: NativeInt; aDeleteCount: NativeInt): TJSTypedArray;

begin
end;

function TJSTypedArray.toLocaleString: string;

begin
end;

function TJSTypedArray.toLocaleString(locales: string): string;

begin
end;

function TJSTypedArray.toLocaleString(locales: string; const Options: TLocaleCompareOptions): string;

begin
end;

function TJSTypedArray.toString: string;

begin
end;

function TJSTypedArray.unshift: NativeInt;

begin
end;

function TJSInt8Array.getTypedValue(Index: Integer): ShortInt;

begin
end;

procedure TJSInt8Array.setTypedValue(Index: Integer; AValue: ShortInt);

begin
end;

function TJSUint8Array.getTypedValue(Index: Integer): Byte;

begin
end;

procedure TJSUint8Array.setTypedValue(Index: Integer; AValue: Byte);

begin
end;

function TJSUint8ClampedArray.getTypedValue(Index: Integer): Byte;

begin
end;

procedure TJSUint8ClampedArray.setTypedValue(Index: Integer; AValue: Byte);

begin
end;

function TJSInt16Array.getTypedValue(Index: Integer): SmallInt;

begin
end;

procedure TJSInt16Array.setTypedValue(Index: Integer; AValue: SmallInt);

begin
end;

function TJSUint16Array.getTypedValue(Index: Integer): Word;

begin
end;

procedure TJSUint16Array.setTypedValue(Index: Integer; AValue: Word);

begin
end;

function TJSInt32Array.getTypedValue(Index: Integer): LongInt;

begin
end;

procedure TJSInt32Array.setTypedValue(Index: Integer; AValue: LongInt);

begin
end;

function TJSUint32Array.getTypedValue(Index: Integer): LongWord;

begin
end;

procedure TJSUint32Array.setTypedValue(Index: Integer; AValue: LongWord);

begin
end;

function TJSFloat32Array.getTypedValue(Index: Integer): Float32;

begin
end;

procedure TJSFloat32Array.setTypedValue(Index: Integer; AValue: Float32);

begin
end;

function TJSFloat64Array.getTypedValue(Index: Integer): Float64;

begin
end;

procedure TJSFloat64Array.setTypedValue(Index: Integer; AValue: Float64);

begin
end;

constructor TJSDataView.new(aBuffer: TJSArrayBuffer);

begin
end;

constructor TJSDataView.new(aBuffer: TJSArrayBuffer; aOffset: NativeInt);

begin
end;

constructor TJSDataView.new(aBuffer: TJSArrayBuffer; aOffset: NativeInt; aByteLength: NativeInt);

begin
end;

function TJSDataView.getFloat32(aByteOffset: NativeInt): Double;

begin
end;

function TJSDataView.getFloat32(aByteOffset: NativeInt; aLittleEndian: Boolean): Double;

begin
end;

function TJSDataView.getFloat64(aByteOffset: NativeInt): Double;

begin
end;

function TJSDataView.getFloat64(aByteOffset: NativeInt; aLittleEndian: Boolean): Double;

begin
end;

function TJSDataView.getInt8(aByteOffset: NativeInt): ShortInt;

begin
end;

function TJSDataView.getInt16(aByteOffset: NativeInt): SmallInt;

begin
end;

function TJSDataView.getInt16(aByteOffset: NativeInt; aLittleEndian: Boolean): SmallInt;

begin
end;

function TJSDataView.getInt32(aByteOffset: NativeInt): LongInt;

begin
end;

function TJSDataView.getInt32(aByteOffset: NativeInt; aLittleEndian: Boolean): LongInt;

begin
end;

function TJSDataView.getUint8(aByteOffset: NativeInt): Byte;

begin
end;

function TJSDataView.getUint16(aByteOffset: NativeInt): Word;

begin
end;

function TJSDataView.getUint16(aByteOffset: NativeInt; aLittleEndian: Boolean): Word;

begin
end;

function TJSDataView.getUint32(aByteOffset: NativeInt): LongWord;

begin
end;

function TJSDataView.getUint32(aByteOffset: NativeInt; aLittleEndian: Boolean): LongWord;

begin
end;

procedure TJSDataView.setFloat32(aByteOffset: NativeInt; aValue: Double);

begin
end;

procedure TJSDataView.setFloat32(aByteOffset: NativeInt; aValue: Double; aLittleEndian: Boolean);

begin
end;

procedure TJSDataView.setFloat64(aByteOffset: NativeInt; aValue: Double);

begin
end;

procedure TJSDataView.setFloat64(aByteOffset: NativeInt; aValue: Double; aLittleEndian: Boolean);

begin
end;

procedure TJSDataView.setInt8(aByteOffset: NativeInt; aValue: ShortInt);

begin
end;

procedure TJSDataView.setInt16(aByteOffset: NativeInt; aValue: SmallInt);

begin
end;

procedure TJSDataView.setInt16(aByteOffset: NativeInt; aValue: SmallInt; aLittleEndian: Boolean);

begin
end;

procedure TJSDataView.setInt32(aByteOffset: NativeInt; aValue: LongInt);

begin
end;

procedure TJSDataView.setInt32(aByteOffset: NativeInt; aValue: LongInt; aLittleEndian: Boolean);

begin
end;

procedure TJSDataView.setUint8(aByteOffset: NativeInt; aValue: Byte);

begin
end;

procedure TJSDataView.setUint16(aByteOffset: NativeInt; aValue: Word);

begin
end;

procedure TJSDataView.setUint16(aByteOffset: NativeInt; aValue: Word; aLittleEndian: Boolean);

begin
end;

procedure TJSDataView.setUint32(aByteOffset: NativeInt; aValue: LongWord);

begin
end;

procedure TJSDataView.setUint32(aByteOffset: NativeInt; aValue: LongWord; aLittleEndian: Boolean);

begin
end;

class function TJSJSON.parse(aJSON: string): TJSObject;

begin
end;

class function TJSJSON.stringify(aValue: JSValue): string;

begin
end;

class function TJSJSON.stringify(aValue: JSValue; aReplacer: JSValue): string;

begin
end;

class function TJSJSON.stringify(aValue: JSValue; aReplacer: JSValue; space: NativeInt): string;

begin
end;

class function TJSJSON.parseObject(aJSON: String): TJSObject;
begin
end;

class function TJSJSON.stringify(aValue: JSValue; aReplacer: JSValue; space: string): string;

begin
end;

function decodeURIComponent(encodedURI: string): string;

begin
end;

function encodeURIComponent(str: string): string;

begin
end;

function parseInt(s: string; Radix: NativeInt): NativeInt;

begin
end;

function parseInt(s: string): NativeInt;

begin
end;

function parseFloat(s: string): Double;

begin
end;

function hasString(const v: JSValue): Boolean;

begin
end;

function isArray(const v: JSValue): Boolean;

begin
end;

function isFunction(const v: JSValue): Boolean;

begin
end;

function isModule(const v: JSValue): Boolean;

begin
end;

function isNumber(const v: JSValue): Boolean;

begin
end;

function isObject(const v: JSValue): Boolean;

begin
end;

function isString(const v: JSValue): Boolean;

begin
end;

function isExt(const InstanceOrClass: JSValue; const aClass: JSValue): Boolean;

begin
end;

function jsTypeOf(const v: JSValue): string;

begin
end;

function jsIsNaN(const v: JSValue): Boolean;

begin
end;


// end of dummy implementations



function new(aElements: TJSValueDynArray): TJSObject;
begin
end;

//{$ASMMODE intel}

function hasValue(const v: JSValue): Boolean; assembler;
asm
end;

function isBoolean(const v: JSValue): Boolean; assembler;
asm
end;

function isCallback(const v: JSValue): Boolean; assembler;
asm
end;

function isChar(const v: JSValue): Boolean; assembler;
asm
end;

function isClass(const v: JSValue): Boolean; assembler;
asm
end;

function isClassInstance(const v: JSValue): Boolean; assembler;
asm
end;

function isInteger(const v: JSValue): Boolean; assembler;
asm
end;

function isNull(const v: JSValue): Boolean; assembler;
asm
end;

function isRecord(const v: JSValue): Boolean; assembler;
asm
end;

function isUndefined(const v: JSValue): Boolean; assembler;
asm
end;

function isDefined(const v: JSValue): Boolean; assembler;
asm
end;

function isUTF16Char(const v: JSValue): Boolean; assembler;
asm
end;

function jsInstanceOf(const aFunction: JSValue; const aFunctionWithPrototype: JSValue): string; assembler;
asm
end;

function toNumber(const v: JSValue): Double; assembler;
asm
end;

function toString(Value: JSValue): string;
begin
  Result := '';
end;

function toObject(Value: JSValue): TJSObject;
begin
  Result := nil;
end;

constructor EJS.Create(const Msg: string);
begin
end;

function Await(const AValue: Boolean): Boolean; overload;
begin
  Result := AValue;
end;

function Await(const AValue: Integer): Integer; overload;
begin
  Result := AValue;
end;

function Await(const AValue: JSValue): JSValue; overload;
begin
  Result := AValue;
end;

function Await(const AValue: string): string; overload;
begin
  Result := AValue;
end;

procedure Await(const AValue: TProc); overload;
begin
end;

{$IFNDEF LCLTMSWEB}
procedure Await(const AValue: TTMSMethod); overload;
begin
end;
{$ENDIF}

function Await(const AValue: Double): Double; overload;
begin
  Result := AValue;
end;

function Await(const AValue: Single): Single; overload;
begin
  Result := AValue;
end;

function Await(const AValue: TDate): TDate; overload;
begin
  Result := AValue;
end;

function Await(const AValue: TDateTime): TDateTime; overload;
begin
  Result := AValue;
end;

function Await(const AValue: TTime): TTime; overload;
begin
  Result := AValue;
end;

{ TJSPromise }

class function TJSPromise.all(arg: TJSPromiseArray): TJSPromise;
begin
  Result := nil;
end;

class function TJSPromise.all(arg: JSValue): TJSPromise;
begin
  Result := nil;
end;

class function TJSPromise.all(arg: array of JSValue): TJSPromise;
begin
  Result := nil;
end;

function TJSPromise.catch(onRejected: TJSPromiseResolver): TJSPromise;
begin
  Result := nil;
end;

constructor TJSPromise.new(Executor: TJSPromiseExecutor);
begin
end;

class function TJSPromise.race(arg: array of JSValue): TJSPromise;
begin
  Result := nil;
end;

class function TJSPromise.race(arg: TJSPromiseArray): TJSPromise;
begin
  Result := nil;
end;

class function TJSPromise.race(arg: JSValue): TJSPromise;
begin
  Result := nil;
end;

class function TJSPromise.reject(reason: JSValue): TJSPromise;
begin
  Result := nil;
end;

class function TJSPromise.resolve: TJSPromise;
begin
  Result := nil;
end;

class function TJSPromise.resolve(value: JSValue): TJSPromise;
begin

end;

function TJSPromise._finally(value: TJSPromiseFinallyHandler): TJSPromise;
begin

end;

function TJSPromise._then(onAccepted,
  OnRejected: TJSPromiseResolver): TJSPromise;
begin

end;

function TJSPromise._then(onAccepted: TJSPromiseResolver): TJSPromise;
begin

end;

{$IFNDEF LCLTMSWEB}
{ TAwait }

class function TAwait.Exec<T>(const AValue: T): T;
begin
  Result := Default(T);
end;

class function TAwait.ExecP<T>(const AValue: TJSPromise): T;
begin
  Result := Default(T);
end;

{$ENDIF}

end.
