unit libmedia;

interface

uses
  JS, Web;
  
type
  TWebMediaStreamTrack = class(TJSObject)
  public
    procedure stop;
    function applyConstraints: TJSPromise; overload;
    function applyConstraints(aConstraints: TJSObject): TJSPromise; overload;
    function getConstraints: TJSObject;
    function getSettings: TJSObject;
  end;

  TWebMediaStream = class(TJSObject)
  public
    function getAudioTracks: TJSArray;
    function getTracks: TJSArray;
    function getVideoTracks: TJSArray;
  end;

  TWebAudioNode = class
  public
    function connect(aDestination: TWebAudioNode): TWebAudioNode;
    procedure disconnect; overload;
    procedure disconnect(aDestination: TWebAudioNode); overload;
  end;

  TWebAnalyserNode = class(TWebAudioNode)
  private
    FFFTSize: NativeInt;
    FFrequencyBinCount: NativeInt;
    FMaxDecibels: Double;
    FMinDecibels: Double;
    FSmoothingTimeConstant: Double;
  public
    procedure getFloatFrequencyData(aData: TJSFloat32Array);
    procedure getByteFrequencyData(aData: TJSUint8Array);
    procedure getFloatTimeDomainData(aData: TJSFloat32Array);
    procedure getByteTimeDomainData(aData: TJSUint8Array);
    property FFTSize: NativeInt read FFFTSize write FFFTSize;
    property FrequencyBinCount: NativeInt read FFrequencyBinCount;
    property MaxDecibels: Double read FMaxDecibels write FMaxDecibels;
    property MinDecibels: Double read FMinDecibels write FMinDecibels;
    property SmoothingTimeConstant: Double read FSmoothingTimeConstant write FSmoothingTimeConstant;
  end;

  TWebMediaStreamAudioSourceNode = class(TWebAudioNode)
  end;

  TWebBaseAudioContext = class
  public
    function createAnalyser: TWebAnalyserNode;
  end;

  TWebAudioContext = class(TWebBaseAudioContext)
  public
    function createMediaStreamSource(aStream: TWebMediaStream): TWebMediaStreamAudioSourceNode;
  end;

  {$IFNDEF LCLTMSWEB}
  TWebMediaRecorderStopEvent = reference to procedure;
  {$ENDIF}
  {$IFDEF LCLTMSWEB}
  TWebMediaRecorderStopEvent = procedure of object;
  {$ENDIF}

  TWebMediaRecorder = class(TJSObject)
  public
    ondataavailable: TJSEventHandler;
    onstop: TWebMediaRecorderStopEvent;
    constructor new (aStream : TWebMediaStream); overload;
    constructor new (aStream : TWebMediaStream; aOptions : TJSObject); overload;
    procedure start;
    procedure stop;
  end;

  TWebHTMLMediaElement = class(TJSHTMLElement)
  public
    paused: Boolean;
    srcObject: TJSObject;
    procedure pause;
    procedure play;
  end;

  TWebHTMLVideoElement = class(TWebHTMLMediaElement)
  public
    videoHeight: NativeInt;
    videoWidth: NativeInt;
  end;

  TWebMediaDevicesInfo = class(TJSObject)
  public
    deviceId: string;
    kind: string;
    &label: string;
  end;

  TWebMediaDevices = class(TJSEventTarget)
  public
    function enumerateDevices: TJSPromise;
    function getUserMedia(aConstraints: TJSObject): TJSPromise;
    function getSupportedConstraints: TJSObject;
  end;

  TWebJSQRResultObject = class(TJSObject)
    binaryData: TJSUint8ClampedArray;
    data: string;
  end;

  TWebQuaggaJSConfigObject = class(TJSObject)
    decoder: TJSObject;
    src: string; //url or base64 data string
  end;

  TWebQuaggaJSResultObject_CodeResult = class(TJSObject)
    code: string;
  end;

  TWebQuaggaJSResultObject = class(TJSObject)
    codeResult: TWebQuaggaJSResultObject_CodeResult;
  end;

  {$IFNDEF LCLTMSWEB}
  TWebQuaggaJSResultCallback = reference to procedure(data: TWebQuaggaJSResultObject);
  {$ENDIF}
  {$IFDEF LCLTMSWEB}
  TWebQuaggaJSResultCallback = procedure(data: TWebQuaggaJSResultObject) of object;
  {$ENDIF}

  TWebQuagga  = class(TJSObject)
    procedure decodeSingle(aConfig: TWebQuaggaJSConfigObject; aResultCallback: TWebQuaggaJSResultCallback);
  end;

function jsQR(aImageData: TJSUint8ClampedArray; aWidth: NativeInt; aHeight: NativeInt): TWebJSQRResultObject;

var
  Quagga: TWebQuagga;

implementation
{$HINTS OFF}
{$WARNINGS OFF}

function jsQR(aImageData: TJSUint8ClampedArray; aWidth: NativeInt; aHeight: NativeInt): TWebJSQRResultObject;
begin

end;

{ TWebMediaStreamTrack }

function TWebMediaStreamTrack.applyConstraints: TJSPromise;
begin

end;

function TWebMediaStreamTrack.applyConstraints(
  aConstraints: TJSObject): TJSPromise;
begin

end;

function TWebMediaStreamTrack.getConstraints: TJSObject;
begin

end;

function TWebMediaStreamTrack.getSettings: TJSObject;
begin

end;

procedure TWebMediaStreamTrack.stop;
begin

end;

{ TWebMediaStream }

function TWebMediaStream.getAudioTracks: TJSArray;
begin

end;

function TWebMediaStream.getTracks: TJSArray;
begin

end;

function TWebMediaStream.getVideoTracks: TJSArray;
begin

end;

{ TWebAudioNode }

function TWebAudioNode.connect(aDestination: TWebAudioNode): TWebAudioNode;
begin

end;

procedure TWebAudioNode.disconnect;
begin

end;

procedure TWebAudioNode.disconnect(aDestination: TWebAudioNode);
begin

end;

{ TWebAnalyserNode }

procedure TWebAnalyserNode.getByteFrequencyData(aData: TJSUint8Array);
begin

end;

procedure TWebAnalyserNode.getByteTimeDomainData(aData: TJSUint8Array);
begin

end;

procedure TWebAnalyserNode.getFloatFrequencyData(aData: TJSFloat32Array);
begin

end;

procedure TWebAnalyserNode.getFloatTimeDomainData(aData: TJSFloat32Array);
begin

end;

{ TWebBaseAudioContext }

function TWebBaseAudioContext.createAnalyser: TWebAnalyserNode;
begin

end;

{ TWebAudioContext }

function TWebAudioContext.createMediaStreamSource(
  aStream: TWebMediaStream): TWebMediaStreamAudioSourceNode;
begin

end;

{ TWebMediaRecorder }

constructor TWebMediaRecorder.new(aStream: TWebMediaStream);
begin

end;

constructor TWebMediaRecorder.new(aStream: TWebMediaStream;
  aOptions: TJSObject);
begin

end;

procedure TWebMediaRecorder.start;
begin

end;

procedure TWebMediaRecorder.stop;
begin

end;

{ TWebHTMLMediaElement }

procedure TWebHTMLMediaElement.pause;
begin

end;

procedure TWebHTMLMediaElement.play;
begin

end;

{ TWebMediaDevices }

function TWebMediaDevices.enumerateDevices: TJSPromise;
begin

end;

function TWebMediaDevices.getSupportedConstraints: TJSObject;
begin

end;

function TWebMediaDevices.getUserMedia(aConstraints: TJSObject): TJSPromise;
begin

end;

{ TWebQuagga }

procedure TWebQuagga.decodeSingle(aConfig: TWebQuaggaJSConfigObject;
  aResultCallback: TWebQuaggaJSResultCallback);
begin

end;

end.
