{$Error This is a closed source unit. You cannot compile it, it was already compiled.}
{$Error Probably the IDE has cleaned up and you have to unpack the zip again.}
{********************************************************************}
{                                                                    }
{ written by TMS Software                                            }
{            copyright (c) 2020                                      }
{            Email : info@tmssoftware.com                            }
{            Web : http://www.tmssoftware.com                        }
{                                                                    }
{ The source code is given as is. The author is not responsible      }
{ for any possible damage done due to the use of this code.          }
{ The complete source code remains property of the author and may    }
{ not be distributed, published, given or sold in any form as such.  }
{ No parts of the source code can be included in any other component }
{ or application without written authorization of the author.        }
{********************************************************************}

unit WEBLib.Bluetooth;

{$DEFINE NOPP}

interface

uses
  Classes, Web, JS;

type
  TJSValueParam = record
     value: JSValue;
  end;

  TBluetoothCharacteristic = class;
  TBluetoothService = class;

  TReadIntEvent = procedure(Sender: TObject; AInt: integer) of object;
  TReadSingleEvent = procedure(Sender: TObject; ASingle: Single) of object;
  TReadDoubleEvent = procedure(Sender: TObject; ADouble: Double) of object;
  TReadStringEvent = procedure(Sender: TObject; AString: string) of object;
  TReadArrayEvent = procedure(Sender: TObject; AArray: array of Byte) of object;

  TGetDeviceErrorEvent = procedure(Sender: TObject; AError: TJSObject) of object;
  TGetDeviceObjectEvent = procedure(Sender:TObject; AObject: TJSObject) of object;
  TGetServiceObjectEvent = procedure(Sender: TObject; AOject: TJSObject) of object;
  TGetCharacteristicObjectEvent = procedure(Sender: TObject; AObject: TJSObject) of object;
  TGetDescriptorObjectEvent = procedure(Sender: TObject; AObject: TJSObject) of object;

  TGetServicesEvent = procedure(Sender: TObject; ServiceArray: array of TJSObject) of object;
  TGetCharacteristicsEvent = procedure(Sender: TObject; CharacteristicArray: array of TJSObject) of object;
  TGetDescriptorsEvent = procedure(Sender: TObject; DescriptorArray: array of TJSObject) of object;

  TGetNotifyIntEvent = procedure(Sender: TObject; AValue: integer) of object;
  TGetNotifyDoubleEvent = procedure(Sender: TObject; AValue: double) of object;
  TGetNotifyArrayEvent = procedure(Sender: TObject; AValue: array of byte) of object;

  TBTRefProc = reference to procedure;

  TBTServiceProc = reference to procedure(AService: TBluetoothService);
  TBTCharacteristicProc = reference to procedure(ACharacteristic: TBluetoothCharacteristic);
  TBTReadValueProc = reference to procedure(AValue: integer);
  TBTReadSingleProc = reference to procedure (AValue: single);
  TBTReadDoubleProc = reference to procedure(AValue: double);
  TBTReadArrayProc = reference to procedure (AValue: array of byte);

  { TBluetoothService }

  TBluetoothService = class(TPersistent)
  protected
    procedure DoGetCharacteristic(AObject: TJSObject);
    procedure DoGetCharacteristics(CharacteristicArray: array of TJSObject);
    procedure DoGetService(AObject: TJSObject);
    procedure SetServiceObject(AService: TJSObject);
    function CreateCharacteristic(AObject: TJSObject): TBluetoothCharacteristic;
  public
    constructor Create; overload;
    constructor Create(AService: TJSObject); overload;
    procedure GetCharacteristic(uuid: string); overload;
    procedure GetCharacteristic(uuid: string; proc: TBTCharacteristicProc); overload;
    procedure GetCharacteristic(uuid: integer); overload;
    procedure GetCharacteristic(uuid: integer; proc: TBTCharacteristicProc); overload;
    procedure GetCharacteristics;
  published
    property UUID: string read FUUID;
    property OnCharacteristic: TGetCharacteristicObjectEvent read FOnCharacteristic write FOnCharacteristic;
    property OnCharacteristics: TGetCharacteristicsEvent read FOnCharacteristics write FOnCharacteristics;
    property OnService: TGetServiceObjectEvent read FOnService write FOnService;
  end;

  { TBluetoothDevice }

  TBluetoothDevice = class(TPersistent)
  protected
    function DoHandleDisconnect(Event: TJSEvent): boolean;
    procedure DoGetService(AObject: TJSObject);
    procedure GetServiceAndConnect(uuid:string; proc: TBTServiceProc);
    procedure AssignDisconnectHandler;
    procedure DoConnect;
    procedure DoGetServices(ServiceArray: array of TJSObject);
    procedure SetDeviceObject(AObject: TJSObject);
    function CreateService(AObject: TJSObject): TBluetoothService;
  public
    constructor Create; overload;
    constructor Create(ADevice: TJSObject); overload;
    destructor Destroy; override;
    procedure GetService(uuid: string); overload;
    procedure GetService(uuid: string; proc: TBTServiceProc); overload;
    procedure GetServices;
    procedure Connect; overload;
    procedure Connect(proc: TBTRefProc); overload;
    function DisconnectDevice:boolean;
    function ReConnectDevice:boolean;
    function Connected: boolean;
    property Service: TBluetoothService read FService;
  published
    property OnConnect : TNotifyEvent read FOnConnect write FOnConnect;
    property OnDisconnect: TNotifyEvent read FOnDisconnect write FOnDisconnect;
    property OnService: TGetServiceObjectEvent read FOnService write FOnService;
    property OnServices: TGetServicesEvent read FOnServices write FOnServices;
  end;

  { TBluetoothCharacteristic }

  TBluetoothCharacteristic = class(TPersistent)
  protected
    procedure DoGetDescriptor(AObject: TJSObject);
    procedure DoGetDescriptors(DescriptorArray: array of TJSObject);
    procedure DoRead(AValue:integer); virtual;
    procedure DoReadSingle(ASingle: single); virtual;
    procedure DoReadDouble(ADouble: double); virtual;
    procedure DoReadArray(AArray: array of byte); virtual;
    procedure DoReadString(a: array of byte); virtual;
    procedure DoWriteDone; virtual;
    procedure DoStartNotify;
    procedure DoStopNotify;
    procedure DoNotify(AValue: integer); virtual;
    procedure DoNotifyDouble(AValue: double); virtual;
    procedure DoNotifyArray(AValue: array of byte); virtual;
    procedure AssignNotifyHandler;
    procedure UnAssignNotifyHandler;
    procedure DoHandleNotifications(Event: TJSEvent);
  public
    constructor Create(ACharacteristic: TJSObject);
    procedure GetDescriptor(uuid: string); overload;
    procedure GetDescriptor(uuid: integer); overload;
    procedure GetDescriptors;
    procedure StartNotify;
    procedure StopNotify;

    procedure Read; overload;
    procedure Read(proc: TBTReadValueProc); overload;
    procedure ReadSingle; overload;
    procedure ReadSingle(proc: TBTReadSingleProc); overload;
    procedure ReadDouble; overload;
    procedure ReadDouble(proc: TBTReadDoubleProc); overload;
    procedure ReadArray; overload;
    procedure ReadArray(proc: TBTReadArrayProc); overload;
    procedure ReadString;
    procedure WriteByte(b: byte);
    procedure WriteSmallInt(s: smallint);
    procedure WriteInt(i: integer);
    procedure WriteArray(a: array of byte);
    procedure WriteString(s: string);
    procedure WriteSingle(s: single);
    procedure WriteDouble(d: double);
  published
    property UUID: string read FUUID;
    property OnReadValue: TReadIntEvent read FOnReadValue write FOnReadValue;
    property OnReadSingle: TReadSingleEvent read FOnReadSingle write FOnReadSingle;
    property OnReadDouble: TReadDoubleEvent read FOnReadDouble write FOnReadDouble;
    property OnReadArray: TReadArrayEvent read FOnReadArray write FOnReadArray;
    property OnReadString: TReadStringEvent read FOnReadString write FOnReadString;
    property OnDescriptor: TGetDescriptorObjectEvent read FOnDescriptor write FOnDescriptor;
    property OnDescriptors: TGetDescriptorsEvent read FOnDescriptors write FOnDescriptors;
    property OnStartNotify: TNotifyEvent read FOnStartNotify write FOnStartNotify;
    property OnHandleNotification: TNotifyEvent read FOnHandleNotification write FOnHandleNotification;
    property OnNotifyInt: TGetNotifyIntEvent read FOnNotifyInt write FOnNotifyInt;
    property OnNotifyDouble: TGetNotifyDoubleEvent read FOnNotifyDouble write FOnNotifyDouble;
    property OnNotifyArray: TGetNotifyArrayEvent read FOnNotifyArray write FOnNotifyArray;
    property OnWriteDone: TNotifyEvent read FOnWriteDone write FOnWriteDone;
  end;

  { TBluetoothDescriptor }

  TBluetoothDescriptor = class(TPersistent)
  protected
     procedure DoReadString(a: array of byte); virtual;
  public
    constructor Create(ADescriptor: TJSObject);
    procedure ReadString;
  published
    property UUID: string read FUUID;
    property OnReadString: TReadStringEvent read FOnReadString write FOnReadString;
  end;

  { TBluetooth }

  TBluetooth = class(TComponent)
  protected
    procedure DoGetDeviceObject(AObject: TJSObject); virtual;
    procedure DoGetDeviceError(AError: TJSObject); virtual;
  public
    constructor Create; overload;
    constructor Create(AOwner: TComponent); overload; override;
    destructor Destroy; override;
    function HasBluetooth: boolean;
    function GetDevice: boolean; overload;
    function GetDevice(proc: TBTRefProc): boolean; overload;
    property Device: TBluetoothDevice read FDevice;
  published
    property DeviceName: string read FDeviceName write FDeviceName;
    property FilterService: TStringList read FFilterService write SetFilterService;
    property OnDeviceObject: TGetDeviceObjectEvent read FOnDeviceObject write FOnDeviceObject;
    property OnDeviceError: TGetDeviceErrorEvent read FOnDeviceError write FOnDeviceError;
  end;

  TWebBluetooth = class(TBluetooth);

implementation
end.
