{$Error This is a closed source unit. You cannot compile it, it was already compiled.}
{$Error Probably the IDE has cleaned up and you have to unpack the zip again.}
{********************************************************************}
{                                                                    }
{ written by TMS Software                                            }
{            copyright (c) 2019 - 2020                                      }
{            Email : info@tmssoftware.com                            }
{            Web : http://www.tmssoftware.com                        }
{                                                                    }
{ The source code is given as is. The author is not responsible      }
{ for any possible damage done due to the use of this code.          }
{ The complete source code remains property of the author and may    }
{ not be distributed, published, given or sold in any form as such.  }
{ No parts of the source code can be included in any other component }
{ or application without written authorization of the author.        }
{********************************************************************}

unit WEBLib.FirestoreCDS;

//{$define cdsdebug}                              t

{
  For soft error testing try this
  1) Cause a signout by TFireBase.auth.Signout
  2) On Open when it asks for login, close that popup.
  This should generate a soft error.

  Other soft error ideas can be found by looking in the
  comments for "soft error testing"

  Hard error testing: Any Delphi Exception will be
  hard error and should cause red alert. Try sending a
  usage error from the app like a KeyFieldName that does
  not match any of the field names.
}

{$DEFINE NOPP}

interface

uses
   Classes, JS, Web, SysUtils, WEBLib.CDS, jsondataset,
   DB, libfirestore, WebLib.REST;

type
  TFirestoreClientDataset = class;
  TFirestoreClientDataProxy = class;

  TFirestoreResultEvent = procedure(success: Boolean; data: JSValue; errorName, errorMsg: String) of object;
  TFireStoreRangeFilterType =  (frStartAt, frEndAt, frStartAfter, frEndBefore);

  TFirestoreDb = class(TObject)
  protected
    procedure AttachSortedQuery(sortFields: TJSArray; var aQuery: TFirestoreQuery);
    function IsWhereWithId(whereFilters: TJSArray; theKeyFieldname: String; var aValue: String; var isInvalidUsage: Boolean): Boolean;
    procedure AttachWhereQuery(whereFilters: TJSArray; var aQuery: TFirestoreQuery);
    procedure AttachRangeQuery(rangeFilters: TJSArray; sortDefs: TJSArray; var aQuery: TFirestoreQuery; var rangeError: String);
    procedure AttachLimitQuery(limitRecords: Integer; var aQuery: TFirestoreQuery);
    procedure AttachContinueQuery(aContinueFlag: Boolean; var aQuery: TFirestoreQuery);
  public
    constructor Create;
    destructor Destroy; override;

    procedure DoSignIn(responseEvent: TFirestoreResultEvent);
    procedure InitFireBaseApp(anApiKey, anAuthDomain, aProjectId: String);
    procedure GetDocs(aCollectionName: String; aClientDataSet: TFirestoreClientDataset; responseEvent: TFirestoreResultEvent);
    procedure GetDoc(aCollectionName: String; aClientDataSet: TFirestoreClientDataset; anId: String; responseEvent: TFirestoreResultEvent);
    procedure UpdateDoc(aCollectionName: String; anId: String; data: TJSObject; responseEvent: TFirestoreResultEvent);
    procedure DeleteDoc(aCollectionName: String; anId: String; responseEvent: TFirestoreResultEvent);
    procedure AddDoc(aCollectionName: String; dataToWrite: TJSObject; responseEvent: TFirestoreResultEvent);
  end;

  TFirestoreEntityUpdate = class(TCollectionItem)
  protected
    procedure HandleInsert(success: Boolean; data: JSValue; errorName, errorMsg: String);
    procedure HandleUpdateDelete(success: Boolean; data: JSValue; errorName, errorMsg: String);
  public
    constructor Create(Collection: TCollection);  override;
    destructor Destroy; override;
    procedure Assign(Source: TPersistent); override;
    function Update: Boolean;
    function Insert: Boolean;
    function Delete: Boolean;
    property CustomData: JSValue read FCustomData write FCustomData;
    property Data: TJSObject read FData write FData;
  end;

  TFirestoreEntityUpdates = class(TOwnedCollection)
  protected
    property DataProxy: TFirestoreClientDataProxy read FDataProxy;
  public
    constructor Create(AOwner: TPersistent); reintroduce;
    function Add: TFirestoreEntityUpdate; reintroduce;
    function Insert(Index: Integer): TFirestoreEntityUpdate; reintroduce;
    property Items[Index: Integer]: TFirestoreEntityUpdate read GetItems write SetItems; default;
  end;

  TFirestoreErrorEvent = procedure(Sender: TObject; errorName, errorMsg: String) of object;

  TFirestoreClientDataProxy = Class(TDataProxy)
 protected
    function GetDataRequest(aOptions: TLoadOptions; aAfterRequest: TDataRequestEvent; aAfterLoad: TDatasetLoadEvent) : TDataRequest; override;
    procedure HandleSignIn(success: Boolean; data: JSValue; errorName, errorMsg: String);
    function DoGetData(aRequest : TDataRequest) : Boolean; override;
    procedure DoOnError(errorName, errorMsg: String);
    procedure DoOnEntityError(anEntityUpdate: TFirestoreEntityUpdate; errorName, errorMsg: String);
    procedure DoOnEntityUpdateDelete(anEntityUpdate: TFirestoreEntityUpdate);
    procedure DoOnEntityInsert(anEntityUpdate: TFirestoreEntityUpdate);
    procedure HandleGetDocs(success: Boolean; data: JSValue; errorName, errorMsg: String);
    procedure ProcessUpdate(desc: TRecordUpdateDescriptor);
    procedure Close;
    // Dummy Abstract method
    function ProcessUpdateBatch(aBatch: TRecordUpdateBatch): Boolean; override;
    function GetKeyFieldName: String;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    property CollectionName: String read FCollectionName write FCollectionName;
    property Db: TFirestoreDb read FDb;
    property KeyFieldName: String read GetKeyFieldName;
  end;

{$ifdef FUTURE_FEATURE}
  TFirestoreFieldMapper = class(TJSONObjectFieldMapper)
  private
    FFieldDefs: TFieldDefs;
  public
    Function CopyRow(aRow : JSValue) : JSValue; override;
    property FieldDefs: TFieldDefs read FFieldDefs write FFieldDefs;
  end;
{$endif}

  TFirestoreSignInChangeEvent = procedure(isSignIn: Boolean; UserName: String; UserEmail: String) of object;

  TFirestoreClientDataset = Class(TClientDataSet)
  protected
    procedure InternalClose; override;
    Function AddToChangeList(aChange : TUpdateStatus) : TRecordUpdateDescriptor ; override;
    procedure SetActive(Value: Boolean); override;
    procedure AfterLoadDFMValues; override;
    procedure DoAfterOpen; override;
    procedure DoOnError(errorName, errorMsg: String);
    procedure UpdateStarts;
    procedure UpdateEnds;
    procedure FixKeyFieldCase;
    function IsKeyFieldValid: Boolean;
    procedure AuthChanged(aUser: TFireBaseAuthUser);
{$ifdef FUTURE_FEATURE}
    Function CreateFieldMapper : TJSONFieldMapper; override;
{$endif}

    property DataLoaded: Boolean read FDataLoaded write FDataLoaded;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure Refresh(Force: Boolean=False); reintroduce;

    // Query related
    procedure AddSortFieldDef(aField: String; isAscending: Boolean);
    procedure ClearSortFieldDefs;

    // Filters
    // Important! The following work on current sorted order
    // The results may be different or even not allowed based 
	// on the field you specified in AddSortFieldDef above
    procedure AddServiceFilterCondition(aField: String; anOperator: String; aValue: JSValue);

    // The value refers to the value of active sort field specified
	// by AddSortFieldDef above
    procedure AddServiceRangeFilter(aRangeFilterType: TFireStoreRangeFilterType; aValue: JSValue);

    procedure AddServiceFilterCount(numRecords: Integer);
    procedure AddServiceFilterContinueAfterLast;
    procedure ClearServiceFilters;

    procedure SubscribeToSignInEvents(doSubscribe: Boolean);

{$ifdef FUTURE_FEATURE}
    function ConvertToDateTime(AField: TField; AValue: JSValue; ARaiseException: Boolean): TDateTime; override;
    function ConvertDateTimeToNative(AField: TField; AValue: TDateTime): JSValue; override;
{$endif}
  published
    property ApiKey: String read FApiKey write FApiKey;
    property CollectionName: String read FCollectionName write FCollectionName;
    property KeyFieldName: String read FKeyFieldName write FKeyFieldName;
    property AuthDomain: String read FAuthDomain write FAuthDomain;
    property ProjectId: String read FProjectId write FProjectId;
    property SignInRequired: Boolean read FSignInRequired write FSignInRequired;
    property AutoGenerateKeys: Boolean read FAutoGenerateKeys write FAutoGenerateKeys;

{$ifdef FUTURE_FEATURE}
    // Flag used to interprete timestamp values in FireStore
    // as datetime and vice versa when writing
    property DateTimeIsTimestamp: Boolean read FDateTimeIsTimestamp write FDateTimeIsTimestamp;
{$endif}

    property OnError: TFirestoreErrorEvent read FOnError write FOnError;
    property OnSignInChange: TFirestoreSignInChangeEvent read FOnSignInChange write FOnSignInChange;
  end;

  TWebFirestoreClientDataset = class(TFirestoreClientDataset);

implementation
end.
