{$Error This is a closed source unit. You cannot compile it, it was already compiled.}
{$Error Probably the IDE has cleaned up and you have to unpack the zip again.}
{********************************************************************}
{                                                                    }
{ written by TMS Software                                            }
{            copyright (c) 2018 - 2020                               }
{            Email : info@tmssoftware.com                            }
{            Web : http://www.tmssoftware.com                        }
{                                                                    }
{ The source code is given as is. The author is not responsible      }
{ for any possible damage done due to the use of this code.          }
{ The complete source code remains property of the author and may    }
{ not be distributed, published, given or sold in any form as such.  }
{ No parts of the source code can be included in any other component }
{ or application without written authorization of the author.        }
{********************************************************************}

unit WEBLib.GoogleCalendar;

{$DEFINE NOPP}

interface

uses
  Classes, WebLib.REST, WebLib.Google, SysUtils, StrUtils, WebLib.Graphics,
  WebLib.Dialogs, WebLib.JSON;

const
  MAJ_VER = 1; // Major version nr.
  MIN_VER = 0; // Minor version nr.
  REL_VER = 1; // Release nr.
  BLD_VER = 0; // Build nr.

  // version history
  // 1.0.0.0 : First release
  // 1.0.1.0 : Fixed : Request issues due to changes in the Google Calendar API

var
  GColorNames: array[1..12] of string = ('Default', 'Blue', 'Green', 'Purple', 'Red', 'Yellow', 'Orange',
    'Turquoise', 'Gray', 'Boldblue', 'BoldGreen', 'BoldRed');

type
  TGoogleCalendar = class;
  TGCalendar = class;
  TGCalendars = class;
  TGCalendarItems = class;
  TGCalendarItem = class;
  TGAttendees = class;
  TGReminders = class;
  TGColors = class;

  TStatus = (stConfirmed, stTentative, stCancelled);
  TVisibility = (viDefault, viPublic, viPrivate);
  TResponseStatus = (rsNeedsAction, rsDeclined, rsTentative, rsAccepted);
  TReminderMethod = (rmPopup, rmEmail, rmSMS);
  TGItemColor = (icDefault, icBlue, icGreen, icPurple, icRed,
    icYellow, icOrange, icTurquoise, icGray, icBoldBlue,
    icBoldGreen, icBoldRed);
  TGCalendarColor = (ccBoldBrown, ccBrown, ccBoldRed, ccBoldOrange, ccBoldYellow,
    ccMediumGreen, ccDarkGreen, ccBoldGreen, ccLimeGreen, ccLightGreen, ccYellow,
    ccOrange, ccGreen, ccTurquoise, ccBlue, ccBoldBlue, ccBoldPurple, ccPurple,
    ccGray, ccLightGray, ccBoldGray, ccPink, ccBoldPing, ccDarkPurple);

  TGReminder = class(TCollectionItem)
  public
    constructor Create(Collection: TCollection);  override;
    destructor Destroy; override;
    procedure Assign(Source: TPersistent); override;
    function GetOwner: TPersistent; override;
  protected
    function GetDisplayName: string; override;
  published
    property Minutes: integer read FMinutes write SetMinutes default 10;
    property Method: TReminderMethod read FMethod write SetMethod default rmPopup;
  end;

  TGReminders = class(TCollection)
  protected
    procedure Update(Item: TCollectionItem); override;
  public
    constructor Create(AOwner: TGCalendarItem); reintroduce;
    function GetOwner: TPersistent; override;
    function Add: TGReminder; reintroduce;
    function Insert(Index: Integer):TGReminder; reintroduce;
    property Items[Index: Integer]: TGReminder read GetItems write SetItems; default;
  end;

  TGDefaultReminders = class(TCollection)
  protected
    procedure Update(Item: TCollectionItem); override;
  public
    constructor Create(AOwner: TGCalendar); reintroduce;
    function GetOwner: TPersistent; override;
    function Add: TGReminder; reintroduce;
    function Insert(Index: Integer):TGReminder; reintroduce;
    property Items[Index: Integer]: TGReminder read GetItems write SetItems; default;
  end;

  TGAttendee = class(TCollectionItem)
  public
    constructor Create(Collection: TCollection);  override;
    destructor Destroy; override;
    procedure Assign(Source: TPersistent); override;
    function GetOwner: TPersistent; override;
  protected
    function GetDisplayName: string; override;
  published
    property Summary: string read FSummary write SetSummary;
    property Email: string read FEmail write SetEmail;
    property Status: TResponseStatus read FStatus write SetStatus default rsNeedsAction;
  end;

  TGAttendees = class(TCollection)
  protected
    procedure Update(Item: TCollectionItem); override;
  public
    constructor Create(AOwner: TGCalendarItem); reintroduce;
    function GetOwner: TPersistent; override;
    function Add: TGAttendee; reintroduce;
    function Insert(Index: Integer):TGAttendee; reintroduce;
    property Items[Index: Integer]: TGAttendee read GetItems write SetItems; default;
  end;

  TGCalendar = class(TCollectionItem)
  public
    constructor Create(Collection: TCollection);  override;
    destructor Destroy; override;
    procedure Assign(Source: TPersistent); override;
    function GetOwner: TPersistent; override;
  protected
    function GetDisplayName: string; override;
  published
    property ID: string read FID write SetID;
    property Description: string read FDescription write SetDescription;
    property ETag: string read FEtag write FEtag;
    property Location: string read FLocation write SetLocation;
    property Primary: boolean read FPrimary write SetPrimary default false;
    property Summary: string read FSummary write SetSummary;
    property TimeZone: string read FTimeZone write SetTimeZone;
    property Color: TGCalendarColor read FColor write FColor default ccGray;
    property BackgroundColor: TColor read FBackgroundColor write FBackgroundColor default clNone;
    property ForeGroundColor: TColor read FForegroundColor write FForegroundColor default clNone;
    property DefaultReminders: TGDefaultReminders read FDefaultReminders write FDefaultReminders;
  end;

  TGCalendars = class(TOwnedCollection)
  protected
    procedure Update(Item: TCollectionItem); override;
  public
    constructor Create(AOwner: TGoogleCalendar); reintroduce;
    function GetOwner: TPersistent; override;
    function Add: TGCalendar; reintroduce;
    function Insert(Index: Integer):TGCalendar; reintroduce;
    property Items[Index: Integer]: TGCalendar read GetItems write SetItems; default;
    function ByID(ID: string): TGCalendar;
  end;

  TGCalendarUser = class(TPersistent)
  public
    constructor Create; reintroduce;
    procedure Assign(Source: TPersistent); override;
  published
    property Email: string read FEmail write FEmail;
    property DisplayName: string read FDisplayName write FDisplayName;
  end;

  TGCalendarItem = class(TCollectionItem)
  public
    constructor Create(Collection: TCollection);  override;
    destructor Destroy; override;
    procedure Assign(Source: TPersistent); override;
    function GetOwner: TPersistent; override;
  protected
    function GetDisplayName: string; override;
  published
    property ID: string read FID write SetID;
    property ETag: string read FETag write FETag;
    property CalendarID: string read FCalendarID write SetCalendarID;
    property Summary: string read FSummary write SetSummary;
    property Description: string read FDescription write SetDescription;
    property StartTime: TDateTime read FStartTime write SetStartTime;
    property EndTime: TDateTime read FEndTime write SetEndTime;
    property Created: TDateTime read FCreated write SetCreated;
    property Updated: TDateTime read FUpdated write SetUpdated;
    property IsAllDay: boolean read FIsAllDay write SetIsAllDay default false;
    property Location: string read FLocation write SetLocation;
    property Status: TStatus read FStatus default stConfirmed;
    property Visibility: TVisibility read FVisibility write SetVisibility default viPublic;
    property Recurrence: string read FRecurrence write SetRecurrence;
    property RecurringID: string read FRecurringID write SetRecurringID;
//    property Overlap: boolean read FOverlap write SetOverlap default false;
    property Sequence: integer read FSequence write SetSequence;
    property Attendees: TGAttendees read FAttendees write FAttendees;
    property Reminders: TGReminders read FReminders write FReminders;
    property UseDefaultReminders: boolean read FUseDefaultReminders write FUseDefaultReminders default true;
    property TimeZone: String read FTimeZone write FTimeZone;
    property Color: TGItemColor read FColor write FColor default icDefault;
    property Creator: TGCalendarUser read FCreator write SetCreator;
    property Organizer: TGCalendarUser read FOrganizer write SetOrganizer;
    property SendNotifications: boolean read FSendNotifications write FSendNotifications default false;
  end;

  TGCalendarItems = class(TOwnedCollection)
  protected
    procedure Update(Item: TCollectionItem); override;
  public
    constructor Create(AOwner: TGoogleCalendar); reintroduce;
    function GetOwner: TPersistent; override;
    function Add: TGCalendarItem; reintroduce;
    function Find(const ID: integer): TGCalendarItem; overload;
    function Find(const ID: string): TGCalendarItem; overload;
    function Insert(Index: Integer): TGCalendarItem; reintroduce;
    function GetItemByID(const ID: string): TGCalendarItem;
    property Items[Index: Integer]: TGCalendarItem read GetItems write SetItems; default;
  end;

  TGColor = class(TCollectionItem)
  public
    constructor Create(Collection: TCollection);  override;
    destructor Destroy; override;
    procedure Assign(Source: TPersistent); override;
    function GetOwner: TPersistent; override;
  protected
    function GetDisplayName: string; override;
  published
    property ID: integer read FID write SetID default -1;
    property BackgroundColor: TColor read FBackgroundColor write SetBackGroundColor default clNone;
    property ForegroundColor: TColor read FForegroundColor write SetForeGroundColor default clNone;
  end;

  TGColors = class(TCollection)
  protected
    procedure Update(Item: TCollectionItem); override;
  public
    constructor Create(AOwner: TGoogleCalendar); reintroduce;
    function GetOwner: TPersistent; override;
    function Add: TGColor; reintroduce;
    function Insert(Index: Integer): TGColor; reintroduce;
    property Items[Index: Integer]: TGColor read GetItems write SetItems; default;
  end;

  TGoogleCalendar = class(TGoogleClient)
  protected
    procedure HandleGetCalendars(Sender: TObject; AResponse: string; var Handled: boolean);
    procedure HandleGetCalendar(Sender: TObject; AResponse: string; var Handled: boolean);
    procedure HandleGetColors(Sender: TObject; AResponse: string; var Handled: boolean);
    procedure HandleGetItemByID(Sender: TObject; AResponse: string; var Handled: boolean);
    procedure HandleUpdateItem(Sender: TObject; AResponse: string; var Handled: boolean);
    procedure HandleUpdateCalendar(Sender: TObject; AResponse: string; var Handled: boolean);
    procedure HandleTokenLifetime(Sender: TObject; AResponse: string; var Handled: boolean);
    procedure GetCalendarInt(ID: string; FromDate, ToDate: TDate; ChangedSince: TDateTime; MaxResults: integer = 250);
    function GetCalendarData(Item: TGCalendarItem): string;
    procedure ParseCalendar(jo: TJSONObject; Item: TGCalendar);
    procedure ParseCalendarItem(CalendarID: string; jo: TJSONObject; Item: TGCalendaritem);
    procedure ParseColor(jo: TJSONObject; ColorType: string);
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure AddCalendar(Calendar: TGCalendar);
    procedure DeleteCalendar(Calendar: TGCalendar);
    procedure UpdateCalendar(Calendar: TGCalendar);
    procedure GetCalendars;
    procedure GetCalendar(ID: string; FromDate, ToDate: TDate; MaxResults: integer = 250); overload;
    procedure GetCalendar(FromDate, ToDate: TDate; MaxResults: integer = 250); overload;
    procedure GetCalendar(ID: string; ChangedSince: TDateTime; MaxResults: integer = 250); overload;
    procedure GetCalendar(ID: string; MaxResults: integer = 250); overload;
    procedure GetColors;
    function GetItemByID(CalendarID, ItemID: string): TGCalendarItem;
    procedure Add(Item: TGCalendarItem);
    procedure Update(Item: TGCalendarItem);
    procedure Delete(Item: TGCalendarItem);
    property ItemColors: TGColors read FItemColors write FItemColors;
    property CalendarColors: TGColors read FCalendarColors write FCalendarColors;
    function AccessTokenLifeTime: integer;
  published
    property Calendars: TGCalendars read FCalendars write FCalendars;
    property Items: TGCalendarItems read FItems write FItems;
    property OnCalendars: TNotifyEvent read FOnCalendars write FOnCalendars;
    property OnCalendar: TNotifyEvent read FOnCalendar write FOnCalendar;
  end;

  TWebGoogleCalendar = class(TGoogleCalendar);

implementation
end.
