{$Error This is a closed source unit. You cannot compile it, it was already compiled.}
{$Error Probably the IDE has cleaned up and you have to unpack the zip again.}
{********************************************************************}
{                                                                    }
{ written by TMS Software                                            }
{            copyright (c) 2019 - 2020                               }
{            Email : info@tmssoftware.com                            }
{            Web : http://www.tmssoftware.com                        }
{                                                                    }
{ The source code is given as is. The author is not responsible      }
{ for any possible damage done due to the use of this code.          }
{ The complete source code remains property of the author and may    }
{ not be distributed, published, given or sold in any form as such.  }
{ No parts of the source code can be included in any other component }
{ or application without written authorization of the author.        }
{********************************************************************}

unit WEBLib.ThreeJsChart;

{$DEFINE NOPP}

interface

uses classes, Sysutils, WEBLib.Controls, WEBLib.webtools, math, strutils,
   WEBLib.graphics, web, js, libthreejs, WEBLib.ThreeJsControls;

type
  TThreeJsChartLegendPosition = (lpNone, lpTopLeft, lpTopRight, lpBottomLeft, lpBottomRight);
  TThreeJsChartLegendItemShape = (lshSquare, lshCircle);

  TThreeJsCustomChart = class(TThreeJsCustomControl)
  protected
    procedure InitThreeJs; override;
    procedure Initialize; virtual;
    function AddChartLegendBoard(
              aPosition: TThreeJsChartLegendPosition;
              lines: string;
              // pass color1, color2, ... corresponding to lines
              colorList: TJSArray;
              shapeList: TJSArray;
              textColor: TColor = $000000;
              fontFace: string = 'Arial';
              fontHeight: double = 1; //in webGL units
              isBold: boolean = false;
              isTransparentFill: boolean = false;
              fillColor: TColor = $ffffff;
              borderWidth: double = 12;
              borderColor: TColor = $E6D8AD;
              marginPixels: double = 24;
              radiusPixels: double = 6): TThreeJsObject3D;
  public
    procedure CreateInitialize; override;

    procedure CreateChart;
    procedure CreateChartFromData; virtual;
    // clears all data and legend
    procedure clearChart; virtual;

    // Chart Legend related
    procedure AddChartLegendItem(aName: string; aColor: TColor; aShape: TThreeJsChartLegendItemShape = lshSquare);
    procedure AddChartLegendBox; 
    procedure RemoveChartLegendBox;

    property ShowShadows: boolean read FShowShadows write FShowShadows;
    property AutoRotateAxisLabels: boolean read FAutoRotateAxisLabels write FAutoRotateAxisLabels;
    property ShowTransparent: boolean read FShowTransparent write FShowTransparent;
    property Opacity: double read FOpacity write FOpacity;
    property LegendPosition: TThreeJsChartLegendPosition read FLegendPosition write FLegendPosition;
    property LegendStyle: TThreeJsTextStyle read FLegendStyle write FLegendStyle;
    // To make Legend Text fixed size, this is the fixed distance kept from Camera
    property LegendDistance: double read FLegendDistance write FLegendDistance;
  end;

  TThreeJsChartType = (ctBarChart, ctCylinderChart, ctConeChart, ctLineChart, ctAreaChart);
  TChartsSet = set of TThreeJsChartType;

  TThreeJsChartSeries = class
  protected
    function GetValue(aLegendIndex, aCategoryIndex: integer): JSValue;
    function GetLegendCount: integer;
    function GetCategoryCount: integer;
    function GetLegendLabel(anIndex: integer): string;
    function GetCategoryLabel(anIndex: integer): string;
    function GetValueStrForValueAxis(aValue: JSValue): string;
    procedure PrepareValueLabelMetrics;

    property LegendCount: integer read getLegendCount;
    property CategoryCount: integer read getCategoryCount;
    property LegendLabels[anIndex: integer]: string read getLegendLabel;
    property CategoryLabels[anIndex: integer]: string read getCategoryLabel;
    property Values[aLegendIndex, aCategoryIndex: Integer]: JSValue read getValue;
    property ValueAxisDivisions: integer read FValueAxisDivisions;
    property ValueAxisRange: double read FValueAxisRange;
  public
    constructor Create(aLegendAxisLabels: TJSArray; aCategoryAxisLabels: TJSArray); virtual;
    // Number of data items in the following has to match number of Category Labels
    procedure AddLegendRow(aLegendLabel: string; data: TJSArray);

    function GetValueStr(aValue: JSValue): string;
    procedure SetColors(colors: array of JSValue);
    procedure ResetColors;

    property ChartType: TThreeJsChartType read FChartType write FChartType;
    property ValueFormatFloat: string read FValueFormatFloat write FValueFormatFloat;
    // specify the following only if needed different from above
    property ValueAxisFormatFloat: string read FValueAxisFormatFloat write FValueAxisFormatFloat;

    // chart item properties
    property ItemWidth: double read FItemWidth write FItemWidth;
    property itemSpace: double read FitemSpace write FitemSpace;
    property ItemTextStyle: TThreeJsTextStyle read FItemTextStyle write FItemTextStyle;
    property PlotWidth: double read FPlotWidth write FPlotWidth;

    // titles
    property Title: string read FTitle write FTitle;
    property TitleTextStyle: TThreeJsTextStyle read FTitleStyle write FTitleStyle;
    property TitleSpace: double read FTitleSpace write FTitleSpace;
    property AxisTitleStyle: TThreeJsTextStyle read FAxisTitleStyle write FAxisTitleStyle;
    property CategoryAxisTitle: string read FCategoryAxisTitle write FCategoryAxisTitle;
    property CategoryAxisTitleSpace: double read FCategoryAxisTitleSpace write FCategoryAxisTitleSpace;
    property LegendAxisTitle: string read FLegendAxisTitle write FLegendAxisTitle;
    property LegendAxisTitleSpace: double read FLegendAxisTitleSpace write FLegendAxisTitleSpace;
    property ValueAxisTitle: string read FValueAxisTitle write FValueAxisTitle;
    property ValueAxisTitleSpace: double read FValueAxisTitleSpace write FValueAxisTitleSpace;

    // Axis properties
    property ValueAxisLength: double read FValueAxisLength write FValueAxisLength;
    // Other axes lengths are calculated based on chart items
    property AutoMarkValueAxis: boolean read FAutoMarkValueAxis write FAutoMarkValueAxis;
    // If not Auto, specify the following values
    property ValueAxisMarkMaximum: double read FValueAxisMarkMaximum write FValueAxisMarkMaximum;
    property ValueAxisMarkMinimum: double read FValueAxisMarkMinimum write FValueAxisMarkMinimum;
    property ValueAxisMarkStep: double read FValueAxisMarkStep write FValueAxisMarkStep;

    property CategoryAxisLabelStyle: TThreeJsTextStyle read FCategoryAxisLabelStyle write FCategoryAxisLabelStyle;
    property LegendAxisLabelStyle: TThreeJsTextStyle read FLegendAxisLabelStyle write FLegendAxisLabelStyle;
    property ValueAxisLabelStyle: TThreeJsTextStyle read FValueAxisLabelStyle write FValueAxisLabelStyle;
    property AxisLabelSpace: double read FAxisLabelSpace write FAxisLabelSpace;
  end;

  TThreeJsChartItemEventArgs = class
  public
    property Item: TThreeJsObject3D read FItem write FItem;
    property LegendLabel: string read FLegendLabel write FLegendLabel;
    property CategoryLabel: string read FCategoryLabel write FCategoryLabel;
    property Value: double read FValue write FValue;
    property ValueStr: string read FValueStr write FValueStr;
    property XIndex: Integer read FXIndex write FXIndex;
    property ZIndex: Integer read FZIndex write FZIndex;
  end;

  //Just sending the object is not enough becauses for plot
  //objects, the xIndex and the itemValue has to be determined
  //based on the click point.
  TThreeJsItemEvent = procedure(Sender: TObject; Args: TThreeJsChartItemEventArgs) of object;
  TThreeJsItemNotifyEvent = procedure(Sender: TObject; chartItem: TThreeJsObject3D) of object;

  TThreeJsChart = class(TThreeJsCustomChart)
  protected
    procedure Initialize; override;
    function GetRaycasterTargetObjects: TJSArray; override;

    function GetColorFromColorSeries(Zindex: integer): TColor;

    function AddChartItem(xIndex, zIndex: integer; X, Z: double; height: double;
                    aValueLabel: string; aColor: TColor): TThreeJsObject3D;

    function ShowChartValueOnItem(xIndex, zIndex: integer; X, Z: double; height: double; aValueLabel: string): TThreeJsObject3D;
    function AddPlot(zIndex: integer; aPlotGeometry: TThreeJsGeometry; aZPos: double; aColor: TColor): TThreeJsObject3D;

    procedure AddTitles;
    procedure EmbedChartItemInformation(theItem: TThreeJsObject3D; zIndex, xIndex: integer);
    procedure EmbedPlotObjectInformation(theItem: TThreeJsObject3D; xFirst, xLast: double; zIndex: integer);
    function HandleObjectClick(chartObject: TThreeJsObject3D; intersection: TThreeJsIntersection): boolean; override;
    procedure HandleObjectExit(chartObject: TThreeJsObject3D); override;
    procedure HandleObjectDblClick(chartObject: TThreeJsObject3D); override;
    procedure HandleObjectMouseMove(chartObject: TThreeJsObject3D; intersection: TThreeJsIntersection); override;
    procedure HandleObjectMouseEnter(chartObject: TThreeJsObject3D); override;
    procedure HandleObjectMouseLeave(chartObject: TThreeJsObject3D); override;
    function GetGlowChartItem(chartItem: TThreeJsObject3D): boolean;
    procedure SetGlowChartItem(chartItem: TThreeJsObject3D; avalue: boolean);
    function GetItemValueLabelObject(xIndex, zIndex: integer): TThreeJsObject3D;
    function GetMaxXExtent: double;
    function GetMaxZExtent: double;
  public
    procedure CreateInitialize; override;

    procedure CreateChartFromData; override;
    procedure ClearItemLabels;

    function IsPlotTypeChart: boolean;

    // For plot type objects, xIndex is ignored
    function GetItemObject(xIndex, zIndex: integer): TThreeJsObject3D;
    function ShowChartValue(xIndex, zIndex: integer; aValueStr: string): TThreeJsObject3D;
    procedure RemoveChartValue(aChartItem: TThreeJsObject3D);

    property Series: TThreeJsChartSeries read FSeries write FSeries;

    property ShowValuesOnItems: boolean read FShowValuesOnItems write FShowValuesOnItems;

    property Glow[chartItem: TThreeJsObject3D]: boolean read getGlowChartItem write setGlowChartItem;

    property SizeXAxis: double read FSizeXAxis write FSizeXAxis;
    property SizeZAxis: double read FSizeZAxis write FSizeZAxis;
  published
    property OnClick;
    property OnDblClick;

    property OnItemClick: TThreeJsItemEvent read FOnItemClick write FOnItemClick;
    property OnItemExit: TThreeJsItemNotifyEvent read FOnItemExit write FOnItemExit;
    property OnItemDblClick: TThreeJsItemNotifyEvent read FOnItemDblClick write FOnItemDblClick;
    property OnItemMouseMove: TThreeJsItemEvent read FOnItemMouseMove write FOnItemMouseMove;
    property OnItemMouseEnter: TThreeJsItemNotifyEvent read FOnItemMouseEnter write FOnItemMouseEnter;
    property OnItemMouseLeave: TThreeJsItemNotifyEvent read FOnItemMouseLeave write FOnItemMouseLeave;
  end;

  TWebThreeJsChart = class(TThreeJsChart);


implementation
end.
